package rwd.concretesyntax;

import java.awt.*;
import java.awt.geom.*;
import java.util.*;

import rwd.concretesyntax.eulerdiagram.*;
import rwd.concretesyntax.spiderdiagram.*;
import rwd.concretesyntax.constraintdiagram.*;

import rwd.abstractsyntax.spiderdiagram.*;

import rwd.utilities.AreaRWD;

/**
 * Contains all the graphic objects contained in the 
 * editor's internal frames.
 * 
 * @author Octavian Patrascoiu, Richard Bosworth
 * @version 25.8.05   11:06
 */
public class ConcreteDiagram
	extends Observable
	implements java.io.Serializable
{
    public static int SCALE = 1500;

    //
    // constructor
    //
    
    public ConcreteDiagram() {
    }
    

    //
	// Decoration box methods
	//
    
	public DecorationSet getDecorationSet() {
		return decorationSet;
	}
	
	public void setDecorationBoxes(DecorationSet Set) {
		decorationSet = Set;
	}
	
	public void addDecorationBox(DecorationBox db) {
		if (!decorationSet.getBoxes().contains(db)) {
			decorationSet.addBox(db);
			addPolygonal(db);
		}
	}
		
	public void removeDecorationBox(DecorationBox db) {
		decorationSet.removeBox(db);
		removePolygonal(db);
	}
	
	
	//
	// Concrete contours methods
	//
	
	public ConcreteContourSet getConcreteContourSet() {
		return concreteContourSet;
	}
	
	public void setConcreteContourSet(ConcreteContourSet concreteContours) {
		this.concreteContourSet = concreteContours;
	}
		
	public Set getConcreteContours() {
		return new LinkedHashSet(concreteContourSet.getContours());
	}
	
	public void addConcreteContours(Set set) {
		Iterator i = set.iterator();
		while (i.hasNext()) {
			ConcreteContour c = (ConcreteContour)i.next();
			addConcreteContour(c);			
		}
	}
	
	public void addConcreteContour(ConcreteContour c) {
		concreteContourSet.addContour(c);
		addPolygonal(c);
	}

	public void addConcreteArc(ConcreteArc ca) {
		arcSet.addArc(ca);
	}
		
	public void removeConcreteContour(ConcreteContour c) {
		concreteContourSet.removeContour(c);
		removePolygonal(c);
	}
	
	private void addPolygonal(Polygonal p) {
		//System.out.println("adding polygonal"+p);
		ConcreteZoneSet newCZones = new ConcreteZoneSet();
		// Remove empty zone
		newCZones.getZones().clear();
		// RECALC CONCRETE ZONES
		// for each old zone, split it into 2 and copy colours
		Iterator i = concreteZoneSet.getZones().iterator();
		while (i.hasNext()) {
			ConcreteZone cz = (ConcreteZone) i.next();
			// build partner zone inside c
			Vector conts = (Vector) cz.getContainingPolygonals().clone();
			//System.out.println("containing ones were "+conts);
			conts.add(p);
			//System.out.println("containing ones are "+conts);
			ConcreteZone inc =
				new ConcreteZone(conts, cz.getExcludingPolygonals());
			inc.setBackColor(cz.getBackColor());
			inc.setForeColor(cz.getForeColor());
			newCZones.addZone(inc);
			//System.out.println("new zone is "+inc);
			Vector excs = (Vector) cz.getExcludingPolygonals().clone();
			//System.out.println("excluding ones were "+excs);
			excs.add(p);
			//System.out.println("excluding ones are "+excs);
			ConcreteZone exc = new ConcreteZone(cz.getContainingPolygonals(), excs);
			exc.setBackColor(cz.getBackColor());
			exc.setForeColor(cz.getForeColor());
			newCZones.addZone(exc);
			//System.out.println("new zone is "+exc);
		}
		setConcreteZoneSet(newCZones);
	}
	
	private void removePolygonal(Polygonal p) {
		ConcreteZoneSet newCZones = new ConcreteZoneSet();
		// RECALC CONCRETE ZONES
		// for each old zone outside p, 
		// flatten into 1 and copy colours
		Iterator i = concreteZoneSet.getZones().iterator();
		while (i.hasNext()) {
			ConcreteZone cz = (ConcreteZone) i.next();
			Vector excs = cz.getExcludingPolygonals();
			if (excs.contains(p)) {
				excs = (Vector) excs.clone();
				excs.remove(p);
				ConcreteZone newcz =
					new ConcreteZone(cz.getContainingPolygonals(), excs);
				newcz.setBackColor(cz.getBackColor());
				newcz.setForeColor(cz.getForeColor());
				newCZones.addZone(newcz);
			}
		}
		setConcreteZoneSet(newCZones);
	}
	
	
	/** Get the contours contained within a box */
	public ConcreteContourSet getConcreteContours(DecorationBox box) {
		ConcreteContourSet result = new ConcreteContourSet();
		Iterator i = getConcreteContours().iterator();
		while (i.hasNext()) {
			ConcreteContour c = (ConcreteContour) i.next();
			Rectangle2D rect = c.getPolygon().getBounds2D();
			if (box.getPolygon().contains(rect)) {
				result.addContour(c);
			}
		}
		return result;
	}

	
	//
	// Concrete zones methods
	//
	
	public ConcreteZoneSet getConcreteZoneSet() {
		return concreteZoneSet;
	}
	
	public void setConcreteZoneSet(ConcreteZoneSet set) {
		concreteZoneSet = set;
	}
    
	public Set getConcreteZones() {
        return new LinkedHashSet(concreteZoneSet.getVisibleConcreteZones());
    }
	
	public Set getVisibleConcreteZones() {
		return concreteZoneSet.getVisibleConcreteZones();
	}


	/** Refresh all the zones */
	public void refreshZones(boolean smoothed) {
        Iterator i = concreteContourSet.getContours().iterator();
        while(i.hasNext()){
            ConcreteContour cc = (ConcreteContour)i.next();
            cc.getPolygon().invalidate();
        }
		i = concreteZoneSet.getZones().iterator();
		while (i.hasNext()) {
			ConcreteZone cz = (ConcreteZone) i.next();
			cz.refreshArea(smoothed);
		}
	}
	
	
	/** Get the zones contained within a box */
	public ConcreteZoneSet getConcreteZones(DecorationBox box) {
		// Compute the cleaning contours
		java.util.List allContours = getConcreteContours(box).getContours();
		allContours.add(box);
		// Scan every zone, clean it, and add it
		ConcreteZoneSet result = new ConcreteZoneSet();
		Set zones = getConcreteZones();
		Iterator i = zones.iterator();
		while (i.hasNext()) {
			ConcreteZone cz = (ConcreteZone) i.next();
			Vector inPolys = cz.getContainingPolygonals(); 
			if (inPolys.contains(box)) {
				// Clean contours and create a new ConcreteZone
				Vector cps = new Vector();
				cps.addAll(cz.getContainingPolygonals());
				Vector eps = new Vector();
				eps.addAll(cz.getExcludingPolygonals());
				cps.retainAll(allContours);
				eps.retainAll(allContours);
				ConcreteZone ncz = new ConcreteZone(cps, eps);
				ncz.setAttributes(cz);
				// Add abstract zone
				result.addZone(ncz);
			}
		}
		return result;
	}
	
	
	/** Get the zones included in a contour */
	public Set getConcreteZones(ConcreteContour contour) {
		Set res = new LinkedHashSet();
		Set zones = getConcreteZones();
		Iterator i = zones.iterator();
		while (i.hasNext()) {
			ConcreteZone cz = (ConcreteZone)i.next();
			Vector inPolys = cz.getContainingPolygonals(); 
			if (inPolys.contains(contour)) {
				res.add(cz);
			}
		}
		return res;
	}
	
	
	/** Get the zone touched by a foot */
	public ConcreteZone getConcreteZone(ConcreteSpiderFoot foot) {
		ConcreteZone result = null;
		Iterator i = getConcreteZones().iterator();
		while (i.hasNext()) {
			// Check if foot is in zone
			Point pFoot = new Point(foot.getX(), foot.getY());
			ConcreteZone cz = (ConcreteZone) i.next();
			AreaRWD zArea = cz.getArea();
			if (zArea.contains(pFoot)) {
				return cz;
			}
		}
		return result;
	}
	
	
	/** Get the feet in a zone */
	public Set getFeet(ConcreteZone cz) {
		Set result = new LinkedHashSet();
		if (cz == null) return result;
		AreaRWD zArea = cz.getArea();
		if (zArea == null) return result;
		Set feet = getConcreteFeet();
		Iterator i = feet.iterator();
		while (i.hasNext()) {
			// Check if foot is in zone
			ConcreteSpiderFoot foot = (ConcreteSpiderFoot)i.next();
			Point pFoot = new Point(foot.getX(), foot.getY());
			if (zArea.contains(pFoot)) {
				result.add(foot);
			}
		}
		return result;
	}
	
	
	/** Get shaded zones */
	public Set getSConcreteZones() {
		LinkedHashSet result = new LinkedHashSet();
		Iterator i = concreteZoneSet.getZones().iterator();
		while(i.hasNext()){
			ConcreteZone cz = (ConcreteZone)i.next();
			if(cz.isShaded()){
				result.add(cz); 
			}
		}
		return result;
	}
	
	
	//
	// Concrete spider feet and legs methods
	//
	
	public ConcreteSpiderFeetAndLegsSet getFeetAndLegsSet() {
		return this.lfs;
	}
	
	public void setFeetAndLegsSet(ConcreteSpiderFeetAndLegsSet lfs) {
		this.lfs = lfs;
//		this.concreteSpiders = computeSpiders(new LinkedHashSet(lfs.getLegs()), new LinkedHashSet(lfs.getFeet()));
	}

	
	//
	// Concrete feet methods
	// 
	
	public Set getConcreteFeet() {
		return new LinkedHashSet(this.lfs.getFeet());
	}
	
	public void addConcreteFoot(ConcreteSpiderFoot csf) {
		lfs.addFoot(csf);
	}
	
	public void removeConcreteFoot(ConcreteSpiderFoot csf) {
		lfs.removeFoot(csf);
	}
	
	
	/** Get all the feet contained into a box */
	public Set getFeet(DecorationBox box) {
		Polygon polyBox = box.getPolygon();
		// Compute the Set of feet
		Set feet = new LinkedHashSet();
		Iterator ife = lfs.getFeet().iterator();
		while (ife.hasNext()) {
			ConcreteSpiderFoot foot = (ConcreteSpiderFoot) ife.next();
			// Check if foot is in zone
			Point pFoot = new Point(foot.getX(), foot.getY());
			if (polyBox.contains(pFoot)) {
				// Add foot to Set
				feet.add(foot);
			}
		}
		return feet;
	}
	
	
	//
	// Concrete legs methods
	//
	
	public Set getConcreteLegs() {
		return new LinkedHashSet(lfs.getLegs());
	}
	
	public void addConcreteLeg(ConcreteSpiderLeg csl) {
		lfs.addLeg(csl);
	}
	
	public void removeConcreteLeg(ConcreteSpiderLeg csl) {
		lfs.removeLeg(csl);
	}
	
	
	/** Get all the legs contained into a box */
	public Set getLegs(DecorationBox box) {
		// Compute legs within box
		Set res = new LinkedHashSet();
		// Scan all the legs
		Set legs = this.getConcreteLegs();
		AreaRWD area = new AreaRWD(box.getPolygon());
		Iterator i = legs.iterator();
		while (i.hasNext()) {
			ConcreteSpiderLeg leg = (ConcreteSpiderLeg) i.next();
			ConcreteSpiderFoot foot1 = leg.getSource();
			ConcreteSpiderFoot foot2 = leg.getTarget();
			if (area.contains(foot1.getX(), foot1.getY())
				&& area.contains(foot2.getX(), foot2.getY())) {
				res.add(leg);
			}
		}
		return res;
	}
	
	
	//
	// Concrete spider methods
	//
    
	public Set getConcreteSpiders() {
    	return new LinkedHashSet(lfs.getSpiders());
    }
    
    
	/**
	 * Post: cs has been added to the diagram.
	 * @param cs concrete spider to be added to this diagram
	 */
	/*
	 * test: 
	 *   spider with no feet and legs
	 *   spider with three feet and two legs
	 */
	public void addConcreteSpider(ConcreteSpider cs){
    	// add feet
		Set feet = cs.getFeet();
		Iterator fit = feet.iterator();
		while (fit.hasNext())
			addConcreteFoot((ConcreteSpiderFoot)fit.next());

		// add legs
		Set legs = cs.getLegs();
		Iterator lit = legs.iterator();
		while (lit.hasNext())
			addConcreteLeg((ConcreteSpiderLeg)lit.next());
    }
    
    public void addAbstractSpider(Spider s) {
    	this.lfs.addAbstractSpider(this, s);
    }
	
    public ConcreteSpider getConcreteSpider(ConcreteSpiderFoot foot) {
		java.util.List spiders = lfs.getSpiders();
		for(int i=0; i<spiders.size(); i++) {
			ConcreteSpider spider = (ConcreteSpider)spiders.get(i);
			if (spider.getFeet().contains(foot))
				return spider;
		}
		return null;
	}
	
    
    /**
	 * Get the spider labeled with spName
	 */
	public ConcreteSpider getConcreteSpider(String spName) {
		// Find the foor
		Iterator i = this.lfs.getFeet().iterator();
		while (i.hasNext()) {
			ConcreteSpiderFoot foot = (ConcreteSpiderFoot)i.next();
			String label = foot.getLabel();
			if (label.equals(spName)) {
				// Found it
				ConcreteSpider sp = getConcreteSpider(foot);
				return sp;				
			}
		}
		return null;
	}
    
	
	/** Compute the spiders starting from legs and feet */
	private Map foot2spider = new HashMap();

	
	//
	// Concrete arcs
	//
	
	public ConcreteArcSet getArcSet() {
		return arcSet;
	}
	
	public void setArcSet(ConcreteArcSet set) {
		arcSet = set;
	}
	
	public void removeConcreteArc(ConcreteArc ca) {
		arcSet.removeArc(ca);
	}
	
	public Set getConcreteArcs() {
		return new LinkedHashSet(this.arcSet.getArcs());
	}
	
	
	/** Get all the arcs contained into a box */
	public Set getConcreteArcs(DecorationBox box) {
		// Compute legs within box
		Set res = new LinkedHashSet();
		// Scan all the arcs
		Set arcs = this.getConcreteArcs();
		Rectangle rect = box.getPolygon().getBoundingBox();
//		AreaRWD area = new AreaRWD(box.getPolygon());
		Iterator i = arcs.iterator();
		while (i.hasNext()) {
			ConcreteArc arc = (ConcreteArc) i.next();
			ConcreteArcEnd end1 = arc.getStart();
//			ConcreteArcEnd end2 = arc.getMiddle();
			ConcreteArcEnd end3 = arc.getEnd();
			if (rect.contains(end1.getX(), end1.getY())
//				&& rect.contains(end2.getX(), end2.getY())
				&& rect.contains(end3.getX(), end3.getY())) {
				res.add(arc);
			}
		}
		return res;
	}

	
	//
	// Text box set methods
	//
	
	public ConcreteTextBoxSet getTextSet() {
		return textSet;
	}
	
	public void setTextSet(ConcreteTextBoxSet set) {
		textSet = set;
	}
	
	public void addTextBox(ConcreteTextBox text) {
		textSet.addTextBox(text);
	}
	
	public void removeTextBox(ConcreteTextBox text) {
		textSet.removeTextBox(text);
	}
	
	public Set getConcreteTextBoxes() {
		return new LinkedHashSet(this.textSet.getTexts());
	}
	
	
	/** Get all the text boxes contained into a box */
	public Set getConcreteTextBoxes(DecorationBox box) {
		// Compute legs within box
		Set res = new LinkedHashSet();
		// Scan all the text boxes
		Set texts = this.getConcreteTextBoxes();
		Polygon poly = box.getPolygon();
		AreaRWD area = new AreaRWD(box.getPolygon());
		Iterator i = texts.iterator();
		while (i.hasNext()) {
			ConcreteTextBox text = (ConcreteTextBox) i.next();
			Shape shape = text.getShape();
			if (poly.contains(shape.getBounds2D())) {
				res.add(text);
			}
		}
		return res;
	}
	
	
	//
	// Diagram items
	//
	
	/** Get the diagram items */
	public java.util.List getDiagramItems() {
		java.util.List res = new Vector();
//		res.addAll(this.getConcreteContours());
//		res.addAll(this.getConcreteZones());
		res.addAll(this.getConcreteLegs());
		res.addAll(this.getConcreteFeet());
		res.addAll(this.getConcreteArcs());
		return res;
	}
	
	
	//
	// Unitary diagrams
	//
	
	/** Get the set of unitary diagrams */
	public Set getConcreteUnitaryDiagrams() {
		Set cuds = new LinkedHashSet();
		// For each box
		Iterator ib = decorationSet.getBoxes().iterator();
		while (ib.hasNext()) {
			DecorationBox box = (DecorationBox) ib.next();
			// Set title
			ConcreteUnitaryDiagram cud = new ConcreteUnitaryDiagram(box);
			// Get contours
			ConcreteContourSet concreteContours = this.getConcreteContours(box);
			// Get zones
			ConcreteZoneSet concreteZones = this.getConcreteZones(box);
			// Set contours, zones, and shaded zones
			cud.setConcreteContourSet(concreteContours);
			cud.setConcreteZoneSet(concreteZones);
			// Set spider feet and legs
			Set feet = getFeet(box);
			Set legs = getLegs(box);
			ConcreteSpiderFeetAndLegsSet lfs = new ConcreteSpiderFeetAndLegsSet(feet, legs); 
			cud.setFeetAndLegsSet(lfs);
			// Set arcs
			Set arcs = getConcreteArcs(box);
			ConcreteArcSet arcSet = new ConcreteArcSet(new Vector(arcs));
			cud.setArcSet(arcSet);
			// Set text
			Set texts = getConcreteTextBoxes(box);
			ConcreteTextBoxSet textBoxSet = new ConcreteTextBoxSet(new Vector(texts));
			cud.setArcSet(arcSet);
			// Add cd to res
			cuds.add(cud);
			// Add observers (viewer and metrics)
			cud.addObservers(observers);
		}
		return cuds;
	}

	
	//
	// Abstract unitary diagrams
	//
	
	public Set getAbstractUnitaryDiagrams() {
		Set cuds = getConcreteUnitaryDiagrams();
		Set auds = new LinkedHashSet();
		// For each concrete unitary diagram
		Iterator i = cuds.iterator();
		while (i.hasNext()) {
			ConcreteUnitaryDiagram cud = (ConcreteUnitaryDiagram) i.next();
			auds.add(cud.getAbstractUnitaryDiagram());
		}
		return cuds;
	}

	
	//
	// Observers
	//
	
	public void deactivateDiagramItemObservers() {
		// For each diagram item
		java.util.List items = getDiagramItems();
		for (int i = 0; i < items.size(); i++) {
			// Remove observers
			DiagramItem item = (DiagramItem) items.get(i);
			item.deleteObservers();
		}
	}

	
	/** Override toString */
	public String toString() {
		return "cd";
//	   return getConcreteUnitaryDiagrams().toString();
	}
	
	
	/** Clone */
	public ConcreteDiagram getClone() {
		ConcreteDiagram res = new ConcreteDiagram();
		res.decorationSet = this.decorationSet.getClone();
		res.concreteContourSet = this.concreteContourSet.getClone();
		res.concreteZoneSet = this.concreteZoneSet.getClone();
		res.lfs = this.lfs.getClone();
		res.arcSet = this.arcSet.getClone();
		res.textSet = this.textSet.getClone();
		return res;
	}
        
    
	//
    // Local properties
    //
	
	/** Concrete diagrams boxes (decoration boxes) */
	protected DecorationSet decorationSet = new DecorationSet();
	
	/** Concrete contours */
	protected ConcreteContourSet concreteContourSet = new ConcreteContourSet();
	
	/** Concrete zones */
	protected ConcreteZoneSet concreteZoneSet = new ConcreteZoneSet();
	
	/** Concrete feet and legs */
	protected ConcreteSpiderFeetAndLegsSet lfs = new ConcreteSpiderFeetAndLegsSet();
	
	/** Concrete spiders */
//	protected Set concreteSpiders = new LinkedHashSet();
	
	/** Arc set */
	protected ConcreteArcSet arcSet = new ConcreteArcSet();
	
	/** Text box set */
	protected ConcreteTextBoxSet textSet = new ConcreteTextBoxSet();
	
	/*
	// test program
    // ------------

    private static void pr(String s) {
      System.out.println(s);
    }

    private static void hd(String s) {
      System.out.println("\n" + s + "\n");
    }

    private static void pg() {
      try {
        int i;
        System.out.print("next page? (CR)");
        do i = System.in.read(); while (i != '\n');
        System.out.println();
      }
      catch (Exception e) {System.out.println("raised exception : " + e);}
    }

    public static void main(String args[]) throws Exception {
    	hd("Tests of addConcreteSpider");
    	ConcreteDiagram cd = new ConcreteDiagram();
    	pr("cd="+cd);
    	pr("cd.getFeetAndLegsSet()="+
    		 cd.getFeetAndLegsSet());
    	pg();
    	
    	ConcreteSpider cs1 = new ConcreteSpider();
    	pr("cs1="+cs1);
    	pr("cd.addConcreteSpider(cs1)");
    	    cd.addConcreteSpider(cs1);
    	pr("cd="+cd);
    	pr("cd.getFeetAndLegsSet()="+
       		 cd.getFeetAndLegsSet());
    	pg();
    	
    	ConcreteSpider cs2 = new ConcreteSpider();
    	ConcreteSpiderFoot csf1 = 
      	  new ConcreteSpiderFoot("csf1",ConcreteSpiderFoot.NORMAL);
    	csf1.setLocation(1,2);
      	cs2.addFoot(csf1);
    	ConcreteSpiderFoot csf2 = 
          new ConcreteSpiderFoot("csf2",ConcreteSpiderFoot.SCHROEDINGER);
    	csf2.setLocation(3,4);
        cs2.addFoot(csf2);
        ConcreteSpiderFoot csf3 = 
            new ConcreteSpiderFoot("csf3",ConcreteSpiderFoot.UNIVERSAL);
    	csf3.setLocation(5,6);
        cs2.addFoot(csf3);
        ConcreteSpiderLeg csl12 = 
          new ConcreteSpiderLeg("csl12",csf1,csf2,ConcreteSpiderLeg.NORMAL);
        cs2.addLeg(csl12);
        ConcreteSpiderLeg csl23 = 
            new ConcreteSpiderLeg("csl23",csf2,csf3,ConcreteSpiderLeg.STRAND);
        cs2.addLeg(csl23);
    	pr("cs2="+cs2);
    	pr("cd.addConcreteSpider(cs2)");
	    cd.addConcreteSpider(cs2);
	    pr("cd="+cd);
	    pr("cd.getFeetAndLegsSet()="+
   		    cd.getFeetAndLegsSet());
	    pg();
    }
    */
}
