package rwd.concretesyntax;

import java.util.Vector;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;

import rwd.utilities.*;
import rwd.abstractsyntax.eulerdiagram.*;
import rwd.abstractsyntax.spiderdiagram.*;
import rwd.concretesyntax.eulerdiagram.ConcreteContour;
import rwd.concretesyntax.eulerdiagram.ConcreteZone;
import rwd.concretesyntax.eulerdiagram.DecorationBox;
import rwd.concretesyntax.eulerdiagram.Polygonal;
import rwd.concretesyntax.spiderdiagram.ConcreteSpiderFoot;
import rwd.concretesyntax.spiderdiagram.ConcreteSpiderLeg;

/**
 * @author jaf28
 *
 */
public class ConcreteUnitaryConverter {
    private static Set extractUnitaryBoxes(ConcreteDiagram cd) {
        // lazy to start with just get boxes
        // in the end, look for boxes which contain syntactically
        // correct unitary diagrams
        return new LinkedHashSet(cd.getDecorationSet().getBoxes());
    }

    public static Vector getNamesOfUnitaryBoxes(ConcreteDiagram cd) {
        Vector result = new Vector();
        Iterator i = extractUnitaryBoxes(cd).iterator();
        while (i.hasNext()) {
            result.add(((DecorationBox) i.next()).getLabel());
        }
        return result;
    }

    private static DecorationBox getUnitaryBoxCalled(
        ConcreteDiagram cd,
        String name) {
        Iterator i = extractUnitaryBoxes(cd).iterator();
        while (i.hasNext()) {
            DecorationBox box = (DecorationBox) i.next();
            System.out.println("checking box " + box.getLabel());
            if (box.getLabel().equals(name)) {
                return box;
            }
        }
        return null;
    }

    private static Vector getConcreteContours(
        ConcreteDiagram cd,
        DecorationBox box) {
        Vector result = new Vector();
        AreaRWD boxInterior = new AreaRWD(box.getShape());
        Iterator i = cd.getConcreteContours().iterator();
        while (i.hasNext()) {
            Object o = i.next();
            if (o instanceof ConcreteContour) {
               ConcreteContour c = (ConcreteContour)o;
               AreaRWD cInterior = new AreaRWD(c.getShape());
               // find out whether cInterior is contained within boxInterior
               cInterior.subtract(boxInterior);
               if (cInterior.isEmpty()) {
                   result.add(c);
               }
            } 
        }
        System.out.println("relevant concrete contours are " + result);
        return result;
    }

    private static Vector getConcreteZones(
        ConcreteDiagram cd,
        DecorationBox box) {

        Vector result = new Vector();
        AreaRWD boxInterior = new AreaRWD(box.getShape());
        Iterator i = cd.getConcreteZones().iterator();
        while (i.hasNext()) {
            ConcreteZone z = (ConcreteZone) i.next();
            AreaRWD zArea = z.getArea();
            if (zArea != null && !zArea.isEmpty()) {
                AreaRWD zInterior = new AreaRWD(z.getArea());
                zInterior.subtract(boxInterior);
                if (zInterior.isEmpty()) {
                    result.add(z);
                }
            }
        }
        System.out.println("relevant zones are  " + result);
        return result;
    }

    private static Vector getConcreteSpiderFeet(
        ConcreteDiagram cd,
        DecorationBox box) {
        System.out.println("getting spider feet");
        Vector result = new Vector();
        AreaRWD boxInterior = new AreaRWD(box.getShape());
        Iterator i = cd.getConcreteFeet().iterator();
        while (i.hasNext()) {
            System.out.println("found a foot : is it in right box?");
            ConcreteSpiderFoot csf = (ConcreteSpiderFoot) i.next();
            if (boxInterior.contains(csf.getX(), csf.getY())) {
                System.out.println("Yes");
                result.add(csf);
            } else{ 
                System.out.println("No");
            }
        }
        return result;
    }

    private static Vector getConcreteSpiderLegs(
        ConcreteDiagram cd,
        DecorationBox box) {
        Vector result = new Vector();
        AreaRWD boxInterior = new AreaRWD(box.getShape());
        Iterator i = cd.getConcreteLegs().iterator();
        while (i.hasNext()) {
            ConcreteSpiderLeg csl = (ConcreteSpiderLeg) i.next();
            AreaRWD legShape = new AreaRWD(csl.getShape());
            legShape.subtract(boxInterior);
            if (legShape.isEmpty()) {
                result.add(csl);
            }
        }
        return result;
    }

    public static UnitaryDiagram makeAUD(ConcreteUnitaryDiagram cud) {
        String name = (String) getNamesOfUnitaryBoxes(cud).get(0);
        return (UnitaryDiagram)
            (mapUnitaryDiagramCalled(cud, name).get("unitaryDiagram"));
    }

    public static HashMap mapUnitaryDiagramCalled(
        ConcreteDiagram cd,
        String name) {
        HashMap result = new HashMap();

        DecorationBox boundingBox = getUnitaryBoxCalled(cd, name);
        System.out.println("box is " + boundingBox);

        if (boundingBox == null) {
            return null;
        }

        List cpolys = getConcreteContours(cd, boundingBox);
        List zAreas = getConcreteZones(cd, boundingBox);
        System.out.println("zAreas is " + zAreas);
        List getSpFt = getConcreteSpiderFeet(cd, boundingBox);
        List getSpLgs = getConcreteSpiderLegs(cd, boundingBox);

        // should check that cpolys and zAreas are consistent...
        // will assume that for now
        Vector zones = new Vector();
        Vector szones = new Vector();
        Vector czones = new Vector();

        // for each ConcreteZone inside box, build an abstract Zone 
        // and maybe shade it
        ZAreasLoop : for (int i = 0; i < zAreas.size(); i++) {
            ConcreteZone cz = (ConcreteZone) zAreas.get(i);
            System.out.println("got a concrete zone " + cz);
            // Compute abstract zone
            Vector containingCs = new Vector();
            Vector containingCCs =
                (Vector) cz.getContainingPolygonals().clone();
            System.out.println("contained by " + containingCCs);

            for (int j = 0; j < containingCCs.size(); j++) {
                Polygonal p = (Polygonal) containingCCs.get(j);
                if (p instanceof ConcreteContour) {
                    ConcreteContour cc = (ConcreteContour) p;
                    if (cpolys.contains(cc)) {
                        Contour c = Contour.getContour(cc.getLabel());
                        result.put(cc, c);
                        result.put(c, cc);
                        containingCs.add(c);
                    }
                }
            }
            Vector excludingCs = new Vector();
            Vector excludingCCs =
                (Vector) cz.getExcludingPolygonals().clone();
            System.out.println("excluded by " + excludingCCs);
            for (int j = 0; j < excludingCCs.size(); j++) {
                Polygonal p = (Polygonal) excludingCCs.get(j);
                if (cpolys.contains(p) && p instanceof ConcreteContour) {
                    ConcreteContour cc = (ConcreteContour) p;
                    Contour c = Contour.getContour(cc.getLabel());
                    result.put(cc, c);
                    result.put(c, cc);
                    excludingCs.add(c);
                }
            }
            Zone z = Zone.getZone(containingCs, excludingCs);
            System.out.println("built an abstract zone " + z);
            zones.add(z);
            if (cz.isShaded()) {
                szones.add(z);
            }
            result.put(cz, z);
            result.put(z, cz);
            czones.add(cz);
        }

        // Create the unitary diagram
        UnitaryDiagram ud = new UnitaryDiagram(zones, boundingBox.getLabel());
        ud.setShadedZones(szones);
        result.put("unitaryDiagram", ud);
        result.put("czones", czones);

        // now build spiders from the feet and legs.
        List feet = (List) ((Vector) getSpFt).clone();
        List legs = (List) ((Vector) getSpLgs).clone();
        System.out.println("we have "+feet.size()+" feet and "+legs.size()+" legs");
        while (feet.size() > 0) {
            // we need to create a spider
            // choose a foot
            Vector connectedFeet = new Vector();
            ConcreteSpiderFoot sf = (ConcreteSpiderFoot) feet.get(0);
            feet.remove(sf);
            connectedFeet.add(sf);
            boolean hasAddedAFoot = true;
            while (hasAddedAFoot) {
                hasAddedAFoot = false;
                // scan through the legs seeing if a leg attaches to anything in connectedFeet
                for (int i = 0; i < legs.size(); i++) {
                    ConcreteSpiderLeg csl = (ConcreteSpiderLeg) legs.get(i);
                    ConcreteSpiderFoot source = csl.getSource();
                    ConcreteSpiderFoot target = csl.getTarget();
                    if (connectedFeet.contains(source)) {
                        connectedFeet.add(target);
                        feet.remove(target);
                        legs.remove(csl);
                        i--;
                        hasAddedAFoot = true;
                    } else if (connectedFeet.contains(target)) {
                        connectedFeet.add(source);
                        feet.remove(source);
                        legs.remove(csl);
                        i--;
                        hasAddedAFoot = true;
                    }
                }
            }
            // have "feet" an Arraylist of ConcreteSpiderFeet
            Vector habitat = new Vector();
            footLoop : for (int i = 0; i < connectedFeet.size(); i++) {
                ConcreteSpiderFoot csf =
                    (ConcreteSpiderFoot) connectedFeet.get(i);
                System.out.println(
                    ""
                        + i
                        + "th spider foot is "
                        + csf
                        + "("
                        + csf.getX()
                        + ","
                        + csf.getY()
                        + ")");
                // work out which zone it's in
                zoneSearch : for (int j = 0; j < zAreas.size(); j++) {
                    ConcreteZone cz = (ConcreteZone) zAreas.get(j);
                    System.out.println("searching in zone " + cz);
                    AreaRWD a = cz.getArea();
                    System.out.println("searching in area " + a);
                    if (a.contains(csf.getX(), csf.getY())) {
                        if (habitat.contains(zones.get(j))) {
                            // should never get here : two feet in one zone
//                            throw new IllegalArgumentException();
                        }
                        System.out.println("add this zone to habitat");
                        habitat.add(zones.get(j));
                        System.out.println("habitat is now " + habitat);
                        continue footLoop;
                    }
                }
                // should never get here : foot in no zone!
                System.out.println("foot is in no zone");
                throw new IllegalArgumentException();
            }
            ud.addSpider(habitat);
        }
        System.out.println("returning ud " + DiagramFileUser.toSaveString(ud));
        return result;
    }
}
