package rwd.concretesyntax;

import java.awt.Polygon;
import java.awt.geom.PathIterator;
import java.util.Vector;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;

import rwd.abstractsyntax.constraintdiagram.Arrow;
import rwd.abstractsyntax.eulerdiagram.Contour;
import rwd.abstractsyntax.eulerdiagram.Zone;
import rwd.abstractsyntax.spiderdiagram.Spider;
import rwd.abstractsyntax.spiderdiagram.UnitaryDiagram;
import rwd.abstractsyntax.spiderdiagram.Utilities;
import rwd.concretesyntax.constraintdiagram.ConcreteArc;
import rwd.concretesyntax.eulerdiagram.ConcreteContour;
import rwd.concretesyntax.eulerdiagram.ConcreteContourSet;
import rwd.concretesyntax.eulerdiagram.ConcreteZone;
import rwd.concretesyntax.eulerdiagram.DecorationBox;
import rwd.concretesyntax.eulerdiagram.Polygonal;
import rwd.concretesyntax.spiderdiagram.ConcreteSpider;
import rwd.concretesyntax.spiderdiagram.ConcreteSpiderFoot;
import rwd.ocl.constraintdiagram.DependenceGraph;
import rwd.ocl.constraintdiagram.Graph;
import rwd.ocl.constraintdiagram.ReadingTree;
import rwd.ocl.constraintdiagram.ReadingTreeNode;
import uk.ac.kent.cs.kmf.util.ILog;

/**
 * @author Octavian Patrascoiu, rnb
 * @version 8.9.05   12:34
 *
 */
public class ConcreteUnitaryDiagram 
	extends ConcreteDiagram 
{
    //
    // Constructors
    //
    public ConcreteUnitaryDiagram(DecorationBox box) {
        this.box = box;
        addDecorationBox(box);
    }    

    /** Decoration box */
    private DecorationBox box;
    public DecorationBox getDecorationBox() {
        return box;
    }

    /** Name */
    public String getName(){
        return box.getLabel();
    }

	public String getReadingForest() {
		if (box != null) {
			String res = box.getReadingForest();
			if (res != null)
				return res;
		}
		return "";
	}

	/** Add points until all contours have n points */
	public void equalisePointNumber() {
		int max = 0;
		Iterator ccIt = getConcreteContours().iterator();
		while (ccIt.hasNext()) {
			ConcreteContour cc = (ConcreteContour) (ccIt.next());
			Polygon p = cc.getPolygon();
			int npoints = p.npoints;
			if (p.npoints > max) {
				max = p.npoints;
			}
		}
		addMorePoints(max);		
	} 
	public void addMorePoints(int n) { 
		// iterate through polys and check they have >=8 points
		// otherwise add some at midpoints 
		Iterator ccIt = getConcreteContours().iterator();
		while (ccIt.hasNext()) {
			boolean changed = false;
			ConcreteContour cc = (ConcreteContour) (ccIt.next());
			Polygon p = cc.getPolygon();
			int npoints;
			while ((npoints = p.npoints) < n) {
				changed = true;
				int[] newxs = new int[2 * npoints];
				int[] newys = new int[2 * npoints];
				int evenIndex = 0;
				PathIterator pi = p.getPathIterator(null);
				double[] sixDoubles = new double[6];
				int oldx, nextx, oldy, nexty;
				pi.currentSegment(sixDoubles);
				nextx = (int) (sixDoubles[0]);
				nexty = (int) (sixDoubles[1]);
				int firstx = nextx;
				int firsty = nexty;
				while (evenIndex < 2 * npoints - 2) {
					oldx = nextx;
					oldy = nexty;
					// use the "last" coords for old(x, y)
					// now pick up the "next" coords for next(x, y)
					pi.next();
					pi.currentSegment(sixDoubles);
					nextx = (int) (sixDoubles[0]);
					nexty = (int) (sixDoubles[1]);
					newxs[evenIndex] = oldx;
					newys[evenIndex] = oldy;
					newxs[evenIndex + 1] = (oldx + nextx) / 2;
					newys[evenIndex + 1] = (oldy + nexty) / 2;
					evenIndex = evenIndex + 2;
				}
				oldx = nextx;
				oldy = nexty;
				nextx = firstx;
				nexty = firsty;
				newxs[evenIndex] = oldx;
				newys[evenIndex] = oldy;
				newxs[evenIndex + 1] = (oldx + nextx) / 2;
				newys[evenIndex + 1] = (oldy + nexty) / 2;

				p = new Polygon(newxs, newys, 2 * npoints);
				cc.setPolygon(p);
			}
			// Refresh zones and fire change event
			if (changed) 
				this.notifyObservers();
		}
	}

	/** Compute all the concrete zones within the concrete diagram */
	public Set makeAllConcreteZones() {
		Set result = new LinkedHashSet();
		// collect together contours and boxes
		Vector polygonals = new Vector();
		polygonals.addAll(getConcreteContours());
		DecorationBox box = getDecorationBox();
		// for each subset of polygons
		List powerSet = Utilities.powerSet(polygonals);
		for (int i = 0; i < powerSet.size(); i++) {
			Vector subset = (Vector) powerSet.get(i);
			Vector rest = (Vector) polygonals.clone();
			rest.removeAll(subset);
			// Compute the corresponding concrete zone
			if (subset.size() > 0) {
				subset.add(box);
				ConcreteZone cz = new ConcreteZone(subset, rest);
				result.add(cz);
			}
		}
		return result;
	}

	/**
	 * 
	 * @return abstract unitary diagram corresponding to this 
	 *           concrete unitary diagram
	 */
	/*
	 * test:
	 *   blank diagram
	 *   diagram with two zones, one shaded
	 *     two spiders, one labelled, one universal, one existential
	 *       two arrows, one labelled
	 *         non-null reading tree
	 */
	public UnitaryDiagram getAbstractUnitaryDiagram() {
		// Compute abstract zones
		Vector aZones = new Vector();
		Vector aShadedZones = new Vector();
		Iterator i = getConcreteZones().iterator();
		while (i.hasNext()) {
			ConcreteZone cz = (ConcreteZone)i.next();
			Zone az = cz.getAbstractZone();
			if (!az.isEmpty())
				aZones.add(az); 
			if (cz.isShaded())
				aShadedZones.add(az);
		}
		// Make an abstract zone
		UnitaryDiagram aud = new UnitaryDiagram(aZones, this.getName());
		// Set shaded zones
		aud.setShadedZones(aShadedZones);
		// Set spiders
		i = getConcreteSpiders().iterator();
		while (i.hasNext()) {
			// Compute habitat for every cs
			ConcreteSpider cs = (ConcreteSpider)i.next();
			
			String label = cs.getLabel();
			Vector habitat = new Vector();
			Iterator j=cs.getFeet().iterator();
			while (j.hasNext()) {
				ConcreteSpiderFoot cf = (ConcreteSpiderFoot)j.next();
				Zone az = this.getConcreteZone(cf).getAbstractZone();
				habitat.add(az);
			}
			// make new spider of correct kind
			Spider s = new Spider(label,habitat);
			if (cs.isExistential())
				s.kind = Spider.EXISTENTIAL;
			else
				s.kind = Spider.UNIVERSAL;
			
			// Add spider			
			aud.addSpider(s);
		}
		// Set arrows
		i = getConcreteArcs().iterator();
		while (i.hasNext()) {
			ConcreteArc arc = (ConcreteArc)i.next();
			Arrow arrow = arc.getAbstractArrow(this);
			aud.addArrow(arrow);
		}
		// Set reading tree
		aud.setReadingForest(getReadingForest());
		
		// Return result
		return aud;  
	}
	
	/** Get the abstract zone */
	public Zone getAbstractZone(ConcreteZone cz) {
		Vector containingContours = new Vector();
		Iterator i = cz.getContainingPolygonals().iterator();
		while (i.hasNext()) {
			Object o = i.next();
			if (o instanceof ConcreteContour) {
				ConcreteContour cc = (ConcreteContour) o;
				Contour c = Contour.getContour(cc.getLabel());
				containingContours.add(c);
			}
		}
		Vector excludingContours = new Vector();
		i = cz.getExcludingPolygonals().iterator();
		while (i.hasNext()) {
			Polygonal p = (Polygonal)i.next();
			if(p instanceof ConcreteContour){
				ConcreteContour cc = (ConcreteContour)p;
				Contour c = Contour.getContour(cc.getLabel());
				excludingContours.add(c);
			}
		}
		return Zone.getZone(containingContours, excludingContours);
	}

	/** Checks if sp1 is a son of sp in the reading tree */
	public boolean isFather(ConcreteSpider sp, ConcreteSpider sp1) {
		DecorationBox box = getDecorationBox();
		String readingForest = box.getReadingForest();
		if (readingForest == null && readingForest.length() == 0) return false;
		ReadingTree tree = new ReadingTree(readingForest, new uk.ac.kent.cs.kmf.util.FileLog("test.tree"));
		return tree.isAncestor(sp.getLabel(), sp1.getLabel());
	}

	/** Checks if the reading tree is valid */
	// Check the following conditions
	// 1. the nodes of RT(d) are the nodes of DG(d) together with the node PTC
	// 2. if n1 is adjancent to n2 in DG(d) then n1 and n2 lie in a common path 
	// in RT(d) directed from the root node PTC
	public boolean isValidTree() {
		return isValidTree(null);
	}
	public boolean isValidTree(ILog log) {
		return isValidTree(getReadingForest(), log);
	}
	public boolean isValidTree(String forest, ILog log) {
		System.out.println("ConcreteUnitaryDiagram.isValidTree:forest="+forest);
		// Compute the reading tree
		ReadingTree tree = new ReadingTree(forest);
		System.out.println("ConcreteUnitaryDiagram.isValidTree:tree="+tree);
		// Compute the dependence graph
		DependenceGraph graph = getGraph();
		System.out.println("ConcreteUnitaryDiagram.isValidTree:graph="+graph);
		// Check conditions
		boolean flag1 = checkNodes(tree, graph, log);
		System.out.println("ConcreteUnitaryDiagram.isValidTree:flag1="+flag1);
		boolean flag2 = checkEdges(tree, graph, log);			
		System.out.println("ConcreteUnitaryDiagram.isValidTree:flag2="+flag2);
		return flag1 && flag2;
	}
	protected boolean checkNodes(ReadingTree tree, DependenceGraph graph, ILog log) {
		// Print message
		printMessage(log, "Checking nodes ...");
		List treeNodes = tree.getNodeNames();
		List graphNodes = graph.getNodes();
		System.out.println("ConcreteUnitaryDiagram.checkNodes:treeNodes="+treeNodes);
		System.out.println("ConcreteUnitaryDiagram.checkNodes:graphNodes="+graphNodes);
		
		
		if (treeNodes.size() != graphNodes.size()) {
			printMessage(log, "Reading tree and dependence graph do not have the same nodes");
			return false;
		}
		for(int i=0; i<graphNodes.size(); i++) {
			Object gNode = graphNodes.get(i);
			String label = "";
			if (gNode instanceof DiagramItem) {
				label = ((DiagramItem)gNode).getLabel();
			}
			if (!treeNodes.contains(label)) {
				printMessage(log, "Cannot find node '"+label+"' in reading tree");
				return false;
			}
		}
		return true;
	}
	protected boolean checkEdges(ReadingTree tree, DependenceGraph graph, ILog log) {
		System.out.println("ConcreteUnitaryDiagram.checkEdges:start");
		
		List graphEdges = graph.getEdges();
		System.out.println("ConcreteUnitaryDiagram.checkEdges:graphEdges="+graphEdges);
		for(int i=0; i<graphEdges.size(); i++) {
			Graph.Pair edge = (Graph.Pair)graphEdges.get(i);
			DiagramItem n1 = (DiagramItem)edge.first;
			DiagramItem n2 = (DiagramItem)edge.second;
			System.out.println("ConcreteUnitaryDiagram.checkEdges:n1="+n1);
			System.out.println("ConcreteUnitaryDiagram.checkEdges:n2="+n2);
			if (!checkEdge(tree, n1.getLabel(), n2.getLabel(), log))
				return false;
		}
		return true;
	}
	protected boolean checkEdge(ReadingTree tree, String label1, String label2, ILog log) {
		ReadingTreeNode n1 = tree.find(label1);
		ReadingTreeNode n2 = tree.find(label2);
		System.out.println("ConcreteUnitaryDiagram.checkEdge:n1="+n1);
		System.out.println("ConcreteUnitaryDiagram.checkEdge:n2="+n2);

		if (n1 == null) {
			printMessage(log, "Cannot find node '"+label1+"' in reading tree");
		}
		if (n2 == null) { 
			printMessage(log, "Cannot find node '"+label2+"' in reading tree");
		}
		System.out.println("ConcreteUnitaryDiagram.checkEdge:tree.isAncestor(label1, label2)=" +
				 											 tree.isAncestor(label1, label2));
		System.out.println("ConcreteUnitaryDiagram.checkEdge:tree.isAncestor(label2, label1))=" +
															 tree.isAncestor(label2, label1));

		if (tree.isAncestor(label1, label2) || tree.isAncestor(label2, label1))
			return true;
		printMessage(log, "Wrong edge ('"+label1+"', '"+label2+"')");
		return false;
	}
	protected void printMessage(ILog log, String msg) {
		if (log != null)
			log.reportMessage(msg);
		System.out.println(msg);	// for debugging only
	}
	
	public DependenceGraph getGraph() {
		return new DependenceGraph(this);
	}

	/** Clone */
	public ConcreteDiagram getClone() {
		DecorationBox newBox = (DecorationBox)box.getClone();
		ConcreteUnitaryDiagram res = new ConcreteUnitaryDiagram(newBox);
		res.decorationSet = this.decorationSet.getClone();
		ConcreteContourSet newCCS = this.concreteContourSet.getClone();
		List ccs = newCCS.getContours();
		for(int i=0; i<ccs.size(); i++) {
			ConcreteContour cc = (ConcreteContour)ccs.get(i);
			res.addConcreteContour(cc);
		}
		res.lfs = this.lfs.getClone();
		res.arcSet = this.arcSet.getClone();
		res.textSet = this.textSet.getClone();
		return res;
	}
	protected Set makeClone(Set set) {
		Set newSet = new LinkedHashSet();
		Iterator i = set.iterator();
		while (i.hasNext()) {
			Object obj = i.next();
			Object newObj = obj;
			if (obj instanceof DiagramItem) {
				newObj = ((DiagramItem)obj).getClone();
			}
			newSet.add(newObj);
		}
		return newSet;
	}

    /** Override to String */
    public String toString() {
        String res = "ConcreteUnitaryDiagram {\n";
        res += "\t label: "+box.getLabel()+"\n";
        res += "\t contours: "+getConcreteContours()+"\n";
        res += "\t zones: "+getConcreteZones()+"\n";
        res += "\t shaded zones: "+getSConcreteZones()+"\n";
        res += "\t feet: "+getConcreteFeet()+"\n";
        res += "\t legs: "+getConcreteLegs()+"\n";
        res += "\t spiders: "+getConcreteSpiders()+"\n";
        res += "\t arrows: "+getConcreteArcs()+"\n";
        res += "}";
        return res;
    }
}
