package rwd.concretesyntax.constraintdiagram;

import java.awt.Color;
import java.awt.Polygon;
import java.awt.Shape;
import java.awt.geom.Ellipse2D;

import rwd.concretesyntax.DiagramItem;
import rwd.concretesyntax.eulerdiagram.ConcreteContour;
import rwd.concretesyntax.spiderdiagram.ConcreteSpiderFoot;
import rwd.gui.editor.Handle;

//import rwd.abstractsyntax.eulerdiagram.Contour;

/**
 * A handle-position for a concrete arc.
 * 
 * @author Octavian Patrascoiu, rnb
 * @version 7.7.05   08:20
 *
 */
public class ConcreteArcEnd 
	extends DiagramItem 
	implements java.io.Serializable 
{
	public ConcreteArcEnd(float x, float y) {
		setLocation(x, y);
		setBackColor(Color.black);
		setThickness(40);
	}

	public ConcreteArcEnd(DiagramItem item) {
		setLocation(item);
		setBackColor(Color.black);
		setThickness(40);
	}
    
	public Shape getShape(boolean smooth) {
		int x = getX();
		int y = getY();
		int size = Handle.SIZE;
		Shape shape = new Ellipse2D.Float(x - size, y - size, size * 2, size * 2);
		return shape;
	}
	public Shape getShape() {
		return getShape(false);
	}

	public Handle getHandle() {
		if (_handle == null)
			_handle = new Handle(getX(), getY(), this);
		return _handle;
	}

	public java.util.Set getHandles() {
		// Set handles
		java.util.Set _handles = new java.util.LinkedHashSet();
		_handles.add(getHandle()); 
		return _handles;
	}
    
	public void setLocation(float x, float y) {
		_centerX = x;
		_centerY = y;
		_handle = null;
	}

	protected void setLocation(DiagramItem item) {
		_attachedItem = item;
		if (item == null) return;
		if (item instanceof ConcreteSpiderFoot) {
			ConcreteSpiderFoot foot = (ConcreteSpiderFoot)item;
			setLocation(foot.getX(), foot.getY());
		} else if (item instanceof ConcreteContour) {
			ConcreteContour poly = (ConcreteContour)item;
			int x = (int)_centerX;
			int y = (int)_centerY;
			Polygon p = poly.getPolygon();
			double distance = Double.MAX_VALUE;
			int point = 0;
			for(int i=0; i<p.npoints; i++) {
				int xi = p.xpoints[i];
				int yi = p.ypoints[i];
				double d = Math.sqrt((x-xi)*(x-xi)+(y-yi)*(y-yi));
				if (d < distance) {
					distance = d;
					point = i;
				}
			}
			// Get closest point
			setLocation(p.xpoints[point], p.ypoints[point]);
		}
		_handle = null;
	}
    
	public void translateTo(float dx, float dy) {
		_centerX += dx;
		_centerY += dy;
		Handle h = getHandle();
		h.translateTo(dx, dy);
	}

	public int getX() {
		setLocation(_attachedItem);
		return (int)_centerX;
	}
    
	public int getY() {
		setLocation(_attachedItem);
		return (int)_centerY;
	}

	public boolean isMiddle() {
		return _isMiddle;
	}
	public void setIsMiddle(boolean b) {
		_isMiddle = b;
	}

	public DiagramItem getAttachedItem() {
		return _attachedItem;
	}
	public void setAttachedItem(DiagramItem item) {
		_attachedItem = item;
	}
	public void attachItem(DiagramItem item) {
		detach();
		if (item != null) {
			_attachedItem = item;
			setLocation(item);
			item.setAttached(true);
		}
	}
	public void detach() {
		if (_attachedItem != null) {
			_attachedItem.setAttached(false);
			_attachedItem = null;		
		}
	}
	
	/**
	 * @return true, iff an item is attached to this end
	 */
	/*
	 * test:
	 *   item attached
	 *   no item attached
	 */
	public boolean isAttached() {
		return (_attachedItem != null);
	}
	
	/*
	 *  post: true.
	 * <P>
	 * Note: this overrides the setAttached method in DiagramItem.
	 * It does nothing, because the attached state depends on whether
	 * an item is attached to this end. 
	 * 
	 */
	public void setAttached(boolean b) {
	}


	/**
	 *  Update location
	 */
	public void updateLocation() {
		if (_attachedItem != null)
			setLocation(_attachedItem);
	}
	
    /**
     * Note: unlike clone(), this returns a DiagramItem, and does not throw
     * an exception.
     * 
     * @return a deep clone of this diagram item
     */
    /*
     * test: once thru
     */
	public DiagramItem getClone() {
		// Create clone
		ConcreteArcEnd res = new ConcreteArcEnd(_centerX, _centerY);
		res._isMiddle = this._isMiddle;
		res._handle = this._handle;
		DiagramItem di = this.getAttachedItem();
		if (di != null)
			res.setAttachedItem(di.getClone());
		else
			res.setAttachedItem(null);
		// Copy attributes
		res.setAttributes(this);
		return res;
	}
	
	
	/**
	 * 
	 * @return a user-friendly description of this item
	 */
	/*
	 * test:
	 *   attached item
	 *   no attached item
	 */
	public String id() {
		String s = "arrow end at (" + getX() + "," + getY() + ")";
		if (this.isAttached())
			s += 
		         " attached to " + this.getAttachedItem().id();
		return s;
	}
	

	    
	// Contour Polygon or Spider Foot
	private DiagramItem _attachedItem;
	private Handle _handle = null;
	private boolean _isMiddle = false;
	
	
	/*
	// test program
    // ------------

    private static void pr(String s) {
      System.out.println(s);
    }

    private static void hd(String s) {
      System.out.println("\n" + s + "\n");
    }

    private static void pg() {
      try {
        int i;
        System.out.print("next page? (CR)");
        do i = System.in.read(); while (i != '\n');
        System.out.println();
      }
      catch (Exception e) {System.out.println("raised exception : " + e);}
    }

    public static void main(String args[]) throws Exception {
    	hd("Test of toString, hashCode, id");
    	ConcreteArcEnd cae = new ConcreteArcEnd(1,2);
    	pr("cae="+cae);
    	pr("cae.hashCode()="+cae.hashCode());
    	pr("cae.id()="+cae.id());
    	pg();
    	
    	ConcreteArcEnd caec = (ConcreteArcEnd) cae.getClone();
    	pr("caec="+caec);
    	pr("caec.hashCode()="+caec.hashCode());
    	pr("caec.id()="+caec.id());
    	pr("cae.equals(caec)="+cae.equals(caec));
    	pr("caec.equals(cae)="+caec.equals(cae));
    	pg();

    	pr("now attach a contour");
    	Polygon p1 = new Polygon();
    	p1.addPoint(1,2);
    	ConcreteContour cc1 = new ConcreteContour(Contour.getContour("a"),p1);
    	pr("cc1="+cc1);
    	cae.setAttachedItem(cc1);
    	pr("cae="+cae);
    	pr("cae.hashCode()="+cae.hashCode());
    	pr("cae.id()="+cae.id());
    	pg();
    	
    	caec = (ConcreteArcEnd) cae.getClone();
    	pr("caec="+caec);
    	pr("caec.hashCode()="+caec.hashCode());
    	pr("caec.id()="+caec.id());
    	pr("cae.equals(caec)="+cae.equals(caec));
    	pr("caec.equals(cae)="+caec.equals(cae));
    	pr("cae.equals(null)="+cae.equals(null));
    	pr("cae.equals(\"jim\")="+cae.equals("jim"));
    	pg();   	
    }
    */
}
