package rwd.deltas;

import java.awt.Polygon;
import java.awt.geom.Line2D;
import java.awt.geom.PathIterator;
import java.util.List;
import java.util.Vector;

import rwd.concretesyntax.DiagramItem;
import rwd.concretesyntax.constraintdiagram.ConcreteArc;
import rwd.concretesyntax.constraintdiagram.ConcreteArcEnd;
import rwd.concretesyntax.eulerdiagram.ConcreteContour;
import rwd.concretesyntax.spiderdiagram.ConcreteSpiderFoot;
import rwd.concretesyntax.spiderdiagram.ConcreteSpiderLeg;
import uk.ac.kent.cs.kmf.util.ILog;
import uk.ac.kent.cs.kmf.util.OutputStreamLog;

/**
 * @author Octavian Patrascoiu
 *
 */
public class EdgeContourCrossingDelta 
	extends Delta
{
	// Constructor
	public EdgeContourCrossingDelta(DiagramItem item, ConcreteContour c, ILog log) {
		this.log = log;
		args.add(item);
		args.add(c);	
		item.addObserver(this);
		c.addObserver(this);
		// Add ends to make observer update when an end is moved
		addEdgeEnds(item);
		computeValue();
	}
	public EdgeContourCrossingDelta(DiagramItem item, ConcreteContour c) {
		this(item, c, new OutputStreamLog(System.out));
	}
	protected void addEdgeEnds(DiagramItem item) {
		if (item instanceof ConcreteSpiderLeg) {
			ConcreteSpiderLeg leg = (ConcreteSpiderLeg)item;
			ConcreteSpiderFoot end1 = leg.getSource();
			ConcreteSpiderFoot end2 = leg.getTarget();
			args.add(end1);
			args.add(end2);
			end1.addObserver(this);
			end2.addObserver(this);
		} else if (item instanceof ConcreteArc) {
			ConcreteArc arc = (ConcreteArc)item;
			ConcreteArcEnd end1 = arc.getStart();
			ConcreteArcEnd end2 = arc.getEnd();
			args.add(end1);
			args.add(end2);
			end1.addObserver(this);
			end2.addObserver(this);
		}		
	}
	protected void computeValue() {
		DiagramItem arg = (DiagramItem)args.get(0);
		ConcreteContour c = (ConcreteContour)args.get(1);
		value = crossings(arg, c);
	}
	protected double crossings(DiagramItem item, ConcreteContour c) {
		// Make a list of segments
		List segs1 = new Vector();
		Polygon poly = c.getPolygon();
//		for (int i=0; i<poly.npoints-1; i++) {
//			double x1 = poly.xpoints[i];
//			double y1 = poly.ypoints[i];
//			double x2 = poly.xpoints[i+1];
//			double y2 = poly.ypoints[i+1];
//			Line2D seg = makeLine(x1, y1, x2, y2);
//			segs1.add(seg);
//		}
		double[] pathPoint = {0.0,0.0,0.0,0.0,0.0,0.0};
		PathIterator pi = poly.getPathIterator(null);
		double x1 = 0;
		double y1 = 0;
		double x2 = 0;
		double y2 = 0;
		while(!pi.isDone()) {
			int segmentType = pi.currentSegment(pathPoint);
			if(segmentType == PathIterator.SEG_MOVETO) {
//				System.out.println("MOVETO x "+pathPoint[0]+"y "+pathPoint[1]+" ");
				x1 = pathPoint[0];
				y1 = pathPoint[1];				
			} else if(segmentType == PathIterator.SEG_LINETO) {
//				System.out.println("LINETO x "+pathPoint[0]+" y "+pathPoint[1]);
				x2 = pathPoint[0];
				y2 = pathPoint[1];				
				Line2D seg = makeLine(x1, y1, x2, y2);
				segs1.add(seg);
				x1 = x2;
				y1 = y2;
			} else if (segmentType == PathIterator.SEG_CLOSE) {
//				System.out.println("CLOSE x "+pathPoint[0]+" y "+pathPoint[1]);
			}
			pi.next();
		}
		// Make the segment associated to the leg/arc
		List segs2 = makeLines(item);
		// Count the number of intersections
		double no = 0;
		for(int i=0; i<segs1.size(); i++) {
			Line2D seg1 = (Line2D)segs1.get(i);
			for(int j=0; j<segs2.size(); j++) {
				Line2D seg2 = (Line2D)segs2.get(j);
				if (seg1.intersectsLine(seg2))
					no++; 
			}
		}
		return no;
	}
	protected List makeLines(DiagramItem item) {
		if (item instanceof ConcreteSpiderLeg) {
			return makeLines((ConcreteSpiderLeg)item);
		} else if (item instanceof ConcreteArc) {
			return makeLines((ConcreteArc)item);
		} else {
			return new Vector();
		}
	}
	protected List makeLines(ConcreteSpiderLeg leg) {
		List segs = new Vector();
		ConcreteSpiderFoot s = leg.getSource();
		ConcreteSpiderFoot t = leg.getTarget();
		double x1 = s.getX();
		double y1 = s.getY();
		double x2 = t.getX();
		double y2 = t.getY();
		Line2D seg = makeLine(x1, y1, x2, y2);
		segs.add(seg);
		return segs;
	}
	protected List makeLines(ConcreteArc arc) {
		if (arc.getType() == ConcreteArc.LINE) {
			List segs = new Vector();
			ConcreteArcEnd s = arc.getStart();
			ConcreteArcEnd t = arc.getEnd();
			double x1 = s.getX();
			double y1 = s.getY();
			double x2 = t.getX();
			double y2 = t.getY();
			Line2D seg = makeLine(x1, y1, x2, y2);
			segs.add(seg);
			return segs;
		} else {
			List segs = new Vector();
			ConcreteArcEnd s = arc.getStart();
			ConcreteArcEnd t = arc.getEnd();
			double x1 = s.getX();
			double y1 = s.getY();
			double x2 = t.getX();
			double y2 = t.getY();
			Line2D seg = makeLine(x1, y1, x2, y2);
			segs.add(seg);
			return segs;
		}
	}
	protected Line2D makeLine(double x1, double y1, double x2, double y2) {
		Line2D.Double seg = new Line2D.Double();
		seg.setLine(x1, y1, x2, y2);
		return seg;
	}
	public String toString() {
		String res = this.getClass().getName().substring(11);
		res += "("+args.get(0).toString()+", "+args.get(1).toString()+")";
		return res;
	}
}
