package rwd.deltas;

import rwd.concretesyntax.DiagramItem;
import rwd.concretesyntax.constraintdiagram.ConcreteArcEnd;
import rwd.concretesyntax.spiderdiagram.ConcreteSpiderFoot;
import uk.ac.kent.cs.kmf.util.ILog;
import uk.ac.kent.cs.kmf.util.OutputStreamLog;

/**
 * @author Octavian Patrascoiu
 *
 */
public class NodeEdgeSeparationDelta 
	extends Delta
{
	// Constructor - item either leg or arc
	public NodeEdgeSeparationDelta(ConcreteSpiderFoot foot, DiagramItem item1, DiagramItem item2, ILog log) {
		this.log = log;
		args.add(foot);
		args.add(item1);
		args.add(item2);
		foot.addObserver(this);
		item1.addObserver(this);
		item2.addObserver(this);
		computeValue();
	}
	public NodeEdgeSeparationDelta(ConcreteSpiderFoot foot, DiagramItem item1, DiagramItem item2) {
		this(foot, item1, item2, new OutputStreamLog(System.out));
	}
	protected void computeValue() {
		ConcreteSpiderFoot foot = (ConcreteSpiderFoot)args.get(0);
		double x1 = foot.getX();
		double y1 = foot.getY();
		DiagramItem arg1 = (DiagramItem)args.get(1);
		DiagramItem arg2 = (DiagramItem)args.get(2);
		if (arg1 instanceof ConcreteSpiderFoot) {
			ConcreteSpiderFoot foot1 = (ConcreteSpiderFoot)arg1;
			ConcreteSpiderFoot foot2 = (ConcreteSpiderFoot)arg2;
			double x2 = foot1.getX();
			double y2 = foot1.getY();
			double x3 = foot2.getX();
			double y3 = foot2.getY();
			double h = distance2Segment(x1, y1, x2, y2, x3, y3);
			if (h != 0)
				value = 1/h;			
		} else if (arg1 instanceof ConcreteArcEnd) {
			ConcreteArcEnd start = (ConcreteArcEnd)arg1;;
			ConcreteArcEnd end = (ConcreteArcEnd)arg2;
			double x2 = start.getX();
			double y2 = start.getY();
			double x3 = end.getX();
			double y3 = end.getY();
			double h = distance2Segment(x1, y1, x2, y2, x3, y3);
			if (h != 0)
				value = 1/h;			
		}
	}

	// Computes the distance from a point to a segment
	protected double distance2Segment(double x1, double y1, double x2, double y2, double x3, double y3) {
		double a = distance(x2, y2, x3, y3);
		double b = distance(x1, y1, x2, y2);
		double c = distance(x1, y1, x3, y3);
		double p = (a+b+c)/2;
		double s = Math.sqrt(p*(p-a)*(p-b)*(p-c));
		double h = s/a;
		return h;
	}
	protected double distance(double x1, double y1, double x2, double y2) {
		return Math.sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2));
	}

	public String toString() {
		String res = this.getClass().getName().substring(11);
		res += "("+args.get(0).toString()+")";
		return res;
	}
}
