package rwd.gui;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Vector;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import rwd.abstractsyntax.spiderdiagram.Diagram;
import rwd.abstractsyntax.spiderdiagram.UnitaryDiagram;
import rwd.rules.NoProofException;
import rwd.rules.Rule;
import rwd.rules.Proof;
import rwd.rules.heuristics.CostCalculator;
//import rwd.rules.heuristics.DefaultCompoundCostCalculator;
import rwd.rules.heuristics.DefaultCostCalculator;
import rwd.rules.heuristics.HeuristicCalculator;
import rwd.rules.heuristics.HeuristicCalculatorCompound;
import rwd.rules.heuristics.HeuristicCalculatorOptimistic;
import rwd.rules.heuristics.HeuristicCalculatorZero;
import rwd.rules.heuristics.HeuristicProofWriter;

/**
 * @author jaf28
 *
 */
public class HeuristicParameterWindow extends FontFrame {

	public HeuristicParameterWindow(final Vector diagrams) {
		super("choose heuristic search parameters");
		Border etchedBorder = BorderFactory.createEtchedBorder();
		Border spaceBorder1 = BorderFactory.createEmptyBorder(1, 1, 1, 1);
		Border spaceBorder2 = BorderFactory.createEmptyBorder(1, 1, 1, 1);
		Border compoundBorder =
			BorderFactory.createCompoundBorder(etchedBorder, spaceBorder1);
		Border panelBorder =
			BorderFactory.createCompoundBorder(spaceBorder2, compoundBorder);

		GridBagLayout gridbagContentPane = new GridBagLayout();
		GridBagConstraints cContentPane = new GridBagConstraints();
		getContentPane().setLayout(gridbagContentPane);

		/////////////////////////////////////////////////////////////////////////
		

		final JTabbedPane typeOfHeuristicPanel = new JTabbedPane();
		typeOfHeuristicPanel.setBorder(panelBorder);
		final JPanel unitaryHeuristicPanel;
		final JRadioButton fullHeuristic;
		final JRadioButton worstHeuristic;
		final JRadioButton ignoreContours;
		final JRadioButton ignoreZones;
		final JRadioButton ignoreShading;
		final JRadioButton ignoreSpiders;
		final JList choiceList3;
		{
			unitaryHeuristicPanel = new JPanel();
			unitaryHeuristicPanel.setBorder(panelBorder);
			typeOfHeuristicPanel.add("unitary", unitaryHeuristicPanel);

			GridBagLayout gridbagUnitary = new GridBagLayout();
			GridBagConstraints cUnitary = new GridBagConstraints();
			unitaryHeuristicPanel.setLayout(gridbagUnitary);

			fullHeuristic = new JRadioButton("best heuristic");
			fullHeuristic.setSelected(true);
			worstHeuristic = new JRadioButton("zero heuristic");
			worstHeuristic.setSelected(false);
			ignoreContours = new JRadioButton("ignore contours");
			ignoreContours.setSelected(false);
			ignoreZones = new JRadioButton("ignore zones");
			ignoreZones.setSelected(false);
			ignoreShading = new JRadioButton("ignore shading");
			ignoreContours.setSelected(false);
			ignoreSpiders = new JRadioButton("ignore spiders");
			ignoreContours.setSelected(false);
			ButtonGroup heuristicChoice = new ButtonGroup();

			JPanel topunitaryHeuristicPanel = new JPanel();
			topunitaryHeuristicPanel.setLayout(
				new BoxLayout(topunitaryHeuristicPanel, BoxLayout.Y_AXIS));
			//topunitaryHeuristicPanel.setAlignmentX(JPanel.LEFT_ALIGNMENT);
			topunitaryHeuristicPanel.setBorder(panelBorder);
			heuristicChoice.add(fullHeuristic);
			heuristicChoice.add(worstHeuristic);
			heuristicChoice.add(ignoreContours);
			heuristicChoice.add(ignoreZones);
			heuristicChoice.add(ignoreShading);
			heuristicChoice.add(ignoreSpiders);
			topunitaryHeuristicPanel.add(fullHeuristic);
			topunitaryHeuristicPanel.add(worstHeuristic);
			topunitaryHeuristicPanel.add(ignoreContours);
			topunitaryHeuristicPanel.add(ignoreZones);
			topunitaryHeuristicPanel.add(ignoreShading);
			topunitaryHeuristicPanel.add(ignoreSpiders);

			cUnitary.gridx = 0;
			cUnitary.gridy = 0;
			gridbagUnitary.setConstraints(topunitaryHeuristicPanel, cUnitary);
			unitaryHeuristicPanel.add(topunitaryHeuristicPanel);

			JPanel unitaryRulesPanel = new JPanel();
			GridBagLayout gridbagURules = new GridBagLayout();
			GridBagConstraints cURules = new GridBagConstraints();
			unitaryRulesPanel.setLayout(gridbagURules);

			JLabel l = new JLabel("select reasoning rules");
			cURules.gridx = 0;
			cURules.gridy = 0;
			gridbagURules.setConstraints(l, cURules);
			unitaryRulesPanel.add(l);

			DefaultListModel listModel2 = new DefaultListModel();
			Vector allURules = HeuristicProofWriter.unitaryHRules;
			int[] selectedOnes = new int[allURules.size()];
			for (int i = 0; i < allURules.size(); i++) {
				selectedOnes[i] = i;
				Rule r = (Rule)allURules.get(i);
				listModel2.add(i, r.name);
			}
			choiceList3 = new JList(listModel2);
			JScrollPane jScrollPane13 = new FontScrollPane(choiceList3);
			jScrollPane13.setMinimumSize(new Dimension(250, 200));
			jScrollPane13.setPreferredSize(new Dimension(250, 200));
			choiceList3.setSelectionMode(
				ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
			choiceList3.setSelectedIndices(selectedOnes);
			cURules.gridx = 0;
			cURules.gridy = 1;
			gridbagURules.setConstraints(jScrollPane13, cURules);
			unitaryRulesPanel.add(jScrollPane13);

			cUnitary.gridx = 0;
			cUnitary.gridy = 1;
			gridbagUnitary.setConstraints(unitaryRulesPanel, cUnitary);
			unitaryHeuristicPanel.add(unitaryRulesPanel);
		}

		//////////////////////////////////////////////////////////////////////////
		final JPanel compoundHeuristicPanel= new JPanel();
		final JRadioButton zeroCHeuristic;
        final JRadioButton fullCHeuristic;
		final JList choiceList4;
		{
			compoundHeuristicPanel.setBorder(panelBorder);
			typeOfHeuristicPanel.add("compound", compoundHeuristicPanel);

			GridBagLayout gridbagCompound = new GridBagLayout();
			GridBagConstraints cCompound = new GridBagConstraints();
			compoundHeuristicPanel.setLayout(gridbagCompound);

			zeroCHeuristic = new JRadioButton("zero heuristic");
			zeroCHeuristic.setSelected(true);
            fullCHeuristic = new JRadioButton("full heuristic");
            fullCHeuristic.setSelected(false);
            
			ButtonGroup heuristicCChoice = new ButtonGroup();

			JPanel topcompoundHeuristicPanel = new JPanel();
			topcompoundHeuristicPanel.setBorder(panelBorder);
			topcompoundHeuristicPanel.setLayout(
				new BoxLayout(topcompoundHeuristicPanel, BoxLayout.Y_AXIS));
			//topcompoundHeuristicPanel.setAlignmentX(JPanel.RIGHT_ALIGNMENT);
			heuristicCChoice.add(zeroCHeuristic);
            heuristicCChoice.add(fullCHeuristic);
			topcompoundHeuristicPanel.add(zeroCHeuristic);
            topcompoundHeuristicPanel.add(fullCHeuristic);
			cCompound.gridx = 0;
			cCompound.gridy = 0;
			gridbagCompound.setConstraints(
				topcompoundHeuristicPanel,
				cCompound);
			compoundHeuristicPanel.add(topcompoundHeuristicPanel);

			cContentPane.gridx = 0;
			cContentPane.gridy = 1;
			cContentPane.gridwidth = 1;
			cContentPane.gridheight = 3;
			gridbagContentPane.setConstraints(
				typeOfHeuristicPanel,
				cContentPane);
			getContentPane().add(typeOfHeuristicPanel);

			JPanel compoundRulesPanel = new JPanel();
			GridBagLayout gridbagcr = new GridBagLayout();
			GridBagConstraints ccr = new GridBagConstraints();
			compoundRulesPanel.setLayout(gridbagcr);

			JLabel cl = new JLabel("select reasoning rules");
			ccr.gridx = 0;
			ccr.gridy = 0;
			gridbagcr.setConstraints(cl, ccr);
			compoundRulesPanel.add(cl);

			DefaultListModel listModel3 = new DefaultListModel();
			Vector allcRules = (Vector)HeuristicProofWriter.compoundHRules.clone();
			int[] selectedcOnes = new int[allcRules.size()];
			for (int i = 0; i < allcRules.size(); i++) {
				selectedcOnes[i] = i;
				Rule r = (Rule)allcRules.get(i);
				listModel3.add(i, r.name);
			}
			choiceList4 = new JList(listModel3);
			JScrollPane jScrollPane14 = new FontScrollPane(choiceList4);
			jScrollPane14.setMinimumSize(new Dimension(250, 200));
			jScrollPane14.setPreferredSize(new Dimension(250, 200));
			choiceList4.setSelectionMode(
				ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
			choiceList4.setSelectedIndices(selectedcOnes);
			ccr.gridx = 0;
			ccr.gridy = 1;
			gridbagcr.setConstraints(jScrollPane14, ccr);
			compoundRulesPanel.add(jScrollPane14);

			cCompound.gridx = 0;
			cCompound.gridy = 1;
			gridbagCompound.setConstraints(compoundRulesPanel, cCompound);
			compoundHeuristicPanel.add(compoundRulesPanel);
		}
		//////////////////////////////////////////////////////////////////////////
		final JList choiceList1;
		final JList choiceList2;
		{
			JPanel diagramsPanel = new JPanel();
			diagramsPanel.setBorder(panelBorder);
			GridBagLayout gridbagDiagramsPanel = new GridBagLayout();
			GridBagConstraints cDiagramsPanel = new GridBagConstraints();
			diagramsPanel.setLayout(gridbagDiagramsPanel);

			JLabel jl1 = new JLabel("premise diagram");
			cDiagramsPanel.gridx = 0;
			cDiagramsPanel.gridy = 0;
			gridbagDiagramsPanel.setConstraints(jl1, cDiagramsPanel);
			diagramsPanel.add(jl1);

			JLabel jl2 = new JLabel("conclusion diagram");
			cDiagramsPanel.gridx = 1;
			cDiagramsPanel.gridy = 0;
			gridbagDiagramsPanel.setConstraints(jl2, cDiagramsPanel);
			diagramsPanel.add(jl2);

			DefaultListModel listModel1 = new DefaultListModel();
			for (int i = 0; i < diagrams.size(); i++) {
				Object o = diagrams.get(i);
				listModel1.add(i, o.toString());
			}
			choiceList1 = new JList(listModel1);
			choiceList1.addListSelectionListener(new ListSelectionListener() {
				public void valueChanged(ListSelectionEvent e) {
					if (e.getValueIsAdjusting()) {
						return;
					}
					int itemNumber = choiceList1.getSelectedIndex();
					Diagram d = (Diagram)diagrams.get(itemNumber);
					if(!(d instanceof UnitaryDiagram))
					{ typeOfHeuristicPanel.setSelectedIndex(1);
					}
				}

			});
			JScrollPane jScrollPane11 = new FontScrollPane(choiceList1);
			jScrollPane11.setMinimumSize(new Dimension(250, 200));
			jScrollPane11.setPreferredSize(new Dimension(250, 200));
			choiceList1.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			choiceList1.setSelectedIndices(new int[] { 0 });
			cDiagramsPanel.gridx = 0;
			cDiagramsPanel.gridy = 1;
			gridbagDiagramsPanel.setConstraints(jScrollPane11, cDiagramsPanel);
			diagramsPanel.add(jScrollPane11);

			choiceList2 = new JList(listModel1);
			choiceList2.addListSelectionListener(new ListSelectionListener() {
				public void valueChanged(ListSelectionEvent e) {
					if (e.getValueIsAdjusting()) {
						return;
					}
					int itemNumber = choiceList2.getSelectedIndex();
					Diagram d = (Diagram)diagrams.get(itemNumber);
					if(!(d instanceof UnitaryDiagram))
					{ typeOfHeuristicPanel.setSelectedIndex(1);
					}
				}

			});
			JScrollPane jScrollPane12 = new FontScrollPane(choiceList2);
			jScrollPane12.setPreferredSize(new Dimension(250, 200));
			jScrollPane12.setMinimumSize(new Dimension(250, 200));
			choiceList2.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			choiceList2.setSelectedIndices(new int[] { diagrams.size() - 1 });
			cDiagramsPanel.gridx = 1;
			cDiagramsPanel.gridy = 1;
			gridbagDiagramsPanel.setConstraints(jScrollPane12, cDiagramsPanel);
			diagramsPanel.add(jScrollPane12);

			cContentPane.gridx = 0;
			cContentPane.gridy = 0;
			cContentPane.gridwidth = 2;
			cContentPane.gridheight = 1;
			
			gridbagContentPane.setConstraints(diagramsPanel, cContentPane);
			getContentPane().add(diagramsPanel);
		}
		//////////////////////////////////////////////////////////////////

		JPanel logFilePanel = new JPanel();
		logFilePanel.setLayout(new GridLayout(0, 1));

		final JRadioButton doSaveLog = new JRadioButton("save a log file");
		doSaveLog.setSelected(false);
		final JRadioButton dontSaveLog =
			new JRadioButton("don't save a log file");
		dontSaveLog.setSelected(true);
		ButtonGroup do_or_dontSave = new ButtonGroup();

		final JFrame thisFrame = this;

		do_or_dontSave.add(doSaveLog);
		do_or_dontSave.add(dontSaveLog);
		logFilePanel.add(doSaveLog);
		logFilePanel.add(dontSaveLog);

		logFilePanel.add(new JLabel("location of log file"));
		final JTextField logFileLocation = new JTextField(15);
		logFileLocation.setText("heuristic.log");
		logFilePanel.add(logFileLocation);
		JButton b = new JButton("change log file location");
		b.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JFileChooser jf = new JFileChooser();
				jf.showDialog(null, "give log file location");
				File f = jf.getSelectedFile();
				boolean fileIsThereAlready = true;
				try {
					new FileInputStream(f);
				} catch (FileNotFoundException ex) {
					fileIsThereAlready = false;
				}
				if (fileIsThereAlready
					&& JOptionPane.showConfirmDialog(
						thisFrame,
						"overwrite:" + f)
						!= JOptionPane.OK_OPTION) {
					return;
				}
				logFileLocation.setText(f.getPath());
			}
		});
		logFilePanel.add(b);

		cContentPane.gridx = 1;
		cContentPane.gridy = 1;
		cContentPane.gridwidth = 1;
		cContentPane.gridheight = 1;
		gridbagContentPane.setConstraints(logFilePanel, cContentPane);
		getContentPane().add(logFilePanel);

		JPanel numberOfStepsPanel = new JPanel();
		numberOfStepsPanel.setLayout(new GridLayout(0, 1));
		numberOfStepsPanel.add(new JLabel("limiting length/cost of proof"));
		final JTextField numberStepsTF = new JTextField(15);
		numberStepsTF.setText("4");
		numberOfStepsPanel.add(numberStepsTF);

		cContentPane.gridx = 1;
		cContentPane.gridy = 2;
		cContentPane.gridwidth = 1;
		cContentPane.gridheight = 1;
		gridbagContentPane.setConstraints(numberOfStepsPanel, cContentPane);
		getContentPane().add(numberOfStepsPanel);

		JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout(new GridLayout(0, 1));

		final JRadioButton justOneProof =
			new JRadioButton("find a shortest proof");
		justOneProof.setSelected(false);
		final JRadioButton getAllProofs =
			new JRadioButton("find all shortest proofs");
		getAllProofs.setSelected(true);
		ButtonGroup numProofs = new ButtonGroup();

		numProofs.add(justOneProof);
		numProofs.add(getAllProofs);
		buttonPanel.add(justOneProof);
		buttonPanel.add(getAllProofs);

		final JRadioButton hideWhenDone =
			new JRadioButton("hide this window when done");
		hideWhenDone.setSelected(false);
		final JRadioButton showWhenDone =
			new JRadioButton("keep this window showing");
		showWhenDone.setSelected(true);
		ButtonGroup hideOrShow = new ButtonGroup();

		hideOrShow.add(hideWhenDone);
		hideOrShow.add(showWhenDone);
		buttonPanel.add(hideWhenDone);
		buttonPanel.add(showWhenDone);

		final JCheckBox showProgressPane = new JCheckBox("show progress count");
		showProgressPane.setSelected(true);
		buttonPanel.add(showProgressPane);

		JButton go = new JButton("search now");
		go.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				new Thread(new Runnable() {
					public void run() {
						// trigger search
						Diagram premis =
							(Diagram) diagrams.get(
								choiceList1.getSelectedIndex());
						Diagram conclusion =
							(Diagram) diagrams.get(
								choiceList2.getSelectedIndex());
						int numSteps = 0;
						try {
							numSteps =
								Integer.parseInt(numberStepsTF.getText());
						} catch (NumberFormatException x) {
							JOptionPane.showMessageDialog(
								thisFrame,
								"not a recognised number");
							return;
						}
						HeuristicCalculator hc = null;
                        CostCalculator cc = new DefaultCostCalculator();
						if (unitaryHeuristicPanel.isShowing()) {
							if (fullHeuristic.isSelected()) {
								hc = new HeuristicCalculatorOptimistic();
							} else if (worstHeuristic.isSelected()) {
								hc = new HeuristicCalculatorZero();
							} else if (ignoreContours.isSelected()) {
								hc = new HeuristicCalculatorOptimistic("fttt");
							} else if (ignoreZones.isSelected()) {
								hc = new HeuristicCalculatorOptimistic("tftt");
							} else if (ignoreShading.isSelected()) {
								hc = new HeuristicCalculatorOptimistic("ttft");
							} else {
								hc = new HeuristicCalculatorOptimistic("tttf");
							}
						} else {
							if(fullCHeuristic.isSelected()){
							  hc = new HeuristicCalculatorCompound();
                              //cc =new DefaultCompoundCostCalculator();
                              cc =new DefaultCostCalculator();
                            } else {
							  hc = new HeuristicCalculatorZero();
                              //cc =new DefaultCompoundCostCalculator();
                              cc =new DefaultCostCalculator();
							}
						}
						Vector ruleSet = new Vector();
						if(unitaryHeuristicPanel.isShowing()) {
							int[] rules = choiceList3.getSelectedIndices();
							for (int i = 0; i < rules.length; i++) {
								ruleSet.add(HeuristicProofWriter.unitaryHRules.get(rules[i]));
							}
						}
						else
						{
							int[] rules = choiceList4.getSelectedIndices();
							for (int i = 0; i < rules.length; i++) {
								ruleSet.add(HeuristicProofWriter.compoundHRules.get(rules[i]));
							}
						}
						StringProgressShower sps = null;
						if (showProgressPane.isSelected()) {
							sps = new FrameProgressShower();
						} else {
							sps = null;
						}

						boolean collectAllProofs = getAllProofs.isSelected();
						Vector a = null;
						try {
							a =
								HeuristicProofWriter.seekProof(
									premis,
									conclusion,
									numSteps,
									hc,
                                    cc,
									ruleSet,
									sps,
									unitaryHeuristicPanel.isShowing(),
									!unitaryHeuristicPanel.isShowing(),
									new BufferedWriter(
										new FileWriter(
											logFileLocation.getText())),
									collectAllProofs);

						} catch (IOException x) {
						} catch (OutOfMemoryError x) {
							JOptionPane.showMessageDialog(
								thisFrame,
								"ran out of memory");
						} catch (NoProofException x) {
							JOptionPane.showMessageDialog(
								thisFrame,
								"can't be proved:\n" + x.getCounterExample());
						}
						//System.out.println("got proof "+p);
						if (a!=null && a.get(0) != null) {
							for (int i = 0; i < a.size() - 1; i++) {
								Proof p = (Proof) a.get(i);
								ProofManager.showNewProof(p, false, true);
							}
						} else {
							JOptionPane.showMessageDialog(
								thisFrame,
								"no proof found");
						}
						if (hideWhenDone.isSelected()) {
							setVisible(false);
							dispose();
						}
					} // end of run method
				}).start(); // end of Thread definition
			}// end of actionPerformed method
		}); // end of addActionListener invocation
		buttonPanel.add(go);

		cContentPane.gridx = 1;
		cContentPane.gridy = 3;
		cContentPane.gridwidth = 1;
		cContentPane.gridheight = 1;
		gridbagContentPane.setConstraints(buttonPanel, cContentPane);
		getContentPane().add(buttonPanel);

		pack();
	}
}
