package rwd.gui;

import java.awt.event.WindowEvent;
import javax.swing.JButton;
import java.awt.Rectangle;

import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JMenuBar;
import java.awt.BorderLayout;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;
import java.util.HashMap;
import java.util.Hashtable;

import javax.swing.JOptionPane;

import rwd.abstractsyntax.eulerdiagram.Contour;
import rwd.abstractsyntax.eulerdiagram.Zone;
import rwd.abstractsyntax.spiderdiagram.Diagram;
import rwd.rules.*;
import rwd.abstractsyntax.spiderdiagram.NotCompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.UnitaryDiagram;
import rwd.abstractsyntax.spiderdiagram.AndCompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.DiagramFileUser;
import rwd.rules.choicespec.RuleWithChoice;
import rwd.rules.application.RuleApplicationSpec;
import rwd.rules.application.RuleAppliesToJustADiagram;
import rwd.rules.autoproof.*;
//import rwd.rules.heuristics.HeuristicCalculatorCompound;
import rwd.rules.heuristics.HeuristicProofWriter;
import rwd.rules.inputspec.*;
import rwd.rules.choicespec.*;
import rwd.rules.application.RuleAppliesWithString;
import rwd.rules.application.RuleAppliesToDiagramAndDiagram;
import rwd.abstractsyntax.spiderdiagram.OrCompoundDiagram;
import javax.swing.ListSelectionModel;
import rwd.abstractsyntax.spiderdiagram.Spider;
import rwd.rules.nospec.*;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import java.awt.Dimension;
import java.awt.Component;
import java.awt.FlowLayout;
import javax.swing.ListModel;
import rwd.abstractsyntax.spiderdiagram.FalseDiagram;

public class MainDiagramFrame extends FontFrame {
    private DiagramShowerTree dstree;

    private Vector unitaryRules;
    private Vector compoundRules;
    private Vector logicRules;
    private Vector derivedRules;

    final private Vector showerTrees = new Vector();
    final private Hashtable panelsHT = new Hashtable();
    final private Vector diagrams = new Vector();
    final private Vector diagramMenuItems = new Vector();

    private Diagram diagram = null; // that in focus
    private Diagram parentOfPart = null;
    private Diagram partDiagram = null;

    private Proof proof;
    private boolean isProving; // for user-composed proving
    private boolean tryToDrawPictures;

    private JMenuBar menu;
    private JMenu fileMenu, editMenu;
    private JMenu currentDiagsMenu;
    private JMenu ruleMenu;
    private JMenuItem newDiagram, openDiagram, saveAsDiagram, closeDiagram;
    private JMenuItem changeShading, editName, changeZones, addSpiderMI;
    private JMenuItem removeSpiderMI, extractCopy;
    private JMenu ruleMenuUnitary;
    private JMenu ruleMenuCompound, ruleMenuLogic, ruleMenuDerived;

    private Vector rules;
    private HashMap ruleMIMap = new HashMap();
    private HashMap ruleMIUHMap = new HashMap();
    private HashMap ruleMICHMap = new HashMap();
    FontPanel diagramPanel = new FontPanel();
    //private JCheckBox storeProof = new JCheckBox();

    private JMenu proofs = new JMenu("Proof    ");
    private JMenu heuristicUnitaryRules,
        heuristicCompoundRules;
        //heuristicMeasures;
    private JMenuItem storeProof, equaliseContours, splitSpiders;
    private JMenuItem disjNF, combiner, falseRemover, betarise, proofWriter;
    private JMenuItem worseProofWriter, betterProofWriter; //heuristicBound
    private JMenuItem shortenedProofWriter;
    
    private JMenuItem shortenedProofWriterWithNot, heuristicProofWriter;
    private JMenuItem 
        //contourMeasures,
        //zoneMeasures,
        //shadingMeasures,
        //spiderMeasures,
        //andOrMeasures,
        hCalc;

    private JMenu displayMenu = new JMenu("Display    ");
    private JMenuItem changeFonts, switchViewType;

    public MainDiagramFrame(final boolean tryToDrawPicturesP) {
        super("Spider diagram reasoning tool");
        this.tryToDrawPictures = tryToDrawPicturesP;
        //System.out.println("pictures? "+tryToDrawPictures);
        //System.out.println("building a main diagram frame");
        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent evt) {
                exitForm(evt);
            }
        });
        diagramPanel = new FontPanel();
        getContentPane().add(diagramPanel, BorderLayout.CENTER);

        menu = new JMenuBar();
        fileMenu = new JMenu("File    ");
        fileMenu.setMnemonic(KeyEvent.VK_F);
        newDiagram = new JMenuItem("new diagram");
        newDiagram.setMnemonic(KeyEvent.VK_N);
        openDiagram = new JMenuItem("open...");
        openDiagram.setMnemonic(KeyEvent.VK_O);
        saveAsDiagram = new JMenuItem("save as...");
        saveAsDiagram.setMnemonic(KeyEvent.VK_S);
        closeDiagram = new JMenuItem("close");
        closeDiagram.setMnemonic(KeyEvent.VK_C);
        editMenu = new JMenu("Edit    ");
        editMenu.setMnemonic(KeyEvent.VK_E);
        changeShading = new JMenuItem("edit shading");
        changeShading.setMnemonic(KeyEvent.VK_S);
        editName = new JMenuItem("edit name");
        editName.setMnemonic(KeyEvent.VK_N);
        changeZones = new JMenuItem("edit zones");
        changeZones.setMnemonic(KeyEvent.VK_Z);
        addSpiderMI = new JMenuItem("add spider");
        addSpiderMI.setMnemonic(KeyEvent.VK_A);
        removeSpiderMI = new JMenuItem("remove spider");
        removeSpiderMI.setMnemonic(KeyEvent.VK_R);
        extractCopy = new JMenuItem("extract copy");
        extractCopy.setMnemonic(KeyEvent.VK_C);
        currentDiagsMenu = new JMenu("Diagrams    ");
        currentDiagsMenu.setMnemonic(KeyEvent.VK_D);
        ruleMenu = new JMenu("Apply rule    ");
        ruleMenu.setMnemonic(KeyEvent.VK_R);
        ruleMenuUnitary = new JMenu("diagrammatic on unitary diagrams");
        ruleMenuCompound = new JMenu("diagrammatic involving compound");
        ruleMenuLogic = new JMenu("logic rules");
        ruleMenuDerived = new JMenu("derived rules for convenience");
        proofs = new JMenu("Proof    ");
        storeProof = new JMenuItem("compose proof by hand");
        equaliseContours = new JMenuItem("equalise contour sets");
        splitSpiders = new JMenuItem("split spiders");
        disjNF = new JMenuItem("to disjunctive normal form");
        combiner = new JMenuItem("combine parts");
        falseRemover = new JMenuItem("remove false diagrams");
        betarise = new JMenuItem("betarise");
        proofWriter = new JMenuItem("write proof");
        worseProofWriter =
            new JMenuItem("get automated proof - simple version");
        betterProofWriter =
            new JMenuItem("get automated proof - improved version");
        shortenedProofWriter =
            new JMenuItem("get automated proof - shortened version");
        shortenedProofWriterWithNot =
            new JMenuItem("get automated proof - shortened version with not");

        //heuristicBound = new JMenuItem("heuristic lower bound");
        heuristicUnitaryRules = new JMenu("unitary heuristic rules");
        heuristicCompoundRules = new JMenu("compound heuristic rules");
        //heuristicMeasures = new JMenu("heuristic measures");
        heuristicProofWriter = new JMenuItem("seek heuristic proof");
        //contourMeasures = new JMenuItem("calculate contour measures");
        //zoneMeasures = new JMenuItem("calculate zone measures");
        //shadingMeasures = new JMenuItem("calculate shading measures");
        //spiderMeasures = new JMenuItem("calculate spider measures");
        //andOrMeasures = new JMenuItem("calculate and/or measures");
        hCalc = new JMenuItem("calculate compound heuristic");
        changeFonts = new JMenuItem("change fonts");
        switchViewType = new JMenuItem("switch view type");
        setJMenuBar(menu);
        menu.add(fileMenu);
        fileMenu.add(newDiagram);
        newDiagram.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                newDiagramRequested();
            }
        });
        fileMenu.add(openDiagram);
        openDiagram.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                openDiagram();
            }
        });
        fileMenu.add(saveAsDiagram);
        saveAsDiagram.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                DiagramFileUser.saveToFile(diagram);
            }
        });
        fileMenu.add(closeDiagram);
        closeDiagram.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                closeCurrentDiagram();
            }
        });
        menu.add(editMenu);
        editMenu.add(editName);
        editName.addActionListener(new EditNameActionListener());
        editMenu.add(changeShading);
        changeShading.addActionListener(new ShadingListener());
        editMenu.add(changeZones);
        changeZones.addActionListener(new ZoneListener());
        editMenu.add(addSpiderMI);
        addSpiderMI.addActionListener(new AddSpiderListener());
        editMenu.add(removeSpiderMI);
        removeSpiderMI.addActionListener(new RemoveSpiderListener());
        editMenu.add(extractCopy);
        extractCopy.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                extractCopy();
            }
        });
        menu.add(currentDiagsMenu);
        menu.add(ruleMenu);

        unitaryRules = new Vector();
        unitaryRules.add(RuleAddContour.getRule()); //////////
        unitaryRules.add(RuleDeleteContour.getRule()); //////////
        // should be "delete contour simple" when we get to compound work
        // revert to "delete contour" just for D2K4 talk version
        unitaryRules.add(RuleDeleteShadedZone.getRule()); //////////
        unitaryRules.add(RuleUnshadeZone.getRule()); //////////
        unitaryRules.add(RuleIntroduceShadedZone.getRule()); //////////
        unitaryRules.add(RuleAddSpiderFoot.getRule()); //////////
        unitaryRules.add(RuleDeleteSpider.getRule()); //////////

        compoundRules = new Vector();
        compoundRules.add(RuleSplitSpider.getRule()); //////////
        compoundRules.add(RuleJoinSpiders.getRule()); //////////
        compoundRules.add(null);
        compoundRules.add(RuleExcludedMiddle.getRule()); //////////
        compoundRules.add(RuleReverseExcludedMiddle.getRule()); /////////
        compoundRules.add(null);
        compoundRules.add(MergeConjunctionRule.getRule()); //////////        
        compoundRules.add(MergeContradictionRule.getRule()); //////////
        compoundRules.add(RuleFalseImpliesAContradiction.getRule());
        //////////       
        compoundRules.add(null);
        compoundRules.add(RemoveNotRule.getRule()); //////////
        compoundRules.add(ReverseNegationEquivalenceRule.getRule()); //////////
        compoundRules.add(TrueWithAndRule.getRule()); //////////
        compoundRules.add(TrueWithOrRule.getRule()); //////////

        logicRules = new Vector();
        logicRules.add(AndIdempotentRule.getRule()); //////////
        logicRules.add(RuleReverseAndIdempotent.getRule()); //////////
        logicRules.add(OrIdempotentRule.getRule()); //////////
        logicRules.add(RuleReverseOrIdempotent.getRule()); //////////
        logicRules.add(null);
        logicRules.add(RuleDistributeAndOverOr.getRule()); //////////
        logicRules.add(RuleDistributeOrOverAnd.getRule()); //////////
        logicRules.add(RuleDistributeMergeAnds.getRule()); //////////
        logicRules.add(RuleDistributeMergeOrs.getRule()); //////////
        //logicRules.add(ToDNFRule.getRule());
        logicRules.add(null);
        //logicRules.add(RuleAndAbsorbtion.getRule()); //////////
        //logicRules.add(RuleReverseAndAbsorbtion.getRule()); //////////     
        //logicRules.add(RuleOrAbsorbtion.getRule()); //////////
        //logicRules.add(RuleReverseOrAbsorbtion.getRule()); //////////
        logicRules.add(RuleAndForgetsInfo.getRule()); //////////
        logicRules.add(RulePToPOrRubbish.getRule()); //////////
        logicRules.add(null);
        logicRules.add(RuleFalseImpliesAnything.getRule());
        logicRules.add(FalseWithAndRule.getRule()); //////////
        logicRules.add(FalseWithOrRule.getRule()); //////////
        logicRules.add(RulePToPorF.getRule()); //////////
        logicRules.add(null);
        logicRules.add(RemoveNotNotRule.getRule()); //////////
        logicRules.add(InsertNotNotRule.getRule()); //////////
        logicRules.add(NotFalseIsTrueRule.getRule()); //////////
        logicRules.add(NotTrueIsFalseRule.getRule()); //////////
        logicRules.add(null);
        logicRules.add(RuleDMPushNotThroughAnd.getRule()); //////////
        logicRules.add(RuleDMPushNotThroughOr.getRule()); //////////
        logicRules.add(RuleDMPullNotThroughAnd.getRule()); //////////
        logicRules.add(RuleDMPullNotThroughOr.getRule()); //////////

        derivedRules = new Vector();
        derivedRules.add(RuleDeleteContour.getRule()); //////////
        derivedRules.add(ShatterSpidersRule.getRule()); //////////
        derivedRules.add(ToDNFRule.getRule()); //////////
        derivedRules.add(RulePushNotsToLeaves.getRule()); //////////
        derivedRules.add(SimpleCombiningRule.getRule()); //////////
        derivedRules.add(RuleDeleteContour.getRule()); //////////
        derivedRules.add(RemoveNotRuleComplex.getRule()); //////////
        derivedRules.add(NotTrueIsFalseComplexRule.getRule()); //////////
        derivedRules.add(MergeContradictionAvoidingFalseRule.getRule());
        derivedRules.add(RuleAndAbsorbtion.getRule()); //////////
        derivedRules.add(RuleReverseAndAbsorbtion.getRule()); //////////     
        derivedRules.add(RuleOrAbsorbtion.getRule()); //////////
        derivedRules.add(RuleReverseOrAbsorbtion.getRule()); //////////
        //////////

        ruleMenu.add(ruleMenuUnitary);
        ruleMenu.add(ruleMenuCompound);
        ruleMenu.add(ruleMenuLogic);
        ruleMenu.add(ruleMenuDerived);
        rules = new Vector();

        rules.addAll(unitaryRules);
        //System.out.println("unitaryRules are "+Rule.getUnitaryRules());
        for (int i = 0; i < unitaryRules.size(); i++) {
            final Rule r = (Rule) (unitaryRules.get(i));
            JMenuItem ruleMI = new JMenuItem(r.name);
            ruleMI.addActionListener(new RuleActionListener(r));
            ruleMenuUnitary.add(ruleMI);
            ruleMIMap.put(r, ruleMI);
        }

        rules.addAll(compoundRules);
        for (int i = 0; i < compoundRules.size(); i++) {
            final Rule r = (Rule) (compoundRules.get(i));
            if (r != null) {
                JMenuItem ruleMI = new JMenuItem(r.name);
                ruleMI.addActionListener(new RuleActionListener(r));
                ruleMenuCompound.add(ruleMI);
                ruleMIMap.put(r, ruleMI);
            } else {
                ruleMenuCompound.addSeparator();
            }
        }

        rules.addAll(logicRules);
        for (int i = 0; i < logicRules.size(); i++) {
            final Rule r = (Rule) (logicRules.get(i));
            if (r != null) {
                JMenuItem ruleMI = new JMenuItem(r.name);
                ruleMI.addActionListener(new RuleActionListener(r));
                ruleMenuLogic.add(ruleMI);
                ruleMIMap.put(r, ruleMI);
            } else {
                ruleMenuLogic.addSeparator();
            }
        }

        rules.addAll(derivedRules);
        for (int i = 0; i < derivedRules.size(); i++) {
            final Rule r = (Rule) (derivedRules.get(i));
            if (r != null) {
                JMenuItem ruleMI = new JMenuItem(r.name);
                ruleMI.addActionListener(new RuleActionListener(r));
                ruleMenuDerived.add(ruleMI);
                ruleMIMap.put(r, ruleMI);
            } else {
                ruleMenuDerived.addSeparator();
            }
        }

        menu.add(proofs);
        proofs.add(storeProof);
        storeProof.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                proof = new Proof(diagram);
                ProofManager.showNewProof(proof, true, tryToDrawPictures);
                isProving = true;
                makeMenusRight();
            }
        });

        proofs.add(worseProofWriter);
        worseProofWriter.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) { //System.out.println("action listener triggered for proof writer");
                new Thread(new Runnable() {
                    public void run() {
                        Diagram premis =
                            (Diagram) StaticObjects
                                .cg
                                .getChoice(
                                    StaticObjects.mdf.getBounds(),
                                    "initial diagram?",
                                    diagrams,
                                    new Vector(),
                                    false,
                                    ListSelectionModel.SINGLE_SELECTION)
                                .get(0);
                        Diagram conclusion =
                            (Diagram) StaticObjects
                                .cg
                                .getChoice(
                                    StaticObjects.mdf.getBounds(),
                                    "concluding diagram?",
                                    diagrams,
                                    new Vector(),
                                    false,
                                    ListSelectionModel.SINGLE_SELECTION)
                                .get(0);
                        //System.out.println("premis is "+premis.toSaveString());
                        //System.out.println("conclusion is "+conclusion.toSaveString());
                        try {
                            Proof p =
                                AutoProofWriterWorse.writeProof(
                                    premis,
                                    conclusion);
                            //System.out.println("got proof "+p);
                            ProofManager.showNewProof(
                                p,
                                false,
                                tryToDrawPictures);
                        } catch (NoProofException npe) {
                            JOptionPane.showMessageDialog(
                                StaticObjects.mdf,
                                "can't be proved:\n" + npe.getCounterExample());
                        }
                    }
                }).start();
            }
        });
        proofs.add(betterProofWriter);
        betterProofWriter.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) { //System.out.println("action listener triggered for proof writer");
                new Thread(new Runnable() {
                    public void run() {
                        Diagram premis =
                            (Diagram) StaticObjects
                                .cg
                                .getChoice(
                                    StaticObjects.mdf.getBounds(),
                                    "initial diagram?",
                                    diagrams,
                                    new Vector(),
                                    false,
                                    ListSelectionModel.SINGLE_SELECTION)
                                .get(0);
                        Diagram conclusion =
                            (Diagram) StaticObjects
                                .cg
                                .getChoice(
                                    StaticObjects.mdf.getBounds(),
                                    "concluding diagram?",
                                    diagrams,
                                    new Vector(),
                                    false,
                                    ListSelectionModel.SINGLE_SELECTION)
                                .get(0);
                        //System.out.println("premis is "+premis.toSaveString());
                        //System.out.println("conclusion is "+conclusion.toSaveString());
                        try {
                            Proof p =
                                AutoProofWriterBetter.writeProof(
                                    premis,
                                    conclusion);
                            //System.out.println("got proof "+p);
                            ProofManager.showNewProof(
                                p,
                                false,
                                tryToDrawPictures);
                        } catch (NoProofException npe) {
                            JOptionPane.showMessageDialog(
                                StaticObjects.mdf,
                                "can't be proved:\n" + npe.getCounterExample());
                        }
                    }
                }).start();
            }
        });
        proofs.add(shortenedProofWriter);
        shortenedProofWriter.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) { //System.out.println("action listener triggered for proof writer");
                new Thread(new Runnable() {
                    public void run() {
                        Diagram premis =
                            (Diagram) StaticObjects
                                .cg
                                .getChoice(
                                    StaticObjects.mdf.getBounds(),
                                    "initial diagram?",
                                    diagrams,
                                    new Vector(),
                                    false,
                                    ListSelectionModel.SINGLE_SELECTION)
                                .get(0);
                        Diagram conclusion =
                            (Diagram) StaticObjects
                                .cg
                                .getChoice(
                                    StaticObjects.mdf.getBounds(),
                                    "concluding diagram?",
                                    diagrams,
                                    new Vector(),
                                    false,
                                    ListSelectionModel.SINGLE_SELECTION)
                                .get(0);
                        //System.out.println("premis is "+premis.toSaveString());
                        //System.out.println("conclusion is "+conclusion.toSaveString());
                        try {
                            Proof p =
                                AutoProofWriterShortened.writeProof(
                                    premis,
                                    conclusion);
                            //System.out.println("got proof "+p);
                            ProofManager.showNewProof(
                                p,
                                false,
                                tryToDrawPictures);
                        } catch (NoProofException npe) {
                            JOptionPane.showMessageDialog(
                                StaticObjects.mdf,
                                "can't be proved:\n" + npe.getCounterExample());
                        }
                    }
                }).start();
            }
        });
        proofs.add(shortenedProofWriterWithNot);
        shortenedProofWriterWithNot.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) { //System.out.println("action listener triggered for proof writer");
                new Thread(new Runnable() {
                    public void run() {
                        Diagram premis =
                            (Diagram) StaticObjects
                                .cg
                                .getChoice(
                                    StaticObjects.mdf.getBounds(),
                                    "initial diagram?",
                                    diagrams,
                                    new Vector(),
                                    false,
                                    ListSelectionModel.SINGLE_SELECTION)
                                .get(0);
                        Diagram conclusion =
                            (Diagram) StaticObjects
                                .cg
                                .getChoice(
                                    StaticObjects.mdf.getBounds(),
                                    "concluding diagram?",
                                    diagrams,
                                    new Vector(),
                                    false,
                                    ListSelectionModel.SINGLE_SELECTION)
                                .get(0);
                        //System.out.println("premis is "+premis.toSaveString());
                        //System.out.println("conclusion is "+conclusion.toSaveString());
                        try {
                            Proof p =
                                AutoProofWriterShortenedWithNegation
                                    .writeProof(
                                    premis,
                                    conclusion);
                            //System.out.println("got proof "+p);
                            ProofManager.showNewProof(
                                p,
                                false,
                                tryToDrawPictures);
                        } catch (NoProofException npe) {
                            JOptionPane.showMessageDialog(
                                StaticObjects.mdf,
                                "can't be proved:\n" + npe.getCounterExample());
                        }
                    }
                }).start();
            }
        });
        proofs.add(heuristicUnitaryRules);
        Vector hur = HeuristicProofWriter.unitaryHRules;
        rules.addAll(hur);
        for (int i = 0; i < hur.size(); i++) {
            final Rule r = (Rule) (hur.get(i));
            JMenuItem ruleMI = new JMenuItem(r.name);
            ruleMI.addActionListener(new RuleActionListener(r));
            heuristicUnitaryRules.add(ruleMI);
            ruleMIUHMap.put(r, ruleMI);
        }
        proofs.add(heuristicCompoundRules);
        hur = HeuristicProofWriter.compoundHRules;
        for (int i = 0; i < hur.size(); i++) {
            final Rule r = (Rule) (hur.get(i));
            JMenuItem ruleMI = new JMenuItem(r.name);
            ruleMI.addActionListener(new RuleActionListener(r));
            heuristicCompoundRules.add(ruleMI);
            ruleMICHMap.put(r, ruleMI);
        }

        proofs.add(heuristicProofWriter);
        heuristicProofWriter.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) { //System.out.println("action listener triggered for proof writer");
                new Thread(new Runnable() {
                    public void run() {
                        //find out which diagrams are unitary
                        FontFrame ff = new HeuristicParameterWindow(diagrams);
                        centre(ff, StaticObjects.mdf);
                        ff.show();
                    }
                }).start();
            }
        });
        //proofs.add(hCalc);
        /*
        heuristicMeasures.add(contourMeasures);
        contourMeasures.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                new Thread(new Runnable() {
                    public void run() {
                        try {
                            Vector[] cs =
                                new CompoundMeasureCalculator(
                                    diagram).calcMeasuresList(
                                    "Contour");
                            JOptionPane.showMessageDialog(
                                StaticObjects.mdf,
                                "C1 is " + cs[0] + " and C2 is " + cs[1]);
                        } catch (ContainsFalseException x) {
                            JOptionPane.showMessageDialog(
                                StaticObjects.mdf,
                                "not implemented for diagrams containing F");
                        }
                    }
                }).start();
            }
        });
        heuristicMeasures.add(zoneMeasures);
        zoneMeasures.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                new Thread(new Runnable() {
                    public void run() {
                        try {
                            Diagram context =
                                (Diagram) StaticObjects
                                    .cg
                                    .getChoice(
                                        StaticObjects.mdf.getBounds(),
                                        "context?",
                                        diagrams,
                                        new Vector(),
                                        false,
                                        ListSelectionModel.SINGLE_SELECTION)
                                    .get(0);
                            Vector[] zs =
                                new CompoundMeasureCalculator(
                                    CompoundMeasureCalculator.calcCF(
                                        diagram,
                                        context)).calcMeasuresList(
                                    "Zone");
                            JOptionPane.showMessageDialog(
                                StaticObjects.mdf,
                                "Z1 is " + zs[0] + " and Z2 is " + zs[1]);
                        } catch (ContainsFalseException x) {
                            JOptionPane.showMessageDialog(
                                StaticObjects.mdf,
                                "not implemented for diagrams containing F");
                        }
                    }
                }).start();
            }
        });
        heuristicMeasures.add(shadingMeasures);
        shadingMeasures.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                new Thread(new Runnable() {
                    public void run() {
                        try {
                            Diagram context =
                                (Diagram) StaticObjects
                                    .cg
                                    .getChoice(
                                        StaticObjects.mdf.getBounds(),
                                        "context?",
                                        diagrams,
                                        new Vector(),
                                        false,
                                        ListSelectionModel.SINGLE_SELECTION)
                                    .get(0);
                            Vector[] ss =
                                new CompoundMeasureCalculator(
                                    CompoundMeasureCalculator.calcVF(
                                        CompoundMeasureCalculator.calcCF(
                                            diagram,
                                            context))).calcMeasuresList(
                                    "Shading");
                            JOptionPane.showMessageDialog(
                                StaticObjects.mdf,
                                "S1 is " + ss[0] + " and S2 is " + ss[1]);
                        } catch (ContainsFalseException x) {
                            JOptionPane.showMessageDialog(
                                StaticObjects.mdf,
                                "not implemented for diagrams containing F");
                        }
                    }
                }).start();
            }
        });
        heuristicMeasures.add(spiderMeasures);
        spiderMeasures.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                new Thread(new Runnable() {
                    public void run() {
                        try {
                            Diagram context =
                                (Diagram) StaticObjects
                                    .cg
                                    .getChoice(
                                        StaticObjects.mdf.getBounds(),
                                        "context?",
                                        diagrams,
                                        new Vector(),
                                        false,
                                        ListSelectionModel.SINGLE_SELECTION)
                                    .get(0);
                            Vector[] ss =
                                new CompoundMeasureCalculator(
                                    CompoundMeasureCalculator.calcCF(
                                        diagram,
                                        context)).calcMeasuresList(
                                    "Spiders");
                            JOptionPane.showMessageDialog(
                                StaticObjects.mdf,
                                "S1 is " + ss[0] + " and S2 is " + ss[1]);
                        } catch (ContainsFalseException x) {
                            JOptionPane.showMessageDialog(
                                StaticObjects.mdf,
                                "not implemented for diagrams containing F");
                        }
                    }
                }).start();
            }
        });
        heuristicMeasures.add(andOrMeasures);
        andOrMeasures.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                new Thread(new Runnable() {
                    public void run() {
                        int c1 =
                            CompoundAnd_OrNumMeasure.getMeasure(diagram, true);
                        int c2 =
                            CompoundAnd_OrNumMeasure.getMeasure(diagram, false);
                        JOptionPane.showMessageDialog(
                            StaticObjects.mdf,
                            "AndNum is " + c1 + " and OrNum is " + c2);
                    }
                }).start();
            }
        });*/
        //heuristicMeasures.add(hCalc);
        /*hCalc.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                new Thread(new Runnable() {
                    public void run() {
                        Diagram context =
                            (Diagram) StaticObjects
                                .cg
                                .getChoice(
                                    StaticObjects.mdf.getBounds(),
                                    "context?",
                                    diagrams,
                                    new Vector(),
                                    false,
                                    ListSelectionModel.SINGLE_SELECTION)
                                .get(0);
                        boolean oldVal = HeuristicCalculatorCompound.debug;
                        HeuristicCalculatorCompound.debug = true;
                        double num =
                            new HeuristicCalculatorCompound(
                                ).calcHeuristic(
                                diagram,
                                context);
                        HeuristicCalculatorCompound.debug = oldVal;
                        JOptionPane.showMessageDialog(
                            StaticObjects.mdf,
                            "num is " + num);
                    }
                }).start();
            }
        });*/
        menu.add(displayMenu);
        displayMenu.add(changeFonts);
        changeFonts.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                changeFonts();
            }
        });
        displayMenu.add(switchViewType);
        switchViewType.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                tryToDrawPictures = !tryToDrawPictures;
            }
        });
        diagramPanel.setPreferredSize(new Dimension(150, 300));
        makeMenusRight();
        pack();
        Dimension screenSize =
            java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setBounds(
            new Rectangle(
                (int) (screenSize.getWidth() / 2 - getWidth() / 2),
                (int) (screenSize.getHeight() / 2 - getHeight() / 2),
                getWidth(),
                getHeight()));
    }

    public void drawPictures(boolean toDrawPictures) {
        tryToDrawPictures = toDrawPictures;
    }

    /** Exit the Application */
    private void exitForm(WindowEvent evt) {
        if (JOptionPane
            .showConfirmDialog(
                this,
                "quit?",
                "really quit?",
                JOptionPane.OK_CANCEL_OPTION)
            == JOptionPane.OK_OPTION) {
            /**/

            System.exit(0);
        }
    }

    public Diagram getPartDiagram() // called by SpiderRemovalFrame
    {
        return partDiagram;
    }

    public Vector getOpenDiagramNames() // used by new diagram frame
    {
        Vector result = new Vector();
        for (int i = 0; i < diagrams.size(); i++) {
            result.add(((Diagram) (diagrams.get(i))).getName());
        }
        return result;
    }

    private void extractCopy() { // sets diagram to be partDiagram but need to check name
        // of partDiagram against existing names
        Diagram newD = partDiagram.getClone();
        newD.renameOutside(getOpenDiagramNames());
        //System.out.println("showing 2");
        showNewDiagram(newD);
    }

    public void doRule(Rule r, RuleApplicationSpec ras) {
        System.out.println(
            "do rule " + r + " to " + ras.d + " in whole " + diagram);
        Diagram oldDiagram = diagram;
        Diagram newDiagram = r.applyRuleToPart(diagram, ras);
        //System.out.println("showing "+newDiagram);
        showNewDiagram(newDiagram);
        if (isProving) { //System.out.println("about to tell proof of step");
            Diagram endOfProof = proof.getLastDiagram();
            //System.out.println("end of proof is "+endOfProof);
            if (endOfProof.equals(oldDiagram)) {
                //System.out.println("about to tell proof to 'doStep'");
                proof.addStepAtEnd(r, ras, diagram);
            }
            //System.out.println("told proof of step");
        }
        //makeMenusRight();
    }

    public void stopWritingProof(Proof p) {
        if (p == proof) {
            isProving = false;
            makeMenusRight();
        }
    }

    private void openDiagram() {
        Vector diagrams = DiagramFileUser.loadFromFile(this);
        if (diagrams == null) {
            return;
        }
        for (int i = 0; i < diagrams.size(); i++) {
            Diagram diagram = (Diagram) diagrams.get(i);
            if (diagram == null) {
                return;
            }
            if (existsDiagramCalled(diagram.getName())) {
                int result =
                    JOptionPane.showOptionDialog(
                        StaticObjects.mdf,
                        "'"
                            + diagram.getName()
                            + "' already open,"
                            + "\nclose existing diagram?",
                        "name in use",
                        JOptionPane.YES_NO_OPTION,
                        JOptionPane.QUESTION_MESSAGE,
                        null,
                        new Object[] {
                            "close existing and open new",
                            "cancel" },
                        "cancel");
                if (result == JOptionPane.YES_OPTION) {
                    //System.out.println("closing diagram called "+diagram.getName());
                    closeDiagram(diagram.getName());
                } else {
                    return;
                }
            }
            showNewDiagram(diagram);
        }
    }

    public void showNewDiagram(Diagram diag) { //
        System.out.println("showing new " + diag);
        final Diagram d = diag.getClone();
        //System.out.println("asked to show "+d.getName());
        //System.out.println("existing diagram names are "+getOpenDiagramNames());
        while (existsDiagramCalled(d.getName())) {
            //System.out.println("extended in showNewDiagram");
            d.setName(
                rwd.abstractsyntax.spiderdiagram.Utilities.makeNameLike(
                    d.getName()));
        }
        d.renameOutside(new Vector());
        //System.out.println("renamed as "+d.getName());
        diagrams.add(d);
        JMenuItem jmi = new JMenuItem(d.getName());
        jmi.addActionListener(new ShowDiagListener());
        currentDiagsMenu.add(jmi);
        diagramMenuItems.add(jmi);
        diagram = d;
        partDiagram = d;
        sayName();
        DiagramShowerTree shower;
        diagramPanel = new FontPanel();
        getContentPane().removeAll();
        getContentPane().add(diagramPanel, BorderLayout.CENTER);
        panelsHT.put(d, diagramPanel);
        //System.out.println("showing new diagram : picture?" +tryToDrawPictures);
        shower =
            new DiagramShowerTree(
                diagramPanel,
                d,
                true,
                tryToDrawPictures,
                300);
        showerTrees.add(shower);
        //System.out.println("showers are " + showers);
        dstree = shower;
        //System.out.println("about to add observer");
        //d.addObserver(ds);
        //System.out.println("about to make menus right");
        makeMenusRight();
        repaint();
        //shower.repaintThings();
    }

    public void refreshOldDiagram() {
        //System.out.println("showing an old diagram");
        dstree = getShowerTreeFor(diagram);
        //System.out.println("getting diagramPanel for "+diagram);
        //System.out.println("panelsHT is "+panelsHT);
        diagramPanel = (FontPanel) panelsHT.get(diagram);
        //System.out.println("diagramPanel extracted is "+diagramPanel);
        getContentPane().removeAll();
        getContentPane().add(diagramPanel, BorderLayout.CENTER);
        pack();
        //System.out.println("dstree is " + dstree);
        makeMenusRight();
        repaint();
    }

    private DiagramShowerTree getShowerTreeFor(Diagram d) {
        for (int i = 0; i < diagrams.size(); i++) {
            if (diagrams.get(i) == d) {
                return (DiagramShowerTree) (showerTrees.get(i));
            }
        }
        return null;
    }

    public void refreshAfterEdit() {
        dstree.refreshDiagramAfterEdit(partDiagram);
        //System.out.println("finished refreshDiagramAfterEdit");
        //diagram.tellObservers();
        if (proof != null) {
            proof.tellObservers();
            //System.out.println("finished telling observers");
        }
        //System.out.println("finished repainting");
        makeMenusRight();
        //System.out.println("finished making menus right");
        repaint();
        //ds.repaintThings();
    }

    public void selectPart(String name) {
        //System.out.println("select '" + name + "'");
        Vector parentAndPart = diagram.findPartCalled(name);
        //System.out.println(" from "+name);
        //System.out.println("found parentAndPart " + parentAndPart);
        parentOfPart = (Diagram) (parentAndPart.get(0));
        partDiagram = (Diagram) (parentAndPart.get(1));
        sayName();
        makeMenusRight();
        repaint();
    }

    private void makeMenusRight() {
        //System.out.println("making menus right for " + partDiagram);
        if (diagram == null) {
            editMenu.setEnabled(false);
            closeDiagram.setEnabled(false);
            storeProof.setEnabled(false);
            ruleMenu.setEnabled(false);
            saveAsDiagram.setEnabled(false);
            proofs.setEnabled(false);
            return;
        }
        // now we can assume there is an open diagram
        editMenu.setEnabled(true);
        closeDiagram.setEnabled(true);
        storeProof.setEnabled(true);
        ruleMenu.setEnabled(true);
        saveAsDiagram.setEnabled(true);
        proofs.setEnabled(true);

        if (partDiagram instanceof UnitaryDiagram) {
            changeShading.setEnabled(true);
            changeZones.setEnabled(true);
            addSpiderMI.setEnabled(true);
            UnitaryDiagram ud = (UnitaryDiagram) (partDiagram);
            if (ud.getSpiders().size() > 0) {
                removeSpiderMI.setEnabled(true);
            } else {
                removeSpiderMI.setEnabled(false);
            }
        } else // partDiagram is compound
            {
            changeShading.setEnabled(false);
            changeZones.setEnabled(false);
            addSpiderMI.setEnabled(false);
            removeSpiderMI.setEnabled(false);
        }
        if (isProving) // || !diagram.isEqualTo(proof.getLastDiagram()))
            { //System.out.println("disabling edit menus");
            changeShading.setEnabled(false);
            changeZones.setEnabled(false);
            addSpiderMI.setEnabled(false);
            removeSpiderMI.setEnabled(false);
            storeProof.setEnabled(false);
        } else {
            storeProof.setEnabled(true);
        }
        // find out which rwd.rules are applicable
        for (int i = 0; i < rules.size(); i++) {
            Rule r = (Rule) rules.get(i);
            //System.out.println("checking rule " + r);
            if (r != null) {
                if (r.canApplyTo(partDiagram)) {
                    //System.out.println(
                    //    "diagram contains negation of part? "
                    //        + diagram.containsNegationOf(partDiagram));
                    if (r.isInvertible()
                        || !diagram.containsNegationOf(partDiagram)) {
                        JMenuItem mi = (JMenuItem) (ruleMIMap.get(r));
                        if (mi != null) {
                            //System.out.println("enable " + mi.getText());
                            mi.setEnabled(true);
                        }
                        mi = (JMenuItem) (ruleMIUHMap.get(r));
                        if (mi != null) {
                            //System.out.println("enable " + mi.getText());
                            mi.setEnabled(true);
                        }
                        mi = (JMenuItem) (ruleMICHMap.get(r));
                        if (mi != null) {
                            //System.out.println("enable " + mi.getText());
                            mi.setEnabled(true);
                        }
                    } else {
                        JMenuItem mi = (JMenuItem) (ruleMIMap.get(r));
                        if (mi != null) {
                            //System.out.println("disable " + mi.getText());
                            mi.setEnabled(false);
                        }
                        mi = (JMenuItem) (ruleMIUHMap.get(r));
                        if (mi != null) {
                            //System.out.println("disable " + mi.getText());
                            mi.setEnabled(false);
                        }
                        mi = (JMenuItem) (ruleMICHMap.get(r));
                        if (mi != null) {
                            //System.out.println("disable " + mi.getText());
                            mi.setEnabled(false);
                        }
                    }
                } else {
                    JMenuItem mi = (JMenuItem) (ruleMIMap.get(r));
                    if (mi != null) {
                        //System.out.println("disable " + mi.getText());
                        mi.setEnabled(false);
                    }
                    mi = (JMenuItem) (ruleMIUHMap.get(r));
                    if (mi != null) {
                        //System.out.println("disable " + mi.getText());
                        mi.setEnabled(false);
                    }
                    mi = (JMenuItem) (ruleMICHMap.get(r));
                    if (mi != null) {
                        //System.out.println("disable " + mi.getText());
                        mi.setEnabled(false);
                    }
                }
            }
        }
        //System.out.println("done making menus right for "+diagram);
    }

    private void closeCurrentDiagram() {
        closeDiagram(diagram.getName());
    }

    public void closeDiagram(String name) { // locate the diagram
        //System.out.println("close diagram :"+name);
        diagramSearch : for (int i = 0; i < diagrams.size(); i++) {
            Diagram d = (Diagram) (diagrams.get(i));
            if (d.getName().equals(name)) {
                DiagramShowerTree s = getShowerTreeFor(d);
                showerTrees.remove(s);
                diagrams.remove(d);
                panelsHT.remove(d);
            }
        }
        diagram = null;
        dstree = null;
        menuSearch : for (int i = 0; i < diagramMenuItems.size(); i++) {
            JMenuItem jmi = (JMenuItem) (diagramMenuItems.get(i));
            if (jmi.getText().equals(name)) {
                diagramMenuItems.remove(jmi);
                currentDiagsMenu.remove(jmi);
                break menuSearch;
            }
        }
        // remove menu item
        editMenu.setEnabled(false);
        saveAsDiagram.setEnabled(false);
        closeDiagram.setEnabled(false);
        storeProof.setEnabled(false);
        // disable edit menus, save option
        if (diagramMenuItems.size() > 0) {
            //System.out.println("there is a menu item left");
             ((JMenuItem) (diagramMenuItems.get(0))).doClick();
        } else { //System.out.println("there is no menu item left");
            showNoDiagram();
        }
    }

    private void showNoDiagram() {
        diagramPanel.removeAll();
        diagramPanel.add(new JLabel("no diagram open"));
        diagramPanel.revalidate();
    }

    public class ShowDiagListener implements ActionListener {
        public void actionPerformed(ActionEvent e) { // find out which diagram in diagrams arraylist it is
            String diagName = ((JMenuItem) (e.getSource())).getText();
            for (int i = 0; i < diagrams.size(); i++) {
                Diagram d = (Diagram) (diagrams.get(i));
                if (d.getName().equals(diagName)) {
                    diagram = d;
                    partDiagram = d;
                    sayName();
                    refreshOldDiagram();
                    //diagramPanel.revalidate();
                    return;
                }
            }
        }
    }

    public class ShadingListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            new Thread(new Runnable() {
                public void run() {
                    UnitaryDiagram ud = (UnitaryDiagram) (partDiagram);
                    Vector newShading =
                        StaticObjects.cg.getChoice(
                            StaticObjects.mdf.getBounds(),
                            "shaded zones?",
                            ud.getCloneOfZoneSet(),
                            ud.getCloneOfShadedZones(),
                            true,
                            ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
                    ud.setShadedZones(newShading);
                    refreshAfterEdit();
                }
            }).start();
        }
    }

    public class ZoneListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            new Thread(new Runnable() {
                public void run() {
                    UnitaryDiagram ud = (UnitaryDiagram) (partDiagram);
                    Vector contourSet = ud.getCloneOfContourSet();
                    Vector zones =
                        UnitaryDiagram.allPossibleZones(contourSet);
                    zones.remove(Zone.getZone(new Vector(), contourSet));
                    Vector newZoneSet =
                        StaticObjects.cg.getChoice(
                            StaticObjects.mdf.getBounds(),
                            "zones?",
                            zones,
                            ud.getCloneOfZoneSet(),
                            false,
                            ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
                    newZoneSet.add(Zone.getZone(new Vector(), contourSet));
                    // check that there's a zone in each contour
                    Vector contoursWithAZoneInside = new Vector();
                    for (int i = 0; i < newZoneSet.size(); i++) {
                        contoursWithAZoneInside.addAll(
                            ((Zone) (newZoneSet.get(i)))
                                .getContainingContours());
                    }
                    if (!contoursWithAZoneInside.containsAll(contourSet)) {
                        JOptionPane.showMessageDialog(
                            StaticObjects.mdf,
                            "insufficient zones for contours");
                        return;
                    }
                    ud.setNewZoneSet(newZoneSet);
                    refreshAfterEdit();
                }
            }).start();
        }
    }

    public class AddSpiderListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            new Thread(new Runnable() {
                public void run() {
                    Vector zoneSet =
                        ((UnitaryDiagram) (partDiagram)).getCloneOfZoneSet();
                    Vector habitat =
                        StaticObjects.cg.getChoice(
                            StaticObjects.mdf.getBounds(),
                            "habitat?",
                            zoneSet,
                            new Vector(),
                            false,
                            ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
                    ((UnitaryDiagram) (partDiagram)).addSpider(habitat);
                    refreshAfterEdit();
                }
            }).start();
        }
    }

    public class RemoveSpiderListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            new Thread(new Runnable() {
                public void run() {
                    Vector spiders =
                        ((UnitaryDiagram) partDiagram).getSpiders();
                    //System.out.println("spiders are "+spiders);
                    //System.out.println("spider is "+oneSpider);
                    Spider s =
                        (Spider)
                            (
                                (
                                    StaticObjects.cg.getChoice(
                                        StaticObjects.mdf.getBounds(),
                                        "spider?",
                                        spiders,
                                        new Vector(),
                                        false,
                                        ListSelectionModel
                                            .SINGLE_SELECTION)))
                                            .get(
                            0);
                    ((UnitaryDiagram) partDiagram).removeSpider(s);
                    refreshAfterEdit();
                }
            }).start();
        }
    }

    public class EditNameActionListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            String s =
                JOptionPane.showInputDialog(
                    StaticObjects.mdf,
                    "type in new name",
                    "rename " + partDiagram.getName(),
                    JOptionPane.PLAIN_MESSAGE);
            if (s == null) {
                return;
            }
            Vector bannedNames = diagram.getAllNames(); // "vertical check"
            boolean nameIsBanned = false;
            nameSearch : for (int i = 0; i < bannedNames.size(); i++) {
                String bannedName = (String) (bannedNames.get(i));
                if (bannedName.equals(s)
                    && !bannedName.equals(partDiagram.getName())) {
                    nameIsBanned = true;
                    break nameSearch;
                }
            }
            if (existsDiagramCalled(s)
                || nameIsBanned) // "across the top check"
                {
                JOptionPane.showMessageDialog(
                    StaticObjects.mdf,
                    "'" + s + "' already in use",
                    "name in use",
                    JOptionPane.ERROR_MESSAGE);
                return;
            }
            if (partDiagram == diagram) {
                // search through diagramMenuItems and rename whatever
                for (int i = 0; i < diagramMenuItems.size(); i++) {
                    JMenuItem jmi = (JMenuItem) (diagramMenuItems.get(i));
                    if (jmi.getText().equals(partDiagram.getName())) {
                        jmi.setText(s);
                        break;
                    }
                }
            }
            System.out.println("setting new name OK");
            partDiagram.setName(s);
            System.out.println("about to refresh");
            refreshAfterEdit();
            System.out.println("done refresh");
        }
    }

    public void centre(FontFrame toBeCentred, FontFrame toCentreUpon) {
        Rectangle r = toCentreUpon.getBounds();
        int centrex = (int) (r.getCenterX());
        int centrey = (int) (r.getCenterY());
        int halfwidth = (int) (((double) (toBeCentred.getWidth())) / 2);
        int halfheight = (int) (((double) (toBeCentred.getHeight())) / 2);
        if (centrex - halfwidth < 0) {
            centrex = halfwidth;
        }
        if (centrey - halfheight < 0) {
            centrey = halfheight;
        }
        toBeCentred.setBounds(
            centrex - halfwidth,
            centrey - halfheight,
            halfwidth * 2,
            halfheight * 2);
    }

    public boolean existsDiagramCalled(String name) {
        for (int i = 0; i < diagrams.size(); i++) {
            Diagram d = (Diagram) (diagrams.get(i));
            //System.out.println("see whether existing name "+d.getName()+" matches "+name);
            if (d.getName().equals(name)) {
                //System.out.println("foudn match");
                return true;
            }
        }
        //System.out.println("no match found called "+name);
        return false;
    }

    class RuleActionListener implements ActionListener {
        Rule r;

        RuleActionListener(Rule r) {
            this.r = r;
        }

        public void actionPerformed(ActionEvent e) {
            applyRule(r);
        }
    }

    private void applyRule(Rule r) { // either r needs no more input
        try {
            RuleNoSpec rns = (RuleNoSpec) r;
            doRule(rns, new RuleAppliesToJustADiagram(partDiagram));
        } catch (ClassCastException e) {
        }
        // or use a choicegetter to get user choice...
        try {
            final RuleWithChoice rwc = (RuleWithChoice) r;
            new Thread(new Runnable() {
                public void run() {
                    Vector a =
                        StaticObjects.cg.getChoice(
                            StaticObjects.mdf.getBounds(),
                            "apply rule how?",
                            rwc.getApplicationPossibilities(partDiagram),
                            new Vector(),
                            false,
                            ListSelectionModel.SINGLE_SELECTION);
                    if (a != null) {
                        RuleApplicationSpec ras =
                            (RuleApplicationSpec) a.get(0);
                        doRule(rwc, ras);
                    }
                }
            }).start();
        } catch (ClassCastException e) {
        }
        // or use input (for add contour)
        try {
            final RuleNeedsInput rni = (RuleNeedsInput) r;
            if (rni.getInputType() == Class.forName("java.lang.String")) {
                UnitaryDiagram ud = (UnitaryDiagram) partDiagram;
                String nameTyped =
                    JOptionPane.showInputDialog(
                        StaticObjects.mdf,
                        "new contour name (not "
                            + ud.getCloneOfContourSet()
                            + ")",
                        "type new contour name");
                for (int i = 0; i < ud.getCloneOfContourSet().size(); i++) {
                    if (((Contour) (ud.getCloneOfContourSet().get(i)))
                        .toString()
                        .equals(nameTyped)) {
                        JOptionPane.showMessageDialog(
                            StaticObjects.mdf,
                            "contour name already present",
                            "can't duplicate contour names",
                            JOptionPane.ERROR_MESSAGE);
                        return;
                    } else if (nameTyped == null || nameTyped.length() > 1) {
                        JOptionPane.showMessageDialog(
                            StaticObjects.mdf,
                            "contour names must be single letters",
                            "single letter only please",
                            JOptionPane.ERROR_MESSAGE);
                        return;
                    }
                }
                RuleApplicationSpec ras =
                    new RuleAppliesWithString(partDiagram, nameTyped);
                doRule(rni, ras);
            } else if (
                rni.getInputType()
                    == Class.forName(
                        "rwd.abstractsyntax.spiderdiagram.Diagram")) {
                final Vector finalDiagrams = diagrams;
                final Rule finalRni = rni;
                new Thread(new Runnable() {
                    public void run() {
                        Diagram otherd =
                            (Diagram) StaticObjects
                                .cg
                                .getChoice(
                                    StaticObjects.mdf.getBounds(),
                                    "diagram?",
                                    finalDiagrams,
                                    new Vector(),
                                    false,
                                    ListSelectionModel.SINGLE_SELECTION)
                                .get(0);
                        RuleApplicationSpec ras =
                            new RuleAppliesToDiagramAndDiagram(
                                partDiagram,
                                otherd);
                        doRule(finalRni, ras);
                    }
                }).start();
            }
        } catch (ClassCastException e) {
        } catch (ClassNotFoundException e) {
        }
    }

    private void newDiagramRequested() {
        // find out unitary or compound
        Object[] possibleValues = { "Unitary", "Compound", "Not", "False" };
        String selectedValue =
            (String) JOptionPane.showInputDialog(
                null,
                "What kind of new diagram?",
                "Input",
                JOptionPane.INFORMATION_MESSAGE,
                null,
                possibleValues,
                possibleValues[0]);
        if (selectedValue != null && selectedValue.equals("False")) {
            showNewDiagram(new FalseDiagram("false"));
        } else if (selectedValue != null && selectedValue.equals("Not")) {
            new Thread(new Runnable() {
                public void run() {
                    Vector chosenDiagrams =
                        StaticObjects.cg.getChoice(
                            StaticObjects.mdf.getBounds(),
                            "Negation of which diagram?",
                            diagrams,
                            new Vector(),
                            false,
                            ListSelectionModel.SINGLE_SELECTION);
                    Diagram choice = (Diagram) chosenDiagrams.get(0);
                    showNewDiagram(
                        new NotCompoundDiagram(
                            chosenDiagrams,
                            "not_" + choice.getName()));
                }
            }).start();
        } else if (selectedValue != null && selectedValue.equals("Unitary")) {
            // if unitary, get input of contour names
            String contourWord =
                JOptionPane.showInputDialog(
                    StaticObjects.mdf,
                    "type in contour names");
            if (contourWord == null) {
                return;
            }
            //extract contours from this
            Vector contours = new Vector();
            for (int i = 0; i < contourWord.length(); i++) {
                contours.add(
                    Contour.getContour(contourWord.substring(i, i + 1)));
            }
            final Vector contours2 = contours;
            // build zone set
            new Thread(new Runnable() {
                public void run() {
                    Vector possibleZones =
                        UnitaryDiagram.allPossibleZones(contours2);
                    possibleZones.remove(
                        Zone.getZone(new Vector(), contours2));
                    Vector zones =
                        StaticObjects.cg.getChoice(
                            StaticObjects.mdf.getBounds(),
                            "zones?",
                            possibleZones,
                            new Vector(),
                            false,
                            ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
                    // get selection of zones
                    // build new unitary diagram
                    zones.add(Zone.getZone(new Vector(), contours2));
                    Vector contoursWithAZoneInside = new Vector();
                    for (int i = 0; i < zones.size(); i++) {
                        contoursWithAZoneInside.addAll(
                            ((Zone) (zones.get(i))).getContainingContours());
                    }
                    if (!contoursWithAZoneInside.containsAll(contours2)) {
                        JOptionPane.showMessageDialog(
                            StaticObjects.mdf,
                            "insufficient zones for contours");
                        return;
                    } else {
                        showNewDiagram(
                            new UnitaryDiagram(zones, "" + contours2));
                    }
                }
            }).start();
        } else if (selectedValue != null && selectedValue.equals("Compound")) {
            new Thread(new Runnable() {
                public void run() {
                    final FontFrame jf =
                        new FontFrame("construct compound diagram");
                    final JList diagList = new JList(new Vector(diagrams));
                    final DefaultListModel chosenOnes = new DefaultListModel();
                    FontScrollPane diagSP = new FontScrollPane(diagList);
                    diagSP.setPreferredSize(new Dimension(100, 200));
                    jf.getContentPane().add(diagSP, BorderLayout.WEST);
                    final JList chosenList = new JList(chosenOnes);
                    FontScrollPane chosenSP = new FontScrollPane(chosenList);
                    chosenSP.setPreferredSize(new Dimension(100, 200));
                    jf.getContentPane().add(chosenSP, BorderLayout.EAST);
                    FontPanel buttonPanel = new FontPanel();
                    jf.getContentPane().add(buttonPanel, BorderLayout.CENTER);
                    buttonPanel.setLayout(
                        new BoxLayout(buttonPanel, BoxLayout.Y_AXIS));
                    JButton addButton = new JButton("add") {
                        public float getAlignmentX() {
                            return Component.CENTER_ALIGNMENT;
                        }
                    };
                    buttonPanel.add(addButton);
                    JButton removeButton = new JButton("remove") {
                        public float getAlignmentX() {
                            return Component.CENTER_ALIGNMENT;
                        }
                    };
                    buttonPanel.add(removeButton);
                    FontPanel lowerp = new FontPanel();
                    jf.getContentPane().add(lowerp, BorderLayout.SOUTH);
                    lowerp.setLayout(new FlowLayout());
                    final JButton andMaker = new JButton("AND");
                    andMaker.setEnabled(false);
                    lowerp.add(andMaker);
                    andMaker.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            ListModel items = chosenList.getModel();
                            Vector diags = new Vector();
                            String newName = "";
                            for (int i = 0; i < items.getSize(); i++) {
                                Diagram d = (Diagram) items.getElementAt(i);
                                diags.add(d.getClone());
                                newName = newName + d.getName() + " and ";
                            }
                            newName =
                                newName.substring(0, newName.length() - 5);
                            Diagram d = new AndCompoundDiagram(diags, newName);
                            d.renameOutside(new Vector());
                            jf.hide();
                            showNewDiagram(d);
                        }
                    });
                    final JButton orMaker = new JButton("OR");
                    orMaker.setEnabled(false);
                    lowerp.add(orMaker);
                    orMaker.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            ListModel items = chosenList.getModel();
                            Vector diags = new Vector();
                            String newName = "";
                            for (int i = 0; i < items.getSize(); i++) {
                                Diagram d = (Diagram) items.getElementAt(i);
                                diags.add(d.getClone());
                                newName = newName + d.getName() + " or ";
                            }
                            newName =
                                newName.substring(0, newName.length() - 4);
                            Diagram d = new OrCompoundDiagram(diags, newName);
                            d.renameOutside(new Vector());
                            jf.hide();
                            showNewDiagram(d);
                        }
                    });
                    addButton.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            Object[] items = diagList.getSelectedValues();
                            for (int i = 0; i < items.length; i++) {
                                chosenOnes.addElement(items[i]);
                            }
                            if (chosenOnes.size() > 1) {
                                andMaker.setEnabled(true);
                                orMaker.setEnabled(true);
                            }
                        }
                    });
                    removeButton.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            int[] items = chosenList.getSelectedIndices();
                            for (int i = items.length - 1; i >= 0; i--) {
                                chosenOnes.removeElementAt(items[i]);
                            }
                            if (chosenOnes.size() < 2) {
                                andMaker.setEnabled(false);
                                orMaker.setEnabled(false);
                            }
                        }
                    });
                    jf.pack();
                    centre(jf, StaticObjects.mdf);
                    jf.show();
                }
            }).start();
        }
    }
    private void changeFonts() {
        try {
            StaticObjects.setFontSize(
                Integer.parseInt(
                    JOptionPane.showInputDialog(
                        StaticObjects.mdf,
                        "font size")));
        } catch (NumberFormatException x) {
            JOptionPane.showMessageDialog(
                StaticObjects.mdf,
                "not a recognised number");
        }
    }

    public static void main(String args[]) {
        StaticObjects.mdf.show();
    }

    private void sayName() {
        if (partDiagram == null) {
            setTitle("spider diagram proving tool");
        } else {
            setTitle("part is " + partDiagram.getName());
        }
    }
}
