package rwd.gui;

import java.awt.event.WindowEvent;
import javax.swing.JButton;
import rwd.rules.Proof;
import rwd.rules.Rule;
import rwd.rules.application.RuleApplicationSpec;
import rwd.abstractsyntax.spiderdiagram.Diagram;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Dimension;
import java.util.Vector;
import java.util.Observer;
import java.util.Observable;
import javax.swing.JLabel;
import java.awt.Rectangle;

public class SmoothProofViewer extends FontFrame implements Observer {
    Proof p; // "current"
    private FontPanel bigPanel = new FontPanel();
    private FontScrollPane jsp;
    private JLabel statusLabel = new JLabel();
    private FontFrame thisFrame = this;

    private boolean isUserExtensible = false;
    private int lastProofStepShown = 0;
    private JButton lastProofStepButton = null;
    private ActionListener extendProofListener = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
            	//
            	//System.out.println("extension listener triggered");
            	
		    extendVisibleProof();
        }
    };
    Vector thingsToRepaint = new Vector();

    public SmoothProofViewer(Proof p, boolean autoUpdate) { //

        //System.out.println("proof viewer about to show proof "+p);
        super("proof with " + p.numSteps() + " steps");
        initGUI();
        isUserExtensible = !autoUpdate;
        this.p = p;
        if (autoUpdate) {
            p.addObserver(this);
            update(null, null);
        } else {
            startWithSome();
        }
        p.goToStart();
        pack();
        Dimension screenSize =
            java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setBounds(
            new Rectangle(
                (int) (screenSize.getWidth() - getWidth()),
                (int) (screenSize.getHeight() - getHeight()),
                getWidth(),
                getHeight()));
        show();
    }

    // for autoUpdate
    public void update(Observable o, Object args) {
        FontPanel jp = new FontPanel();
        thisFrame.getContentPane().remove(jsp);
        jsp = new FontScrollPane(jp);
        jsp.setBounds(0, 0, 700, 330);
        jsp.setMaximumSize(new Dimension(700, 330));
        jsp.setPreferredSize(new Dimension(700, 330));
        jsp.setMinimumSize(new Dimension(700, 330));
        jp.setLayout(new FlowLayout());
        ////
        thisFrame.getContentPane().add(jsp, BorderLayout.CENTER);
        bigPanel = jp;
        //System.out.println("set bigPanel to be new jp");
        //bigPanel.setBackground(Color.red);
        thingsToRepaint = new Vector();
        thingsToRepaint.add(addDiagramPanel(p.getFirstDiagram()));
        //System.out.println("added a diagram");
        p.goToStart();
        int stepNumber = 0;
        while (p.goToNextStep()) //&& stepNumber<20)
            {
            stepNumber++;
            Rule r = p.getPreviousRule();
            RuleApplicationSpec ras = p.getPreviousRuleApplicationSpec();
            addStepPanel(r, ras);
            thingsToRepaint.add(addDiagramPanel(p.getCurrentDiagram()));
            //System.out.print(","+stepNumber);
        }
        setTitle("proof with " + p.numSteps() + " steps");
        repaintThings();
    }

    // for user-extensible diagrams
    public void startWithSome() { //System.out.println("start with some");
        FontPanel jp = new FontPanel();
        thisFrame.getContentPane().remove(jsp);
        jsp = new FontScrollPane(jp);
        jsp.setBounds(0, 0, 700, 330);
        jsp.setMaximumSize(new Dimension(700, 330));
        jsp.setPreferredSize(new Dimension(700, 330));
        jsp.setMinimumSize(new Dimension(700, 330));
        jp.setLayout(new FlowLayout());
        ////
        thisFrame.getContentPane().add(jsp, BorderLayout.CENTER);
        bigPanel = jp;
        //System.out.println("set bigPanel to be new jp");
        //bigPanel.setBackground(Color.red);
        //System.out.println("adding one panel");
        thingsToRepaint = new Vector();
        thingsToRepaint.add(addDiagramPanel(p.getFirstDiagram()));
        //System.out.println("added a diagram");
        p.goToStart();
        if (p.goToNextStep()) //&& stepNumber<20)
            {
            Rule r = p.getPreviousRule();
            RuleApplicationSpec ras = p.getPreviousRuleApplicationSpec();
            addStepPanel(r, ras);
            //System.out.println("adding a 2nd panel");
            thingsToRepaint.add(addDiagramPanel(p.getCurrentDiagram()));
            lastProofStepShown = 1;
        }
        if (p.goToNextStep()) //&& stepNumber<20)
            {
            Rule r = p.getPreviousRule();
            RuleApplicationSpec ras = p.getPreviousRuleApplicationSpec();
            addStepPanel(r, ras);
        }
        repaintThings();
    }

    private void extendVisibleProof() {
        p.goToStart();
        int n = 0;
        while (n < lastProofStepShown) {
            p.goToNextStep();
            n++;
        }
        // add diagram
		thingsToRepaint.add(addDiagramPanel(p.getCurrentDiagram()));
		// if possible, add next step
		if (p.goToNextStep()) {
			Rule r = p.getPreviousRule();
			RuleApplicationSpec ras = p.getPreviousRuleApplicationSpec();
			addStepPanel(r, ras);
		}
		repaintThings();
    }

    private SmoothDiagramShowerTree addDiagramPanel(Diagram d) {
        if (lastProofStepButton != null) {
            lastProofStepButton.removeActionListener(extendProofListener);
        }
        //System.out.println("adding diagram panel for "+d);
        FontPanel proofPanel = new FontPanel();
        SmoothDiagramShowerTree dst =
            new SmoothDiagramShowerTree(proofPanel, d, false, true,280);
        proofPanel.setPreferredSize(new Dimension(300, 300));
        bigPanel.add(proofPanel);
        jsp.validate();
        //System.out.println("scrolling to show panel at "+bigPanel.getWidth());
        bigPanel.scrollRectToVisible(
            new Rectangle(bigPanel.getWidth(), 0, 0, 0));
        repaintThings();
        return dst;
    }

    private void addStepPanel(final Rule r, final RuleApplicationSpec ras) {
        JButton step = new JButton(">");
        step.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) { //System.out.println("description writing listener triggered");
                String description = "rwd.rules: " + r.name + "   ";
                description =
                    description
                        + "\napplied to (sub)diagram: "
                        + ras.d.getName()
                        + "   ";
                description = description + "\napplied in case: " + ras + "   ";
                statusLabel.setText(description);
            }
        });
        lastProofStepShown++;
        lastProofStepButton = step;
        lastProofStepButton.addActionListener(extendProofListener);
        step.setPreferredSize(new Dimension(30, 50));
        bigPanel.add(step);
        jsp.validate();
        //System.out.println("scrolling to show button at "+bigPanel.getWidth());
        bigPanel.scrollRectToVisible(
            new Rectangle(bigPanel.getWidth(), 0, 0, 0));
        repaintThings();
    }

    /** This method is called from within the constructor to initialize the form. */
    private void initGUI() {
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
        });
        bigPanel.setLayout(new FlowLayout());
        jsp = new FontScrollPane(bigPanel);
        jsp.setBounds(0, 0, 700, 330);
        jsp.setMaximumSize(new Dimension(700, 330));
        jsp.setMinimumSize(new Dimension(700, 330));
        jsp.setPreferredSize(new Dimension(700, 330));
        getContentPane().add(jsp, BorderLayout.CENTER);
        statusLabel.setPreferredSize(new Dimension(700, 50));
        getContentPane().add(statusLabel, BorderLayout.SOUTH);
    }

    /** Exit the Application */
    private void exitForm(WindowEvent evt) {
        //System.exit(0);
        ProofManager.close(p);
    }

    public void repaintThings() { // who should call this, when?
        System.out.println("repainting things in proofviewer");
        for (int i = 0; i < thingsToRepaint.size(); i++) {
            ((SmoothDiagramShowerTree) thingsToRepaint.get(i)).repaintThings();
        }
    }

    public void show() {
        super.show();
        repaintThings();
    }
}
