package rwd.gui.editor;

import java.awt.*;
import java.awt.event.*;

import java.io.File;

import java.util.*;
import java.util.List;

import javax.swing.*;

import rwd.abstractsyntax.spiderdiagram.*;
import rwd.concretesyntax.*;
import rwd.concretesyntax.constraintdiagram.*;
import rwd.concretesyntax.spiderdiagram.*;

import rwd.gui.FontFrame;
import rwd.gui.editor.listeners.*;
import rwd.gui.editor.listeners.DrawNewTextListener;
import rwd.gui.editor.viewers.*;

import uk.ac.kent.cs.kmf.util.*;

public class EditorFrame 
	extends FontFrame 
	implements rwd.views.abstracttoconcrete.forcemodel.Constants 
{
	protected final Component thisFrame = this;

	public static Options options = new Options();

	// for centering JOptionPanes in anonymous actionListeners

	public EditorFrame(int width, int height) {
		super("Constraint diagram editor");
		this.setSize(new Dimension(width, height));
		this.setResizable(true);
		initComponents();
		initMenu();
		this.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				System.exit(0);
			}
		});
		this.setVisible(true);
		setExtendedState(JFrame.MAXIMIZED_BOTH);
		//*** Added by op
		//--- Create INPUT file chooser ---
		xmiFilter = new EditorFileFilter("*.xml, *.xmi");
		xmiFilter.addExtension("xmi");
		xmiFilter.addExtension("xml");
		File dir = new File(System.getProperty("user.dir"));
		inputChooser = new JFileChooser(dir);
		inputChooser.setFileFilter(xmiFilter);
		//***
	}

	protected void initComponents() {
		screenSize = getToolkit().getScreenSize();
		screenSize.setSize(screenSize.width, screenSize.height);
		setSize(screenSize);

		//--- Create SPLIT PANELS ---
		mainPanel = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
		mainPanel.setDividerSize(dividerSize);
		mainPanel.setDividerLocation((int) ((int) (screenSize.height * rightDiv)));
		mainPanel.setContinuousLayout(true);

		//--- Create frames for editor and log
		JInternalFrame editorFrame = new JInternalFrame();
		editorFrame.getContentPane().add(desktopPane);
		JInternalFrame consoleFrame = new JInternalFrame();
		consoleFrame.getContentPane().add((JTextArea) log);

		//--- Add components to SPLIT PANELS ---
		mainPanel.setTopComponent(new JScrollPane(desktopPane,
				JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED));
		mainPanel.setBottomComponent(new JScrollPane((JTextArea) log,
				JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED));
		getContentPane().add(mainPanel, BorderLayout.CENTER);
	}

	protected void initMenu() {
		// Create main menu, submenus, and menu items
		JMenuBar menuBar = new JMenuBar();
		JMenuItem menuItem;

		//
		// File menu
		//
		JMenu fileMenu = new JMenu("File");
		menuItem = makeMenuItem("New", new FileNewListener(this));
		fileMenu.add(menuItem);
		menuItem = makeMenuItem("Open", new FileOpenListener(this));
		fileMenu.add(menuItem);
		menuItem = makeMenuItem("Save", new FileSaveListener(this));
		fileMenu.add(menuItem);
		menuItem = makeMenuItem("Save As", new FileSaveListener(this, true));
		fileMenu.add(menuItem);
		fileMenu.addSeparator();
		menuItem = makeMenuItem("Exit", new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// Should ask for confirmation first!
				System.exit(0);
			}
		});
		fileMenu.add(menuItem);

		//
		// Edit menu
		//
		JMenu editMenu = new JMenu("Edit");
		menuItem = makeMenuItem("Undo", new EditUndoRedoListener(this,
				EditUndoRedoListener.UNDO));
		editMenu.add(menuItem);
		menuItem = makeMenuItem("Redo", new EditUndoRedoListener(this,
				EditUndoRedoListener.REDO));
		editMenu.add(menuItem);
		editMenu.addSeparator();
		menuItem = makeMenuItem("Cut", new EditCopyCutPasteListener(this,
				EditCopyCutPasteListener.CUT));
		editMenu.add(menuItem);
		menuItem = makeMenuItem("Copy", new EditCopyCutPasteListener(this,
				EditCopyCutPasteListener.COPY));
		editMenu.add(menuItem);
		menuItem = makeMenuItem("Paste", new EditCopyCutPasteListener(this,
				EditCopyCutPasteListener.PASTE));
		editMenu.add(menuItem);
		editMenu.addSeparator();
		menuItem = makeMenuItem("Delete", new EditDeleteListener(this));
		editMenu.add(menuItem);
		menuItem = makeMenuItem("Edit", new EditDiagramItemListener(this));
		editMenu.add(menuItem);
		editMenu.addSeparator();
		menuItem = makeMenuItem("Zoom in", new EditZoomListener(this,
				EditZoomListener.ZOOM_IN, -1));
		editMenu.add(menuItem);
		menuItem = makeMenuItem("Zoom out", new EditZoomListener(this,
				EditZoomListener.ZOOM_OUT, 1));
		editMenu.add(menuItem);
		menuItem = makeMenuItem("Zoom", new EditZoomListener(this,
				EditZoomListener.ZOOM, 100));
		editMenu.add(menuItem);

		//
		// Draw menu
		//
		JMenu drawMenu = new JMenu("Draw");
		menuItem = makeMenuItem("Decoration box", new DrawNewBoxListener(this));
		drawMenu.add(menuItem);
		menuItem = makeMenuItem("Contour", new DrawNewContourListener(this));
		drawMenu.add(menuItem);
		menuItem = makeMenuItem("Foot", new DrawNewFootListener(this,
				ConcreteSpiderFoot.NORMAL));
		drawMenu.add(menuItem);
		menuItem = makeMenuItem("Leg", new DrawNewLegListener(this,
				ConcreteSpiderLeg.NORMAL));
		drawMenu.add(menuItem);
		menuItem = makeMenuItem("Tie", new DrawNewLegListener(this,
				ConcreteSpiderLeg.TIE));
		drawMenu.add(menuItem);
		menuItem = makeMenuItem("Strand", new DrawNewLegListener(this,
				ConcreteSpiderLeg.STRAND));
		drawMenu.add(menuItem);
		menuItem = makeMenuItem("Universal Spider", new DrawNewFootListener(
				this, ConcreteSpiderFoot.UNIVERSAL));
		drawMenu.add(menuItem);
		menuItem = makeMenuItem("Schrodinger Spider", new DrawNewFootListener(
				this, ConcreteSpiderFoot.SCHROEDINGER));
		drawMenu.add(menuItem);
		menuItem = makeMenuItem("Arc", new DrawNewArcListener(this,
				ConcreteArc.QUAD));
		drawMenu.add(menuItem);
		menuItem = makeMenuItem("Line", new DrawNewArcListener(this,
				ConcreteArc.LINE));
		drawMenu.add(menuItem);
		menuItem = makeMenuItem("Text", new DrawNewTextListener(this,
				ConcreteTextBox.NORMAL));
		drawMenu.addSeparator();
		menuItem = makeMenuItem("From abstract diagram",
				new DrawCS2AS2CSListener(this));
		drawMenu.add(menuItem);

		//
		// Format menu
		//
		JMenu formatMenu = new JMenu("Format");
		menuItem = makeMenuItem("Label", new FormatSetLabelListener(this));
		formatMenu.add(menuItem);
		menuItem = makeMenuItem("Font", new FormatFontListener(this));
		formatMenu.add(menuItem);
		formatMenu.addSeparator();
		menuItem = makeMenuItem("Foreground Color",
				new FormatForeColorListener(this));
		formatMenu.add(menuItem);
		menuItem = makeMenuItem("Background Color",
				new FormatBackColorListener(this));
		formatMenu.add(menuItem);
		formatMenu.addSeparator();
		menuItem = makeMenuItem("Thickness", new FormatThicknessListener(this));
		formatMenu.add(menuItem);
		menuItem = makeMenuItem("Type", new FormatTypeListener(this));
		formatMenu.add(menuItem);
		formatMenu.addSeparator();
		menuItem = makeMenuItem("Add points to contours", new FormatAddPointsListener(this));
		formatMenu.add(menuItem);
		menuItem = makeMenuItem("Add points to diagram", new FormatAddPointsToDiagramListener(this));
		formatMenu.add(menuItem);

		//
		// Generator menu
		//
		JMenu generatorMenu = new JMenu("Generator");
		menuItem = makeMenuItem("Circuliser", new GeneratorCirculiserListener(this));
		generatorMenu.add(menuItem);
		//*** Added by scc to call UVic generator
		menuItem = makeMenuItem("Radialiser", new GeneratorRadialiserListener(this));
		generatorMenu.add(menuItem);
		menuItem = makeMenuItem("Force model", new GeneratorForceListener(this));
		generatorMenu.add(menuItem);
		menuItem = makeMenuItem("Random unitary spider diagram", new GeneratorRandomListener(this));
		generatorMenu.add(menuItem);
		menuItem = makeMenuItem("Random unitary constraint diagram", new GeneratorRandomCDListener(this));
		generatorMenu.add(menuItem);

		//
		// Resoning menu
		//
		JMenu reasoningMenu = new JMenu("Reasoning");
		menuItem = makeMenuItem("Check reading tree", new ReasoningCheckReadingTreeListener(this));
		reasoningMenu.add(menuItem);
		menuItem = makeMenuItem("Apply rule on constraint diagrams", new ReasoningApplyRuleListener(this));
		reasoningMenu.add(menuItem);
		menuItem = makeMenuItem("Proof (unitary)", new ReasoningProofListener(this));
		reasoningMenu.add(menuItem);
		menuItem = makeMenuItem("Proof (unitary) - smooth", new ReasoningSmoothProofListener(this));
		reasoningMenu.add(menuItem);
		menuItem = makeMenuItem("Exploring tableau", new ReasoningETableauListener(this));
		reasoningMenu.add(menuItem);

		//
		// Ocl menu
		//
		JMenu mappingMenu = new JMenu("Mappings");
		menuItem = makeMenuItem("Euler Diagrams to UML", new MapEd2UmlListener(this));
		mappingMenu.add(menuItem);
		menuItem = makeMenuItem("UML to Euler Diagrams", new MapUml2EdListener(this));
		mappingMenu.add(menuItem);
		mappingMenu.addSeparator();
		menuItem = makeMenuItem("Spider Diagrams to OCL", new MapSd2OclListener(this));
		mappingMenu.add(menuItem);
		menuItem = makeMenuItem("OCL to Spider Diagrams", new MapOcl2SdListener(this));
		mappingMenu.add(menuItem);
		menuItem = makeMenuItem("", new ReasoningProofListener(this));
		mappingMenu.add(menuItem);
		mappingMenu.addSeparator();
		menuItem = makeMenuItem("Dependence graph", new MapCd2DGListener(this));
		mappingMenu.add(menuItem);
		menuItem = makeMenuItem("PTC in FOPL", new MapCd2PtcListener(this));
		mappingMenu.add(menuItem);
		menuItem = makeMenuItem("Constraint Diagrams to FOPL", new MapCd2FOPLListener(this));
		mappingMenu.add(menuItem);
		menuItem = makeMenuItem("Constraint Diagrams to OCL", new MapCd2OclListener(this));
		mappingMenu.add(menuItem);

		// Metrics menu
		JMenu metricsMenu = new JMenu("Metrics");
		menuItem = makeMenuItem("Compute metrics", new MetricsComputeListener(this));
		metricsMenu.add(menuItem);
		menuItem = makeMenuItem("Compute average weight", new MetricsAverageWeightListener(this));
		metricsMenu.add(menuItem);

		// Layout menu
		JMenu layoutMenu = new JMenu("Layout");
		menuItem = makeMenuItem("Dynamic", new LayoutDynamicListener(this));
		layoutMenu.add(menuItem);
        menuItem = makeMenuItem("Place feet", new PlaceFeetListener(this));
		layoutMenu.add(menuItem);
        menuItem = makeMenuItem("Auto-embed feet", new AutoSpiderListener(this));
		layoutMenu.add(menuItem);
        menuItem = makeMenuItem("Smooth", new SmoothListener(this));
		layoutMenu.add(menuItem);
        menuItem = makeMenuItem("Add random spider", new RandomSpiderListener(this));
		layoutMenu.add(menuItem);
		menuItem = makeMenuItem("Toggle Movement", new LayoutMotionListener(this));
		layoutMenu.add(menuItem);

		// Tools menu
		JMenu toolsMenu = new JMenu("Tools");
		menuItem = makeMenuItem("Options", new ToolsOptionsListener(this));
		toolsMenu.add(menuItem);

		// Window menu
		JMenu windowMenu = new JMenu("Window");
		menuItem = makeMenuItem("Box Tiling", new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				tileInternalFramesBoxed();
			}
		});
		windowMenu.add(menuItem);

		//
		// Main menu
		//
		menuBar.add(fileMenu);
		menuBar.add(editMenu);
		menuBar.add(drawMenu);
		menuBar.add(formatMenu);
		menuBar.add(generatorMenu);
		menuBar.add(reasoningMenu);
		menuBar.add(mappingMenu);
		menuBar.add(metricsMenu);
		menuBar.add(layoutMenu);
		menuBar.add(toolsMenu);
		menuBar.add(windowMenu);
		// Set main menu
		this.setJMenuBar(menuBar);
	}

	protected JMenuItem makeMenuItem(String label, ActionListener action) {
		JMenuItem item = new JMenuItem(label);
		item.addActionListener(action);
		return item;
	}

	// Get the current editor frame
	public EditorInternalFrame getCurrentEditor() {
		Object frame = desktopPane.getSelectedFrame();
		if (frame instanceof EditorInternalFrame) {
			return (EditorInternalFrame) frame;
		}
		return null;
	}

	// Get the current viewer
	public DiagramViewerInterface getCurrentViewer() {
		EditorInternalFrame editor = getCurrentEditor();
		if (editor != null)
			return editor.getViewer();
		return null;
	}

	/** Changed by PJR to return the viewer. */
	public DiagramViewerComponent addViewer() {
		return addViewer("", new ConcreteDiagram());
	}

	/** Changed by PJR to return the viewer. */
	public DiagramViewerComponent addViewer(ConcreteUnitaryDiagram cud) {
		return addViewer(cud.getName(), cud);
	}

	public DiagramViewerComponent addViewer(String title, ConcreteDiagram cd) {
		DiagramViewerComponent viewer = new DiagramViewerComponent(cd,
				WINDOW_WIDTH, WINDOW_HEIGHT, false);
		if (title != null) {
			viewer.setConcreteDiagram(cd);
			cd.addObserver(viewer);
		}
		return addViewer(title, viewer);
	}

	public DiagramViewerComponent addViewer(String title,
			DiagramViewerComponent viewer) {
		JInternalFrame editor = new EditorInternalFrame(1000, 700, viewer);
		editor.setTitle(title);
		desktopPane.add(editor);
		// Draw the diagram
		editor.moveToFront();
		try {
			editor.setSelected(true);
		} catch (Exception e) {
		}
		desktopPane.setSelectedFrame(editor);
		return viewer;
	}

	//    public DiagramViewerComponent addViewer(ConcreteDiagram cd) {
	//        return addViewer(
	//          cd.getConcreteContours(),
	//          cd.toString(),
	//          new ConcreteSpiderFeetAndLegsSet(
	//          new Vector(cd.getConcreteFeet()),new Vector(cd.getConcreteLegs())),
	//          cd.getDecorationSet(),
	//          new ConcreteArcSet(),
	//          new ConcreteTextBoxSet(),
	//          cd.getConcreteZones()
	//        );
	//    }
	//
	//    /** Changed by PJR to return the viewer. */
	//    public DiagramViewerComponent addViewer(
	//        Set concreteContours,
	//        String title,
	//        ConcreteSpiderFeetAndLegsSet concreteSpiderSet,
	//        DecorationSet decorationSet,
	//        ConcreteArcSet arcSet,
	//        ConcreteTextBoxSet textSet,
	//        Set concreteZones)
	//    {
	//        DiagramViewerComponent viewer = new DiagramViewerComponent(WIDTH, HEIGHT,
	// false);
	//        if (title != null) {
	//            viewer.setDecorationSet(decorationSet);
	//            viewer.setConcreteContours(concreteContours);
	//            viewer.setConcreteSpiderFeetAndLegsSet(concreteSpiderSet);
	//            viewer.setArcSet(arcSet);
	//            viewer.setTextSet(textSet);
	//        }
	//        JInternalFrame editor = new EditorInternalFrame(1000, 700, viewer);
	//        desktopPane.add(editor);
	//        // Create a concrete diagram
	//        ConcreteDiagram cd = new ConcreteDiagram();
	//        cd.setDecorationBoxes(decorationSet);
	//        cd.setConcreteContourSet(concreteContours);
	//        cd.setConcreteFeet(new LinkedHashSet(concreteSpiderSet.getFeet()));
	//        cd.setConcreteLegs(new LinkedHashSet(concreteSpiderSet.getLegs()));
	//        cd.setConcreteZones(concreteZones);
	//        // Draw the diagram
	//        viewer.draw(cd);
	//        editor.moveToFront();
	//        try {
	//            editor.setSelected(true);
	//        } catch (Exception e) {
	//        }
	//        desktopPane.setSelectedFrame(editor);
	//
	//        return viewer;
	//    }

	protected void tileInternalFramesBoxed() {
		int desktopWidth = desktopPane.getWidth();
		int desktopHeight = desktopPane.getHeight();
		JInternalFrame[] frames = desktopPane.getAllFrames();
		int frameCount = frames.length;
		int totalRows = (int) Math.sqrt((double) frameCount);
		int totalCols = 1;
		while (totalCols * totalRows < frameCount) {
			totalCols++;
		}
		int windowsDrawn = 0;
		for (int row = 0; row < totalRows; row++) {
			for (int col = 0; col < totalCols; col++) {
				if (windowsDrawn == frameCount) {
					break;
				}
				JInternalFrame frame = frames[row * totalCols + col];
				try {
					frame.setIcon(false);
				} catch (java.beans.PropertyVetoException e) {
					System.out.println(e);
				}
				frame.setVisible(true);
				frame.toFront();
				frame.reshape((col * desktopWidth) / totalCols,
						(row * desktopHeight) / totalRows, desktopWidth
								/ totalCols, desktopHeight / totalRows);
				windowsDrawn++;
			}
		}
	}

	//*** added by jaf
	public UnitaryDiagram getUDCalled(String s) {
		//System.out.println("getting UD called "+s);
		List diagrams = new Vector();
		Object[] comps = desktopPane.getComponents();
		for (int i = 0; i < comps.length; i++) {
			Object comp = comps[i];
			if (comp instanceof EditorInternalFrame) {
				EditorInternalFrame editor = (EditorInternalFrame) comp;
				DiagramViewerInterface viewer = editor.getViewer();

				ConcreteDiagram cd = viewer.getConcreteDiagram();
				//System.out.println("got cd : "+cd.toString());
				HashMap hm = ConcreteUnitaryConverter.mapUnitaryDiagramCalled(
						cd, s);
				try {
					return (UnitaryDiagram) hm.get("unitaryDiagram");
				} catch (NullPointerException x) {
				}
			}
		}
		return null;
	}

	//*** added by jaf
	public ConcreteUnitaryDiagram getCUDCalled(String s) {
		System.out.println("getting CUD called " + s);
		List diagrams = new Vector();
		Object[] comps = desktopPane.getComponents();
		for (int i = 0; i < comps.length; i++) {
			Object comp = comps[i];
			if (comp instanceof EditorInternalFrame) {
				EditorInternalFrame editor = (EditorInternalFrame) comp;
				DiagramViewerInterface viewer = editor.getViewer();
				ConcreteDiagram cd = viewer.getConcreteDiagram();
				// Get the unitary diagrams
				Set unitaryDiags = cd.getConcreteUnitaryDiagrams();
				Iterator it = unitaryDiags.iterator();
				while (it.hasNext()) {
					ConcreteUnitaryDiagram cud = (ConcreteUnitaryDiagram) it
							.next();
					if (cud.getDecorationBox().getLabel().equals(s)) {
						//HashMap existingColorMap =
						// ConcreteZone.getZoneColors();
						//HashMap relevantZoneColors = viewer.getZoneColors();
						//ConcreteZone.setZoneColors(relevantZoneColors);
						// need to get the cd to recalculate its shaded zones
						cud.setConcreteZoneSet(cud.getConcreteZoneSet());
						//ConcreteZone.setZoneColors(existingColorMap);
						return cud;
					}
				}
			}
		}
		return null;
	}

	//*** added by op
	// Get the concrete diagrams within current internal frame
	protected List getCurrentConcreteDiagrams(JDesktopPane desktopPane) {
		List diagrams = new Vector();
		Object frame = desktopPane.getSelectedFrame();
		if (frame instanceof EditorInternalFrame) {
			EditorInternalFrame editor = (EditorInternalFrame) frame;
			DiagramViewerInterface viewer = editor.getViewer();
			ConcreteDiagram cd = viewer.getConcreteDiagram();
			// Add the unitary diagrams
			diagrams.add(cd);
		}
		return diagrams;
	}

	// Get all the concrete diagrams
	public List getConcreteDiagrams(JDesktopPane desktopPane) {
		List diagrams = new Vector();
		Object[] comps = desktopPane.getComponents();
		for (int i = 0; i < comps.length; i++) {
			Object comp = comps[i];
			if (comp instanceof EditorInternalFrame) {
				EditorInternalFrame editor = (EditorInternalFrame) comp;
				DiagramViewerInterface viewer = editor.getViewer();
				ConcreteDiagram cd = viewer.getConcreteDiagram();
				// Add the unitary diagrams
				diagrams.add(cd);
			}
		}
		return diagrams;
	}

	// Get the concrete diagrams within current internal frame
	protected List getCurrentConcreteUnitaryDiagrams(JDesktopPane desktopPane) {
		List cds = getCurrentConcreteDiagrams(desktopPane);
		List cuds = new Vector();
		for (int i = 0; i < cds.size(); i++) {
			ConcreteDiagram cd = (ConcreteDiagram) cds.get(i);
			cuds.addAll(cd.getConcreteUnitaryDiagrams());
		}
		return cuds;
	}

	// Get the unitary concrete diagrams
	protected List getConcreteUnitaryDiagrams(JDesktopPane desktopPane) {
		List cds = getConcreteDiagrams(desktopPane);
		List cuds = new Vector();
		for (int i = 0; i < cds.size(); i++) {
			ConcreteDiagram cd = (ConcreteDiagram) cds.get(i);
			cuds.addAll(cd.getConcreteUnitaryDiagrams());
		}
		return cuds;

	}

	// Get the current abstract unitary diagrams
	protected List getCurrentAbstractUnitaryDiagrams(JDesktopPane desktopPane) {
		System.out.println("getting current abstract unitary diagrams");
		List cuds = getCurrentConcreteUnitaryDiagrams(desktopPane);
		List auds = new Vector();
		for (int i = 0; i < cuds.size(); i++) {
			ConcreteUnitaryDiagram cud = (ConcreteUnitaryDiagram) cuds.get(i);
			UnitaryDiagram aud = ConcreteUnitaryConverter.makeAUD(cud);
			auds.add(aud);
		}
		return auds;
	}

	// Get all the abstract unitary diagrams
	protected List getAbstractUnitaryDiagrams(JDesktopPane desktopPane) {
		List cuds = getConcreteUnitaryDiagrams(desktopPane);
		List auds = new Vector();
		for (int i = 0; i < cuds.size(); i++) {
			ConcreteUnitaryDiagram cud = (ConcreteUnitaryDiagram) cuds.get(i);
			UnitaryDiagram aud = ConcreteUnitaryConverter.makeAUD(cud);
			auds.add(aud);
		}
		return auds;
	}

	public DiagramViewerInterface showAbstractDiagram(UnitaryDiagram ud,
			boolean doneNicely, boolean justForViewing) {
		try {
			if (ud == null)
				return null;
			DiagramViewerInterface viewer = new DiagramViewerComponent(
					WINDOW_WIDTH, WINDOW_HEIGHT, ud, justForViewing, false);
			JInternalFrame editor = new EditorInternalFrame(WINDOW_WIDTH,
					WINDOW_HEIGHT, viewer);
			desktopPane.add(editor);
			editor.moveToFront();
			viewer.fitToWidth(PICTURE_SIZE);
			try {
				viewer.redrawImage();
			} catch (Exception e) {
				System.out.println("Viewer can't repaint image");
			}
			try {
				viewer.redrawImage();
			} catch (Exception e) {
				System.out.println("Viewer can't draw image now");
			}
			return viewer;
		} catch (IllegalArgumentException x) {
			System.out
					.println("Illegal argument thrown in showAbstractDiagram");
			return null;
		}
		//    try {
		//      DiagramViewerInterface viewer =
		//        new DiagramViewerComponent(600, 400, ud, justForViewing, true);
		//      // viewer.repaintImage();
		//      viewer.redrawImage();
		//      // viewer.fitToWidth(diagramSize);
		//      JInternalFrame editor = new EditorInternalFrame(600, 400, viewer);
		//      desktopPane.add(editor);
		//      editor.moveToFront();
		//      viewer.fitToWidth(300);
		//      try {
		//        viewer.redrawImage();
		//      } catch (Exception e) {
		//        System.out.println("Viewer can't draw image now");
		//      }
		//      return viewer;
		//    } catch (IllegalArgumentException x) {
		//      System.out.println("Illegal argument thrown in showAbstractDiagram");
		//      return null;
		//    }
	}

	//
	// Getters and setters
	//
	public File getDirectory() {
		return directory;
	}

	public void setDirectory(File file) {
		directory = file;
	}

	public ILog getLog() {
		return log;
	}

	public void setLog(ILog log) {
		this.log = log;
	}

	public JDesktopPane getDesktopPane() {
		return desktopPane;
	}

	public void setDesktopPane(JDesktopPane pane) {
		desktopPane = pane;
	}

	public JFileChooser getInputChooser() {
		return inputChooser;
	}

	//--- Layout panels ---
	protected JSplitPane mainPanel;

	protected JSplitPane leftPanel;

	protected JSplitPane rightPanel;

	protected JFileChooser inputChooser;

	protected EditorFileFilter xmiFilter;

	protected Dimension screenSize;

	protected int barHeight = 70;

	protected double mainDiv = 0.22;

	protected double rightDiv = 0.70;

	protected int dividerSize = 4;

	protected JDesktopPane desktopPane = new JDesktopPane();

	protected ILog log = new ConsoleLog();

	protected File directory = null;

	protected static int WINDOW_WIDTH = 1000;

	protected static int WINDOW_HEIGHT = 700;

	protected static int PICTURE_SIZE = 400;
}