package rwd.gui.editor;

import java.awt.*;

import rwd.concretesyntax.*;

/**
 * @author Octavian Patrascoiu, Richard Bosworth
 * @version 6.7.05   14:16
 *
 */
public class Handle 
	extends DiagramItem 
{    
	public Handle(float x, float y, DiagramItem item) {
		_centerX = x;
		_centerY = y;
		_item = item;
        
		setForeColor(Color.black);
		setBackColor(Color.yellow);
	}

	public void translateTo(float dx, float dy) {
		_centerX += dx;
		_centerY += dy;
	}    
    
	public Shape getShape(boolean smooth) {
		int size = SIZE;
		return new Rectangle((int)_centerX - size, (int)_centerY - size, size * 2, size * 2);
	}

	public Shape getShape() {
		return getShape(false);
	}

	public void draw(Graphics2D g, boolean smoothed) {
		// Draw shape
		Shape shape = getShape();
		g.setColor(getBackColor());
		g.fill(shape);
		g.setColor(getForeColor());
		g.draw(shape);
	}  

	public java.util.Set getHandles() {
		// Set handles
		java.util.Set _handles = new java.util.LinkedHashSet();
		_handles.add(this);
		return _handles;
	}

	public void setLocation(int x, int y) {
		_centerX = x;
		_centerY = y;
	}
    
	public int getX() {
		return (int)_centerX;
	}
    
	public int getY() {
		return (int)_centerY;
	}
	
	public DiagramItem getItem() {
		return _item;
	}

	public void setItem(DiagramItem item) {
		_item = item;
	}
    
	public DiagramItem getClone() {
		Handle res = new Handle(_centerX, _centerY, _item);
		// Set attributes
		setAttributes(this);
		return res;
	}

	/**
	 * 
	 * @return a user-friendly description of this item
	 */
	/*
	 * test: once thru
	 */
	public String id() {
		return "handle at (" + getX() + "," + getY() + ")";
	}
	

	// Local properties
	protected DiagramItem _item;
	public static final int SIZE = 40;
	
	/*
	// test program
    // ------------

    private static void pr(String s) {
      System.out.println(s);
    }

    private static void hd(String s) {
      System.out.println("\n" + s + "\n");
    }

    private static void pg() {
      try {
        int i;
        System.out.print("next page? (CR)");
        do i = System.in.read(); while (i != '\n');
        System.out.println();
      }
      catch (Exception e) {System.out.println("raised exception : " + e);}
    }

    public static void main(String args[]) throws Exception {
    	hd("Test of toString, hashCode, id");
    	Handle h = new Handle(1,2,null);
    	pr("h="+h);
    	pr("h.hashCode()="+h.hashCode());
    	pr("h.id()="+h.id());
    	pg();    	

    	{
    		hd("Test of equals, getClone");
    		pr("h.equals(null)="+h.equals(null));
    		Object o = new Object();
    		pr("o="+o);
    		pr("h.equals(o)="+h.equals(o));
    		pg();
    		
    		Handle hc = (Handle) h.getClone();
    		pr("hc="+hc);
    		pr("hc.id()="+hc.id());
    		pr("h.equals(hc)="+h.equals(hc));
    		pg();
    		
    	}
    	
    }
    */
}
