package rwd.gui.editor;

import java.io.*;
import java.util.List;
import java.util.Vector;

import rwd.concretesyntax.ConcreteDiagram;
import rwd.gui.editor.viewers.DiagramViewerInterface;

/**
 * @author Octavian Patrascoiu
 *
 */
public class History {
	public History() {
	}
	
	public void store(DiagramViewerInterface viewer) {
		if (viewer == null) return;
		if (viewer.getConcreteDiagram() == null) return;
		// History is filled
		if (cursor == max) {
			// Shift left
			File file = (File)files.get(0);
			file.delete();
			files.remove(0);
		} else {
			cursor++;
		}
		// Store state - diagram
		try {
			File file = new File("History"+cursor+".bak");
			ObjectOutputStream str = new ObjectOutputStream(new FileOutputStream(file));
			str.writeObject(viewer.getConcreteDiagram());
			str.close();
			files.add(file);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public void undo(DiagramViewerInterface viewer) {
		if (cursor > 0) {
			cursor--;	
			copy(viewer);
			viewer.getDiagramCanvas().deselect();
		}		
	}

	public void redo(DiagramViewerInterface viewer) {
		if (cursor < size()-1) {
			cursor++;	
			copy(viewer);
		}
	}
	
	public void reset() {
		for(int i=0; i<files.size(); i++) {
			File file = (File)files.get(i);
			file.delete();
		}
		files = new Vector();
	}

	public void copy(DiagramViewerInterface viewer) {
		try {
			File file = (File)files.get(cursor);
			ObjectInputStream input = new ObjectInputStream(new FileInputStream(file));
			ConcreteDiagram cd = (ConcreteDiagram)input.readObject();
			viewer.setConcreteDiagram(cd);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public int size() {
		return files.size();
	}
	
	//
	// Local properties
	//
	protected int max = 10;
	protected int cursor = -1;
	protected List files = new Vector();
}

//public class History {
//	public History() {
//	}
//	
//	public void store(DiagramViewerInterface viewer) {
//		if (viewer == null) return;
//		if (viewer.getConcreteDiagram() == null) return;
//		// History is filled
//		if (cursor == max) {
//			// Shift left
//			diagrams.remove(0);
//			contZones.remove(0);
//		} else {
//			cursor++;
//		}
//		// Store state
//		diagrams.add(viewer.getConcreteDiagram().getClone());
//		contZones.add(((TreeSet)viewer.getContradictionZones()).clone());
//	}
//	
//	public void undo(DiagramViewerInterface viewer) {
//		if (cursor > 0) {
//			cursor--;	
//			copy(viewer);
//			viewer.getDiagramCanvas().deselect();
//		}		
//	}
//
//	public void redo(DiagramViewerInterface viewer) {
//		if (cursor < size()-1) {
//			cursor++;	
//			copy(viewer);
//		}
//	}
//	
//	public void reset() {
//		diagrams = new Vector();
//		contZones = new Vector();
//	}
//
//	public void copy(DiagramViewerInterface viewer) {
//		try {
//			viewer.setConcreteDiagram((ConcreteDiagram)diagrams.get(cursor));
//			viewer.setContradictionZones((TreeSet)contZones.get(cursor));
//		} catch (Exception e) {
//			e.printStackTrace();
//		}
//	}
//
//	public int size() {
//		return diagrams.size();
//	}
//	
//	//
//	// Local properties
//	//
//	protected int max = 10;
//	protected int cursor = -1;
//	protected List diagrams = new Vector();
//	protected List contZones = new Vector();
//}
