package rwd.gui.editor.dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import rwd.tableaux.semantic.MatrixPanel;

import uk.ac.kent.cs.kmf.util.ILog;
import uk.ac.kent.cs.kmf.util.OutputStreamLog;

/**
 * @author Octavian Patrascoiu
 *
 */
public class DecorationBoxDialog
	extends JDialog
{
	public DecorationBoxDialog(JFrame parent, String title) {
		this(parent, title, new OutputStreamLog(System.out));
	}
	public DecorationBoxDialog(JFrame parent, String title, ILog log) {
		super(parent, title);	
		getContentPane().setLayout(new GridBagLayout());
		this.log = log;
		JDialog dialog = new JDialog();

		//
        // Create and fill CONTOUR panel
        //
		contourPanel = new MatrixPanel();
		// Add labels
		int line = 0;
		int column = 0;
		contourPanel.addLabel("Name", line, column);
		nameField = contourPanel.addTextField("d", line, column+1);
		line++;
		contourPanel.addLabel("Reading forest", line, column);
		readingForestArea = new JTextArea(5, 30);
		readingForestArea.setText("");
		JScrollPane scroll = new JScrollPane(readingForestArea); 
		contourPanel.add(scroll, line, column+1);
		line++;
		    
		//
        // Create and fill CONFIRM panel 
        //
        MatrixPanel confirmPanel = new MatrixPanel();
        // OK button 
		JButton okButton = confirmPanel.addButton("OK", 0, 0);
        okButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                okAction(evt);
            }
        });
        // Cancel button 
		JButton cancelButton = confirmPanel.addButton("Cancel", 0, 1);
        cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelAction(evt);
            }
        });

	    // Add TAB panel to MAIN panel
		GridBagConstraints c = new GridBagConstraints(); 
	    c.gridx = 0;
	    c.gridy = 0;
		getContentPane().add(contourPanel, c);

		// Add CONFIRM to MAIN panel 		
	    c.gridx = 0;
	    c.gridy = 1;
	    c.anchor = GridBagConstraints.CENTER;
	    getContentPane().add(confirmPanel, c);
    		
        // Put frame in the middle of the screen
        pack();
        Dimension screen = getToolkit().getScreenSize();
        int width  = this.getSize().width+20;
        int height = this.getSize().height+20;
        int left = (screen.width - width)/2;
        int top = (screen.height - height)/2;
        setBounds(left, top, width, height);
	}

	// OK action 
	protected int returnVal = JFileChooser.CANCEL_OPTION;
    protected void okAction(java.awt.event.ActionEvent evt) {
    	returnVal = JFileChooser.APPROVE_OPTION;
		dispose();
    }

	// Cancel action 
    protected void cancelAction(java.awt.event.ActionEvent evt) {
		returnVal = JFileChooser.CANCEL_OPTION;   	
    	dispose();
    }

	// Show dialog
	public int showOpenDialog(JFrame parent) {
		this.setModal(true);
		show();
		return returnVal;
	}
	
	// Get the label
	public String getLabel() {
		String text = nameField.getText(); 
		return text;
	}
	public void setLabel(String label) {
		nameField.setText(label);
	}

	// Get the reading forest
	public String getReadingForest() {
		String text = readingForestArea.getText(); 
		return text;
	}

	protected MatrixPanel contourPanel;
	protected JTextField nameField;
	protected JTextArea readingForestArea;

	protected ILog log;
	/**
	 * @param readingTree
	 */
	public void setReadingForest(String readingForest) {
		readingForestArea.setText(readingForest);
	}
}
