package rwd.gui.editor.dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.border.EmptyBorder;

import rwd.tableaux.semantic.MatrixPanel;

/**
 * @author Octavian Patrascoiu
 *
 */
public class TextBoxDialog 
	extends JDialog
{
	public TextBoxDialog(JFrame parent, String title, String content) {
		super(parent, title);
		getContentPane().setLayout(new GridBagLayout());

		//
		//--- Create and fill panel
		//
		generalPanel = new MatrixPanel();
		//--- Give the panel a border gap of 5 pixels ---
		generalPanel.setBorder(new EmptyBorder(new Insets(10, 10, 10, 10)));
		//--- Get the constraints ---
		GridBagConstraints c = new GridBagConstraints();
		//--- Add space around all components to avoid clutter ---
		c.insets = new Insets(2, 2, 2, 2);
		c.ipadx = 5;
		c.ipady = 5;
		c.weightx = 0.0;
		c.weighty = 0.0;
		//--- Initialize common variables ---
		int line = 0;
		int column = 0;
		EmptyBorder border = new EmptyBorder(new Insets(0, 0, 0, 10));
		Dimension buttonSize = new Dimension(80, 25);

		//--- Text area ---
		textArea = new JTextArea(5, 30);
		textArea.setText(content);
		JScrollPane scroll = new JScrollPane(textArea); 
		c.gridx = column+1;
		c.gridy = line;
		generalPanel.add(scroll, c);
		line++;
    
		//
        // Create and fill CONFIRM panel 
        //
        MatrixPanel confirmPanel = new MatrixPanel();
        // OK button 
		JButton okButton = confirmPanel.addButton("OK", 0, 0);
        okButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                okAction(evt);
            }
        });
        // Cancel button 
		JButton cancelButton = confirmPanel.addButton("Cancel", 0, 1);
        cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelAction(evt);
            }
        });

		// Add TAB panel to MAIN panel ---
		c.gridx = 0;
		c.gridy = 0;
		getContentPane().add(generalPanel, c);

		// Add CONFIRM to MAIN panel 		
		c.gridx = 0;
		c.gridy = 1;
		c.anchor = GridBagConstraints.CENTER;
		getContentPane().add(confirmPanel, c);

		// Put frame in the middle of the screen
		pack();
		Dimension screen = getToolkit().getScreenSize();
		int width  = this.getSize().width+20;
		int height = this.getSize().height+20;
		int left = (screen.width - width)/2;
		int top = (screen.height - height)/2;
		setBounds(left, top, width, height);
	}

	// Show the dialog
	public int showOpenDialog(JFrame parent) {
		this.setModal(true);
		show();
		return returnVal;
	}

	//--- OK action ---
	protected int returnVal = JFileChooser.CANCEL_OPTION;
	protected void okAction(java.awt.event.ActionEvent evt) {
		returnVal = JFileChooser.APPROVE_OPTION;
		String input = getText();
		if (input != null && input.length() != 0)  	
			dispose();
	}

	//--- Cancel action ---
	protected void cancelAction(java.awt.event.ActionEvent evt) {
		returnVal = JFileChooser.CANCEL_OPTION;   	
		dispose();
	}
	
	//--- Get the comment ----
	public String getText() {
		return textArea.getText();
	}

	// Local properties	
	protected MatrixPanel generalPanel;
	protected JTextArea textArea;
}
