package rwd.gui.editor.listeners;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JFileChooser;

import rwd.concretesyntax.DiagramItem;
import rwd.concretesyntax.constraintdiagram.ConcreteTextBox;
import rwd.concretesyntax.eulerdiagram.DecorationBox;
import rwd.gui.editor.EditorFrame;
import rwd.gui.editor.dialogs.DecorationBoxDialog;
import rwd.gui.editor.dialogs.TextBoxDialog;
import rwd.gui.editor.viewers.DiagramViewerInterface;

/**
 * @author Octavian Patrascoiu
 *
 */
public class EditDiagramItemListener implements ActionListener {
	public EditDiagramItemListener(EditorFrame frame) {
		this.frame = frame;
	}
	
	public EditDiagramItemListener(DiagramViewerInterface viewer) {
		this.viewer = viewer;
	}

	public void actionPerformed(ActionEvent e) {
		if (frame != null) viewer = frame.getCurrentViewer();
		if (viewer == null) return;
		edit();
	}
	
	public void edit() {
		// Get the text box
		java.util.List items = viewer.getDiagramCanvas().getSelectedItems();
		if (items != null && !items.isEmpty() && items.size() == 1) {
			DiagramItem item = (DiagramItem)items.iterator().next();
			if (item instanceof DecorationBox) {
				// Create a dialog to input text
				DecorationBox box = (DecorationBox)item;
				DecorationBoxDialog dialog = new DecorationBoxDialog(frame, "Box");
				dialog.setLabel(box.getLabel());
				dialog.setReadingForest(box.getReadingForest());
				int returnVal = dialog.showOpenDialog(frame);
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					String label = dialog.getLabel();
					String readingForest = dialog.getReadingForest();
					if (label != null && label.length() != 0) {
						// Update properties
						box.setLabel(label);
						box.setReadingForest(new String(readingForest));
						
						// Repaint image
						viewer.redrawImage();
					}
				}						
			} else if (item instanceof ConcreteTextBox) {
				// Create a dialog to input text
				ConcreteTextBox textBox = (ConcreteTextBox)item;
				TextBoxDialog dialog = new TextBoxDialog(frame, "Comment", textBox.getText());
				int returnVal = dialog.showOpenDialog(frame);
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					String comment = dialog.getText();
					if (comment != null && comment.length() != 0) {
						// Set new text
						textBox.setText(comment);
						
						// Repaint image
						viewer.redrawImage();
					}
				}						
			}
		}
	}

	// Local variables
	private EditorFrame frame;
	private DiagramViewerInterface viewer;
}
