package rwd.gui.editor.listeners;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;
import java.util.Random;
import java.util.Vector;

import javax.swing.JFileChooser;

import rwd.abstractsyntax.spiderdiagram.UnitaryDiagram;
import rwd.concretesyntax.ConcreteUnitaryDiagram;
import rwd.concretesyntax.DiagramItem;
import rwd.concretesyntax.constraintdiagram.ConcreteArc;
import rwd.concretesyntax.constraintdiagram.ConcreteArcEnd;
import rwd.gui.editor.EditorFrame;
import rwd.gui.editor.dialogs.RandomCDDialog;
import rwd.gui.editor.viewers.DiagramViewerInterface;
import rwd.rules.UnitaryDiagramGetterRandom;

/**
 * @author Octavian Patrascoiu
 *
 */
public class GeneratorRandomCDListener implements ActionListener {
	public GeneratorRandomCDListener(EditorFrame frame) {
		this.frame = frame;
	}

	public void actionPerformed(ActionEvent e) {
		// Launch dialog
		RandomCDDialog dialog = new RandomCDDialog(frame, "Random constraint diagram", frame.getLog());
		int returnVal = dialog.showOpenDialog(frame);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			int numContours = dialog.getNumContours();
			int numSpiders = dialog.getNumSpiders();
			int numArcs = dialog.getNumArcs();
			int i = 0;
			whileLoop : while (true) {
				UnitaryDiagram ud =
					new UnitaryDiagramGetterRandom(
						numContours,
						numSpiders,
						true,
						false)
						.getDiagram();
//				if (ud.getCloneOfContourSet().size() < 2
//						|| ud.getSpiders().size() == 0) {
//						continue whileLoop;
//				
//				}
				DiagramViewerInterface viewer = frame.showAbstractDiagram(ud, true, false); 
				if (viewer != null) {
					// Add random arrows
					ConcreteUnitaryDiagram cud = (ConcreteUnitaryDiagram)viewer.getConcreteDiagram();
					addRandomArcs(cud, numArcs);
					// Add more points
					cud.addMorePoints(dialog.getPointNum());
					if (dialog.isEqualised()) {
						cud.equalisePointNumber();
					}
					viewer.redrawImage();
					break whileLoop;
				}
			}
		}
	}

	protected void addRandomArcs(ConcreteUnitaryDiagram cud, int no) {
		List items = new Vector();
		items.addAll(cud.getConcreteContours());
		items.addAll(cud.getConcreteFeet());
		for(int i=0; i<no; i++) {
			ConcreteArc arc = generateRandomArc(items);
			cud.addConcreteArc(arc);
		}
	}
	
	protected ConcreteArc generateRandomArc(List items) {
		// Generate random source and target
		int min = 0;
		int max = items.size();
		int iSource = generateInt(min, max);
		int iTarget = 0;
		int i =0;
		do {
			i++;
			iTarget = generateInt(min, max);			
		} while (iTarget == iSource && i<10000);
		DiagramItem source = (DiagramItem)items.get(iSource);
		DiagramItem target = (DiagramItem)items.get(iTarget);
		ConcreteArcEnd start = new ConcreteArcEnd(source);
		ConcreteArcEnd end = new ConcreteArcEnd(target);
		
		// Make arc
		ConcreteArc arc = new ConcreteArc(start, end);
		return arc;
	}
	
	protected int generateInt(int min, int max) {
		int size = max - min;
		int i = Math.abs(r.nextInt());
		i = (i % size) + min;
		if (i < min) i = min;
		if (i > max) i = max;
		return i;
	}
	
	// Local variables
	protected EditorFrame frame;
	protected static Random r = new Random(1);
	protected int numArcs = 1;
	protected int numSpiders = 2;
	protected int numContours = 3;
}
