package rwd.gui.editor.listeners;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.JOptionPane;

import rwd.concretesyntax.ConcreteDiagram;
import rwd.concretesyntax.ConcreteUnitaryDiagram;
import rwd.concretesyntax.eulerdiagram.DecorationBox;
import rwd.gui.editor.EditorFrame;
import rwd.gui.editor.EditorInternalFrame;
import rwd.gui.editor.viewers.DiagramViewerComponent;
import rwd.gui.editor.viewers.DiagramViewerInterface;
import rwd.views.smoother.ComparisonParameterWindow;

/**
 * @author Octavian Patrascoiu
 *
 */
public class LayoutDynamicListener implements ActionListener {
	public LayoutDynamicListener(EditorFrame frame) {
		this.frame = frame;
	}

	public void actionPerformed(ActionEvent e) {
		String staticString =
			JOptionPane.showInputDialog(
				frame,
				"Please enter name of target (unmoving) diagram",
				"d1");

		if(staticString == null) {
			return;
		}

		String modifyString =
			JOptionPane.showInputDialog(
				frame,
				"Please enter name of diagram to change",
				"d2");

		if(modifyString == null) {
			return;
		}

		// Get diagrams and viewers
		ConcreteUnitaryDiagram cudStatic = null;
		ConcreteUnitaryDiagram cudModify = null;
		DiagramViewerInterface viewerStatic = null;
		DiagramViewerInterface viewerModify = null;
		Object[] comps = frame.getDesktopPane().getComponents();
		for (int i = 0; i < comps.length; i++) {
			Object comp = comps[i];
			if (comp instanceof EditorInternalFrame) {
				EditorInternalFrame editor = (EditorInternalFrame) comp;
                System.out.println("found an internal editor frame");
				DiagramViewerInterface viewer = editor.getViewer();
				ConcreteDiagram cd = viewer.getConcreteDiagram();
				// Get the unitary diagrams
				Vector cudList =	new Vector(cd.getConcreteUnitaryDiagrams());
                System.out.println("found "+cudList.size()+" cuds");
				for (int j = 0; j < cudList.size(); j++) {
                    System.out.println("found a CUD");
					ConcreteUnitaryDiagram cud =
						(ConcreteUnitaryDiagram) cudList.get(j);
					DecorationBox b = cud.getDecorationBox();
					if (b.getLabel().equals(staticString)) {
						cudStatic = cud;
						viewerStatic = viewer;
					} else if (b.getLabel().equals(modifyString)) {
						cudModify = cud;
						viewerModify = viewer;
					}
				}
			}
		}

		if (cudStatic == null) {
			JOptionPane.showMessageDialog(
				frame,
				"can't identify unitary diagram called "
					+ staticString);
			return;
		}
		if (cudModify == null) {
			JOptionPane.showMessageDialog(
				frame,
				"can't identify unitary diagram called "
					+ modifyString);
			return;
		}

		ComparisonParameterWindow pcw =
			new ComparisonParameterWindow(
				frame,
				(ConcreteUnitaryDiagram)cudModify,
				(ConcreteUnitaryDiagram)cudStatic,
				(DiagramViewerComponent)viewerModify,
				modifyString + " like " + staticString,
				200);

	}

	// Local variables
	protected EditorFrame frame;
}
