package rwd.gui.editor.listeners;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JFileChooser;

import rwd.concretesyntax.ConcreteUnitaryDiagram;
import rwd.gui.editor.EditorFrame;
import rwd.gui.editor.dialogs.CDReadingDialog;
import rwd.ocl.constraintdiagram.ConstraintDiagram2Ocl;

/**
 * @author Octavian Patrascoiu
 *
 */
public class MapCd2OclListener implements ActionListener {
	public MapCd2OclListener(EditorFrame frame) {
		this.frame = frame;
	}

	public void actionPerformed(ActionEvent e) {
		// Launch dialog
		CDReadingDialog dialog = new CDReadingDialog(frame, "Mapping", frame.getLog());
		int returnVal = dialog.showOpenDialog(frame);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			String inputString = dialog.getDiagramName();
			
			// Get all the abstract unitary diagrams
			ConcreteUnitaryDiagram cud = frame.getCUDCalled(inputString);

			// Reset log
			frame.getLog().reset();

			// Create an OCL generator
			ConstraintDiagram2Ocl gen = new ConstraintDiagram2Ocl(cud, frame.getLog());

			// Generate the corresponding ocl
			String ocl = gen.getOCL(cud.getReadingForest(), cud);

			// Print it out
			String message = "OCL for diagram '"+inputString+"'\n";
			message += ocl;
			frame.getLog().reportMessage(message);
		}
	}

	// Local variables
	protected EditorFrame frame;
}
