package rwd.gui.editor.listeners;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;
import java.util.List;

import javax.swing.JOptionPane;

import rwd.abstractsyntax.spiderdiagram.UnitaryDiagram;
import rwd.gui.SmoothProofViewer;
import rwd.gui.editor.EditorFrame;
import rwd.rules.NoProofException;
import rwd.rules.Proof;
import rwd.rules.heuristics.HeuristicCalculatorOptimistic;
import rwd.rules.heuristics.HeuristicProofWriter;

/**
 * @author Octavian Patrascoiu
 *
 */
public class ReasoningSmoothProofListener implements ActionListener {
    public ReasoningSmoothProofListener(EditorFrame frame) {
        this.frame = frame;
    }

    public void actionPerformed(ActionEvent e) {
        String d1String =
            JOptionPane.showInputDialog(
                frame,
                "Please enter name of premise",
                "d1");
        String d2String =
            JOptionPane.showInputDialog(
                frame,
                "Please enter name of conclusion",
                "d2");
        List diagrams = frame.getConcreteDiagrams(frame.getDesktopPane());

        // get all the unitary diagrams
        Vector unitaryDiagrams = new Vector();

        UnitaryDiagram ud1 = frame.getUDCalled(d1String);
        if (ud1 == null) {
            JOptionPane.showMessageDialog(
                frame,
                "can't identify unitary diagram called " + d1String);
            return;
        }
        UnitaryDiagram ud2 = frame.getUDCalled(d2String);
        if (ud2 == null) {
             JOptionPane.showMessageDialog(
                frame,
                 "can't identify unitary diagram called " + d2String);
             return;
        }

        try {
            Vector result =
                HeuristicProofWriter.seekProof(
                    ud1,
                    ud2,
                    6,
                    new HeuristicCalculatorOptimistic(),
                    null,
                    HeuristicProofWriter.unitaryHRules,
                    null,
                    true,
                    false,
                    false);
            Proof p = (Proof) result.get(0);
            if (p == null) {
                throw new NoProofException("no proof");
            }
            SmoothProofViewer pv = new SmoothProofViewer(p, false);
            pv.show();
        } catch (NoProofException x) {
            JOptionPane.showMessageDialog(
                frame,
                "no proof found");
        }
    }

    // Local variables
    protected EditorFrame frame;
}
