package rwd.gui.editor.viewers;

import rwd.abstractsyntax.spiderdiagram.*;
import rwd.concretesyntax.ConcreteDiagram;
import rwd.concretesyntax.ConcreteUnitaryDiagram;
import rwd.concretesyntax.eulerdiagram.*;
import rwd.gui.editor.ControlPanel;
import rwd.gui.editor.DiagramCanvas;
import rwd.views.abstracttoconcrete.converting.DrawableResult;
import rwd.views.abstracttoconcrete.converting.Result;
import rwd.views.abstracttoconcrete.converting.ResultDirector;
import rwd.views.abstracttoconcrete.forcemodel.SpiderPlacer;
import rwd.views.smoother.MetricSmoother;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

import rwd.abstractsyntax.eulerdiagram.Contour;
import rwd.abstractsyntax.eulerdiagram.Zone;

/**
 * @author RWD, Richard Bosworth
 * @version 21.4.05   20:31
 */
public class DiagramViewerComponent
    extends AbstractDiagramViewer
    implements DiagramViewerInterface 
{
	//
	// Constructors
	//
    public DiagramViewerComponent(ConcreteDiagram cd, int width, int height, boolean smooth) {
        super(cd);
        
        System.out.println("new DiagramViewerComponent: "+ strId);
        System.out.println("isRequestFocusEnabled()="+isRequestFocusEnabled());
		this.addFocusListener(new FocusAdapter() {
			public void focusGained(FocusEvent e) {
				System.out.println("Viewer " + strId + " gained focus");
			}
		});
        
        setPreferredSize(new Dimension(width, height));
        setMinimumSize(new Dimension(width, height));
        _canvas = new DiagramCanvas(this);
        _controls = new ControlPanel(this);
		setBezier(smooth);

        this.setLayout(new BorderLayout());
        this.add(_controls, BorderLayout.NORTH);
        this.add(_canvas, BorderLayout.CENTER);
    }
    
	public DiagramViewerComponent(int width, int height, boolean smooth) {
		super();
		
        System.out.println("new DiagramViewerComponent: "+ strId);
        System.out.println("isRequestFocusEnabled()="+isRequestFocusEnabled());
		this.addFocusListener(new FocusAdapter() {
			public void focusGained(FocusEvent e) {
				System.out.println("Viewer " + strId + " gained focus");
			}
		});

		setPreferredSize(new Dimension(width, height));
		setMinimumSize(new Dimension(width, height));
		_canvas = new DiagramCanvas(this);
		_controls = new ControlPanel(this);
		setBezier(smooth);

		this.setLayout(new BorderLayout());
		this.add(_controls, BorderLayout.NORTH);
		this.add(_canvas, BorderLayout.CENTER);
	}

    public DiagramViewerComponent(int width, int height, boolean justForViewing, boolean smooth) {
        super();
        
        System.out.println("new DiagramViewerComponent: "+ strId);
        System.out.println("isRequestFocusEnabled()="+isRequestFocusEnabled());
		this.addFocusListener(new FocusAdapter() {
			public void focusGained(FocusEvent e) {
				System.out.println("Viewer " + strId + " gained focus");
			}
		});

		setPreferredSize(new Dimension(width, height));
        setMinimumSize(new Dimension(width, height));
        _canvas = new DiagramCanvas(this);
        _controls = new ControlPanel(this);
		setBezier(smooth);

        this.setLayout(new BorderLayout());
        if(!justForViewing) 
        	this.add(_controls, BorderLayout.NORTH);
        this.add(_canvas, BorderLayout.CENTER);        

		this.addFocusListener(new FocusAdapter() {
			public void focusGained(FocusEvent e) {
				System.out.println("Viewer " + strId + " gained focus");
			}
		});
    }

	public DiagramViewerComponent(int width, int height, UnitaryDiagram ud, boolean justForViewing, boolean smooth, int numPoints) {
		this(width, height, ud, justForViewing, smooth);
		ConcreteUnitaryDiagram cud = (ConcreteUnitaryDiagram)_diagram;
		cud.addMorePoints(numPoints);
		cud.refreshZones(smooth);
		cud.notifyObservers();
	}

    public DiagramViewerComponent(int width, int height, UnitaryDiagram ud, boolean justForViewing, boolean smooth) {
        
        System.out.println("new DiagramViewerComponent: "+ strId);
        System.out.println("isRequestFocusEnabled()="+isRequestFocusEnabled());
		this.addFocusListener(new FocusAdapter() {
			public void focusGained(FocusEvent e) {
				System.out.println("Viewer " + strId + " gained focus");
			}
		});
        
		this._justForViewing = justForViewing;
		Result result = new ResultDirector().makeResult(ud);
		if (!(result instanceof DrawableResult)) {
			// do something sensible
			throw new IllegalArgumentException();
		}

		// Set canvas si controls in AbstractViewer
		setPreferredSize(new Dimension(width, height));
		setMinimumSize(new Dimension(width, height));
		_canvas = new DiagramCanvas(this, justForViewing);
		this.setLayout(new BorderLayout());
		this.add(_canvas, BorderLayout.CENTER);
		_controls = new ControlPanel(this);
		if (!justForViewing) {
			this.add(_controls, BorderLayout.NORTH);
		}
		setBezier(smooth);

		// Compute concrete contours
		DrawableResult dr = (DrawableResult) result;
		AEDView aedv = dr.getAEDV();

		// Compute decoration box	
		Set _polygonSet = aedv.getConcreteContours();
		DecorationBox box = getBoxAround(_polygonSet, 1.0/6.0);
		box.setLabel(ud.getName()); 

		// Create a concrete diagram
		ConcreteUnitaryDiagram cud = new ConcreteUnitaryDiagram(box);

		// Add decoration box
		cud.addDecorationBox(box);

		// Add contours and zones
		cud.addConcreteContours(_polygonSet);

		// Shade zones
		Iterator it = ud.getSZIterator();
		Set czones = cud.getConcreteZones();
		while(it.hasNext()){
			Zone z = (Zone)it.next();
			// to shade "outside zone"
			// need to find out a suitable DecorationBox and
			// include this with the contourPolygons to build
			// the concrete zone (use arraylists of Polygonal objects)
			// ConcreteZone.setZoneColors(_zoneColors);
			Iterator j = czones.iterator();
			while (j.hasNext()) {
				ConcreteZone cz = (ConcreteZone) j.next();
				if (cz.getAbstractZone() == z) {
					cz.setBackColor(Color.gray);
				}
			}
		}

		// Add spiders and their feet
		Vector abstractSpiders = ud.getSpiders();
		for (int i = 0; i < abstractSpiders.size(); i++) {
			Spider s = (Spider) abstractSpiders.get(i);
			//System.out.println("adding a spider:" + s);
			cud.addAbstractSpider(s);
		}
		
		// Set properties within the AbstractViewer
		_diagram = cud;
		
		// Smooth 
		if(_smoothed && _polygonSet.size()>0) {
			// Refresh zones and notify observers
			cud.refreshZones(true);
			cud.notifyObservers();
			
            MetricSmoother smoother = new MetricSmoother(cud, this, 10);
			Thread smootherThread = new Thread(smoother);
			smootherThread.start();
			//--- Join thread
			try {
				smootherThread.join();
			} catch (Exception e) {
				System.out.println("Smoother thread be joined");
				e.printStackTrace();
			}
		}
		
		// Embed spider		
		SpiderPlacer sp = new SpiderPlacer(this);
		sp.embed();
		// Register as observer
		cud.addObserver(this);
		// Notify observers
		cud.notifyObservers();
    }

    public Vector getAllConcreteZones() {
        return new Vector(_diagram.getConcreteZones());
        /*Vector result = new Vector();
        // collect together contours and boxes
        Vector polygonals = new Vector();
        polygonals.addAll(_polygonSet.getContourPolys());
        polygonals.addAll(_decorations.getBoxes());
        Vector powerSet = Utilities.powerSet(polygonals);
        for (int i = 0; i < powerSet.size(); i++) {
            Vector subset = (Vector) powerSet.get(i);
            Vector rest = (Vector) polygonals.clone();
            rest.removeAll(subset);
            if (subset.size() > 0) {
                ConcreteZone cz = new ConcreteZone(subset, rest, true);
                //if (!cz.getArea().isEmpty()) {
                result.add(cz);
                //System.out.println("add zone:"+cz.getZoneString());
                //}
            }
        }
        return result;*/
    }

    public void fitToWidth(int width) {
        DecorationBox box = getBoxAround(_diagram.getConcreteContours(), 1.0/10.0);
        // use the rectangle for this box ro
        // set scaling and transforming in canvas
        Rectangle r = box.getPolygon().getBounds();
        int widthBox = r.width;
        float scaleFactor = (float)width / (float)widthBox;
        //System.out.println("bounding box width is "+r.width);
        //System.out.println("given width is "+width);
        //System.out.println("so scale factor is "+scaleFactor);
        float shiftFactorX = -r.x;
        float shiftFactorY = -r.y;
        _canvas.setTranslation(shiftFactorX,shiftFactorY);
        _canvas.setScale(scaleFactor);
    }

	public java.util.List getSelectedItems() {
		if (_canvas == null) return new Vector();
		java.util.List res = new Vector(_canvas.getSelectedItems());
		return res;
	}

//	public void trashSpiderLegs(Set badLegs){
//		_diagram.getFeetAndLegsSet().removeLegs(badLegs);
//	}
//	public void addSpiderLegs(Set goodLegs){
//		_diagram.getFeetAndLegsSet().addLegs(goodLegs);        
//	}

	// A test method for this class.
	public static void main(String[] args) {
		DiagramViewerComponent viewer 
		  = new DiagramViewerComponent(640, 480, false);
		JFrame frame = new JFrame("Test Frame");
		frame.getContentPane().add(viewer);
		frame.pack();
		frame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				System.exit(0);
			}
		});
		frame.setVisible(true);

		Random rand = new Random();
		ConcreteUnitaryDiagram cud = new ConcreteUnitaryDiagram(
                    new DecorationBox("box", "", new Polygon(new int[]{100,200,200,100},new int[]{100,100,200,200},4)));
        cud.addConcreteContour(
			new ConcreteContour(
				Contour.getContour(String.valueOf(Math.random())),
				new Polygon(
					new int[] {
						rand.nextInt(400),
						rand.nextInt(400),
						rand.nextInt(400)},
					new int[] {
						rand.nextInt(400),
						rand.nextInt(400),
						rand.nextInt(400)},
					3)));
		viewer.draw(cud);

        cud.addConcreteContour(
			new ConcreteContour(
				Contour.getContour(String.valueOf(Math.random())),
				new Polygon(
					new int[] {
						rand.nextInt(400),
						rand.nextInt(400),
						rand.nextInt(400)},
					new int[] {
						rand.nextInt(400),
						rand.nextInt(400),
						rand.nextInt(400)},
					3)));
        viewer.draw(cud);
	}
}
