package rwd.gui.editor.writers;

import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import java.io.File;
import java.util.Iterator;

import javax.imageio.ImageIO;
import javax.swing.JOptionPane;

import rwd.gui.editor.viewers.DiagramViewerInterface;
import uk.ac.kent.cs.kmf.util.ILog;

/**
 * @author Octavian Patrascoiu
 *
 */
public class ImageWriter 
	extends DiagramWriter
{
	public ImageWriter(ILog log, String ext) {
		super(log);
		this.ext = ext;
	}
	public void write(String fileName, DiagramViewerInterface viewer) {
		write(new File(fileName), viewer);
	}
	public void write(File file, DiagramViewerInterface viewer) {
		// Check if the platform supports the suffix
		Iterator imageWriterIt =
			ImageIO.getImageWritersBySuffix(ext);
		if (!imageWriterIt.hasNext()) {
			JOptionPane.showMessageDialog(
				null,
				"Your Java platform does not support the writing of '"
					+ ext
					+ "' files.",
				"Could not write image",
				JOptionPane.ERROR_MESSAGE);
		}
		// Save the file
		Rectangle rect = viewer.getBoxAround(0);
		BufferedImage image =
			viewer.getDiagramCanvas().getBufferedImage(rect);
		try {
			ImageIO.write(image, ext, file);
		} catch (Exception anye) {
			JOptionPane.showMessageDialog(
				null,
				"The image could not be written:\n\n" + anye,
				"Could not write image",
				JOptionPane.ERROR_MESSAGE);
		}
	}
	
	// Local properties
	private String ext = "jpg";
}
