package rwd.gui.editor.writers;

//import java.awt.Polygon;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.ObjectOutputStream;
//import java.util.Iterator;
//import java.util.Vector;

import javax.swing.JOptionPane;

//import rwd.abstractsyntax.eulerdiagram.Contour;
//import rwd.concretesyntax.ConcreteDiagram;
//import rwd.concretesyntax.eulerdiagram.ConcreteContour;
//import rwd.concretesyntax.eulerdiagram.ConcreteZone;
//import rwd.concretesyntax.eulerdiagram.ConcreteZoneSet;
//import rwd.gui.editor.viewers.DiagramViewerComponent;
import rwd.gui.editor.viewers.DiagramViewerInterface;
import uk.ac.kent.cs.kmf.util.ILog;

/**
 * Writer for serialized (cd) files.
 *  
 * @author Octavian Patrascoiu, rnb
 * 
 * @version 14.6.05   12:19
 *
 */
public class SerialWriter 
	extends DiagramWriter
{
	// Constructor
	// -----------
	
	/**
	 * post: SerialWriter object has been instantiated with given log.
	 * @param log console log for the current frame
	 * 
	 */
	public SerialWriter(ILog log) {
		super(log);
	}
	
	/**
	 * post: contents of viewer have been serialized to fileName
	 * @param viewer viewer whose contents are to be serialized
	 * @param fileName name of file to which the viewer is to be serialized
	 */
	public void write(String fileName, DiagramViewerInterface viewer) {
		write(new File(fileName), viewer);
	}
	
	/**
	 * post: contents of viewer have been serialized to file
	 * @param viewer viewer whose contents are to be serialized
	 * @param file file to which the viewer is to be serialized
	 */
	public void write(File file, DiagramViewerInterface viewer) {
		try {
			ObjectOutputStream output =
				new ObjectOutputStream(
					new BufferedOutputStream(
						new FileOutputStream(file)));
			output.writeObject(viewer.getTitle());
			output.writeObject(viewer.getConcreteDiagram());
			output.flush();
			output.close();
		} catch (Exception anye) {
			anye.printStackTrace();
			JOptionPane.showMessageDialog(
				null,
				"The CD file could not be created:\n\n" + anye,
				"Could not write CD",
				JOptionPane.ERROR_MESSAGE);
		}
	}

	/*
	/**
	 * post: links to abstract zones from the concrete zones of this viewer
	 *       have been cleared.
	 * @param viewer viewer whose abstract zones are to be cleared
	 */
	/*
	 * test:
	 *   viewer has no concrete zones
	 *   viewer has two concrete zones
	 */
	/*
	public void clearAbstractZones(DiagramViewerInterface viewer){
		Iterator it = 
			viewer.getConcreteDiagram().getConcreteZoneSet().getZones().iterator();
		while (it.hasNext()){
			ConcreteZone cz = (ConcreteZone) it.next();
			cz.clearAbstractLink();		
		}
	}
	*/
	
	/*
	// test program
    // ------------

    private static void pr(String s) {
      System.out.println(s);
    }

    private static void hd(String s) {
      System.out.println("\n" + s + "\n");
    }

    private static void pg() {
      try {
        int i;
        System.out.print("next page? (CR)");
        do i = System.in.read(); while (i != '\n');
        System.out.println();
      }
      catch (Exception e) {System.out.println("raised exception : " + e);}
    }

    public static void main(String args[]) throws Exception {
    	hd("Tests of clearAbstractZones");
    	SerialWriter sw = new SerialWriter(null);
    	pr("sw="+sw);
    	ConcreteDiagram cd1 = new ConcreteDiagram();
    	DiagramViewerComponent dvc1 = new DiagramViewerComponent(cd1,100,200,false);
    	pr("dvc1="+dvc1);
    	pr("sw.clearAbstractZones(dvc1)");
    	    sw.clearAbstractZones(dvc1);
    	pg();
    	
    	Polygon p = new Polygon();
    	p.addPoint(1,2);
    	ConcreteContour cca = new ConcreteContour(Contour.getContour("a"),p);
    	Vector va = new Vector();
    	va.add(cca);
    	Vector v = new Vector();   	
    	
    	ConcreteZone cz1 = new ConcreteZone(v,va);
    	ConcreteZone cz2 = new ConcreteZone(va,v);
    	
    	Vector zv = new Vector();
    	zv.add(cz1);
    	zv.add(cz2);
    	pr("zv="+zv);
    	pg();
    	
    	ConcreteZoneSet czs = new ConcreteZoneSet(zv);
    	ConcreteDiagram cd2 = new ConcreteDiagram();
    	cd2.setConcreteZoneSet(czs);
    	DiagramViewerComponent dvc2 = new DiagramViewerComponent(cd2,100,200,false);
    	pr("dvc2="+dvc2);
		Iterator it = 
			dvc2.getConcreteDiagram().getConcreteZoneSet().getZones().iterator();
		while (it.hasNext()){
			ConcreteZone cz = (ConcreteZone) it.next();
			pr("  cz.abstractZone="+cz.abstractZone);	
		}
		pg();
    	
    	pr("sw.clearAbstractZones(dvc2)");
    	    sw.clearAbstractZones(dvc2);
    	pg();
    	
		it = 
			dvc2.getConcreteDiagram().getConcreteZoneSet().getZones().iterator();
		while (it.hasNext()){
			ConcreteZone cz = (ConcreteZone) it.next();
			pr("  cz.abstractZone="+cz.abstractZone);	
		}
    	pg();
    }
    */
}
