package rwd.gui.editor.writers;

import java.awt.Color;
import java.awt.Polygon;
import java.awt.geom.Point2D;
import java.io.File;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.util.Vector;
import java.util.HashMap;
import java.util.IdentityHashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.swing.JOptionPane;

import rwd.concretesyntax.DiagramItem;
import rwd.concretesyntax.constraintdiagram.ConcreteArc;
import rwd.concretesyntax.constraintdiagram.ConcreteArcEnd;
import rwd.concretesyntax.constraintdiagram.ConcreteArcSet;
import rwd.concretesyntax.constraintdiagram.ConcreteTextBox;
import rwd.concretesyntax.constraintdiagram.ConcreteTextBoxSet;
import rwd.concretesyntax.eulerdiagram.ConcreteContour;
import rwd.concretesyntax.eulerdiagram.ConcreteZone;
import rwd.concretesyntax.eulerdiagram.DecorationBox;
import rwd.concretesyntax.eulerdiagram.DecorationSet;
import rwd.concretesyntax.spiderdiagram.ConcreteSpiderFoot;
import rwd.concretesyntax.spiderdiagram.ConcreteSpiderLeg;
import rwd.concretesyntax.spiderdiagram.ConcreteSpiderFeetAndLegsSet;
import rwd.gui.editor.viewers.DiagramViewerInterface;
import uk.ac.kent.cs.kmf.util.ILog;

/**
 * @author Octavian Patrascoiu
 *
 */
public class XmlWriter 
	extends DiagramWriter
{
	/** Constructor */
	public XmlWriter(ILog log) {
		super(log);
	}
	
	/** Write a list of unitary diagrams */
	public void write(String fileName, DiagramViewerInterface viewer) {
		write(new File(fileName), viewer);
	}
	public void write(File file, DiagramViewerInterface viewer) {
		try {
			PrintWriter out = new PrintWriter(new FileWriter(file));
			out.println("<?xml version=\"1.0\"?>");
			out.println("<content>");
			write(out, tab, viewer);
			out.println("</content>");
			out.close();
		} catch (Exception anye) {
			anye.printStackTrace();
			JOptionPane.showMessageDialog(
				null,
				"The CD file could not be created:\n\n" + anye,
				"Could not write CD",
				JOptionPane.ERROR_MESSAGE);
		}
	}
	
	/** Specialized write methods */
	// Write the viewer
	protected void write(PrintWriter out, String indent, DiagramViewerInterface host) {
		out.println(indent+"<viewer title='"+host.getTitle()+"'>");
		write(out, indent+tab, host.getConcreteDiagram().getDecorationSet());
		writeConcreteContours(out, indent+tab, host.getConcreteDiagram().getConcreteContours());
		writeConcreteZones(out, indent+tab, host.getConcreteDiagram().getConcreteZones());
		write(out, indent+tab, host.getConcreteDiagram().getFeetAndLegsSet());
		write(out, indent+tab, host.getConcreteDiagram().getArcSet());
		write(out, indent+tab, host.getConcreteDiagram().getTextSet());
		out.println(indent+"</viewer>");
	}
	// Write decoration boxes
	protected void write(PrintWriter out, String indent, DecorationSet host) {
		List list = host.getBoxes();
		for(int i=0; i<list.size(); i++) {
			write(out, indent, (DecorationBox)list.get(i));			
		}
	}
	protected void write(PrintWriter out, String indent, DecorationBox host) {
		// Store box
		Integer contourId = newId();
		item2id.put(host, contourId);
		// Write box
		out.println(indent+"<box label='"+host.getLabel()+"' id='"+contourId+"'>");
		out.print(indent+tab+"<reading>");
		out.print(host.getReadingForest());		
		out.println(indent+tab+"</reading>");
		write(out, indent+tab, host.getPolygon());
		out.println(indent+"</box>");
	}
	protected void write(PrintWriter out, String indent, Polygon host) {
		out.println(indent+"<polygon>");
		for(int i=0; i<host.npoints; i++) {
			out.println(indent+tab+"<point x='"+host.xpoints[i]+"' y='"+host.ypoints[i]+"'/>");
		}
		out.println(indent+"</polygon>");
	}

	// Write contours
	protected void writeConcreteContours(PrintWriter out, String indent, Set host) {
		List list = new Vector(host);
		for(int i=0; i<list.size(); i++) {
			write(out, indent, (ConcreteContour)list.get(i));			
		}
	}
	protected void write(PrintWriter out, String indent, ConcreteContour host) {
		// Store contour
		Integer contourId = newId();
		item2id.put(host, contourId);
		// Write contour
		out.println(indent+"<contour label='"+host.getLabel()+"' id='"+contourId+"'>");
		writeLabel(out, indent+tab, host);
		write(out, indent+tab, host.getPolygon());
		out.println(indent+"</contour>");
	}

	// Write concrete zones
	protected void writeConcreteZones(PrintWriter out, String indent, Set host) {
		List list = new Vector(host);
		for(int i=0; i<list.size(); i++) {
			write(out, indent, (ConcreteZone)list.get(i));			
		}
	}
	protected void write(PrintWriter out, String indent, ConcreteZone host) {
		out.println(indent+"<zone>");
		// Write including
		out.println(indent+tab+"<in>");
		writeRefContours(out, indent+tab, host.getContainingPolygonals());
		out.println(indent+tab+"</in>");
		// Write excluding
		out.println(indent+tab+"<out>");
		writeRefContours(out, indent+tab, host.getExcludingPolygonals());
		out.println(indent+tab+"</out>");
		// Write color
		Color c = (Color)host.getBackColor();
		int r = c.getRed();
		int g = c.getGreen();
		int b = c.getBlue();
		out.println(indent+tab+"<color>"+r+","+g+","+b+"</color>");
		out.println(indent+"</zone>");
	}
	protected void writeRefContours(PrintWriter out, String indent, List host) {
		for(int i=0; i<host.size(); i++) {
			Object contour = host.get(i);
			// Get contour's id
			Integer contourId = (Integer)item2id.get(contour);
			checkId(contour, contourId);
			// Write contour
			out.println(indent+tab+"<zone-contour ref='"+contourId+"'/>");
		}
	}

	// Write spiders
	protected void write(PrintWriter out, String indent, ConcreteSpiderFeetAndLegsSet host) {
		out.println(indent+"<spiders>");
		List feet = host.getFeet();
		for(int i=0; i<feet.size(); i++) {
			write(out, indent+tab, (ConcreteSpiderFoot)feet.get(i));
		}
		List legs = host.getLegs();
		for(int i=0; i<legs.size(); i++) {
			write(out, indent+tab, (ConcreteSpiderLeg)legs.get(i));
		}
		out.println(indent+"</spiders>");
	}
	protected void write(PrintWriter out, String indent, ConcreteSpiderLeg host) {
		ConcreteSpiderFoot source = host.getSource();
		ConcreteSpiderFoot target = host.getTarget();
		Integer sourceId = (Integer)item2id.get(source);
		Integer targetId = (Integer)item2id.get(target);
		checkId(source, sourceId);
		checkId(target, sourceId);
		out.println(indent+"<leg type='"+host.getType()+"' source='"+sourceId+"' target='"+targetId+"'/>");
	}
	protected void write(PrintWriter out, String indent, ConcreteSpiderFoot host) {
		Integer id = newId();
		item2id.put(host, id);
		out.println(indent+"<foot id='"+id+"' type='"+host.getType()+"' x='"+host.getX()+"' y='"+host.getY()+"'>");
		writeLabel(out, indent+tab, host);
		out.println(indent+"</foot>");
	}
	
	// Write arcs
	protected void write(PrintWriter out, String indent, ConcreteArcSet host) {
		out.println(indent+"<arcs>");
		List arcs = host.getArcs();
		for(int i=0; i<arcs.size(); i++) {
			write(out, indent+tab, (ConcreteArc)arcs.get(i));
		}
		out.println(indent+"</arcs>");
	}
	protected void write(PrintWriter out, String indent, ConcreteArc host) {
		out.println(indent+"<arc type='"+host.getType()+"'>");
		write(out, indent+tab, host.getStart());
		if (host.getType() == ConcreteArc.QUAD)
			write(out, indent+tab, host.getMiddle());
		write(out, indent+tab, host.getEnd());
		writeLabel(out, indent+tab, host);
		out.println(indent+"</arc>");
	}
	protected void write(PrintWriter out, String indent, ConcreteArcEnd host) {
		DiagramItem item = host.getAttachedItem();
		if (item == null)
			out.println(indent+"<end x='"+host.getX()+"' y='"+host.getY()+"'/>");
		else {
			// Search for id
			Integer id = (Integer)item2id.get(item);
			checkId(host, id);
			out.println(indent+"<end id='"+id+"' x='"+host.getX()+"' y='"+host.getY()+"'/>");
		}
	}
	
	// Write label
	protected void writeLabel(PrintWriter out, String indent, DiagramItem host) {
		Point2D loc = host.getLabelPosition();
		double x = loc.getX();
		double y = loc.getY();
		out.println(indent+"<label name='"+host.getLabel()+"' x='"+x+"' y='"+y+"'/>");
	}
	
	// Write comments
	protected void write(PrintWriter out, String indent, ConcreteTextBoxSet host) {
		out.println(indent+"<comments>");
		List texts = host.getTexts();
		for(int i=0; i<texts.size(); i++) {
			ConcreteTextBox text = (ConcreteTextBox)texts.get(i);
			write(out, indent+tab, text);
		}
		out.println(indent+"</comments>");
	}
	protected void write(PrintWriter out, String indent, ConcreteTextBox host) {
		out.println(indent+"<comment type='"+host.getBoxType()+"'>");
		write(out, indent+tab, host.getPolygon());
		out.println(indent+tab+"<text>");
		out.println(indent+tab+tab+host.getText());
		out.println(indent+tab+"</text>");
		out.println(indent+"</comment>");
	}
	
	// Write zone colors
	protected void write(PrintWriter out, String indent, HashMap host) {
		out.println(indent+"<colors>");
		Iterator i = host.keySet().iterator();
		while (i.hasNext()) {
			String key = (String)i.next();
			out.println(indent+tab+"<key>"+key+"</key>");
			Color c = (Color)host.get(key);
			int r = c.getRed();
			int g = c.getGreen();
			int b = c.getBlue();
			out.println(indent+tab+"<value>"+r+","+g+","+b+"</value>");
		}
		out.println(indent+"</colors>");
	}

	protected void checkId(Object source, Object id) {
		if (id == null) {
			log.reportError("Cannot find id for '"+source+"' in XML writer");
		}
	}

	protected static Integer newId() {
		return new Integer(++id);
	}
	
	// Local properties
	protected String tab = "\t";
	protected static Map item2id = new IdentityHashMap();
	protected static int id = 0;
}
