package rwd.metrics;

import java.awt.Polygon;
import java.awt.geom.PathIterator;
import java.awt.geom.Point2D;
import java.util.Iterator;
import java.util.Set;

import rwd.concretesyntax.ConcreteUnitaryDiagram;
import rwd.concretesyntax.eulerdiagram.ConcreteContour;


/**
 * Find the difference between two polygons based on
 * their relative difference in points.
 * First the polygons are overlaid, with the centres of
 * their bounding boxes matched. The polygon (LINE_TO) points are
 * then compared in the order they appear in the polygon points list.
 * This assumes the polygons have the same number of points and
 * that the polygons are simple, with no extra MOVE_TO points except at
 * the start.
 **/
public class ContourPointsDifferenceComparison 
	extends Comparison 
{
	public ContourPointsDifferenceComparison(ConcreteUnitaryDiagram cud, double weighting, ConcreteUnitaryDiagram moveCud) {
		super(cud, weighting,moveCud);
		usedByPolygonMove = false;
	}


	protected double giveScore() {
		
		double total = 0.0;

        Set moveContourPolys = cud.getConcreteContours();
		Set stillContourPolys = stillCud.getConcreteContours();

        Iterator i = stillContourPolys.iterator();
        while(i.hasNext()){
			ConcreteContour stillcp = (ConcreteContour)i.next();
			Polygon stillp = stillcp.getPolygon();

			Point2D.Double stillCentre = ContourPositionComparison.polygonCentre(stillp);
            
            Iterator j = moveContourPolys.iterator();
            while(j.hasNext()){

				ConcreteContour movecp = (ConcreteContour)j.next();

				if(stillcp.getContour().equals(movecp.getContour())) {

					Polygon movep = movecp.getPolygon();

					Point2D.Double moveCentre = ContourPositionComparison.polygonCentre(movep);

					double diffX = stillCentre.x-moveCentre.x;
					double diffY = stillCentre.y-moveCentre.y;

// copy one polygon, because it will move to get an accurate overlay
					Polygon newMovep = new Polygon(movep.xpoints, movep.ypoints, movep.npoints);
					ContourAreaDifferenceComparison.shiftPolygon(newMovep,diffX,diffY);

// do the area difference calculation
					double diff = findPointsDifference(stillp,newMovep);

					total += diff;
				}
			}
		}
		
// for scaling
		double polygonArea = ContourPositionComparison.getPolygonAreaSum(stillContourPolys);
        if(polygonArea!=0){
            return total/polygonArea;
        } else {
            return total;
        }

	}	



	public static double findPointsDifference(Polygon poly1,Polygon poly2) {

		double ret = 0.0;

		double[] pathPoint1 = {0.0,0.0,0.0,0.0,0.0,0.0};
		double[] pathPoint2 = {0.0,0.0,0.0,0.0,0.0,0.0};
		PathIterator pi1 = poly1.getPathIterator(null);
		PathIterator pi2 = poly2.getPathIterator(null);
		while(!pi1.isDone() && !pi2.isDone()) {
			int segmentType1 = pi1.currentSegment(pathPoint1);
			int segmentType2 = pi2.currentSegment(pathPoint2);

			if(segmentType1 == PathIterator.SEG_LINETO || segmentType1 == PathIterator.SEG_MOVETO) {
				double x1 = pathPoint1[0];
				double y1 = pathPoint1[1];

				double x2 = pathPoint2[0];
				double y2 = pathPoint2[1];

				double xDiff = x1-x2;
				double yDiff = y1-y2;

				double distanceSquared = xDiff*xDiff+yDiff*yDiff;

				ret += distanceSquared;

			}

			pi1.next();
			pi2.next();
		}


		return ret;
	}


}




