package rwd.metrics;

import java.awt.Polygon;
import java.util.Iterator;
import java.util.Set;

import rwd.concretesyntax.ConcreteUnitaryDiagram;
import rwd.concretesyntax.eulerdiagram.ConcreteContour;

public class DiagramAspectRatio
	extends Metric 
{
	public DiagramAspectRatio(ConcreteUnitaryDiagram cud, double weighting) {
		super(cud, weighting);
	}

	public void setIdealAspectRatio(double ratio) {
		idealAspectRatio = ratio;
	}
	public double getIdealAspectRatio() {
		return idealAspectRatio;
	}

	protected double giveScore() {
		Set contourPolys = cud.getConcreteContours();

		if (contourPolys.size() == 0) {
			return (0.0);
		}

		double minX = Double.MAX_VALUE;
		double minY = Double.MAX_VALUE;
		double maxX = -Double.MAX_VALUE;
		double maxY = -Double.MAX_VALUE;
        
        Iterator i = contourPolys.iterator();
        while(i.hasNext()){

			ConcreteContour cp = (ConcreteContour) i.next();
			Polygon poly = cp.getPolygon();
			int[] xs = poly.xpoints;
			int[] ys = poly.ypoints;

			for (int j = 0; j < poly.npoints; j++) {

				int x = xs[j];
				int y = ys[j];
				if (x > maxX) {
					maxX = x;
				}
				if (x < minX) {
					minX = x;
				}
				if (y > maxY) {
					maxY = y;
				}
				if (y < minY) {
					minY = y;
				}
			}
		}

		double ar = (maxX - minX) / (maxY - minY);
		double result = Math.abs(ar - idealAspectRatio);
		//System.out.println("DiagramAspectRatioMetric returning "+result);
		return result;
	}

	/** this is the aspect ratio that will give a score of 0 */
	double idealAspectRatio = 1.61;
}
