package rwd.metrics;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.util.List;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JTextField;

import rwd.tableaux.semantic.MatrixPanel;

import uk.ac.kent.cs.kmf.util.ILog;

/**
 * @author Octavian Patrascoiu
 *
 */
public class MetricsDialog
	extends JDialog
{
	public MetricsDialog(JFrame parent, String title, ILog log) {
		super(parent, title);	
		getContentPane().setLayout(new GridBagLayout());
		this.log = log;
		
		//
        // Create and fill PROJECT panel
        //
		MatrixPanel diagramPanel = new MatrixPanel();
		// Initialize common variables 
		int line = 0;
		int column = 0;

    	// Diagrams
    	JLabel label = diagramPanel.addLabel("Diagram", line, column);
        diagramField = diagramPanel.addTextField("d1", line, column+1);
        diagramField.setColumns(10);
		line++;

		// Metrics
		MatrixPanel metricsPanel = new MatrixPanel();
		for(int i=0; i<metrics.length; i++) {
			addMetric(metricsPanel, metrics[i], boxes[i], line, column);
			line++;			
		}
		
		// Active deltas
		metricsPanel.add(activeDeltas, line, column);
		metricsPanel.addLabel("Active deltas", line, column+1);
		line++;

		//
        // Create and fill CONFIRM panel 
        //
        MatrixPanel confirmPanel = new MatrixPanel();
        // OK button 
		JButton okButton = confirmPanel.addButton("OK", 0, 0);
        okButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                okAction(evt);
            }
        });
        // Cancel button 
		JButton cancelButton = confirmPanel.addButton("Cancel", 0, 1);
		cancelButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				cancelAction(evt);
			}
		});

	    // Init MAIN panel 
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(2, 2, 2, 2);
		c.ipadx = 5;
		c.ipady = 5;
		c.weightx = 0.0;
		c.weighty = 0.0;
		
		// Add DIAGRAM panel
	    c.gridx = 0;
	    c.gridy = 0;
		getContentPane().add(diagramPanel, c);

		// Add METRICS panel
		c.gridx = 0;
		c.gridy = 1;
		getContentPane().add(metricsPanel, c);
		
		// Add CONFIRM to MAIN panel 		
	    c.gridx = 0;
	    c.gridy = 2;
	    c.anchor = GridBagConstraints.CENTER;
	    getContentPane().add(confirmPanel, c);
    		
        // Put frame in the middle of the screen
        pack();
        Dimension screen = getToolkit().getScreenSize();
        int width  = this.getSize().width+20;
        int height = this.getSize().height+20;
        int left = (screen.width - width)/2;
        int top = (screen.height - height)/2;
        setBounds(left, top, width, height);
	}

	// OK action 
	protected int returnVal = JFileChooser.CANCEL_OPTION;
    protected void okAction(java.awt.event.ActionEvent evt) {
    	returnVal = JFileChooser.APPROVE_OPTION;
    	String input = getDiagramName();
		if (input != null && input.length() != 0)  	
			dispose();
    }

	// Cancel action 
    protected void cancelAction(java.awt.event.ActionEvent evt) {
		returnVal = JFileChooser.CANCEL_OPTION;   	
    	dispose();
    }

	// Get XMI file name -
	public int showOpenDialog(JFrame parent) {
		this.setModal(true);
		show();
		return returnVal;
	}
	
	// Get the name of the unitary diagram  -
	public String getDiagramName() {
		return diagramField.getText();
	}
	
	// Get metrics name
	public List getMetricsName() {
		List res = new Vector();
		for(int i=0; i<boxes.length; i++) {
			if (boxes[i].isSelected())
				res.add(metrics[i]);
		}
		return res;
	}

	// Add a label and a box
	protected void addMetric(MatrixPanel panel, String name, JCheckBox box, int line, int column) {
		box.setSelected(true);
		panel.add(box, line, column);
		panel.addLabel(name, line, column+1);
	}

	public boolean isActiveDeltas() {
		return this.activeDeltas.isSelected();
	}
	
	protected JTextField diagramField;
	protected String metrics[] = {
		"NodeNodeSeparationMetric",
		"NodeZoneSeparationMetric",
		"EdgeLengthMetric",
		"AngularResolutionMetric",
		"EdgeEdgeCrossingMetric",
		"EdgeContourCrossingMetric"
	};
	protected JCheckBox boxes[] = {
		new JCheckBox(),
		new JCheckBox(),
		new JCheckBox(),
		new JCheckBox(),
		new JCheckBox(),
		new JCheckBox()
	};
	protected JCheckBox activeDeltas = new JCheckBox();
	protected ILog log;
}
