package rwd.metrics;

import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.Set;
import java.util.Vector;

import rwd.concretesyntax.ConcreteUnitaryDiagram;
import rwd.concretesyntax.DiagramItem;
import rwd.concretesyntax.constraintdiagram.ConcreteArc;
import rwd.concretesyntax.spiderdiagram.ConcreteSpiderFoot;
import rwd.concretesyntax.spiderdiagram.ConcreteSpiderLeg;
import rwd.deltas.*;

/**
 * @author Octavian Patrascoiu
 *
 * Node Edge Separation. Small distances between legs and feet are bad. 
 * 
 * Summed of 1/square of distance
 */
public class NodeEdgeSeparation
	extends Metric 
{
	public NodeEdgeSeparation(ConcreteUnitaryDiagram cud, double weighting) {
		super(cud, weighting);
		area = getArea();
	}
	public NodeEdgeSeparation(ConcreteUnitaryDiagram cud) {
		super(cud);
		area = getArea();
	}

	protected void initDeltas(ConcreteUnitaryDiagram cud) {
		if (cud == null) return;
		// For each pair (spider leg, edge)
		deltas = new Vector();
		Set nodes = cud.getConcreteFeet();
		Set items = new LinkedHashSet(cud.getConcreteLegs());
		items.addAll(cud.getConcreteArcs());
		Iterator i = nodes.iterator();
		while (i.hasNext()) {
			ConcreteSpiderFoot foot = (ConcreteSpiderFoot)i.next();
			Iterator j = items.iterator();
			while (j.hasNext()) {
				// Make a specialized delta delta(item) where item is either leg or arc  
				DiagramItem item = (DiagramItem)j.next();
				if (item instanceof ConcreteSpiderLeg) {
					ConcreteSpiderLeg leg = (ConcreteSpiderLeg)item;
					ConcreteSpiderFoot foot1 = leg.getSource();
					ConcreteSpiderFoot foot2 = leg.getTarget();
					if (foot1 != foot && foot2 != foot) {
						Delta delta = new NodeEdgeSeparationDelta(foot, foot1, foot2); 
						deltas.add(delta);
					}
				} else if (item instanceof ConcreteArc) {
					ConcreteArc arc = (ConcreteArc)item;
					DiagramItem source = arc.getSource();
					DiagramItem target = arc.getTarget();
					if (source != foot && target != foot) {
						Delta delta = new NodeEdgeSeparationDelta(foot, arc.getStart(), arc.getEnd()); 
						deltas.add(delta);
					}
				}
			}				
		}
	}
	public double giveScore() {
		double score = super.giveScore();
		if (area != 0)
			score /= area;
		return score;
	}

}
