package rwd.metrics;

import java.awt.Polygon;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

import rwd.abstractsyntax.eulerdiagram.Contour;
import rwd.concretesyntax.ConcreteUnitaryDiagram;
import rwd.concretesyntax.eulerdiagram.ConcreteContour;
import rwd.concretesyntax.eulerdiagram.ConcreteZone;
import rwd.concretesyntax.eulerdiagram.DecorationBox;

public class ZoneAreaProportional 
	extends SpiderMetric 
{
    public ZoneAreaProportional(
		ConcreteUnitaryDiagram cud, 
	    double weighting,
        HashMap abZonesToPoints) {
        super(cud, weighting);
        this.abZonesToPoints = abZonesToPoints;
    }

    /**
     * Score is given by calculating the difference in squares from
     * the mean area. Ergo, the lowest (best) score is achieved when
     * all zones have the same area.
     */
    public double giveScore() {
        Set zoneAreas = cud.getConcreteZones();
        int numZones = zoneAreas.size();

        double[] weightedAreas = new double[numZones];
        double[] points = new double[numZones];
        double weightedAreaAverage = 0.0;

        Iterator i = zoneAreas.iterator();
        int counter = 0;
        int numRelevantZones = 0;
        while (i.hasNext()) {
            ConcreteZone zoneArea = (ConcreteZone) i.next();
            //Polygon poly = zonePoly.getPoly();
            //System.out.println("ZAM assessing poly:"+AreaPolygonAdapter.niceString(poly));
            //area[i] =AreaPolygonAdapter.getArea(poly);
            //System.out.println("considering zone area "+zoneArea.getZoneString());
            try {
                points[counter] =
                    ((Double) abZonesToPoints.get(zoneArea.getAbstractZone()))
                        .doubleValue();
                //System.out.println("got "+points[counter]+ " for "+zoneArea.getAbstractZone());
            } catch (NullPointerException x) {
                System.out.println(
                    "no points for zone " + zoneArea.getAbstractZone());
                return 0;
            }
            if (points[counter] > 0) {
                double area =
                    AreaPolygonAdapter.getAreaOfConnectedArea(
                        zoneArea.getArea());
                area = area / 100000;
                //System.out.print(
                //    "area of " + zoneArea.getAbstractZone() + " is " + area);
                weightedAreas[counter] = area / points[counter];
                //System.out.println(" -> " + weightedAreas[counter]);
                weightedAreaAverage =
                    weightedAreaAverage + weightedAreas[counter];
                numRelevantZones++;
            }
            counter++;
        }
        weightedAreaAverage = weightedAreaAverage / numRelevantZones;
        //System.out.println("average is " + weightedAreaAverage);
        //System.out.println("ZAM returning score:"+score*totalArea);
        i = zoneAreas.iterator();
        counter = 0;
        double score = 0.0;
        while (i.hasNext()) {
            i.next();
            if (points[counter] > 0) {
                score =
                    score
                        + (weightedAreas[counter] - weightedAreaAverage)*points[counter]
                        * (weightedAreas[counter] - weightedAreaAverage)*points[counter]
                * (weightedAreas[counter] - weightedAreaAverage)*points[counter]
                * (weightedAreas[counter] - weightedAreaAverage)*points[counter];
            }
            counter++;
        }
        //System.out.println("score is " + score);
        return score;
    }

    public static void main(String[] args) { //build PolygonSet with null zones is OK
        Polygon p1 =
            new Polygon(
                new int[] { 0, 20, 20, 0 },
                new int[] { 0, 0, 20, 20 },
                4);
        Polygon p2 =
            new Polygon(
                new int[] { 5, 35, 30, 10 },
                new int[] { 5, 10, 30, 25 },
                4);
        ConcreteContour cp1 = new ConcreteContour(Contour.getContour("a"), p1);
        ConcreteContour cp2 = new ConcreteContour(Contour.getContour("b"), p2);
        ConcreteUnitaryDiagram cud =
            new ConcreteUnitaryDiagram(
                new DecorationBox(
                    "box",
					"",
                    new Polygon(
                        new int[] { 0, 50, 50, 0 },
                        new int[] { 0, 0, 50, 50 },
                        4)));
        cud.addConcreteContour(cp1);
        cud.addConcreteContour(cp2);
        Set czones = cud.getConcreteZones();
        HashMap zonesToPoints = new HashMap();
        Iterator i = czones.iterator();
        double[] numbers = new double[] { 1.0, 2.0, 4.0, 5.0 };
        int counter = 0;
        while (i.hasNext()) {
            ConcreteZone cz = (ConcreteZone) i.next();
            System.out.println("for map, cz is " + cz.hashCode());
            zonesToPoints.put(cz, new Double(numbers[counter]));
            counter++;
        }
        System.out.println(
            ""
                + new ZoneAreaProportional(cud, 1.0, zonesToPoints).giveScore());

        p1.xpoints[0] = 10;
        cud.refreshZones(true);
        cud.notifyObservers();
        System.out.println(
            ""
                + new ZoneAreaProportional(cud, 1.0, zonesToPoints).giveScore());
        ;

        p1 =
            new Polygon(
                new int[] { 0, 200, 200, 0 },
                new int[] { 0, 0, 200, 200 },
                4);
        p2 =
            new Polygon(
                new int[] { 50, 350, 300, 100 },
                new int[] { 50, 100, 300, 250 },
                4);
        cp1 = new ConcreteContour(Contour.getContour("a"), p1);
        cp2 = new ConcreteContour(Contour.getContour("b"), p2);
        cud =
            new ConcreteUnitaryDiagram(
                new DecorationBox(
                    "box",
					"",
                    new Polygon(
                        new int[] { 0, 400, 400, 0 },
                        new int[] { 0, 0, 400, 400 },
                        4)));
        cud.addConcreteContour(cp1);
        cud.addConcreteContour(cp2);
        czones = cud.getConcreteZones();
        zonesToPoints = new HashMap();
        i = czones.iterator();
        counter = 0;
        while (i.hasNext()) {
            ConcreteZone cz = (ConcreteZone) i.next();
            System.out.println("for map, cz is " + cz.hashCode());
            zonesToPoints.put(cz, new Double(numbers[counter]));
            counter++;
        }
        /**/
        System.out.println(
            ""
                + new ZoneAreaProportional(cud, 1.0, zonesToPoints).giveScore());
    }

	//
	// Local properties
	//
	protected HashMap abZonesToPoints;
}
