package rwd.rules;
import java.util.Vector;
import rwd.abstractsyntax.spiderdiagram.Diagram;
import rwd.rules.application.RuleApplicationSpec;
import java.util.Observable;

public class Proof extends Observable
{ // a proof is a sequence of diagrams and rwd.rules applications

	Vector proofSequence = new Vector();
	int currentStep = 0;

	public Proof(Diagram start) {
		proofSequence.add(start);
	}

	private Proof(Vector steps) {
		proofSequence = steps;
	}
	
	public int numSteps() {
		return (proofSequence.size() - 1) / 3;
	}

	public Vector getRules() { // extract 2nd, 5th,...
		Vector result = new Vector();
		for (int i = 1; i < proofSequence.size(); i = i + 3) {
			result.add(proofSequence.get(i));
		}
		return result;
	}

	public void tellObservers() {
		setChanged();
		notifyObservers();
	}

	public Proof cloneProof() {
		return new Proof((Vector) proofSequence.clone());
	}

	public void addStepAtStart(
		Diagram start,
		Rule r,
		RuleApplicationSpec ras) {
		proofSequence.add(0, ras);
		proofSequence.add(0, r);
		proofSequence.add(0, start);
		//System.out.println("proof about to tell observers");
		tellObservers();
		//System.out.println("proof told observers");
	}

	public void addStepAtEnd(
		Rule r,
		RuleApplicationSpec ras,
		Diagram outcome) {
		proofSequence.add(r);
		proofSequence.add(ras);
		proofSequence.add(outcome);
		//System.out.println("proof about to tell observers");
		tellObservers();
		//System.out.println("proof told observers");
	}

	public Diagram getLastDiagram() {
		return (Diagram) proofSequence.get(proofSequence.size() - 1);
	}
	public void setLastDiagram(Diagram d) {
		proofSequence.remove(proofSequence.size() - 1);
		proofSequence.add(d);
	}
	public Diagram getFirstDiagram() {
		return (Diagram) proofSequence.get(0);
	}

	public boolean trashLastStep() {
		if (proofSequence.size() > 1) {
			proofSequence.remove(proofSequence.size() - 1);
			proofSequence.remove(proofSequence.size() - 1);
			proofSequence.remove(proofSequence.size() - 1);
			return true;
		} else {
			return false;
		}
	}

	public void trashFirstStep() {
		if (proofSequence.size() > 1) {
			proofSequence.remove(0);
			proofSequence.remove(0);
			proofSequence.remove(0);
		}
	}
	public Proof extract(Diagram start, Diagram stop) {
		Proof result = new Proof(start);
		int i = 0;
		while (!((Diagram) (proofSequence.get(i))).equals(start)
			&& i < proofSequence.size() + 3) {
			i = i + 3;
		}
		if (i > proofSequence.size()) {
			return null;
		}
		while (!((Diagram) (proofSequence.get(i))).equals(stop)
			&& i < proofSequence.size()) {
			result.addStepAtEnd(
				(Rule) (proofSequence.get(i + 1)),
				(RuleApplicationSpec) (proofSequence.get(i + 2)),
				(Diagram) (proofSequence.get(i + 3)));
			i = i + 3;
		}
		return result;
	}

	public void goToStart() {
		currentStep = 0;
	}
	public Diagram getCurrentDiagram() {
		return (Diagram) (proofSequence.get(currentStep));
	}
	public Diagram getNextDiagram() {
		try {
			return (Diagram) (proofSequence.get(currentStep + 3));
		} catch (Exception e) {
			return null;
		}
	}
	public Rule getImminentRule() {
		return (Rule) (proofSequence.get(currentStep + 1));
	}
	public RuleApplicationSpec getImminentRuleApplicationSpec() {
		return (RuleApplicationSpec) (proofSequence.get(currentStep + 2));
	}

	public Rule getPreviousRule() {
		return (Rule) (proofSequence.get(currentStep - 2));
	}
	public RuleApplicationSpec getPreviousRuleApplicationSpec() {
		return (RuleApplicationSpec) (proofSequence.get(currentStep - 1));
	}

	public boolean goToNextStep() {
		if (currentStep < proofSequence.size() - 1) {
			currentStep = currentStep + 3;
			return true;
		} else {
			return false;
		}
	}

	public boolean goBackAStep() {
		if (currentStep > 2) {
			currentStep = currentStep - 3;
			return true;
		} else {
			return false;
		}
	}
	
}
