package rwd.rules;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

import java.io.File;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.awt.Component;
import java.util.Vector;
import java.util.Hashtable;

import rwd.abstractsyntax.eulerdiagram.*;
import rwd.abstractsyntax.spiderdiagram.*;
//import rwd.gui.ProofManager;
import rwd.gui.StaticObjects;
import rwd.rules.application.RuleApplicationSpec;
import rwd.rules.application.RuleAppliesToContour;
import rwd.rules.application.RuleAppliesToDandDandD;
import rwd.rules.application.RuleAppliesToDiDiAL;
import rwd.rules.application.RuleAppliesToDiagramAndDiagram;
import rwd.rules.application.RuleAppliesToDiagramAndDiagrams;
import rwd.rules.application.RuleAppliesToJustADiagram;
import rwd.rules.application.RuleAppliesToSpider;
import rwd.rules.application.RuleAppliesToSpiderAndRegion;
import rwd.rules.application.RuleAppliesToSpiderAndZone;
import rwd.rules.application.RuleAppliesToZone;
import rwd.rules.application.RuleAppliesWithString;
import rwd.rules.choicespec.RuleDistributeAndOverOr;
public class ProofFileUser {
    static File directory = new File(System.getProperty("user.dir"));
    static private boolean debug = false;

    static private Hashtable readers = new Hashtable();
    static private Hashtable writers = new Hashtable();

    public static void saveToFile(Proof p) {
        JFileChooser chooser = new JFileChooser(directory);
        chooser.setSelectedFile(new File(".proof"));
        //System.out.println("built FileChooser");
        int returnVal = chooser.showSaveDialog(StaticObjects.mdf);
        if (returnVal != JFileChooser.APPROVE_OPTION) {
            return;
        }
        File f = chooser.getSelectedFile();
        directory = chooser.getCurrentDirectory();
        //System.out.println("got file "+f+" from FileChooser");
        saveToFile(p, f);
    }

    private static void saveToFile(Proof p, File f) {
        boolean fileIsThereAlready = true;
        try {
            new FileInputStream(f);
        } catch (FileNotFoundException e) {
            fileIsThereAlready = false;
        }
        if (fileIsThereAlready
            && JOptionPane.showConfirmDialog(StaticObjects.mdf, "overwrite:" + f)
                != JOptionPane.OK_OPTION) {
            return;
        }
        //System.out.println("save to file "+f);
        try {
            FileWriter fw = new FileWriter(f);
            BufferedWriter b = new BufferedWriter(fw);
            writeToFile(p, b);
            b.flush();
            b.close();
        } catch (IOException e) {
            System.out.println(
                "An IO exception occured when executing "
                    + "saveToFile(..): error is "
                    + e
                    + "\n");
        }
    }

    public static void addProofAppend(Proof p, String filename) {
        try {
            // look for filename in writers
            BufferedWriter bf = (BufferedWriter) writers.get(filename);
            // if it's not there, build one and add it to tha hashtable
            if (bf == null) {
                bf = new BufferedWriter(new FileWriter(new File(filename)));
                writers.put(filename, bf);
            }
            writeToFile(p, bf);
        } catch (IOException e) {
            System.out.println("IO exception: " + e);
            e.printStackTrace();
            System.exit(0);
        } catch (NullPointerException e) {
            System.out.println("couldn't save proof");
            Vector pfseq = p.proofSequence;
            for (int i = 0; i < pfseq.size(); i++) {
                System.out.println("" + pfseq.get(i));
            }
        }
    }

    public static void addProofFreshFile(Proof p, String filename) {
        try {
            // look for filename in writers
            BufferedWriter bf = (BufferedWriter) writers.get(filename);
            // if it's not there, build one and add it to tha hashtable
            if (bf != null) {
                writers.remove(bf);
                bf.flush();
                bf.close();
            }
            // create a fresh writer
            bf = new BufferedWriter(new FileWriter(new File(filename)));
            writers.put(filename, bf);
            writeToFile(p, bf);
        } catch (IOException e) {
            System.out.println("IO exception: " + e);
            e.printStackTrace();
            System.exit(0);
        }
    }

    public static void clearFile(String filename) {
        try {
            // look for filename in writers
            BufferedWriter bf = (BufferedWriter) writers.get(filename);
            // if it is there clear and delete
            if (bf != null) {
                writers.remove(bf);
                bf.flush();
                bf.close();
            }
            // create a fresh writer
            bf = new BufferedWriter(new FileWriter(new File(filename)));
            bf.flush();
            bf.close();
            bf = new BufferedWriter(new FileWriter(new File(filename)));
            writers.put(filename, bf);
            // look for filename in readers
            BufferedReader bfr = (BufferedReader) readers.get(filename);
            // if it is there close and delete
            if (bfr != null) {
                readers.remove(bfr);
                bfr.close();
            }
            // create a fresh writer
            bfr = new BufferedReader(new FileReader(new File(filename)));
            readers.put(filename, bfr);
        } catch (IOException e) {
            System.out.println("IO exception: " + e);
            e.printStackTrace();
            System.exit(0);
        }
    }

    private static void writeToFile(Proof p, BufferedWriter b) {
        if (debug) {
            System.out.println(
                "in PFU, about to write to file ~~~~~~~~~~~~~~~~~~");
        }
        try {
            // save the outcome of toString()
            //System.out.println("get proof string");
            String s = toSaveString(p);
            //System.out.println("got proof string:\n" + s);
            //System.out.println("saving "+s);
            int newLinePlace = s.indexOf('\n');
            while (newLinePlace != -1) {
                b.write(s.substring(0, newLinePlace));
                b.newLine();
                s = s.substring(newLinePlace + 1, s.length());
                newLinePlace = s.indexOf('\n');
            };
            b.write(s);
            b.newLine();
            b.flush();
        } catch (IOException e) {
            System.out.println(
                "An IO exception occured when executing "
                    + "writeToFile(..): error is "
                    + e
                    + "\n");
        } catch (NullPointerException e) {
            System.out.println("null pointer in writeToFile !!!!!!!!" + e);
            System.exit(0);
        }
        //System.out.println("in PFU, finished writing to file");
    }

    public static Vector loadFromFile(Component c) {
        JFileChooser chooser = new JFileChooser(directory);
        chooser.setMultiSelectionEnabled(true);
        if (chooser.showOpenDialog(c) != JFileChooser.APPROVE_OPTION) {
            //System.out.println("other than approval from file opener");
            return null;
        }
        File[] fs = chooser.getSelectedFiles();
        directory = chooser.getCurrentDirectory();
        Vector result = new Vector();
        for (int i = 0; i < fs.length; i++) {
            File f = fs[i];
            InputStream inStream;
            try {
                inStream = new FileInputStream(f);
                //BufferedReader b = new BufferedReader(new FileReader(file));
                InputStreamReader isReader = new InputStreamReader(inStream);
                BufferedReader b = new BufferedReader(isReader);
                //Diagram diag = getDiagram(b);
                Proof p = getProof(b);
                //System.out.println("about to return:"+result);
                result.add(p);
            } catch (Exception e) {
                System.out.println("exception thrown " + e);
            }
        }
        return result;
    }

    public static Proof getProofFromWherever(String fileName) {
        try {
            BufferedReader bf;
            //look for filename in readers
            bf = (BufferedReader) readers.get(fileName);
            // if it's not there, build one and add it to tha hashtable
            if (bf == null) {
                bf = new BufferedReader(new FileReader(new File(fileName)));
                readers.put(fileName, bf);
            }
            // use it to get a diagram and return it
            return getProof(bf);
        } catch (IOException e) {
            System.out.println("IO exception: " + e);
            e.printStackTrace();
            System.exit(0);
            return null;
        }
    }

    public static Proof getProofFromStart(String fileName) {
        try {
            BufferedReader bf;
            //look for filename in readers
            bf = (BufferedReader) readers.get(fileName);
            // if it's not there, build one and add it to tha hashtable
            if (bf != null) {
                readers.remove(bf);
                bf.close();
            }
            bf = new BufferedReader(new FileReader(new File(fileName)));
            readers.put(fileName, bf);
            // use it to get a diagram and return it
            return getProof(bf);
        } catch (IOException e) {
            System.out.println("IO exception: " + e);
            e.printStackTrace();
            System.exit(0);
            return null;
        }
    }

    private static Proof getProof(BufferedReader b) {
        if (debug) {
            System.out.println(
                "getting proof from file::::::::::::::::::::::::::");
        }
        Diagram d = DiagramFileUser.getDiagram(b);
        //if (d == null) {
        //	System.out.println("first diagram in proof can't be read");
        //}
        Proof result = null;
        if (d != null) {
            result = new Proof(d);
            if (debug) {
                System.out.println(
                    "first diagram in proof is "
                        + DiagramFileUser.toSaveString(d));
            }
        } else {
            //System.out.println("first diagram in proof is null: return null proof");			
            return result;
        }
        while (true) {
            try {
                String line = b.readLine();
                if (debug) {
                    System.out.println("rule description is " + line);
                }
                if (line.equals("EndOfProof")) {
                    if (debug) {
                        System.out.println("finished the proof");
                        System.out.println("returning this:::::");
                        System.out.println(ProofFileUser.toSaveString(result));
                        System.out.println(":::::");                        
                    }
                    return result;
                } else {
                    // recreate Rule from line
                    if (debug) {
                        System.out.println("work out which rule it means");
                    }
                    Rule r = getRule(line);
                    if (debug) {
                        System.out.println(
                            "rule line is " + line + " means " + r);
                    }
                    //read next line is spec
                    if (debug) {
                        System.out.println("work out which spec: pass buffer");
                    }
                    RuleApplicationSpec ras = getSpec(b, d);
                    if (debug) {
                        System.out.println("spec is " + ras);
                    }
                    // get next diagram
                    //System.out.println("work out which diagram we end up at");

                    d = r.applyRuleToPart(d, ras);
                    //System.out.println("build up the proof");
                    result.addStepAtEnd(r, ras, d);
                    if (debug) {
                        System.out.println("built up the proof");
                    }
                    //System.out.println("d is "+DiagramFileUser.toSaveString(d));
                }
            } catch (IOException e) { /**/
                System.out.println("exception in getProof : " + e);
                System.exit(0);
                return result;
            } catch (OutOfMemoryError e) {
                System.out.println("out of memory in getProof : " + e);
                //System.exit(0);
                throw new OutOfMemoryError();
                //return result;
            }

        }
    }

    public static String toSaveString(Proof p) {

        // strategy - store 1st diagram as diagram
        // store rule
        // store ras using numerical codes for part-diagrams
        //System.out.println("}}}}}}}}}}}}}}} SAVING::: ");
        try {
            //System.out.println("converting proof to string~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~");
            //System.out.println("proof is "+p);
            p.goToStart();
            //System.out.println("at start of proof");
            //System.out.println("proof has steps " + p.proofSequence);
            Diagram d = p.getFirstDiagram();
            //System.out.println("d is " + d.toSaveString());
            String result = DiagramFileUser.toSaveString(d);
            while (p.goToNextStep()) {
                //result = result + "\n";
                //result = result + "next step:";
                result = result + "\n";
                //System.out.println("rule is " + p.getPreviousRule().getClass());
                //result = result + p.getPreviousRule().toSaveString();
                result = result + p.getPreviousRule().name;
                //System.out.println(
                //	"got a rule:\n~~~~~~~~~~~\n"
                //		+ p.getPreviousRule().toSaveString()
                //		+ "\n~~~~~~~~~~~~~~");
                result = result + "\n";
                //System.out.println(
                //	"app spec is "+ p.getPreviousRuleApplicationSpec());
                result =
                    result+toSaveString(p.getPreviousRuleApplicationSpec(),d);
                //System.out.println("got a spec:\n~~~~~~~~~~~\n"+result+"\n~~~~~~~~~~~~~~");
                //result = result + "\n";
                //result = result + p.getCurrentDiagram().toSaveString();
                d = p.getCurrentDiagram();
                //System.out.println("d is " + d.toSaveString());
            }
            //System.out.println("and wrap up end");
            //System.out.println("}}}}}}}}}}}}}}} FINISHED SAVING::: ");
            return result + "\nEndOfProof";
        } catch (Exception e) {

            System.out.println("exception in toSaveString for proofs" + e);
            Vector a = p.proofSequence;
            for (int i = 0; i < a.size(); i++) {
                System.out.println("" + i + ":" + a.get(i));
            }
            e.printStackTrace();
            System.exit(0);
            return "";
        }
    }

    private static String toSaveString(RuleApplicationSpec ras, Diagram d) {
        // remembers spec diagrams relative to whole
        //System.out.println("converting ras to string:" + ras.getClass());
        try {
            RuleAppliesToContour rac = (RuleAppliesToContour) ras;
            //System.out.println("!!!racon\n" + d.toSaveString()+"\n"+rac.d.toSaveString());
            return "racon\n" + string(d.getCoords(rac.d)) + "\n" + rac.c;
        } catch (ClassCastException x) {
        }
        try {
            RuleAppliesToDandDandD raddd = (RuleAppliesToDandDandD) ras;
            return "raddd\n"
                + string(d.getCoords(raddd.d))
                + "\n"
                + string(d.getCoords(raddd.other1))
                + "\n"
                + string(d.getCoords(raddd.other2));
        } catch (ClassCastException x) {
        }
        try {
            RuleAppliesToDiagramAndDiagram raddi =
                (RuleAppliesToDiagramAndDiagram) ras;
            return "raddi\n"
                + string(d.getCoords(raddi.d))
                + "\n"
                + string(d.getCoords(raddi.other));
        } catch (ClassCastException x) {
        }
        try {
            RuleAppliesToDiagramAndDiagrams radds =
                (RuleAppliesToDiagramAndDiagrams) ras;
            String result = "radds\n" + string(d.getCoords(radds.d)) + "\n";
            for (int i = 0; i < radds.others.size(); i++) {
                Diagram dtmp = (Diagram) radds.others.get(i);
                int[] position = d.getCoords(dtmp);
                if (position == null) {
                    throw new IllegalArgumentException();
                }
                result = result + string(position) + "\n";
            }
            result = result + "end of list";
            return result;
        } catch (ClassCastException x) {
        }
        try {
            RuleAppliesToDiDiAL radda = (RuleAppliesToDiDiAL) ras;
            String result = "radda\n" + string(d.getCoords(radda.d)) + "\n";
            result = result + string(d.getCoords(radda.d1)) + "\n";
            result = result + string(d.getCoords(radda.d2)) + "\n";
            for (int i = 0; i < radda.others.size(); i++) {
                Diagram dtmp = (Diagram) radda.others.get(i);
                result = result + string(d.getCoords(dtmp)) + "\n";
            }
            result = result + "end of list";
            return result;
        } catch (ClassCastException x) {
        }
        try {
            RuleAppliesToJustADiagram rajad = (RuleAppliesToJustADiagram) ras;
            String result = "rajad\n" + string(d.getCoords(rajad.d));
            return result;
        } catch (ClassCastException x) {
        }
        try {
            RuleAppliesToSpider rasp = (RuleAppliesToSpider) ras;
            String result = "raspi\n" + string(d.getCoords(rasp.d)) + "\n";
            result = result + DiagramFileUser.toSaveString(rasp.s.getHabitat());
            return result;
        } catch (ClassCastException x) {
        }
        try {
            RuleAppliesToSpiderAndRegion raspr =
                (RuleAppliesToSpiderAndRegion) ras;
            String result = "raspr\n" + string(d.getCoords(raspr.d)) + "\n";
            result =
                result
                    + DiagramFileUser.toSaveString(raspr.s.getHabitat())
                    + "\n";
            result = result + DiagramFileUser.toSaveString(raspr.region);
            return result;
        } catch (ClassCastException x) {
        }
        try {
            RuleAppliesToSpiderAndZone raspz = (RuleAppliesToSpiderAndZone) ras;
            String result = "raspz\n" + string(d.getCoords(raspz.d)) + "\n";
            result =
                result
                    + DiagramFileUser.toSaveString(raspz.s.getHabitat())
                    + "\n";
            result = result + DiagramFileUser.toSaveString(raspz.z);
            return result;
        } catch (ClassCastException x) {
        }
        try {
            RuleAppliesToZone raz = (RuleAppliesToZone) ras;
            String result = "razon\n" + string(d.getCoords(raz.d)) + "\n";
            result = result + DiagramFileUser.toSaveString(raz.z);
            return result;
        } catch (ClassCastException x) {
        }
        try {
            RuleAppliesWithString rastr = (RuleAppliesWithString) ras;
            String result = "rastr\n" + string(d.getCoords(rastr.d)) + "\n";
            result = result + rastr.s;
            return result;
        } catch (ClassCastException x) {
        }
        try {
            RuleAppliesToDiagramAndDiagrams radds =
                (RuleAppliesToDiagramAndDiagrams) ras;
            String result =
                "raddsExplicit\n" + string(d.getCoords(radds.d)) + "\n";
            for (int i = 0; i < radds.others.size(); i++) {
                Diagram dtmp = (Diagram) radds.others.get(i);
                result = result + DiagramFileUser.toSaveString(dtmp) + "\n";
            }
            result = result + "end of list";
            return result;
        } catch (ClassCastException ex) {
        }
        System.out.println("couldn't convert ras->String " + ras.getClass());
        System.exit(0);
        return "couldn't convert ras to string";
    }

    private static String string(int[] array) {
        String result = "[";
        for (int i = 0; i < array.length; i++) {
            result = result + array[i] + ",";
        }
        return result + "]";
    }

    private static Rule getRule(String s) {
        Vector rules = Rule.getAllSDRules();
        for (int i = 0; i < rules.size(); i++) {
            Rule r = (Rule) rules.get(i);
            String s2 = r.name;
            //String s2 = r.toString();
            //System.out.println("test whether "+s+" matches "+s2);
            if (s.equals(s2)) {
                return r;
            }
        }
        return null;
    }

    private static RuleApplicationSpec getSpec(BufferedReader b, Diagram d) {
        String s = "";
        String diagString = "";
        try {
            s = b.readLine();
            diagString = b.readLine();
            if (debug) {
                System.out.println("spec line is " + s);
            }
            if (debug) {
                System.out.println("diag text is " + diagString);
            }
            Diagram part = d.getPartAt(toIntArray(diagString));
            if (debug) {
                System.out.println(
                    "so part is " + DiagramFileUser.toSaveString(part));
            }
            if (s.equals("racon")) {
                if (debug) {
                    System.out.println("get contour for racon");
                }
                // next line is a string representation of contour
                Contour c = Contour.getContour(b.readLine());
                return new RuleAppliesToContour(part, c);
            } else if (s.equals("raddd")) { // next lines are other1 and other2
                if (debug) {
                    System.out.println("get parts for raddd");
                }
                Diagram other1 = d.getPartAt(toIntArray(b.readLine()));
                Diagram other2 = d.getPartAt(toIntArray(b.readLine()));
                if (debug) {
                    System.out.println("other1 is " + other1);
                    System.out.println("other2 is " + other2);
                }
                return new RuleAppliesToDandDandD(part, other1, other2);
            } else if (s.equals("raddi")) {
                if (debug) {
                    System.out.println("get part for raddi");
                }
                Diagram other = d.getPartAt(toIntArray(b.readLine()));
                return new RuleAppliesToDiagramAndDiagram(part, other);
            } else if (s.equals("radds")) { //
                if (debug) {
                    System.out.println("get parts for radds");
                }
                // lines are diagram coordinates except for "end of list"
                Vector diags = new Vector();
                String line = b.readLine();
                while (!line.equals("end of list")) {
                    if (debug) {
                        System.out.println("read line '" + line + "'");
                    }
                    int[] intarray = toIntArray(line);
                    Diagram dtmp = d.getPartAt(intarray);
                    if (debug) {
                        System.out.println(
                            "got part " + DiagramFileUser.toSaveString(dtmp));
                    }
                    if(dtmp==null){
                        System.out.println("extracted null from file");
                        System.out.println("intarray is "+string(intarray));
                                                
                        System.exit(0);
                    }
                    diags.add(dtmp);
                    line = b.readLine();
                }
                //System.out.println("return RulAppDDs spec");
                return new RuleAppliesToDiagramAndDiagrams(part, diags);
            } else if (s.equals("raddsExplicit")) { //
                if (debug) {
                    System.out.println("get parts for raddsExplicit");
                }
                // lines are diagram coordinates except for "end of list"
                Vector diags = new Vector();
                Diagram diag = DiagramFileUser.getDiagram(b);
                while (diag != null) {
                    //System.out.println("read line '"+line+"'");
                    diags.add(diag);
                    diag = DiagramFileUser.getDiagram(b);
                }
                //System.out.println("return RulAppDDs spec");
                return new RuleAppliesToDiagramAndDiagrams(part, diags);
            } else if (s.equals("radda")) {
                Diagram other1 = d.getPartAt(toIntArray(b.readLine()));
                String whatsthis = b.readLine();
                Diagram other2 = d.getPartAt(toIntArray(whatsthis));
                Vector diags = new Vector();
                String line = b.readLine();
                while (!line.equals("end of list")) {
                    diags.add(d.getPartAt(toIntArray(line)));
                    line = b.readLine();
                }
                if (other2 == null) {
                    System.out.println(
                        "part is " + DiagramFileUser.toSaveString(part));
                    System.out.println(
                        "other1 is " + DiagramFileUser.toSaveString(other1));
                    System.out.println("other2 is from " + whatsthis);
                    System.exit(0);
                }
                return new RuleAppliesToDiDiAL(part, other1, other2, diags);
            } else if (s.equals("rajad")) {
                return new RuleAppliesToJustADiagram(part);
            } else if (s.equals("raspi")) {
                Vector contours = ((UnitaryDiagram) part).getCloneOfContourSet();
                Spider sp =
                    new Spider(
                        DiagramFileUser.getRegionFrom(b.readLine(), contours));
                return new RuleAppliesToSpider(part, sp);
            } else if (s.equals("raspr")) {
                Vector contours = ((UnitaryDiagram) part).getCloneOfContourSet();
                Spider sp =
                    new Spider(
                        DiagramFileUser.getRegionFrom(b.readLine(), contours));
                Vector a =
                    DiagramFileUser.getRegionFrom(b.readLine(), contours);
                return new RuleAppliesToSpiderAndRegion(part, sp, a);
            } else if (s.equals("raspz")) {
                Vector contours = ((UnitaryDiagram) part).getCloneOfContourSet();
                Spider sp =
                    new Spider(
                        DiagramFileUser.getRegionFrom(b.readLine(), contours));
                //System.out.println("Spider is "+DiagramFileUser.toSaveString(sp));
                Zone z = DiagramFileUser.getZoneFrom(b.readLine(), contours);
                //System.out.println("Zone is "+DiagramFileUser.toSaveString(z));
                return new RuleAppliesToSpiderAndZone(part, sp, z);
            } else if (s.equals("razon")) {
                
                Vector contours = ((UnitaryDiagram) part).getCloneOfContourSet();
                String line = b.readLine();
                //System.out.println("getting zones from "+line);
                Zone z = DiagramFileUser.getZoneFrom(line, contours);
                return new RuleAppliesToZone(part, z);
            } else if (s.equals("rastr")) {
                return new RuleAppliesWithString(part, b.readLine());
            }
            System.out.println("can't make spec from " + s);
            //System.exit(0);
            return null;
        } catch (IOException e) {
            return null;
        } catch (Exception e) {
            System.out.println("exception in getSpec:" + e);
            e.printStackTrace();
            System.exit(0);
            return null;
        }
    }

    private static int[] toIntArray(String s) { // expect [1,3,2,]
        Vector integers = new Vector();
        //System.out.println("start with '" + s + "'");
        s = s.substring(1, s.length() - 1);
        //System.out.println("eliminate brackets: '" + s + "'");
        while (s.length() > 0) {
            String start = s.substring(0, s.indexOf(","));
            s = s.substring(s.indexOf(",") + 1, s.length());
            //System.out.println("number is '"+start+"'");
            //System.out.println("rest is '"+s+"'");
            integers.add(new Integer(Integer.parseInt(start)));
        }
        int[] result = new int[integers.size()];
        for (int i = 0; i < result.length; i++) {
            result[i] = ((Integer) integers.get(i)).intValue();
        }
        return result;
    }

    public static void main(String args[]) {
        // build a proof, save it and reload it.
        Vector cs = new Vector();
        cs.add(Contour.getContour("a"));
        UnitaryDiagram uda =
            new UnitaryDiagram(UnitaryDiagram.allPossibleZones(cs), "a");
        cs = new Vector();
        cs.add(Contour.getContour("b"));
        UnitaryDiagram udb =
            new UnitaryDiagram(UnitaryDiagram.allPossibleZones(cs), "b");
        cs = new Vector();
        cs.add(Contour.getContour("c"));
        UnitaryDiagram udc =
            new UnitaryDiagram(UnitaryDiagram.allPossibleZones(cs), "c");
        Vector children = new Vector();
        children.add(uda);
        children.add(udb);
        OrCompoundDiagram aorb = new OrCompoundDiagram(children, "aorb");
        children = new Vector();
        children.add(aorb);
        children.add(udc);
        AndCompoundDiagram aorb_and_c =
            new AndCompoundDiagram(children, "aorb_and_c");
        RuleDistributeAndOverOr r =
            (RuleDistributeAndOverOr) RuleDistributeAndOverOr.getRule();
        Vector specs = r.getApplicationPossibilities(aorb_and_c);
        System.out.println("specs are " + specs);
        Proof p = new Proof(aorb_and_c);
        RuleAppliesToDiagramAndDiagrams ras =
            (RuleAppliesToDiagramAndDiagrams) specs.get(0);
        Diagram outcome = r.applyRuleToPart(aorb_and_c, ras);
        p.addStepAtEnd(r, ras, outcome);
        System.out.println("radds is " + ras);
        System.out.println("radds.others is " + ras.others);
        System.out.println("proof is " + ProofFileUser.toSaveString(p));

    }
}
