package rwd.rules.autoproof;

import rwd.abstractsyntax.eulerdiagram.*;
import rwd.abstractsyntax.spiderdiagram.*;
import java.util.Vector;

import rwd.rules.*;
import rwd.rules.Proof;
import rwd.rules.Rule;
import rwd.rules.application.RuleApplicationSpec;
import rwd.rules.application.RuleAppliesToDandDandD;
import rwd.rules.application.RuleAppliesToDiagramAndDiagrams;
import rwd.rules.application.RuleAppliesToZone;
import rwd.rules.choicespec.RuleUnshadeZone;
import rwd.abstractsyntax.spiderdiagram.Spider;
//import rwd.gui.ProofManager;
import rwd.rules.application.RuleAppliesToSpider;
import rwd.rules.choicespec.RuleDeleteSpider;
import rwd.rules.application.RuleAppliesToDiagramAndDiagram;
import rwd.rules.inputspec.RuleFalseImpliesAnything;
import rwd.rules.choicespec.RuleExcludedMiddle;
import rwd.rules.choicespec.RuleReverseExcludedMiddle;
import rwd.rules.choicespec.RuleWithChoice;
import rwd.rules.choicespec.RuleReverseOrIdempotent;
import rwd.rules.inputspec.RulePToPOrRubbish;

public class AutoProofWriterBetter {

    private static Proof concatenateProofs(
        Proof p1,
        Proof p2) { //ProofManager.showNewProof(p1,false);
        //ProofManager.showNewProof(p2,false);
        Proof result = p1.cloneProof();
        //ProofManager.showNewProof(p1,false);
        //ProofManager.showNewProof(p2,false);
        if (!result.getLastDiagram().equals(p2.getFirstDiagram())) {
            //System.out.println("result.getLastDiagram() is "+result.getLastDiagram().toDebugString());
            //System.out.println("p2.getFirstDiagram() is "+p2.getFirstDiagram().toDebugString());
            //Diagram d1 = (Diagram)result.getLastDiagram();
            //Diagram d2 = (Diagram)p2.getFirstDiagram();
            //System.out.println(""+cd1.getChildren().size());
            //System.out.println(""+cd2.getChildren().size());
            return null;
        }
        while (p1.goToNextStep()) {
        }
        p2.goToStart();
        if (result.trashLastStep()) {
            result.addStepAtEnd(
                p1.getPreviousRule(),
                p1.getPreviousRuleApplicationSpec(),
                p2.getFirstDiagram());
        } else {
            return p2;
        }
        while (p2.goToNextStep()) {
            result.addStepAtEnd(
                p2.getPreviousRule(),
                p2.getPreviousRuleApplicationSpec(),
                p2.getCurrentDiagram());
        }
        return result;
    }

    private static Proof startToAlpharised(Diagram d, Diagram other) {

        //System.out.println("proving from "+DiagramFileUser.toSaveString(d)+" to "+DiagramFileUser.toSaveString(other));
        //System.out.print("extending zone set ...");
        Proof result = ZoneEqualiser.makeZonesEqual(d, other);
        //System.out.println("done");
        //System.out.print("splitting spiders ...");
        result =
            concatenateProofs(
                result,
                SlowSpiderSplitter.splitAllSpiders(result.getLastDiagram()));
        //System.out.println("done");
        //System.out.print("turning to DNF ...");
        result =
            concatenateProofs(
                result,
                SlowMakerOfDisNF.turnToDisjNF(result.getLastDiagram()));
        //System.out.println("after making DNF ~~~~~~"+result.getLastDiagram().toDebugString());
        //System.out.println("done");
        //System.out.print("applying 'combine' ...");
        result =
            concatenateProofs(
                result,
                Combiner.combineWherePossible(result.getLastDiagram()));
        //System.out.println("done");
        //System.out.println("after combining where possible ~~~~~~~~~~~~~~~~~~");
        //System.out.println(""+result.getLastDiagram().toDebugString());
        //System.out.print("removing occurences of False ...");
        result =
            concatenateProofs(
                result,
                FalseRemover.removeFalse(result.getLastDiagram()));
        //System.out.println("done");
        //System.out.println("after removing false ~~~~~~~~~~~~~~~~~~");
        //System.out.println(""+result.getLastDiagram().toDebugString());
        //System.out.print("removing duplicates ...");
        result =
            concatenateProofs(
                result,
                DuplicatesRemover.removeDupes(result.getLastDiagram()));
        //System.out.println("done");
        //ProofManager.showNewProof(result, false, true);
        return result;
    }

    public static Proof writeProof(Diagram premis, Diagram conclusion)
        throws NoProofException {
        //System.out.println("trying to generate proof from ");
        //System.out.println("premis =  "+premis.toDebugString());
        //System.out.println("conclusion =  "+conclusion.toDebugString());
        //System.out.print("go through to alpharise with premis...");

        Proof start =
            ContourSetEqualiser.makeContourSetsEqual(premis, conclusion);
        Proof endBackwards =
            ContourSetEqualiser.makeContourSetsEqual(conclusion, premis);
        //System.out.println("made contour sets equal");
        Diagram lhstmp = start.getLastDiagram();
        Diagram rhstmp = endBackwards.getLastDiagram();
        start = concatenateProofs(start, startToAlpharised(lhstmp, rhstmp));
        //System.out.println("gone through to alpharised for lhs");
        endBackwards =
            concatenateProofs(endBackwards, startToAlpharised(rhstmp, lhstmp));
        //System.out.println("gone through to alpharised for rhs");
        //ProofManager.showNewProof(start, false, true);
        //ProofManager.showNewProof(endBackwards, false, true);
        //if(1==1){return null;}
        Vector rhsChildren;
        Diagram rhs = endBackwards.getLastDiagram();
        try { // we know that this is an or-junction of unitary alpha-diagrams
            rhsChildren = ((OrCompoundDiagram) rhs).getChildren();
        } catch (ClassCastException ex) {
            rhsChildren = new Vector();
            rhsChildren.add(endBackwards.getLastDiagram());
        }
        int[] numberOfTimesUsedInExcMiddleReversed =
            new int[rhsChildren.size()];
        for (int i = 0; i < rhsChildren.size(); i++) {
            numberOfTimesUsedInExcMiddleReversed[i] = 0;
        }
        Vector appPossibilities =
            (
                (RuleWithChoice) RuleReverseExcludedMiddle
                    .getRule())
                    .getApplicationPossibilities(
                rhs);
        for (int i = 0; i < appPossibilities.size(); i++) {
            RuleAppliesToDandDandD raddd =
                (RuleAppliesToDandDandD) appPossibilities.get(i);
            UnitaryDiagram ud1 = (UnitaryDiagram) raddd.other1;
            UnitaryDiagram ud2 = (UnitaryDiagram) raddd.other2;
            //System.out.println("Rule app has ");
            //System.out.println("ud1 is "+ud1);
            //System.out.println("ud2 is "+ud2);
            for (int j = 0; j < rhsChildren.size(); j++) {
                UnitaryDiagram inArray = (UnitaryDiagram) rhsChildren.get(j);
                if (inArray == ud1) {
                    numberOfTimesUsedInExcMiddleReversed[j]++;
                }
                if (inArray == ud2) {
                    numberOfTimesUsedInExcMiddleReversed[j]++;
                }
            }
        }

        //System.out.println("number of times it's possible to apply");
        //System.out.print("rev exc middle = ");
        //for(int i=0; i<rhsChildren.size(); i++)
        //{ /**/System.out.print(""+numberOfTimesUsedInExcMiddleReversed[i]); }
        //System.out.println("");

        //System.out.println("rhs before duplication is "+DiagramFileUser.toSaveString(rhs));
        int shift = 0;
        for (int i = 0; i < rhsChildren.size(); i++) {
            while (numberOfTimesUsedInExcMiddleReversed[i] > 1) {
                numberOfTimesUsedInExcMiddleReversed[i]--;
                Rule r = RulePToPOrRubbish.getRule();
                rhs = endBackwards.getLastDiagram();
                Diagram currentPart = rhs.getPartAt(new int[] { shift + i });
                RuleApplicationSpec ras =
                    new RuleAppliesToDiagramAndDiagram(
                        currentPart,
                        currentPart);
                shift++;
                endBackwards.addStepAtEnd(r, ras, r.applyRuleToPart(rhs, ras));
            }
        }
        //System.out.println("rhs after duplication is "+DiagramFileUser.toSaveString(rhs));
        rhs = endBackwards.getLastDiagram();
        try {
            rhs = (OrCompoundDiagram) endBackwards.getLastDiagram();
            //System.out.println("rhs after duplication is "+rhs.toDebugString());

            RuleWithChoice rrem =
                ((RuleWithChoice) RuleReverseExcludedMiddle.getRule());
            Vector apps = rrem.getApplicationPossibilities(rhs);
            while (apps.size() > 0) {
                RuleAppliesToDandDandD raddd =
                    (RuleAppliesToDandDandD) apps.get(0);
                //System.out.println("before applying reverse excluded middle");
                //System.out.println("rhs is "+rhs.toDebugString());
                //System.out.println("before applying reverse excluded middle");
                //System.out.println("spec is or,p,q:"+raddd);

                endBackwards.addStepAtEnd(
                    rrem,
                    raddd,
                    rrem.applyRuleToPart(rhs, raddd));
                rhs = (OrCompoundDiagram) endBackwards.getLastDiagram();
                //System.out.println("after applying reverse excluded middle");
                //System.out.println("rhs is "+rhs.toDebugString());
                apps = rrem.getApplicationPossibilities(rhs);
            }
        } catch (ClassCastException ex) {
        }
        rhs = endBackwards.getLastDiagram();

        //System.out.println(
        //    "rhs after contraction is " + DiagramFileUser.toSaveString(rhs));

        try {
            rhsChildren = ((OrCompoundDiagram) rhs).getChildren();
        } catch (ClassCastException ex) {
            rhsChildren = new Vector();
            rhsChildren.add(rhs);
        }
        Vector zones = ((UnitaryDiagram) (rhsChildren.get(0))).getCloneOfZoneSet();
        boolean[][] rhsShadedZones =
            new boolean[rhsChildren.size()][zones.size()];
        int[][] rhsSpiderCount = new int[rhsChildren.size()][zones.size()];
        for (int i = 0; i < rhsChildren.size(); i++) {
            UnitaryDiagram ud = (UnitaryDiagram) rhsChildren.get(i);
            for (int j = 0; j < zones.size(); j++) {
                Zone z = (Zone) zones.get(j);
                rhsShadedZones[i][j] = ud.containsShaded(z);
                rhsSpiderCount[i][j] = 0;
                for (int k = 0; k < ud.getSpiders().size(); k++) {
                    Spider s = (Spider) ud.getSpiders().get(k);
                    if (s.getHabitat().contains(z)) {
                        rhsSpiderCount[i][j]++;
                    }
                }
            }
        }
        boolean[] hasBeenMatchedBySomethingOnLHS =
            new boolean[rhsChildren.size()];
        for (int i = 0; i < rhsChildren.size(); i++) {
            hasBeenMatchedBySomethingOnLHS[i] = false;
        }
        boolean[] isASecondMatchForSomethingOnTheRHS;
        //System.out.println("have built info arrays for rhs");
        //System.out.println("before entering bigLoop:");
        //System.out.println("lhs is :"+DiagramFileUser.toSaveString(start.getLastDiagram()));
        //System.out.println("rhs is :"+DiagramFileUser.toSaveString(rhs));

        bigLoop : while (true) // enables start-again
            {
            //System.out.println("at start of bigLoop while(true)"+new Date());
            for (int i = 0; i < rhsChildren.size(); i++) {
                hasBeenMatchedBySomethingOnLHS[i] = false;
            }
            Vector lhsChildren;
            Diagram lhs = start.getLastDiagram();
            //System.out.println("lhs is "+lhs);
            try {
                lhsChildren = ((OrCompoundDiagram) lhs).getChildren();
            } catch (ClassCastException ex) {
                lhsChildren = new Vector();
                lhsChildren.add(lhs);
            }
            isASecondMatchForSomethingOnTheRHS =
                new boolean[lhsChildren.size()];
            for (int i = 0; i < lhsChildren.size(); i++) {
                isASecondMatchForSomethingOnTheRHS[i] = false;
            }
            //System.out.println("the rhs is made up of .........................");
            // take di on the left - iterate through
            // just for debugging...................
            //for(int i=0; i<lhsChildren.size(); i++)
            //{ Diagram temp = (Diagram)lhsChildren.get(i);
            //System.out.println("temp is "+temp.toDebugString());
            //}
            // just for debugging..................
            lhsLoop : for (
                int i = 0;
                    i < lhsChildren.size();
                    i++) { //Diagram temp = (Diagram)lhsChildren.get(i);
                //System.out.println("temp is "+temp.toDebugString());
                // Question - how can this be an And-compound diagram???
                // this causes a crash
                // qjfeL;JF;JFE;Lfj
                // 2j2  pijfi   J
                UnitaryDiagram di = (UnitaryDiagram) lhsChildren.get(i);
                // if di matches any Dj then take next di
                for (int j = 0; j < rhsChildren.size(); j++) {
                    UnitaryDiagram dj = (UnitaryDiagram) rhsChildren.get(j);
                    if (di.equals(dj)) {
                        if (hasBeenMatchedBySomethingOnLHS[j]) {
                            isASecondMatchForSomethingOnTheRHS[i] = true;
                        }
                        hasBeenMatchedBySomethingOnLHS[j] = true;
                        continue lhsLoop;
                    }
                }
                if (di instanceof FalseDiagram) {
                    // there was no F on rhs so apply F->Anything and
                    // start bigLoop again

                    Rule rfia = RuleFalseImpliesAnything.getRule();
                    RuleAppliesToDiagramAndDiagram radd =
                        new RuleAppliesToDiagramAndDiagram(
                            di,
                            (Diagram) rhsChildren.get(0));
                    lhs = (UnitaryDiagram) rfia.applyRuleToPart(lhs, radd);
                    start.addStepAtEnd(rfia, radd, lhs);
                    continue bigLoop;
                }
                boolean[] isShaded = new boolean[zones.size()];
                int[] spiderCount = new int[zones.size()];
                for (int j = 0; j < zones.size(); j++) {
                    Zone z = (Zone) zones.get(j);
                    isShaded[j] = di.containsShaded(z);
                    spiderCount[j] = 0;
                    for (int k = 0; k < di.getSpiders().size(); k++) {
                        Spider s = (Spider) di.getSpiders().get(k);
                        if (s.getHabitat().contains(z)) {
                            spiderCount[j]++;
                        }
                    }
                    //System.out.println("spider count for zone "+zones.get(j)+" is "+spiderCount[j]);
                }
                //System.out.println("have built info arrays");
                // if di can reduce to any Dj then reduce it and start again
                for (int j = 0; j < rhsChildren.size(); j++) {
                    //System.out.println("considering rhs diag #"+j);
                    boolean canReduceDiToDj = true;
                    for (int k = 0; k < zones.size(); k++) {
                        if (spiderCount[k] < rhsSpiderCount[j][k]
                            || (isShaded[k]
                                && rhsShadedZones[j][k]
                                && spiderCount[k] != rhsSpiderCount[j][k])
                            || (rhsShadedZones[j][k] && !isShaded[k])) {
                            canReduceDiToDj = false;
                            break;
                        }
                    }
                    if (canReduceDiToDj) { // do the reduction here and then start again
                        // delete shading if necessary
                        // delete spiders
                        //System.out.println("found a rhs partner for lhs diag "+i);
                        for (int k = 0; k < zones.size(); k++) {
                            if (isShaded[k] && !rhsShadedZones[j][k]) {
                                Rule r = RuleUnshadeZone.getRule();
                                RuleApplicationSpec ras =
                                    new RuleAppliesToZone(
                                        di,
                                        (Zone) zones.get(k));
                                int[] coords = lhs.getCoords(di);
                                //System.out.println("about to unshade a zone");
                                lhs = r.applyRuleToPart(lhs, ras);
                                di = (UnitaryDiagram) lhs.getPartAt(coords);
                                start.addStepAtEnd(r, ras, lhs);
                            }
                        }
                        for (int k = 0; k < zones.size(); k++) {
                            int numExtraSpiders =
                                spiderCount[k] - rhsSpiderCount[j][k];
                            while (numExtraSpiders > 0) {
                                //System.out.println("about to remove a spider");
                                numExtraSpiders--;
                                Spider s = null;
                                for (int q = 0;
                                    q < di.getSpiders().size();
                                    q++) {
                                    s = (Spider) di.getSpiders().get(q);
                                    if (s
                                        .getHabitat()
                                        .contains(zones.get(k))) {
                                        break;
                                    }
                                }
                                Rule r = RuleDeleteSpider.getRule();
                                RuleApplicationSpec ras =
                                    new RuleAppliesToSpider(di, s);
                                int[] coords = lhs.getCoords(di);
                                lhs = r.applyRuleToPart(lhs, ras);
                                di = (UnitaryDiagram) lhs.getPartAt(coords);
                                start.addStepAtEnd(r, ras, lhs);
                            }
                        }
                        continue bigLoop;
                    }
                }
                //System.out.println("need to work out metrics");
                // build a set forMetricMeasurement containing all Dj
                Vector forMetricMeasurement =
                    (Vector) rhsChildren.clone();
                // if di contradicts any Dj then remove Dj then forMetricMeasurement
                for (int j = 0; j < rhsChildren.size(); j++) {
                    UnitaryDiagram dj = (UnitaryDiagram) rhsChildren.get(j);
                    //System.out.println("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
                    //System.out.println("looking for a contradiction between");
                    //System.out.println("di="+di.toDebugString());
                    //System.out.println("dj="+dj.toDebugString());
                    boolean diContradictsDj = false;
                    for (int k = 0; k < zones.size(); k++) {
                        if ((isShaded[k]
                            && spiderCount[k] < rhsSpiderCount[j][k])
                            || (rhsShadedZones[j][k]
                                && rhsSpiderCount[j][k] < spiderCount[k])) {
                            diContradictsDj = true;
                            break;
                        }
                    }
                    if (diContradictsDj) {
                        forMetricMeasurement.remove(dj);
                    }
                }
                // if forMetricMeasurement is empty then use di to build a model c/ex
                if (forMetricMeasurement.size() == 0) {
                    //System.out.println("can't prove this!!! see "+di.toDebugString());
                    throw new NoProofException(di);
                }
                // measure metrics for di and all Dj in ForMetricMeasurement
                Vector metrics = new Vector();
                rhsChildLoop : for (int j = 0; j < rhsChildren.size(); j++) {
                    UnitaryDiagram dj = (UnitaryDiagram) rhsChildren.get(j);
                    if (!forMetricMeasurement.contains(dj)) {
                        continue rhsChildLoop;
                    }
                    //System.out.println("calculating metrics for "+di.toDebugString()+
                    //                                      " and "+dj.toDebugString());
                    int[] metric = new int[zones.size()];
                    for (int k = 0; k < zones.size(); k++) {
                        metric[k] =
                            rhsSpiderCount[rhsChildren.indexOf(dj)][k]
                                - spiderCount[k];
                        //System.out.println("lhs has "+spiderCount[k]+" spiders in "+zones.get(k));
                        //System.out.println("rhs has "+rhsSpiderCount[rhsChildren.indexOf(dj)][k]+" spiders in "+zones.get(k));
                        if (metric[k] < 0) {
                            metric[k] = 0;
                        }
                        //System.out.println("metric for "+zones.get(k)+" for spiders is "+metric[k]);
                        //System.out.println("shaded on left :"+isShaded[k]);
                        //System.out.println("shaded on right :"+rhsShadedZones[j][k]);
                        if (!isShaded[k]
                            && rhsShadedZones[j][k]) { //System.out.println("increment metric for shading");
                            metric[k]++;
                        }
                        //System.out.println("metric["+k+"] is "+metric[k]);
                    }
                    metrics.add(metric);
                }
                //System.exit(0);
                int lowestTotal = Integer.MAX_VALUE;
                int lowestIndex = 0;
                // pick the Dj with smallest metric sum
                for (int j = 0; j < metrics.size(); j++) {
                    int[] metric = (int[]) metrics.get(j);
                    int total = 0;
                    for (int k = 0; k < metric.length; k++) {
                        total = total + metric[k];
                    }
                    if (total < lowestTotal) {
                        lowestTotal = total;
                        lowestIndex = j;
                    }
                }
                // use the metric outcome for di, Dj to apply excluded middle to di
                int[] metric = (int[]) metrics.get(lowestIndex);
                UnitaryDiagram target =
                    (UnitaryDiagram) rhsChildren.get(lowestIndex);
                UnitaryDiagram source = di;
                for (int j = 0; j < metric.length; j++) {
                    while (metric[j] > 0) {
                        //System.out.println("metric["+j+"] is "+metric[j]);
                        // metric measurement is +ve
                        // find jth zone
                        Zone z = (Zone) zones.get(j);
                        Rule r = RuleExcludedMiddle.getRule();
                        RuleApplicationSpec ras =
                            new RuleAppliesToZone(source, z);
                        int[] coords = lhs.getCoords(source);
                        // apply excluded middle to source
                        //System.out.println("about to apply rule to " + lhs);
                        lhs = (OrCompoundDiagram) r.applyRuleToPart(lhs, ras);
                        //System.out.println("lhs is now " + lhs);
                        // add rwd.rules to "start"
                        start.addStepAtEnd(r, ras, lhs);
                        if (metric[j] != 1
                            || !rhsShadedZones[rhsChildren.indexOf(target)][j]) {
                            try {
                                coords[coords.length - 1]++;
                            } catch (ArrayIndexOutOfBoundsException x) {
                                coords = new int[] { 1 };
                            }
                        }
                        if (coords.length == 0) {
                            coords = new int[] { 0 };
                        }
                        source = (UnitaryDiagram) lhs.getPartAt(coords);
                        metric[j]--;
                    }
                    //System.out.println("metric["+j+"] is reduced to zero");
                }
                //System.out.println("source should reduce to target next time round");
                //System.out.println("source is "+source.toDebugString());
                //System.out.println("target equals "+target.toDebugString());
                // start again
                continue bigLoop;
            }
            // if you get here then the proof is done
            break;
        }
        // each lhs is equal to an rhs component
        // but the lhs may contain duplicates and may not hit some rhs ones

        Diagram lhs = start.getLastDiagram();
        rhs = endBackwards.getLastDiagram();
        //System.out.println("lhs matched is "+lhs.toDebugString());
        //System.out.println("rhs matched is "+rhs.toDebugString());
        Vector lhsParts = lhs.getUnitaryParts();
        Vector rhsParts = rhs.getUnitaryParts();
        //System.out.println("lhs has "+lhsParts.size()+" children");
        //System.out.println("rhs has "+rhsParts.size()+" children");
        //System.out.println("second matches for rhs:");
        //for(int i=0; i<lhsParts.size(); i++)
        //{ //System.out.print(""+isASecondMatchForSomethingOnTheRHS[i]); }
        //System.out.println("have been hit on rhs:");
        //for(int i=0; i<rhsParts.size(); i++)
        //{ //System.out.print(""+hasBeenMatchedBySomethingOnLHS[i]); }

        Vector thingsToRemove = new Vector();
        for (int i = 0; i < lhsParts.size(); i++) {
            if (isASecondMatchForSomethingOnTheRHS[i]) {
                thingsToRemove.add(lhsParts.get(i));
            }
        }
        Vector thingsToAdd = new Vector();
        for (int i = 0; i < rhsParts.size(); i++) {
            if (!hasBeenMatchedBySomethingOnLHS[i]) {
                thingsToAdd.add(rhsParts.get(i));
            }
        }
        // pair off equal things to remove and things to add
        Vector dontNeedToRemove = new Vector();
        Vector dontNeedToAdd = new Vector();
        removeLoop : for (int i = 0; i < thingsToRemove.size(); i++) {
            UnitaryDiagram toRemove = (UnitaryDiagram) thingsToRemove.get(i);
            for (int j = 0; j < thingsToAdd.size(); j++) {
                UnitaryDiagram toAdd = (UnitaryDiagram) thingsToAdd.get(j);
                if (toRemove.equals(toAdd) && !dontNeedToAdd.contains(toAdd)) {
                    dontNeedToRemove.add(toRemove);
                    dontNeedToAdd.add(toAdd);
                    continue removeLoop;
                }
            }
        }
        thingsToRemove.removeAll(dontNeedToRemove);
        thingsToAdd.removeAll(dontNeedToAdd);
        //System.out.println("must remove "+thingsToRemove.size()+" things from lhs");
        for (int i = 0; i < thingsToRemove.size(); i++) {
            UnitaryDiagram toRemove = (UnitaryDiagram) thingsToRemove.get(i);
            Vector children = new Vector(); // will find 2 matches
            boolean hasFoundFirst = false;
            for (int j = 0;
                j < ((OrCompoundDiagram) lhs).getChildren().size();
                j++) {
                UnitaryDiagram child =
                    (UnitaryDiagram) (((OrCompoundDiagram) lhs)
                        .getChildren()
                        .get(j));
                if (child.equals(toRemove)) {
                    children.add(child);
                    if (hasFoundFirst) {
                        break;
                    } // arraylist children number 2 already
                    else {
                        hasFoundFirst = true;
                    }
                }
            }
            //System.out.println("identify "+children.size()+" children");
            Rule rroi = RuleReverseOrIdempotent.getRule();
            //System.out.println("2. provides d,[d2,d3,..]");
            RuleAppliesToDiagramAndDiagrams radd =
                new RuleAppliesToDiagramAndDiagrams(lhs, children);
            lhs = rroi.applyRuleToPart(lhs, radd);
            start.addStepAtEnd(rroi, radd, lhs);
        } // end of thingsToRemove loop
        //System.out.println("must add "+thingsToAdd.size()+" things to lhs");
        for (int i = 0; i < thingsToAdd.size(); i++) {
            UnitaryDiagram toAdd = (UnitaryDiagram) thingsToAdd.get(i);
            Rule rptprr = RulePToPOrRubbish.getRule();
            RuleAppliesToDiagramAndDiagram radd =
                new RuleAppliesToDiagramAndDiagram(lhs, toAdd);
            lhs = rptprr.applyRuleToPart(lhs, radd);
            start.addStepAtEnd(rptprr, radd, lhs);
        } // end of thingsToRemove loop
        //System.out.println("after addition, lhs has "+lhsParts.size()+" children");

        // invert end-bit and concatenate and show proof

        start =
            concatenateProofs(start, ProofInverter.invertProof(endBackwards));
        //ProofManager.showNewProof(start,false);
        //ProofManager.showNewProof(invertProof(endBackwards),false);
        //System.out.println("returning proof "+start);
        return start;
    }
}
