package rwd.rules.choicespec;

import java.util.Vector;
import rwd.abstractsyntax.spiderdiagram.Diagram;
import rwd.abstractsyntax.spiderdiagram.FalseDiagram;
import rwd.rules.Rule;
import rwd.rules.application.RuleAppliesToDiagramAndDiagrams;
import rwd.rules.application.RuleApplicationSpec;
import rwd.abstractsyntax.spiderdiagram.OrCompoundDiagram;

public class FalseWithOrRule extends RuleWithChoice {
	private FalseWithOrRule() {
		super("sd: PorF->P");
	}
	private static Rule example = null;
	public static Rule getRule() {
		if (example == null) {
			example = new FalseWithOrRule();
		}
		return example;
	}
    public boolean isInvertible(){
        return true;
    }

	public boolean canApplyTo(Diagram d) {
		if (!(d instanceof OrCompoundDiagram)) {
			return false;
		}
		OrCompoundDiagram ocd = (OrCompoundDiagram) d;
		Vector children = (Vector) (ocd.getChildren().clone());
		Vector falses = new Vector();
		for (int i = 0; i < children.size(); i++) {
			Diagram d2 = (Diagram) (children.get(i));
			if (d2 instanceof FalseDiagram) {
				children.remove(d2);
				falses.add(d2);
			}
		}
		if (falses.size() > 0) {
			return true;
		} else {
			return false;
		}
	}

	public Vector getApplicationPossibilities(Diagram d) {
		Vector result = new Vector();
		if (!(d instanceof OrCompoundDiagram)) {
			return result;
		}
		OrCompoundDiagram ocd = (OrCompoundDiagram) d;
		Vector children = (Vector) (ocd.getChildren().clone());
		Vector falses = new Vector();
		for (int i = 0; i < children.size(); i++) {
			Diagram d2 = (Diagram) (children.get(i));
			if (d2 instanceof FalseDiagram) {
				children.remove(d2);
				falses.add(d2);
			}
		}
		//System.out.println("4. providing d,[f,f,..]");
		if (falses.size() > 0) {
			result.add(new RuleAppliesToDiagramAndDiagrams(ocd, falses));
		}
		return result;
	}

	public Diagram applyRule(RuleApplicationSpec r) {
		RuleAppliesToDiagramAndDiagrams radd =
			(RuleAppliesToDiagramAndDiagrams) r;
		OrCompoundDiagram ocd = (OrCompoundDiagram) radd.d;
		Vector children = (Vector) ocd.getChildren().clone();
		children.removeAll(radd.others);
		if (children.size() > 1) {
			return new OrCompoundDiagram(children, ocd.getName() + "-F");
		} else {
			return (Diagram) children.get(0);
		}
	}
}
