package rwd.rules.choicespec;
import rwd.abstractsyntax.eulerdiagram.Zone;
import rwd.abstractsyntax.spiderdiagram.Diagram;
import rwd.rules.Rule;
import rwd.rules.application.RuleApplicationSpec;
import rwd.rules.application.RuleAppliesToDandDandD;
import rwd.abstractsyntax.spiderdiagram.AndCompoundDiagram;
import java.util.Vector;
import java.util.Iterator;

import rwd.abstractsyntax.spiderdiagram.FalseDiagram;
import rwd.abstractsyntax.spiderdiagram.UnitaryDiagram;
import rwd.abstractsyntax.spiderdiagram.Spider;

public class MergeConjunctionRule extends RuleWithChoice {
    public MergeConjunctionRule() {
        super("sd: combining");
    }
    private static Rule example = null;

    public static Rule getRule() {
        if (example == null) {
            example = new MergeConjunctionRule();
        }
        return example;
    }
    public boolean isInvertible() {
        return true;
    }

    public boolean canApplyTo(Diagram d) {
        // d must be an And-Diagram, look at its children
        // there must be 2 non-false components which are alpha-diagrams
        // with same contours and zones
        // also, for this rule, the diagrams shouldn't be in 
        // contradiction with each other
        if (d instanceof AndCompoundDiagram) {
            AndCompoundDiagram acd = (AndCompoundDiagram) d;
            Vector children = acd.getChildren();
            iLoop : for (int i = 0; i < children.size(); i++) {
                Diagram ditmp = (Diagram) (children.get(i));
                if(!(ditmp instanceof UnitaryDiagram)){
                    return false;
                }
                UnitaryDiagram di = (UnitaryDiagram)ditmp;                
                if (di instanceof FalseDiagram) {
                    continue iLoop;
                }
                Vector spiders = di.getSpiders();
                for (int si = 0; si < spiders.size(); si++) {
                    Spider s = (Spider) spiders.get(si);
                    if (s.getHabitat().size() > 1) {
                        continue iLoop;
                    }
                }
                Vector contours0 = di.getCloneOfContourSet();
                Vector zones0 = di.getCloneOfZoneSet();
                jLoop : for (int j = i + 1; j < children.size(); j++) {
                    Diagram dothertmp = (Diagram) (children.get(j));
                    if(!(dothertmp instanceof UnitaryDiagram)){
                        return false;
                    }
                    UnitaryDiagram dother = (UnitaryDiagram) (dothertmp);
                    if (dother instanceof FalseDiagram) {
                        continue jLoop;
                    }
                    Vector contoursOther = dother.getCloneOfContourSet();
                    if (!(contours0.containsAll(contoursOther)
                        && contoursOther.containsAll(contours0))) {
                        continue jLoop;
                    }
                    Vector zonesOther = dother.getCloneOfZoneSet();
                    if (!(zones0.containsAll(zonesOther)
                        && zonesOther.containsAll(zones0))) {
                        continue jLoop;
                    }
                    spiders = dother.getSpiders();
                    for (int sj = 0; sj < spiders.size(); sj++) {
                        Spider s = (Spider) spiders.get(sj);
                        if (s.getHabitat().size() > 1) {
                            continue jLoop;
                        }
                    }
                    // now we know di and other have same contours and zones
                    // and both are alpha diagrams
                    // check that they are not contradictory
                    // count spiders
                    int[] numSpidersInDi =new int[zonesOther.size()];
                    for(int si=0; si < di.getSpiders().size(); si++){
                        Spider s = (Spider)di.getSpiders().get(si);
                        Zone hab = (Zone)s.getHabitat().get(0);
                        numSpidersInDi[zonesOther.indexOf(hab)]++;
                    }
                    int[] numSpidersInOther =new int[zonesOther.size()];
                    for(int si=0; si < dother.getSpiders().size(); si++){
                        Spider s = (Spider)dother.getSpiders().get(si);
                        Zone hab = (Zone)s.getHabitat().get(0);
                        numSpidersInOther[zonesOther.indexOf(hab)]++;
                    }
                    // scan through shaded zones in di
                    Iterator it=di.getSZIterator();
                    while(it.hasNext()){
                        Zone z = (Zone)it.next();
                        int index = zonesOther.indexOf(z);
                        if(numSpidersInOther[index]>numSpidersInDi[index]){
                            //System.out.println(""+dother+" has more spiders than "+di+" in "+z);                            
                            continue jLoop;
                        }
                    }
                    // scan through shaded zones in other
                    it=dother.getSZIterator();
                    while(it.hasNext()){
                        Zone z = (Zone)it.next();
                        int index = zonesOther.indexOf(z);
                        if(numSpidersInOther[index]<numSpidersInDi[index]){
                            //System.out.println(""+dother+" has fewer spiders than "+di+" in "+z);
                            continue jLoop;
                        }
                    }
                    return true;
                }
            }
        }
        return false;
    }

    public Vector getApplicationPossibilities(Diagram d) {
        // d must be an And-Diagram, look at its children
        // there must be 2 non-false components which are alpha-diagrams
        // with same contours and zones
        // also, for this rule, the diagrams shouldn't be in 
        // contradiction with each other
        Vector result = new Vector();
        if (d instanceof AndCompoundDiagram) {
            AndCompoundDiagram acd = (AndCompoundDiagram) d;
            Vector children = acd.getChildren();
            iLoop : for (int i = 0; i < children.size(); i++) {
                Diagram tmp = (Diagram)children.get(i);
                if(!(tmp instanceof UnitaryDiagram)){
                    continue iLoop;
                }
                UnitaryDiagram di = (UnitaryDiagram) tmp;
                if (di instanceof FalseDiagram) {
                    continue iLoop;
                }
                Vector spiders = di.getSpiders();
                for (int si = 0; si < spiders.size(); si++) {
                    Spider s = (Spider) spiders.get(si);
                    if (s.getHabitat().size() > 1) {
                        continue iLoop;
                    }
                }
                Vector contours0 = di.getCloneOfContourSet();
                Vector zones0 = di.getCloneOfZoneSet();
                jLoop : for (int j = i + 1; j < children.size(); j++) {
                    Diagram dothertmp = (Diagram) (children.get(j));
                    if(!(dothertmp instanceof UnitaryDiagram)){
                        continue jLoop;
                    }
                    UnitaryDiagram dother = (UnitaryDiagram) (dothertmp);
                    if (dother instanceof FalseDiagram) {
                        continue jLoop;
                    }
                    Vector contoursOther = dother.getCloneOfContourSet();
                    if (!(contours0.containsAll(contoursOther)
                        && contoursOther.containsAll(contours0))) {
                        continue jLoop;
                    }
                    Vector zonesOther = dother.getCloneOfZoneSet();
                    if (!(zones0.containsAll(zonesOther)
                        && zonesOther.containsAll(zones0))) {
                        continue jLoop;
                    }
                    spiders = dother.getSpiders();
                    for (int sj = 0; sj < spiders.size(); sj++) {
                        Spider s = (Spider) spiders.get(sj);
                        if (s.getHabitat().size() > 1) {
                            continue jLoop;
                        }
                    }
                    // now we know di and other have same contours and zones
                    // and both are alpha diagrams
                    // check that they are not contradictory
                    // count spiders
                    int[] numSpidersInDi =new int[zonesOther.size()];
                    for(int si=0; si < di.getSpiders().size(); si++){
                        Spider s = (Spider)di.getSpiders().get(si);
                        Zone hab = (Zone)s.getHabitat().get(0);
                        numSpidersInDi[zonesOther.indexOf(hab)]++;
                    }
                    int[] numSpidersInOther =new int[zonesOther.size()];
                    for(int si=0; si < dother.getSpiders().size(); si++){
                        Spider s = (Spider)dother.getSpiders().get(si);
                        Zone hab = (Zone)s.getHabitat().get(0);
                        numSpidersInOther[zonesOther.indexOf(hab)]++;
                    }
                    // scan through shaded zones in di
                    Iterator it = di.getSZIterator();
                    while(it.hasNext()){
                        Zone z = (Zone)it.next();
                        int index = zonesOther.indexOf(z);
                        if(numSpidersInOther[index]>numSpidersInDi[index]){
                            continue jLoop;
                        }
                    }
                    // scan through shaded zones in other
                    Iterator itr=dother.getSZIterator();
                    while(itr.hasNext()){
                        Zone z = (Zone)itr.next();
                        int index = zonesOther.indexOf(z);
                        if(numSpidersInOther[index]<numSpidersInDi[index]){
                            continue jLoop;
                        }
                    }
                    result.add(new RuleAppliesToDandDandD(d, di, dother));
                }
            }
        }
        return result;
    }

    public Diagram applyRule(RuleApplicationSpec r) { //System.out.println("about to apply Simple combining rwd.rules to "+r.d.toSaveString());
        RuleAppliesToDandDandD raddd = (RuleAppliesToDandDandD) r;
        AndCompoundDiagram d = (AndCompoundDiagram) raddd.d;
        UnitaryDiagram d1 = (UnitaryDiagram) raddd.other1;
        UnitaryDiagram d2 = (UnitaryDiagram) raddd.other2;
        int[] coords1 = d.getCoords(d1); 
        int[] coords2 = d.getCoords(d2);
        AndCompoundDiagram result = (AndCompoundDiagram)d.getClone();
        // remove two children and build a new child
        UnitaryDiagram toRemove1 = (UnitaryDiagram)result.getPartAt(coords1); 
        UnitaryDiagram toRemove2 = (UnitaryDiagram)result.getPartAt(coords2);
        Vector children = result.getChildren();
        children.remove(toRemove1); 
        children.remove(toRemove2); 
        // the new child has the same contours and zones as toRemove1 (and 2)
        UnitaryDiagram newChild = (UnitaryDiagram)toRemove1.getClone();
        // the shading in newChild should be the union of shading
        Iterator it = toRemove2.getSZIterator();
        while(it.hasNext()){
            Zone z = (Zone)it.next();
            if(!newChild.containsShaded(z)){
                newChild.shadeZone(z);
            }
        }
        // the number of spiders in newChild should be the max
        // in toRemove1 and toRemove2
        Vector zoneSet = toRemove1.getCloneOfZoneSet();
        int[] numSpidersIn1 =new int[zoneSet.size()];
        for(int si=0; si < toRemove1.getSpiders().size(); si++){
            Spider s = (Spider)toRemove1.getSpiders().get(si);
            Zone hab = (Zone)s.getHabitat().get(0);
            numSpidersIn1[zoneSet.indexOf(hab)]++;
        }
        int[] numSpidersIn2 =new int[zoneSet.size()];
        for(int si=0; si < toRemove2.getSpiders().size(); si++){
            Spider s = (Spider)toRemove2.getSpiders().get(si);
            Zone hab = (Zone)s.getHabitat().get(0);
            numSpidersIn2[zoneSet.indexOf(hab)]++;
        }
        for(int sz = 0; sz<zoneSet.size(); sz++){
            for(int newSp = 0; newSp<numSpidersIn2[sz]-numSpidersIn1[sz]; newSp++){
                Vector hab = new Vector();
                hab.add(zoneSet.get(sz));
                newChild.addSpider(hab);
            }
        }
        children.add(newChild);
        if(children.size()>1){
            return result;
        } else {
            return newChild;
        }
    }
}
