package rwd.rules.choicespec;
import java.util.Vector;

import rwd.abstractsyntax.eulerdiagram.Zone;
import rwd.abstractsyntax.spiderdiagram.Diagram;
import rwd.abstractsyntax.spiderdiagram.UnitaryDiagram;
import rwd.abstractsyntax.spiderdiagram.Spider;
import rwd.rules.Rule;
import rwd.rules.application.RuleApplicationSpec;
import rwd.rules.application.RuleAppliesToSpiderAndZone;
import rwd.abstractsyntax.spiderdiagram.*;

public class RuleAddSpiderFoot extends RuleWithChoice {
	private RuleAddSpiderFoot() {
		super("sd: add spider foot");
	}
	private static Rule example = null;
	public static Rule getRule() {
		if (example == null) {
			example = new RuleAddSpiderFoot();
		}
		return example;
	}
    public boolean isInvertible(){
        return false;
    }

	public boolean canApplyTo(Diagram d) {
		if (d instanceof UnitaryDiagram) {
			UnitaryDiagram ud = (UnitaryDiagram) d;
			// get all spiders
			Vector spiders = ud.getSpiders();
			Vector zones = ud.getCloneOfZoneSet();
			for (int i = 0;
				i < spiders.size();
				i++) { // get all present zones which aren't in habitat
				Spider s = (Spider) (spiders.get(i));
				for (int j = 0; j < zones.size(); j++) {
					Zone z = (Zone) (zones.get(j));
					if (!s.getHabitat().contains(z)) {
						return true;
					}
				}
			}
		}
		return false;
	}

	public Vector getApplicationPossibilities(Diagram d) {
		Vector result = new Vector();
		if (d instanceof UnitaryDiagram) {
			UnitaryDiagram ud = (UnitaryDiagram) d;
			// get all spiders
			Vector spiders = ud.getSpiders();
			Vector zones = ud.getCloneOfZoneSet();
			for (int i = 0;
				i < spiders.size();
				i++) { // get all present zones which aren't in habitat
				Spider s = (Spider) (spiders.get(i));
				for (int j = 0; j < zones.size(); j++) {
					Zone z = (Zone) (zones.get(j));
					if (!s.getHabitat().contains(z)) {
						result.add(new RuleAppliesToSpiderAndZone(ud, s, z));
					}
				}
			}
		}
		return result;
	}

	public Diagram applyRule(RuleApplicationSpec r) {
		RuleAppliesToSpiderAndZone rasz = (RuleAppliesToSpiderAndZone) r;
		Diagram d = rasz.d;
		if (d instanceof UnitaryDiagram) {
			UnitaryDiagram ud = (UnitaryDiagram) rasz.d;
			Zone z = rasz.z;
			Spider s = rasz.s;
			UnitaryDiagram result = (UnitaryDiagram) (ud.getClone());
			// find a spider in result which equals s
			for (int i = 0; i < result.getSpiders().size(); i++) {
				Spider newS = (Spider) (result.getSpiders().get(i));
				if (newS.equals(s)) {
					newS.getHabitat().add(z);
					return result;
				}
			}
		}
		System.out.println("applying rule with spec:"+r);
		System.out.println("part was "+DiagramFileUser.toSaveString(d));
		System.out.println("about to return null :-(");
		return null;
	}
}
