package rwd.rules.choicespec;
import java.util.Vector;
import rwd.abstractsyntax.spiderdiagram.Diagram;
import rwd.abstractsyntax.spiderdiagram.AndCompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.NotCompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.OrCompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.Utilities;
import rwd.rules.Rule;
import rwd.rules.application.RuleApplicationSpec;
import rwd.rules.application.RuleAppliesToDiagramAndDiagrams;

public class RuleDMPushNotThroughAnd extends RuleWithChoice {
    // takes Not (Q and R)
    // takes NotQ or NotR 
    private RuleDMPushNotThroughAnd() {
        super("sd: not(QandR) -> notQ or notR");
    }
    private static Rule example = null;

    public static Rule getRule() {
        if (example == null) {
            example = new RuleDMPushNotThroughAnd();
        }
        return example;
    }
    public boolean isInvertible() {
        return true;
    }

    public boolean canApplyTo(Diagram d) {
        if (!(d instanceof NotCompoundDiagram)) {
            //System.out.println("not a not diagram");
            return false;
        }
        NotCompoundDiagram ncd = (NotCompoundDiagram) d;
        // search for a child which is an "and"
        Vector children = ncd.getChildren();
        Diagram child = (Diagram) (children.get(0));
        //System.out.println("got a child of type "+child.getClass());
        return child instanceof AndCompoundDiagram;
    }

    public Vector getApplicationPossibilities(Diagram d) {
        Vector result = new Vector();
        if (!(d instanceof NotCompoundDiagram)) {
            return result;
        }
        NotCompoundDiagram ncd = (NotCompoundDiagram) d;
        // search for a child which is an "or"
        Vector children = ncd.getChildren();
        Diagram child = (Diagram) (children.get(0));
        if(!(child instanceof AndCompoundDiagram)){
            return result;
        }        
        AndCompoundDiagram acd = (AndCompoundDiagram) child;
        children = (Vector) ((acd.getChildren()).clone());
        int numChildren = children.size();
        Vector QCandidates = Utilities.powerSet(children);
        Object faveObj = children.get(0);
        Vector toRemove = new Vector();
        for (int j = 0; j < QCandidates.size(); j++) {
            Vector QCandidate = (Vector) (QCandidates.get(j));
            if (QCandidate.size()==numChildren || !QCandidate.contains(faveObj)) {
                toRemove.add(QCandidate);
            }
        }
        QCandidates.removeAll(toRemove);
        for (int k = 0; k < QCandidates.size(); k++) {
            Vector Qchildren = (Vector) QCandidates.get(k);
            Vector spec = new Vector();
            spec.add(acd);
            spec.addAll(Qchildren);   
            //System.out.println("children.containsAll(Qchildren):"+children.containsAll(Qchildren));
            result.add(new RuleAppliesToDiagramAndDiagrams(d, spec));
        }
        return result;
    }
    // takes P and (Q or R) and ...
    // becomes ((P and Q) or (P and R)) and ...
    public Diagram applyRule(RuleApplicationSpec r) {
        RuleAppliesToDiagramAndDiagrams radds = (RuleAppliesToDiagramAndDiagrams) r;
        NotCompoundDiagram ncd = (NotCompoundDiagram) radds.d;
        //System.out.println("radds.others is "+radds.others);
        AndCompoundDiagram QandR = (AndCompoundDiagram)(radds.others.get(0));
        Vector QChildren = (Vector)(radds.others.clone());
        //System.out.println("remove "+QandR+" : "+QandR.hashCode());
        QChildren.remove(QandR);
        Vector RChildren = (Vector)(QandR.getChildren().clone());
        RChildren.removeAll(QChildren);
        
        Vector justQ = new Vector();
        if(QChildren.size()>1){
            justQ.add(new AndCompoundDiagram(QChildren,"Q"));
        } else {
            justQ=QChildren;
        }
        NotCompoundDiagram notQ =
            new NotCompoundDiagram(justQ, "not Q");
        Vector justR = new Vector();
        if(RChildren.size()>1){
            justR.add(new AndCompoundDiagram(RChildren,"R"));
        } else {
            justR=RChildren;
        }
        NotCompoundDiagram notR =
            new NotCompoundDiagram(justR, "not R");
        Vector notdiags = new Vector();
        notdiags.add(notQ);
        notdiags.add(notR);
        return new OrCompoundDiagram(
            notdiags,
            "or(" + notQ.getName() + "," + notR.getName() + ")");
    }
}
