package rwd.rules.choicespec;
import java.util.Vector;

import rwd.abstractsyntax.spiderdiagram.CompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.Diagram;
import rwd.abstractsyntax.spiderdiagram.AndCompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.OrCompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.Utilities;
import rwd.rules.Rule;
import rwd.rules.application.RuleAppliesToDiagramAndDiagrams;
import rwd.rules.application.RuleApplicationSpec;

public class RuleDistributeAndOverOr
    extends RuleWithChoice { // takes P and (Q or R) and ...
    // becomes ((P and Q) or (P and R)) and ...
    // specify P, (Q or R) and Qchildren
    private RuleDistributeAndOverOr() {
        super("sd: Pand(QorR)and... -> ((PandQ)or(PandR))and...");
    }
    private static Rule example = null;

    public static Rule getRule() {
        if (example == null) {
            example = new RuleDistributeAndOverOr();
        }
        return example;
    }
    public boolean isInvertible() {
        return true;
    }

    public boolean canApplyTo(Diagram d) {
        if (!(d instanceof AndCompoundDiagram)) {
            return false;
        }
        AndCompoundDiagram acd = (AndCompoundDiagram) d;
        //System.out.println("search for a child which is an \"or\"");
        Vector children = acd.getChildren();
        childLoop : for (int i = 0; i < children.size(); i++) {
            Diagram child = (Diagram) (children.get(i));
            if (!(child instanceof OrCompoundDiagram)) {
                //System.out.println("isn't an OrCompoundDiagram");
                continue childLoop;
            }
            OrCompoundDiagram orChild = (OrCompoundDiagram) child;
            // we have to choose P, one of the other children
            Vector PCandidates = (Vector) (children.clone());
            PCandidates.remove(orChild);
            // now choose Q, a subset of the or-children
            Vector orChildren = orChild.getChildren();
            int numOrChildren = orChildren.size();
            if (numOrChildren == 0) {
                continue childLoop;
            }
            Vector QCandidates = Utilities.powerSet(orChildren);
            Diagram qpart = (Diagram) orChildren.get(0);
            Vector toRemove = new Vector();
            for (int j = 0; j < QCandidates.size(); j++) {
                Vector QCandidate = (Vector) (QCandidates.get(j));
                if (QCandidate.size() == 0
                    || QCandidate.size() == numOrChildren
                    || !QCandidate.contains(qpart)) {
                    toRemove.add(QCandidate);
                }
            }
            QCandidates.removeAll(toRemove);
            //System.out.println("Q-candidates are "+PCandidates);
            for (int j = 0; j < PCandidates.size(); j++) {
                Diagram P = (Diagram) PCandidates.get(j);
                for (int k = 0; k < QCandidates.size(); k++) {
                    Vector Qchildren = (Vector) QCandidates.get(k);
                    return true;
                }
            }
        }
        return false;
    }

    public Vector getApplicationPossibilities(Diagram d) {
        Vector result = new Vector();
        if (!(d instanceof AndCompoundDiagram)) {
            return result;
        }
        AndCompoundDiagram acd = (AndCompoundDiagram) d;
        //System.out.println("search for a child which is an \"or\"");
        Vector children = acd.getChildren();
        childLoop : for (int i = 0; i < children.size(); i++) {
            Diagram child = (Diagram) (children.get(i));
            if (!(child instanceof OrCompoundDiagram)) {
                //System.out.println("isn't an OrCompoundDiagram");
                continue childLoop;
            }
            OrCompoundDiagram orChild = (OrCompoundDiagram) child;
            // we have to choose P, one of the other children
            Vector PCandidates = (Vector) (children.clone());
            PCandidates.remove(orChild);
            // now choose Q, a subset of the or-children
            Vector orChildren = orChild.getChildren();
            int numOrChildren = orChildren.size();
            if (numOrChildren == 0) {
                continue childLoop;
            }
            Vector QCandidates = Utilities.powerSet(orChildren);
            Diagram qpart = (Diagram) orChildren.get(0);
            Vector toRemove = new Vector();
            for (int j = 0; j < QCandidates.size(); j++) {
                Vector QCandidate = (Vector) (QCandidates.get(j));
                if (QCandidate.size() == 0
                    || QCandidate.size() == numOrChildren
                    || !QCandidate.contains(qpart)) {
                    toRemove.add(QCandidate);
                }
            }
            QCandidates.removeAll(toRemove);
            //System.out.println("Q-candidates are "+PCandidates);
            for (int j = 0; j < PCandidates.size(); j++) {
                Diagram P = (Diagram) PCandidates.get(j);
                for (int k = 0; k < QCandidates.size(); k++) {
                    Vector Qchildren = (Vector) QCandidates.get(k);
                    Vector spec = new Vector();
                    spec.add(P);
                    spec.add(orChild);
                    spec.addAll(Qchildren);
                    //System.out.println("spec has "+spec.size()+" children");
                    result.add(new RuleAppliesToDiagramAndDiagrams(d, spec));
                }
            }
        }
        return result;
    }
    public RuleApplicationSpec findApplicableSpec(Diagram d) {
        
        if (!(d instanceof AndCompoundDiagram)) {
            return null;
        }
        AndCompoundDiagram acd = (AndCompoundDiagram) d;
        //System.out.println("search for a child which is an \"or\"");
        Vector children = acd.getChildren();
        childLoop : for (int i = 0; i < children.size(); i++) {
            Diagram child = (Diagram) (children.get(i));
            if (!(child instanceof OrCompoundDiagram)) {
                //System.out.println("isn't an OrCompoundDiagram");
                continue childLoop;
            }
            OrCompoundDiagram orChild = (OrCompoundDiagram) child;
            // we have to choose P, one of the other children
            Vector PCandidates = (Vector) (children.clone());
            PCandidates.remove(orChild);
            // now choose Q, a subset of the or-children
            Vector orChildren = orChild.getChildren();
            int numOrChildren = orChildren.size();
            if (numOrChildren == 0) {
                continue childLoop;
            }
            Vector QCandidates = Utilities.powerSet(orChildren);
            Diagram qpart = (Diagram) orChildren.get(0);
            Vector toRemove = new Vector();
            for (int j = 0; j < QCandidates.size(); j++) {
                Vector QCandidate = (Vector) (QCandidates.get(j));
                if (QCandidate.size() == 0
                    || QCandidate.size() == numOrChildren
                    || !QCandidate.contains(qpart)) {
                    toRemove.add(QCandidate);
                }
            }
            QCandidates.removeAll(toRemove);
            //System.out.println("Q-candidates are "+PCandidates);
            for (int j = 0; j < PCandidates.size(); j++) {
                Diagram P = (Diagram) PCandidates.get(j);
                for (int k = 0; k < QCandidates.size(); k++) {
                    Vector Qchildren = (Vector) QCandidates.get(k);
                    Vector spec = new Vector();
                    spec.add(P);
                    spec.add(orChild);
                    spec.addAll(Qchildren);
                    //System.out.println("spec has "+spec.size()+" children");
                    return new RuleAppliesToDiagramAndDiagrams(d, spec);
                }
            }
        }
        return null;
    }

    // takes P and (Q or R) and ...
    // becomes ((P and Q) or (P and R)) and ...
    public Diagram applyRule(RuleApplicationSpec r) {
        RuleAppliesToDiagramAndDiagrams radd =
            (RuleAppliesToDiagramAndDiagrams) r;
        AndCompoundDiagram acd = (AndCompoundDiagram) radd.d;
        Vector spec = (Vector) radd.others.clone();
        Diagram P = (Diagram) spec.get(0);
        //P is one of the toplevel And-Children
        CompoundDiagram orChild = (CompoundDiagram) spec.get(1);
        // orChild is (QorR)
        spec.remove(P);
        spec.remove(orChild);
        // now spec is just Q-children.
        Vector rchildren = (Vector) orChild.getChildren().clone();
        rchildren.removeAll(spec);
        //now rchildren is just R-children.

        Vector PandQAL = new Vector();
        PandQAL.add(P);
        // used to say PandQAL.addAll(spec_
        PandQAL.add(new OrCompoundDiagram(spec,"old"));
        Diagram PandQ = new AndCompoundDiagram(PandQAL, "PandQ");
        Vector PandRAL = new Vector();
        PandRAL.add(P);
        PandRAL.add(new OrCompoundDiagram(rchildren, "RC"));
        Diagram PandR = new AndCompoundDiagram(PandRAL, "PandR");
        Vector a = new Vector();
        a.add(PandQ);
        a.add(PandR);
        Diagram PandQ_or_PandR = new OrCompoundDiagram(a, "PandQ_or_PandR");
        Vector originalChildren = (Vector) (acd.getChildren().clone());
        originalChildren.remove(P);
        originalChildren.remove(orChild);
        originalChildren.add(PandQ_or_PandR);
        if (originalChildren.size() > 1) {
            return new AndCompoundDiagram(originalChildren, "outcome");
        } else {
            return (Diagram) originalChildren.get(0);
        }
    }
}
