package rwd.rules.choicespec;

import java.util.Vector;
import rwd.abstractsyntax.spiderdiagram.Diagram;
import rwd.abstractsyntax.spiderdiagram.AndCompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.OrCompoundDiagram;
import rwd.rules.Rule;
import rwd.rules.application.RuleAppliesToDiDiAL;
import rwd.rules.application.RuleApplicationSpec;

public class RuleDistributeMergeOrs
	extends RuleWithChoice { // takes (P or Q) and (P or R) and ...
	// becomes (P or (Q and R)) and  ...
	// specify (P or Q) and (P or R) and P
	private RuleDistributeMergeOrs() {
		super("sd: (PorQ)and(PorR)or... -> (Por(QandR))and...");
	}
	private static Rule example = null;

	public static Rule getRule() {
		if (example == null) {
			example = new RuleDistributeMergeOrs();
		}
		return example;
	}
    public boolean isInvertible(){
        return true;
    }
    
	public boolean canApplyTo(Diagram d) {
		if (!(d instanceof AndCompoundDiagram)) {
			return false;
		}
		AndCompoundDiagram acd = (AndCompoundDiagram) d;
		// search for a child which is an "or"
		Vector children = acd.getChildren();
		child1Loop : for (int i = 0; i < children.size(); i++) {
			Diagram child1 = (Diagram) (children.get(i));
			if (!(child1 instanceof OrCompoundDiagram)) {
				continue child1Loop;
			}
			OrCompoundDiagram orChild1 = (OrCompoundDiagram) child1;
			Vector grandchildren1 = orChild1.getChildren();
			child2Loop : for (int j = i + 1; j < children.size(); j++) {
				Diagram child2 = (Diagram) (children.get(j));
				if (!(child2 instanceof OrCompoundDiagram)) {
					continue child2Loop;
				}
				OrCompoundDiagram orChild2 = (OrCompoundDiagram) child2;
				Vector grandchildren2 = orChild2.getChildren();
				// see whether grandchildren1 and grandchildren2 have any common children
				Vector childrenInCommon = new Vector();
				boolean hasMatched[] = new boolean[grandchildren2.size()];
				for (int k = 0; k < hasMatched.length; k++) {
					hasMatched[k] = false;
				}
				for (int k = 0; k < grandchildren1.size(); k++) {
					Diagram gc1 = (Diagram) grandchildren1.get(k);
					findMatch : for (
						int l = 0; l < grandchildren2.size(); l++) {
						if (!hasMatched[l]) {
							Diagram gc2 = (Diagram) grandchildren2.get(l);
							if (gc1.equals(gc2)) {
								childrenInCommon.add(gc1);
								hasMatched[l] = true;
								break findMatch;
							}
						}
					}
				}
				// know commonChildren - find subsets of it
				Vector commonParts =
					rwd.abstractsyntax.spiderdiagram.Utilities.powerSet(
						childrenInCommon);
				for (int k = 0; k < commonParts.size(); k++) {
					Vector commonPart = (Vector) (commonParts.get(k));
					if (commonPart.size() == 0
						|| commonPart.size() == grandchildren1.size()
						|| commonPart.size() == grandchildren2.size()) {
						commonParts.remove(commonPart);
					}
				}
				for (int k = 0; k < commonParts.size(); k++) {
					Vector commonPart = (Vector) (commonParts.get(k));
					Vector spec = new Vector();
					spec.add(orChild1);
					spec.add(orChild2);
					spec.add(commonPart);
					return true;
				}
			}
		}
		return false;
	}

	public Vector getApplicationPossibilities(Diagram d) {
		Vector result = new Vector();
		if (!(d instanceof AndCompoundDiagram)) {
			return result;
		}
		AndCompoundDiagram acd = (AndCompoundDiagram) d;
		// search for a child which is an "or"
		Vector children = acd.getChildren();
		child1Loop : for (int i = 0; i < children.size(); i++) {
			Diagram child1 = (Diagram) (children.get(i));
			if (!(child1 instanceof OrCompoundDiagram)) {
				continue child1Loop;
			}
			OrCompoundDiagram orChild1 = (OrCompoundDiagram) child1;
			Vector grandchildren1 = orChild1.getChildren();
			child2Loop : for (int j = i + 1; j < children.size(); j++) {
				Diagram child2 = (Diagram) (children.get(j));
				if (!(child2 instanceof OrCompoundDiagram)) {
					continue child2Loop;
				}
				OrCompoundDiagram orChild2 = (OrCompoundDiagram) child2;
				Vector grandchildren2 = orChild2.getChildren();
				// see whether grandchildren1 and grandchildren2 have any common children
				Vector childrenInCommon = new Vector();
				boolean hasMatched[] = new boolean[grandchildren2.size()];
				for (int k = 0; k < hasMatched.length; k++) {
					hasMatched[k] = false;
				}
				for (int k = 0; k < grandchildren1.size(); k++) {
					Diagram gc1 = (Diagram) grandchildren1.get(k);
					findMatch : for (
						int l = 0; l < grandchildren2.size(); l++) {
						if (!hasMatched[l]) {
							Diagram gc2 = (Diagram) grandchildren2.get(l);
							if (gc1.equals(gc2)) {
								childrenInCommon.add(gc1);
								hasMatched[l] = true;
								break findMatch;
							}
						}
					}
				}
				// know commonChildren - find subsets of it
				Vector commonParts =
					rwd.abstractsyntax.spiderdiagram.Utilities.powerSet(
						childrenInCommon);
				for (int k = 0; k < commonParts.size(); k++) {
					Vector commonPart = (Vector) (commonParts.get(k));
					if (commonPart.size() == 0
						|| commonPart.size() == grandchildren1.size()
						|| commonPart.size() == grandchildren2.size()) {
						commonParts.remove(commonPart);
					}
				}
				for (int k = 0; k < commonParts.size(); k++) {
					Vector commonPart = (Vector) (commonParts.get(k));
					result.add(
						new RuleAppliesToDiDiAL(
							d,
							orChild1,
							orChild2,
							commonPart));
				}
			}
		}
		return result;
	}

	// takes (P or Q) and (P or R) and ...
	// becomes (P or (Q and R)) and  ...
	// specify (P or Q) and (P or R) and P
	public Diagram applyRule(RuleApplicationSpec r) {
		RuleAppliesToDiDiAL radda =
			(RuleAppliesToDiDiAL) r;
		AndCompoundDiagram acd = (AndCompoundDiagram) radda.d;
		Vector commonPart = (Vector) radda.others;
		OrCompoundDiagram orChild1 = (OrCompoundDiagram) radda.d1;
		OrCompoundDiagram orChild2 = (OrCompoundDiagram) radda.d2;
		// we need to build or( and( andchild1 less commonPart),and(andChild2 less commonpart))
		// then "and" it with or-ed of commonpart
		// then "or" that with whatever's left of original children
		Vector gChildren1 = (Vector) (orChild1.getChildren().clone());
		Vector childrenToRemove = new Vector();
		for (int j = 0; j < commonPart.size(); j++) {
			Diagram d2 = (Diagram) (commonPart.get(j));
			matchSearch : for (int i = 0; i < gChildren1.size(); i++) {
				Diagram d = (Diagram) gChildren1.get(i);
				if (d.equals(d2)) {
					childrenToRemove.add(d);
					break matchSearch;
				}
			}
		}

		gChildren1.removeAll(childrenToRemove);
		Vector gChildren2 = (Vector) (orChild2.getChildren().clone());
		childrenToRemove = new Vector();
		for (int j = 0; j < commonPart.size(); j++) {
			Diagram d2 = (Diagram) (commonPart.get(j));
			matchSearch : for (int i = 0; i < gChildren2.size(); i++) {
				Diagram d = (Diagram) gChildren2.get(i);
				if (d.equals(d2)) {
					childrenToRemove.add(d);
					break matchSearch;
				}
			}
		}
		gChildren2.removeAll(childrenToRemove);
		//System.out.println("after removing common part, grandchildren1 are "+gChildren1);
		//System.out.println("after removing common part, grandchildren2 are "+gChildren2);
		// we need to build and( or( orchild1 less commonPart),or(orChild2 less commonpart))
		Diagram ocd1;
		if (gChildren1.size() > 1) {
			ocd1 = new OrCompoundDiagram(gChildren1, "Q");
		} else {
			ocd1 = (Diagram) gChildren1.get(0);
		}
		Diagram ocd2;
		if (gChildren2.size() > 1) {
			ocd2 = new OrCompoundDiagram(gChildren2, "R");
		} else {
			ocd2 = (Diagram) gChildren2.get(0);
		}
		Vector temp = new Vector();
		temp.add(ocd1);
		temp.add(ocd2);
		//System.out.println("ocd1 is "+ocd1);
		//System.out.println("ocd2 is "+ocd2);
		AndCompoundDiagram acd2 = new AndCompoundDiagram(temp, "QandR");
		// then "or" it with and-ed of commonpart
		Diagram rest;
		if (commonPart.size() > 1) {
			rest = new OrCompoundDiagram(commonPart, "P");// changed late at night
		} else {
			rest = (Diagram) commonPart.get(0);
		}
		temp = new Vector();
		temp.add(acd2);
		temp.add(rest);
		OrCompoundDiagram QandR_or_P =
			new OrCompoundDiagram(temp, "QandR or P");
		// then "and" that with whatever's left of original children
		Vector originalChildren = (Vector) (acd.getChildren().clone());
		originalChildren.remove(orChild1);
		originalChildren.remove(orChild2);
		originalChildren.add(QandR_or_P);
		if (originalChildren.size() == 1) {
			return QandR_or_P;
		} else {
			return new AndCompoundDiagram(originalChildren, "outcome");
		}
	}
}
