package rwd.rules.choicespec;
import java.util.Vector;

import rwd.abstractsyntax.eulerdiagram.Zone;
import rwd.abstractsyntax.spiderdiagram.AndCompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.Diagram;
import rwd.abstractsyntax.spiderdiagram.UnitaryDiagram;
import rwd.rules.Rule;
import rwd.rules.application.RuleAppliesToDiagramAndDiagrams;
import rwd.rules.application.RuleApplicationSpec;

public class TrueWithAndRule extends RuleWithChoice {
	private TrueWithAndRule() {
		super("PorT->T");
	}
	private static Rule example = null;
	public static Rule getRule() {
		if (example == null) {
			example = new TrueWithAndRule();
		}
		return example;
	}
    public boolean isInvertible(){
        return true;
    }

	public boolean canApplyTo(Diagram d) {
		if (d instanceof AndCompoundDiagram) {
			AndCompoundDiagram acd = (AndCompoundDiagram) d;
			Vector children = (Vector) (acd.getChildren().clone());
			for (int i = 0; i < children.size(); i++) {
				Diagram d2 = (Diagram) (children.get(i));
                if(isTrue(d2)){ 
                    return true;
                }
			}
		}
		return false;
	}

	public Vector getApplicationPossibilities(Diagram d) { //System.out.println("getting app possiobilities for remove false");
		//System.out.println("from diagram "+d.toSaveString());
		Vector result = new Vector();
		if (d instanceof AndCompoundDiagram) {
			AndCompoundDiagram acd = (AndCompoundDiagram) d;
			Vector children = (Vector) (acd.getChildren().clone());
			int numTrues = 0;
			for (int i = 0; i < children.size(); i++) {
				Diagram d2 = (Diagram) (children.get(i));
				if (isTrue(d2)) {
					numTrues++;
				}
			}
			if (numTrues == 0) {
				return result;
			}
			for (int i = 0; i < children.size(); i++) {
				Diagram d2 = (Diagram) (children.get(i));
				if (!(isTrue(d2) && numTrues == 1)) {
					Vector childrenCopy = (Vector) children.clone();
					childrenCopy.remove(d2);
					//System.out.println("3. providing d,[d2,d3,...]");
					result.add(
						new RuleAppliesToDiagramAndDiagrams(acd, childrenCopy));
				}
			}
		}
		//System.out.println("result is "+result);
		return result;
	}

	public Diagram applyRule(RuleApplicationSpec r) {
        Vector justOutside = new Vector();
        justOutside.add(Zone.getZone(new Vector(),new Vector()));                    
		return new UnitaryDiagram(justOutside,"t");
	}
    
    private boolean isTrue(Diagram d){
        if (d instanceof UnitaryDiagram) {
            UnitaryDiagram ud = (UnitaryDiagram)d;                    
            return ud.getNumZones()==1
            && ud.getNumShadedZones()==0
            && ud.getSpiders().size()==0;
        } else {
            return false;
        }        
    }
}
