package rwd.rules.constraintdiagrams;

import java.util.List;
import java.util.Vector;

import rwd.abstractsyntax.spiderdiagram.Diagram;

import rwd.ocl.constraintdiagram.ReadingTree;
import rwd.ocl.constraintdiagram.ReadingTreeNode;
import rwd.rules.CDRule;
import rwd.rules.Rule;
import rwd.rules.application.RuleApplicationSpec;

import rwd.concretesyntax.*;
import uk.ac.kent.cs.kmf.util.ConsoleLog;

public class RuleMoveBranch 
	extends CDRule 
{
	private RuleMoveBranch() {
		super("cd: Move Branch");
	}
	private static Rule example = null;

	public static Rule getRule() {
		if (example == null) {
			example = new RuleMoveBranch();
		}
		return example;
	}

	public boolean needsExtraArgs() {
		return true;
	}
	
	public boolean canApplyTo(List args, String extraArgs) {
		// Check if there is a diagram
		ConcreteUnitaryDiagram cud = getCUD(args);
		if (cud == null) return false;
		// Check extra arguments
		String xs[] = extraArgs.split(",");
		if (xs.length == 1) {
			String readingForest = cud.getReadingForest();
			String s1 = xs[0].trim();
			ReadingTree tree = new ReadingTree(readingForest, new ConsoleLog());
			ReadingTreeNode node1 = tree.find(s1);
			if (node1 == null) return false;
			return true;
		} else if (xs.length == 2) {
			String readingForest = cud.getReadingForest();
			String s1 = xs[0].trim();
			String s2 = xs[1].trim();
			ReadingTree tree = new ReadingTree(readingForest, new ConsoleLog());
			ReadingTreeNode node1 = tree.find(s1);
			ReadingTreeNode node2 = tree.find(s2);
			if (node1 == null || node2 == null) return false;
			return true;
		} else {
			return false;
		}
	}

	// Move branch labeled with S1 to be a son of the node labeled S2Delete spider s if none of the following conditions hold
	// 1. s is the source or target of an arrow
	// 2. the habitat of s includes a shaded zones
	// 3. there is a universal spider x whose domain includes a foot of the spider s 
	// and the node s is ordered before the node x in the tree.
	public ConcreteUnitaryDiagram applyRule(List args, String extraArgs) {
		// Get cud
		ConcreteUnitaryDiagram cud = getCUD(args);
		// New tree
		List forest = null;
		// Check extra arguments
		String xs[] = extraArgs.split(",");
		// Move branch to top level
		if (xs.length == 1) {
			String readingForest = cud.getReadingForest();
			String s1 = xs[0].trim();
			ReadingTree tree = new ReadingTree(readingForest, new ConsoleLog());
			ReadingTreeNode node1 = tree.find(s1);
			// Remove node1 from his father's list of sons
			ReadingTree.removeSonFromFather(forest, node1);
			// Add branch to top
			forest.add(node1);
		}
		// Move branch s1 as son of s2
		if (xs.length == 2) {
			String readingForest = cud.getReadingForest();
			String s1 = xs[0].trim();
			String s2 = xs[1].trim();
			ReadingTree tree = new ReadingTree(readingForest, new ConsoleLog());
			ReadingTreeNode node1 = tree.find(s1);
			ReadingTreeNode node2 = tree.find(s2);
			// Remove s1 from his father
			ReadingTree.removeSonFromFather(forest, node1);
			// Add s1 as son of s2
			node2.addSon(node1);
			node1.setFather(node2);
		}
		// Make reading tree
		String newTree = ReadingTree.toString(forest);
		// Set new reading forest
		cud.getDecorationBox().setReadingForest(newTree);
		return cud;
	}
	
	// Inherited from Rule
	public Diagram applyRule(RuleApplicationSpec r) {
		return null;
	}

	public boolean canApplyTo(Diagram ad) {
		return false;
	}

	public RuleApplicationSpec findApplicableSpec(Diagram part) {
		return null;
	}

	public boolean isInvertible() {
		return false;
	}

	public Vector getApplicationPossibilities(Diagram d) {
		return null;
	}
}
