/*
 * Created on 22-Nov-2003
 *
 */
package rwd.rules.heuristics;

import java.util.Vector;
import java.util.HashMap;

import rwd.abstractsyntax.eulerdiagram.Contour;
import rwd.abstractsyntax.eulerdiagram.Zone;
import rwd.abstractsyntax.spiderdiagram.AndCompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.CompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.Diagram;
import rwd.abstractsyntax.spiderdiagram.DiagramFileUser;
import rwd.abstractsyntax.spiderdiagram.FalseDiagram;
import rwd.abstractsyntax.spiderdiagram.NotCompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.Spider;
import rwd.abstractsyntax.spiderdiagram.UnitaryDiagram;
import rwd.abstractsyntax.spiderdiagram.Utilities;
import rwd.rules.Proof;
import rwd.rules.ProofGetterRandom;
import rwd.rules.Rule;
import rwd.rules.application.RuleAppliesToZone;
import rwd.rules.application.RuleAppliesWithString;
import rwd.rules.choicespec.MergeConjunctionRule;
import rwd.rules.choicespec.MergeContradictionAvoidingFalseRule;
import rwd.rules.choicespec.RuleAddSpiderFoot;
import rwd.rules.choicespec.RuleAndAbsorbtion;
import rwd.rules.choicespec.RuleDMPullNotThroughAnd;
import rwd.rules.choicespec.RuleDMPullNotThroughOr;
import rwd.rules.choicespec.RuleDMPushNotThroughAnd;
import rwd.rules.choicespec.RuleDMPushNotThroughOr;
import rwd.rules.choicespec.RuleDeleteContourSimple;
import rwd.rules.choicespec.RuleDeleteShadedZone;
import rwd.rules.choicespec.RuleDeleteSpider;
import rwd.rules.choicespec.RuleDistributeAndOverOr;
import rwd.rules.choicespec.RuleDistributeOrOverAnd;
import rwd.rules.choicespec.RuleExcludedMiddle;
import rwd.rules.choicespec.RuleIntroduceShadedZone;
import rwd.rules.choicespec.RuleJoinSpiders;
import rwd.rules.choicespec.RuleOrAbsorbtion;
import rwd.rules.choicespec.RuleReverseAndIdempotent;
import rwd.rules.choicespec.RuleReverseExcludedMiddle;
import rwd.rules.choicespec.RuleReverseOrIdempotent;
import rwd.rules.choicespec.RuleSplitSpider;
import rwd.rules.choicespec.RuleUnshadeZone;
import rwd.rules.inputspec.RuleAddContour;
import rwd.rules.nospec.AndIdempotentRule;
import rwd.rules.nospec.InsertNotNotRule;
import rwd.rules.nospec.OrIdempotentRule;
import rwd.rules.nospec.RemoveNotNotRule;
import rwd.rules.nospec.RemoveNotRule;

/**
 * @author jaf28
 */
public class CompoundMeasureCalculator {
    private Diagram d;

    public CompoundMeasureCalculator(Diagram d) {
        this.d = d;
    }

    public static Diagram calcCF(Diagram cf, Diagram context) {
        //System.out.println("cf is "+DiagramFileUser.toSaveString(cf));
        Vector leaves = new Vector();
        leaves.addAll(cf.getUnitaryParts());
        leaves.addAll(context.getUnitaryParts());
        Vector allContours = new Vector();
        for (int i = 0; i < leaves.size(); i++) {
            Vector theseContours =
                ((UnitaryDiagram) leaves.get(i)).getCloneOfContourSet();
            for (int j = 0; j < theseContours.size(); j++) {
                Contour c = (Contour) theseContours.get(j);
                if (!allContours.contains(c)) {
                    allContours.add(c);
                }
            }
        }
        // turn to contour form
        contourFormLoop : while (true) {
            Vector leaves3 = cf.getUnitaryParts();
            for (int i = 0; i < leaves3.size(); i++) {
                UnitaryDiagram leaf = (UnitaryDiagram) leaves3.get(i);
                if (!(leaf instanceof FalseDiagram)) {
                    Vector contours = leaf.getCloneOfContourSet();
                    //System.out.println(
                    //    "leaf is " + DiagramFileUser.toSaveString(leaf));
                    //System.out.println(
                    //    "leaf has zone " + leaf.getCloneOfZoneSet());
                    //System.out.println(
                    //    "leaf has contours " + leaf.getCloneOfContourSet());
                    Vector otherCs = (Vector) allContours.clone();
                    //System.out.println("all cs are  " + otherCs);
                    otherCs.removeAll((Vector) contours.clone());
                    //System.out.println("can add  " + otherCs);
                    //System.out.println(
                    //    "to  " + DiagramFileUser.toSaveString(leaf));
                    //contours.removeAll((Vector) otherCs.clone());
                    if (otherCs.size() > 0) {
                        Contour toAdd = (Contour) otherCs.get(0);
                        cf =
                            RuleAddContour.getRule().applyRuleToPart(
                                cf,
                                new RuleAppliesWithString(
                                    leaf,
                                    toAdd.toString()));
                        continue contourFormLoop;
                    }
                }
            }
            break contourFormLoop;
        }
        //System.out.println("got CF");
        return cf;
    }

    public static Diagram calcVF(Diagram vf) {
        Vector allContours =
            ((UnitaryDiagram) vf.getUnitaryParts().get(0))
                .getCloneOfContourSet();
        //System.out.println(
        //    "a component is "
        //        + DiagramFileUser.toSaveString(
        //            (UnitaryDiagram) vf.getUnitaryParts().get(0)));
        //System.out.println("contour set is " + allContours);
        Vector allZones = UnitaryDiagram.allPossibleZones(allContours);
        //System.out.println("all poss zones are " + allZones);
        // turn to venn form
        vennFormLoop : while (true) {
            Vector leaves5 = vf.getUnitaryParts();
            for (int i = 0; i < leaves5.size(); i++) {
                UnitaryDiagram leaf = (UnitaryDiagram) leaves5.get(i);
                if (!(leaf instanceof FalseDiagram)) {
                    Vector zones = leaf.getCloneOfZoneSet();
                    //System.out.println(
                    //    "~~~~~leaf is " + DiagramFileUser.toSaveString(leaf));
                    //System.out.println(
                    //    "~~~~~leaf zones " + leaf.getCloneOfZoneSet());
                    Vector otherZones = (Vector) allZones.clone();
                    otherZones.removeAll(zones);
                    if (otherZones.size() > 0) {
                        Zone toAdd = (Zone) otherZones.get(0);
                        vf =
                            RuleIntroduceShadedZone.getRule().applyRuleToPart(
                                vf,
                                new RuleAppliesToZone(leaf, toAdd));
                        continue vennFormLoop;
                    }
                }
            }
            break vennFormLoop;
        }
        return vf;
    }

    public Vector[] calcMeasuresList(String type)
        throws ContainsFalseException {
        Vector list1 = null;
        Vector list2 = null;

        boolean isDebug = false;
        if (d instanceof FalseDiagram) {
            throw new ContainsFalseException();
        } else if (d instanceof UnitaryDiagram) {
            Vector list = null;
            if (type.equals("Contour")) {
                list = ((UnitaryDiagram) d).getCloneOfContourSet();
            } else if (type.equals("Zone")) {
                list = ((UnitaryDiagram) d).getCloneOfZoneSet();
            } else if (type.equals("Shading")) {
                list = ((UnitaryDiagram) d).getCloneOfShadedZones();
            } else if (type.equals("Spiders")) {
                list = ((UnitaryDiagram) d).getSpiders();
            }
            list1 = (Vector)list.clone();
            list2 = (Vector)list.clone();
        } else if (d instanceof NotCompoundDiagram) {
            CompoundMeasureCalculator cmc =
                new CompoundMeasureCalculator(
                    (Diagram) (((NotCompoundDiagram) d).getChildren().get(0)));
            Vector[] result = cmc.calcMeasuresList(type);
            list1 = result[1];
            list2 = result[0];

        } else if (d instanceof CompoundDiagram) {
            CompoundDiagram cd = (CompoundDiagram) d;
            Vector children = cd.getChildren();
            Vector list1s = new Vector();
            Vector list2s = new Vector();
            for (int i = 0; i < children.size(); i++) {
                CompoundMeasureCalculator cmc =
                    new CompoundMeasureCalculator((Diagram) (children.get(i)));
                Vector[] result = cmc.calcMeasuresList(type);
                list1s.add(result[0]);
                list2s.add(result[1]);
            }
            if (type.equals("Spiders")) {
                // work out the max number in a region
                // and min while I'm there
                list1 = new Vector();
                list2 = new Vector();
                Vector allZones =
                    ((UnitaryDiagram) (cd.getUnitaryParts().get(0)))
                        .getCloneOfZoneSet();
                Vector allRegions = Utilities.powerSet(allZones);
                for (int j = 0; j < allRegions.size(); j++) {
                    Vector region = (Vector) allRegions.get(j);
                    // count the max/min number of spiders in this region
                    int maxNum = 0;
                    int minNum = Integer.MAX_VALUE;
                    for (int i = 0; i < list1s.size(); i++) {
                        int numHere = 0;
                        Vector spiders = (Vector) list1s.get(i);
                        for (int k = 0; k < spiders.size(); k++) {
                            Spider s = (Spider) spiders.get(k);
                            Vector hab = s.getHabitat();
                            if (hab.containsAll(region)
                                && region.containsAll(hab)) {
                                numHere++;
                            }
                        }
                        if (numHere > maxNum) {
                            maxNum = numHere;
                        }
                        if (numHere < minNum) {
                            minNum = numHere;
                        }
                    }
                    if (d instanceof AndCompoundDiagram) {
                        for (int i = 0; i < maxNum; i++) {
                            list1.add(new Spider(region));
                            // max num of spiders in region
                        }
                    } else {
                        for (int i = 0; i < minNum; i++) {
                            list1.add(new Spider(region));
                            // min num of spiders in region
                        }
                    }
                    maxNum = 0;
                    minNum = Integer.MAX_VALUE;
                    for (int i = 0; i < list2s.size(); i++) {
                        int numHere = 0;
                        Vector spiders = (Vector) list2s.get(i);
                        for (int k = 0; k < spiders.size(); k++) {
                            Spider s = (Spider) spiders.get(k);
                            Vector hab = s.getHabitat();
                            if (hab.containsAll(region)
                                && region.containsAll(hab)) {
                                numHere++;
                            }
                        }
                        if (numHere > maxNum) {
                            maxNum = numHere;
                        }
                        if (numHere < minNum) {
                            minNum = numHere;
                        }
                    }
                    if (d instanceof AndCompoundDiagram) {
                        for (int i = 0; i < minNum; i++) {
                            list2.add(new Spider(region));
                            // min num of spiders in region
                        }
                    } else {
                        for (int i = 0; i < maxNum; i++) {
                            list2.add(new Spider(region));
                            // max num of spiders in region
                        }
                    }
                }
            } else {
                if (d instanceof AndCompoundDiagram) {
                    list1 = union(list1s);
                    list2 = intersection(list2s);
                } else {
                    list1 = intersection(list1s);
                    list2 = union(list2s);
                }
            }
        }
        return new Vector[] { list1, list2 };
    }

    private Vector union(Vector lists) {
        Vector result = new Vector();
        for (int j = 0; j < lists.size(); j++) {
            Vector set = (Vector) lists.get(j);
            for (int i = 0; i < set.size(); i++) {
                Object o = set.get(i);
                if (!result.contains(o)) {
                    result.add(o);
                }
            }
        }
        return result;
    }
    private Vector intersection(Vector lists) {
        Vector result = new Vector();
        Vector firstChildSet = (Vector) lists.get(0);
        for (int i = 0; i < firstChildSet.size(); i++) {
            Object o = firstChildSet.get(i);
            boolean isEverywhere = true;
            for (int j = 1; j < lists.size(); j++) {
                if (!((Vector) (lists.get(j))).contains(o)) {
                    isEverywhere = false;
                }
            }
            if (isEverywhere) {
                result.add(o);
            }
        }
        return result;
    }

    public int[] calcMeasuresNum(String type) throws ContainsFalseException {
        int n1 = 0;
        int n2 = 0;

        boolean isDebug = false;
        if (d instanceof FalseDiagram) {
            throw new ContainsFalseException();
        } else if (d instanceof UnitaryDiagram) {
            if (type.equals("AndOrNum")) {
                n1 = 0;
                n2 = 0;
            } else if (type.equals("NotNum")) {
                n1 = 0;
                n2 = 1;
            }
        } else if (d instanceof NotCompoundDiagram) {
            Diagram child =
                (Diagram) (((NotCompoundDiagram) d).getChildren().get(0));
            CompoundMeasureCalculator cmc =
                new CompoundMeasureCalculator(child);
            int[] result = cmc.calcMeasuresNum(type);
            if (type.equals("AndOrNum")) {
                n1 = result[1];
                n2 = result[0];
            } else if (type.equals("NotNum")) {
                if (child instanceof UnitaryDiagram) {
                    n1 = 1;
                    n2 = 0;
                } else {
                    n1 = 1 + result[1];
                    n2 = 1 + result[0];
                }
            }
        } else if (d instanceof CompoundDiagram) {
            CompoundDiagram cd = (CompoundDiagram) d;
            Vector children = (Vector) (cd.getChildren().clone());

            if (type.equals("AndOrNum")) {

                // strip away unitary children
                Vector unitaryChildren = new Vector();
                for (int i = 0; i < children.size(); i++) {
                    Diagram d = (Diagram) children.get(i);
                    if (isUnitaryOrNotNotNotUnitaryETC(d)) {
                        unitaryChildren.add(d);
                    }
                }
                //System.out.println("num children = "+children.size());
                children.removeAll(unitaryChildren);
                //System.out.println("num compound children = "+children.size());

                int[] n1Cs = new int[children.size()];
                int[] n2Cs = new int[children.size()];
                for (int i = 0; i < children.size(); i++) {
                    CompoundMeasureCalculator cmc =
                        new CompoundMeasureCalculator(
                            (Diagram) (children.get(i)));
                    int[] result = cmc.calcMeasuresNum(type);
                    n1Cs[i] = result[0];
                    n2Cs[i] = result[1];
                }

                if (d instanceof AndCompoundDiagram) {
                    // n1=And is max of compound parts and 1 for each stripped away unitary parts
                    if (children.size() == 0) {
                        //System.out.println("andNum(d1 and d2 and d3) = 2");
                        n1 = unitaryChildren.size() - 1;
                    } else {
                        //System.out.println("andNum(d1 and d2 and D...) = 2 + andNum(D and ...)");
                        //System.out.println("num compound children = "+n2Cs.length);
                        int max = 0;
                        for (int i = 0; i < n1Cs.length; i++) {
                            if (n1Cs[i] > max) {
                                max = n1Cs[i];
                            }
                        }
                        n1 = max + unitaryChildren.size();
                    }
                    // n2=Or is the minimum of all
                    if (unitaryChildren.size() > 0) {
                        //System.out.println("orNum(d1 and d2 and d3) = 0");
                        n2 = 0;
                    } else {
                        //System.out.println("orNum(d1 and d2 and D...) = orNum(D and ...)");
                        //System.out.println("num compound children = "+n2Cs.length);
                        int min = Integer.MAX_VALUE;
                        for (int i = 0; i < n2Cs.length; i++) {
                            if (n2Cs[i] < min) {
                                min = n2Cs[i];
                            }
                        }
                        n2 = min;
                    }
                } else {
                    // n2=And is max of compound parts and 1 for each stripped away unitary parts
                    if (children.size() == 0) {
                        //System.out.println("orNum(d1 or d2 or d3) = 2");
                        n2 = unitaryChildren.size() - 1;
                    } else {
                        //System.out.println("orNum(d1 or d2 or D...) = 2 + orNum(D and ...)");
                        //System.out.println("num compound children = "+n2Cs.length);

                        int max = 0;
                        for (int i = 0; i < n2Cs.length; i++) {
                            if (n2Cs[i] > max) {
                                max = n2Cs[i];
                            }
                        }
                        n2 = max + unitaryChildren.size();
                    }
                    // n1=Or is the minimum of all
                    if (unitaryChildren.size() > 0) {
                        //System.out.println("andNum(d1 or d2 or d3) = 0");
                        n1 = 0;
                    } else {
                        //System.out.println("andNum(d1 or d2 or D...) = orNum(D and ...)");
                        //System.out.println("num compound children = "+n2Cs.length);
                        int min = Integer.MAX_VALUE;
                        for (int i = 0; i < n1Cs.length; i++) {
                            if (n1Cs[i] < min) {
                                min = n1Cs[i];
                            }
                        }
                        n1 = min;
                    }
                }
            } else if (type.equals("NotNum")) {
                int[] n1Cs = new int[children.size()];
                int[] n2Cs = new int[children.size()];
                //System.out.println("children.size()==0:"+(children.size()==0));
                for (int i = 0; i < children.size(); i++) {
                    CompoundMeasureCalculator cmc =
                        new CompoundMeasureCalculator(
                            (Diagram) (children.get(i)));
                    int[] result = cmc.calcMeasuresNum(type);
                    n1Cs[i] = result[0];
                    n2Cs[i] = result[1];
                }
                if (d instanceof AndCompoundDiagram) {
                    // n1 is max of n1Cs
                    int max = 0;
                    for (int i = 0; i < n1Cs.length; i++) {
                        if (n1Cs[i] > max) {
                            max = n1Cs[i];
                        }
                    }
                    n1 = max;
                    // n2 is min of n2Cs
                    int min = Integer.MAX_VALUE;
                    for (int i = 0; i < n2Cs.length; i++) {
                        if (n2Cs[i] < min) {
                            min = n2Cs[i];
                        }
                    }
                    n2 = min;
                } else {
                    // n2 is max of n2Cs
                    int max = 0;
                    for (int i = 0; i < n2Cs.length; i++) {
                        if (n2Cs[i] > max) {
                            max = n2Cs[i];
                        }
                    }
                    n2 = max;
                    // n1 is min of n1Cs
                    int min = Integer.MAX_VALUE;
                    for (int i = 0; i < n1Cs.length; i++) {
                        if (n1Cs[i] < min) {
                            min = n1Cs[i];
                        }
                    }
                    n1 = min;
                }
            }
            //System.out.println("n1 is "+n1);
            //System.out.println("n2 is "+n2);
        }
        return new int[] { n1, n2 };
    }
    private static Vector c1Before, c1After, c2Before, c2After;
    private static Vector z1Before, z1After, z2Before, z2After;
    private static Vector s1Before, s1After, s2Before, s2After;
    private static Vector sp1Before, sp1After, sp2Before, sp2After;
    private static int andBefore, andAfter, orBefore, orAfter;

    private static Diagram start, end;
    private static boolean c1Unchanged() {
        return c1HasntGrown() && c1HasntShrunk();
    }
    private static boolean c2Unchanged() {
        return c2HasntGrown() && c2HasntShrunk();
    }
    private static boolean c1HasntGrown() {
        return c1Before.containsAll(c1After);
    }
    private static boolean c2HasntGrown() {
        return c2Before.containsAll(c2After);
    }
    private static boolean c1HasntShrunk() {
        return c1After.containsAll(c1Before);
    }
    private static boolean c2HasntShrunk() {
        return c2After.containsAll(c2Before);
    }
    private static boolean c1HasntChangedByMoreThanOne() {
        return Math.abs(c1After.size() - c1Before.size()) <= 1;
    }
    private static boolean c2HasntChangedByMoreThanOne() {
        return Math.abs(c2After.size() - c2Before.size()) <= 1;
    }
    private static boolean z1Unchanged() {
        return z1HasntGrown() && z1HasntShrunk();
    }
    private static boolean z2Unchanged() {
        return z2HasntGrown() && z2HasntShrunk();
    }
    private static boolean z1HasntGrown() {
        return z1Before.containsAll(z1After);
    }
    private static boolean z2HasntGrown() {
        return z2Before.containsAll(z2After);
    }
    private static boolean z1HasntShrunk() {
        return z1After.containsAll(z1Before);
    }
    private static boolean z2HasntShrunk() {
        return z2After.containsAll(z2Before);
    }
    private static boolean z1HasntChangedByMoreThanOne() {
        return Math.abs(z1After.size() - z1Before.size()) <= 1;
    }
    private static boolean z2HasntChangedByMoreThanOne() {
        return Math.abs(z2After.size() - z2Before.size()) <= 1;
    }
    private static boolean s1Unchanged() {
        return s1HasntGrown() && s1HasntShrunk();
    }
    private static boolean s2Unchanged() {
        return s2HasntGrown() && s2HasntShrunk();
    }
    private static boolean s1HasntGrown() {
        return s1Before.containsAll(s1After);
    }
    private static boolean s2HasntGrown() {
        return s2Before.containsAll(s2After);
    }
    private static boolean s1HasntShrunk() {
        return s1After.containsAll(s1Before);
    }
    private static boolean s2HasntShrunk() {
        return s2After.containsAll(s2Before);
    }
    private static boolean s1HasntChangedByMoreThanOne() {
        return Math.abs(s1After.size() - s1Before.size()) <= 1;
    }
    private static boolean s2HasntChangedByMoreThanOne() {
        return Math.abs(s2After.size() - s2Before.size()) <= 1;
    }
    private static boolean sp1Unchanged() {
        return sp1HasntGrown() && sp1HasntShrunk();
    }
    private static boolean sp2Unchanged() {
        return sp2HasntGrown() && sp2HasntShrunk();
    }
    private static boolean sp1HasntGrown() {
        //return sp1Before.containsAll(sp1After); // doesn't work very well!!
        return containsAllSpiders(sp1Before, sp1After);
    }
    private static boolean containsAllSpiders(Vector y, Vector x) {
        x = (Vector) x.clone();
        y = (Vector) y.clone();
        // pick out an element of x and see if it's in y
        xLoop : while (x.size() > 0) {
            Spider s = (Spider) x.get(0);
            //System.out.println("seek match");
            x.remove(s);
            for (int j = 0; j < y.size(); j++) {
                Spider s2 = (Spider) y.get(j);
                if (s.getHabitat().containsAll(s2.getHabitat())
                    && s2.getHabitat().containsAll(s.getHabitat())) {
                    y.remove(s2);
                    //System.out.println("found match");
                    continue xLoop;
                }
            }
            //System.out.println("not found match");
            return false;
        }
        return true;
    }
    private static boolean sp2HasntGrown() {
        //return sp2Before.containsAll(sp2After);
        return containsAllSpiders(sp2Before, sp2After);
    }
    private static boolean sp1HasntShrunk() {
        //return sp1After.containsAll(sp1Before);
        return containsAllSpiders(sp1After, sp1Before);
    }
    private static boolean sp2HasntShrunk() {
        //return sp2After.containsAll(sp2Before);
        return containsAllSpiders(sp2After, sp2Before);
    }

    private static boolean sp1HasntChangedByMoreThanOne() {
        return Math.abs(sp1After.size() - sp1Before.size()) <= 1;
    }
    private static boolean sp2HasntChangedByMoreThanOne() {
        return Math.abs(sp2After.size() - sp2Before.size()) <= 1;
    }
    private static boolean andUnchanged() {
        return andHasntGrown() && andHasntShrunk();
    }
    private static boolean orUnchanged() {
        return orHasntGrown() && orHasntShrunk();
    }
    private static boolean andHasntGrown() {
        return andBefore >= andAfter;
    }
    private static boolean orHasntGrown() {
        return orBefore >= orAfter;
    }
    private static boolean andHasntShrunk() {
        return andAfter >= andBefore;
    }
    private static boolean orHasntShrunk() {
        return orAfter >= orBefore;
    }
    private static boolean andHasntChangedByMoreThanOne() {
        return Math.abs(andAfter - andBefore) <= 1;
    }
    private static boolean orHasntChangedByMoreThanOne() {
        return Math.abs(orAfter - orBefore) <= 1;
    }

    private static void showCreport(String message) {
        System.out.println(message);
        System.out.println(
            "start diagram is " + DiagramFileUser.toSaveString(start));
        System.out.println(
            "result diagram is " + DiagramFileUser.toSaveString(end));
        System.out.println("c1Before " + c1Before);
        System.out.println("c1After " + c1After);
        System.out.println("c2Before " + c2Before);
        System.out.println("c2After " + c2After);
    }

    private static void showZreport(String message) {
        System.out.println(message);
        System.out.println(
            "start diagram is " + DiagramFileUser.toSaveString(start));
        System.out.println(
            "result diagram is " + DiagramFileUser.toSaveString(end));
        System.out.println("z1Before " + z1Before);
        System.out.println("z1After " + z1After);
        System.out.println("z2Before " + z2Before);
        System.out.println("z2After " + z2After);
    }

    private static void showSreport(String message) {
        System.out.println(message);
        System.out.println(
            "start diagram is " + DiagramFileUser.toSaveString(start));
        System.out.println(
            "result diagram is " + DiagramFileUser.toSaveString(end));
        System.out.println("s1Before " + s1Before);
        System.out.println("s1After " + s1After);
        System.out.println("s2Before " + s2Before);
        System.out.println("s2After " + s2After);
    }
    private static void showSPreport(String message) {
        System.out.println(message);
        System.out.println(
            "start diagram is " + DiagramFileUser.toSaveString(start));
        System.out.println(
            "result diagram is " + DiagramFileUser.toSaveString(end));
        System.out.println("sp1Before " + sp1Before);
        System.out.println("sp1After " + sp1After);
        System.out.println("sp2Before " + sp2Before);
        System.out.println("sp2After " + sp2After);
    }
    private static void showAOreport(String message) {
        System.out.println(message);
        //System.out.println(
        //    "start diagram is " + DiagramFileUser.toSaveString(start));
        //System.out.println(
        //    "result diagram is " + DiagramFileUser.toSaveString(end));
        System.out.println("andBefore " + andBefore);
        System.out.println("andAfter " + andAfter);
        System.out.println("orBefore " + orBefore);
        System.out.println("orAfter " + orAfter);
    }

    public static void main(String args[]) {
        // build suitable rule set
        HashMap resultsHM = new HashMap();
        Vector HRules = HeuristicProofWriter.compoundHRules;
        Rule[] rules = new Rule[HRules.size()];
        for (int i = 0; i < HRules.size(); i++) {
            rules[i] = (Rule) HRules.get(i);
            resultsHM.put(rules[i].name, new Integer(0));
        }

        // build one-step proofs        
        ProofGetterRandom pgr1 = new ProofGetterRandom(2, 3, 2, 1, rules);
        ProofGetterRandom pgr2 = new ProofGetterRandom(1, 1, 2, 6, rules);
        ProofGetterRandom pgr3 = new ProofGetterRandom(0, 3, 2, 2, rules);

        // see how the measures change
        loop : for (int i = 0; i < 5000000; i++) {
            if (i % 100000 == 0) {
                System.out.println("" + i);
            }
            Proof p = null;
            if (i % 3 == 0) {
                p = pgr1.getProof(1);
            } else if (i % 3 == 1) {
                p = pgr2.getProof(1);
            } else {
                p = pgr3.getProof(1);
            }
            if (p.numSteps() == 0) {
                i--;
                continue loop;
            } else {
                start = p.getFirstDiagram();
                end = p.getLastDiagram();
                ///////////////////////////////////////////////
                try {
                    CompoundMeasureCalculator cmc1 =
                        new CompoundMeasureCalculator(start);
                    CompoundMeasureCalculator cmc2 =
                        new CompoundMeasureCalculator(end);
                    Vector[] resultsBefore =
                        cmc1.calcMeasuresList("Contour");
                    Vector[] resultsAfter = cmc1.calcMeasuresList("Contour");
                    c1Before = resultsBefore[0];
                    c2Before = resultsBefore[1];
                    c1After = resultsAfter[0];
                    c2After = resultsAfter[1];
                } catch (ContainsFalseException x) {
                    i--;
                    continue loop;
                }
                ///////////////////////////////////////////////
                try {
                    CompoundMeasureCalculator cmc1 =
                        new CompoundMeasureCalculator(calcCF(start, end));
                    CompoundMeasureCalculator cmc2 =
                        new CompoundMeasureCalculator(calcCF(end, start));
                    Vector[] resultsBefore = cmc1.calcMeasuresList("Zone");
                    Vector[] resultsAfter = cmc2.calcMeasuresList("Zone");
                    z1Before = resultsBefore[0];
                    z2Before = resultsBefore[1];
                    z1After = resultsAfter[0];
                    z2After = resultsAfter[1];
                } catch (ContainsFalseException x) {
                    i--;
                    continue loop;
                }
                ///////////////////////////////////////////////
                try {
                    CompoundMeasureCalculator cmc1 =
                        new CompoundMeasureCalculator(
                            calcVF(calcCF(start, end)));
                    CompoundMeasureCalculator cmc2 =
                        new CompoundMeasureCalculator(
                            calcVF(calcCF(end, start)));
                    Vector[] resultsBefore =
                        cmc1.calcMeasuresList("Shading");
                    Vector[] resultsAfter = cmc2.calcMeasuresList("Shading");
                    s1Before = resultsBefore[0];
                    s2Before = resultsBefore[1];
                    s1After = resultsAfter[0];
                    s2After = resultsAfter[1];
                } catch (ContainsFalseException x) {
                    i--;
                    continue loop;
                }
                ///////////////////////////////////////////////
                try {
                    CompoundMeasureCalculator cmc1 =
                        new CompoundMeasureCalculator(calcCF(start, end));
                    CompoundMeasureCalculator cmc2 =
                        new CompoundMeasureCalculator(calcCF(end, start));
                    Vector[] resultsBefore =
                        cmc1.calcMeasuresList("Spiders");
                    Vector[] resultsAfter = cmc2.calcMeasuresList("Spiders");
                    sp1Before = resultsBefore[0];
                    sp2Before = resultsBefore[1];
                    sp1After = resultsAfter[0];
                    sp2After = resultsAfter[1];
                } catch (ContainsFalseException x) {
                    i--;
                    continue loop;
                }
                ///////////////////////////////////////////////
                try {
                    CompoundMeasureCalculator cmc1 =
                        new CompoundMeasureCalculator(start);
                    CompoundMeasureCalculator cmc2 =
                        new CompoundMeasureCalculator(end);
                    int[] resultsBefore = cmc1.calcMeasuresNum("AndOrNum");
                    int[] resultsAfter = cmc2.calcMeasuresNum("AndOrNum");
                    andBefore = resultsBefore[0];
                    orBefore = resultsBefore[1];
                    andAfter = resultsAfter[0];
                    orAfter = resultsAfter[1];
                } catch (ContainsFalseException x) {
                    i--;
                    continue loop;
                }
                ///////////////////////////////////////////////

                Rule r = p.getImminentRule();
                // log which rules we have experienced
                resultsHM.put(
                    r.name,
                    new Integer(
                        ((Integer) (resultsHM.get(r.name))).intValue() + 1));
                // by default we expect the measure to be left unchanged
                ///////////////////////////////////////////////////////
                if (c1Unchanged() && c2Unchanged()) {
                } else if (r instanceof RuleUnshadeZone) {
                    if (c1HasntShrunk()
                        && c1HasntChangedByMoreThanOne()
                        && c1HasntGrown()
                        && c2HasntChangedByMoreThanOne()
                        && c2HasntShrunk()
                        && c2HasntGrown()) {
                    } else {
                        showCreport("" + r);
                    }
                } else if (r instanceof RuleAddContour) {
                    if (c1HasntShrunk()
                        && c1HasntChangedByMoreThanOne()
                        && c2HasntChangedByMoreThanOne()
                        && c2HasntShrunk()) {
                    } else {
                        showCreport("" + r);
                    }
                } else if (r instanceof RuleDeleteContourSimple) {
                    if (c1HasntChangedByMoreThanOne()
                        && c1HasntGrown()
                        && c2HasntChangedByMoreThanOne()
                        && c2HasntGrown()) {
                    } else {
                        showCreport("" + r);
                    }
                } else if (r instanceof MergeContradictionAvoidingFalseRule) {
                    if (c1HasntChangedByMoreThanOne()
                        && c1HasntGrown()
                        && c2HasntChangedByMoreThanOne()
                        && c2HasntGrown()) {
                    } else {
                        showCreport("" + r);
                    }
                } else {
                    showCreport(" unexpected change : " + r);
                }
                ///////////////////////////////////////////////////////
                if (z1Unchanged() && z2Unchanged()) {
                } else if (r instanceof RuleUnshadeZone) {
                    if (z1HasntShrunk()
                        && z1HasntChangedByMoreThanOne()
                        && z1HasntGrown()
                        && z2HasntChangedByMoreThanOne()
                        && z2HasntShrunk()
                        && z2HasntGrown()) {
                    } else {
                        showZreport("" + r);
                    }
                } else if (r instanceof RuleIntroduceShadedZone) {
                    if (z1HasntShrunk()
                        && z1HasntChangedByMoreThanOne()
                        && z2HasntChangedByMoreThanOne()
                        && z2HasntShrunk()) {
                    } else {
                        showZreport("" + r);
                    }
                } else if (r instanceof RuleDeleteShadedZone) {
                    if (z1HasntChangedByMoreThanOne()
                        && z1HasntGrown()
                        && z2HasntChangedByMoreThanOne()
                        && z2HasntGrown()) {
                    } else {
                        showZreport("" + r);
                    }
                } else {
                    showZreport(" unexpected change : " + r);
                }
                ///////////////////////////////////////////////////////
                if (s1Unchanged() && s2Unchanged()) {
                } else if (r instanceof RuleAddContour) {
                    if (s1HasntShrunk()
                        && s1HasntChangedByMoreThanOne()
                        && s1HasntGrown()
                        && s2HasntChangedByMoreThanOne()
                        && s2HasntShrunk()
                        && s2HasntGrown()) {
                    } else {
                        showSreport("" + r);
                    }
                } else if (r instanceof RuleUnshadeZone) {
                    if (s1HasntChangedByMoreThanOne()
                        && s1HasntGrown()
                        && s2HasntChangedByMoreThanOne()
                        && s2HasntGrown()) {
                    } else {
                        showSreport("" + r);
                    }
                } else if (r instanceof RuleExcludedMiddle) {
                    if (s1HasntShrunk()
                        && s1HasntChangedByMoreThanOne()
                        && s2HasntChangedByMoreThanOne()
                        && s2HasntShrunk()) {
                    } else {
                        showSreport("" + r);
                    }
                } else if (r instanceof RuleReverseExcludedMiddle) {
                    if (s1HasntChangedByMoreThanOne()
                        && s1HasntGrown()
                        && s2HasntChangedByMoreThanOne()
                        && s2HasntGrown()) {
                    } else {
                        showSreport("" + r);
                    }
                } else if (r instanceof MergeConjunctionRule) {
                    if (s1HasntShrunk() && s2HasntShrunk()) {
                    } else {
                        showSreport("" + r);
                    }
                } else if (r instanceof MergeContradictionAvoidingFalseRule) {
                    if (s1HasntGrown() && s2HasntGrown()) {
                    } else {
                        showSreport("" + r);
                    }
                } else if (r instanceof RemoveNotRule) {
                    if (s1HasntChangedByMoreThanOne()
                        && s2HasntChangedByMoreThanOne()) {
                    } else {
                        showSreport("" + r);
                    }
                } else {
                    showSreport(" unexpected change : " + r);
                }
                ///////////////////////////////////////////////////////
                if (sp1Unchanged() && sp2Unchanged()) {
                } else if (r instanceof RuleUnshadeZone) {
                    if (sp1HasntShrunk()
                        && sp1HasntChangedByMoreThanOne()
                        && sp1HasntGrown()
                        && sp2HasntChangedByMoreThanOne()
                        && sp2HasntShrunk()
                        && sp2HasntGrown()) {
                    } else {
                        showSPreport("" + r);
                    }
                } else if (r instanceof RuleAddSpiderFoot) {
                    if (true) {
                    } else {
                        showSPreport("" + r);
                    }
                } else if (r instanceof RuleDeleteSpider) {
                    if (sp1HasntChangedByMoreThanOne()
                        && sp1HasntGrown()
                        && sp2HasntChangedByMoreThanOne()
                        && sp2HasntGrown()) {
                    } else {
                        showSPreport("" + r);
                    }
                } else if (r instanceof RuleSplitSpider) {
                    if (true) {
                    } else {
                        showSPreport("" + r);
                    }
                } else if (r instanceof RuleJoinSpiders) {
                    if (true) {
                    } else {
                        showSPreport("" + r);
                    }
                } else if (r instanceof RuleExcludedMiddle) {
                    if (sp1HasntShrunk()
                        && sp1HasntChangedByMoreThanOne()
                        && sp2HasntChangedByMoreThanOne()
                        && sp2HasntShrunk()) {
                    } else {
                        showSPreport("" + r);
                    }
                } else if (r instanceof RuleReverseExcludedMiddle) {
                    if (sp1HasntChangedByMoreThanOne()
                        && sp1HasntGrown()
                        && sp2HasntChangedByMoreThanOne()
                        && sp2HasntGrown()) {
                    } else {
                        showSPreport("" + r);
                    }
                } else if (r instanceof RemoveNotRule) {
                    if (sp1HasntChangedByMoreThanOne()
                        && sp2HasntChangedByMoreThanOne()) {
                    } else {
                        showSPreport("" + r);
                    }
                } else if (r instanceof MergeConjunctionRule) {
                    if (sp1HasntShrunk()
                        && sp1HasntChangedByMoreThanOne()
                        && sp2HasntShrunk()) {
                    } else {
                        showSPreport("" + r);
                    }
                } else if (r instanceof MergeContradictionAvoidingFalseRule) {
                    if (sp1HasntGrown() && sp2HasntGrown()) {
                    } else {
                        showSPreport("" + r);
                    }
                } else {
                    showSPreport(" unexpected change : " + r);
                }
                ///////////////////////////////////////////////////////
                if (andUnchanged() && orUnchanged()) {
                } else if (r instanceof RuleUnshadeZone) {
                    if (andHasntShrunk()
                        && andHasntChangedByMoreThanOne()
                        && andHasntGrown()
                        && orHasntChangedByMoreThanOne()
                        && orHasntShrunk()
                        && orHasntGrown()) {
                    } else {
                        showAOreport("" + r);
                    }
                } else if (r instanceof RuleSplitSpider) {
                    if (andHasntChangedByMoreThanOne()
                        && orHasntChangedByMoreThanOne()) {
                    } else {
                        showAOreport("" + r);
                    }
                } else if (r instanceof RuleJoinSpiders) {
                    if (andHasntChangedByMoreThanOne()
                        && orHasntChangedByMoreThanOne()
                        && orHasntGrown()) {
                    } else {
                        showAOreport("" + r);
                    }
                } else if (r instanceof RuleExcludedMiddle) {
                    if (andHasntChangedByMoreThanOne()
                        && orHasntChangedByMoreThanOne()) {
                    } else {
                        showAOreport("" + r);
                    }
                } else if (r instanceof RuleReverseExcludedMiddle) {
                    if (andHasntChangedByMoreThanOne()
                        && orHasntChangedByMoreThanOne()
                        && orHasntGrown()) {
                    } else {
                        showAOreport("" + r);
                    }
                } else if (r instanceof MergeContradictionAvoidingFalseRule) {
                    if (andHasntGrown() && orHasntGrown()) {
                    } else {
                        showAOreport("" + r);
                    }
                } else if (r instanceof MergeConjunctionRule) {
                    if (andHasntChangedByMoreThanOne()
                        && andHasntGrown()
                        && orHasntChangedByMoreThanOne()) {
                    } else {
                        showAOreport("" + r);
                    }
                } else if (r instanceof RemoveNotRule) {
                    if (andHasntChangedByMoreThanOne()
                        ) {
                    } else {
                        showAOreport("" + r);
                    }
                } else if (r instanceof AndIdempotentRule) {
                    if (true) {
                    } else {
                        showAOreport("" + r);
                    }
                } else if (r instanceof RuleReverseAndIdempotent) {
                    if (andHasntGrown()) {
                    } else {
                        showAOreport("" + r);
                    }
                } else if (r instanceof OrIdempotentRule) {
                    if (true) {
                    } else {
                        showAOreport("" + r);
                    }
                } else if (r instanceof RuleReverseOrIdempotent) {
                    if (andHasntChangedByMoreThanOne() && orHasntGrown()) {
                    } else {
                        showAOreport("" + r);
                    }
                } else if (r instanceof RuleDMPullNotThroughAnd) {
                    if (andHasntShrunk()
                        && andHasntChangedByMoreThanOne()
                        && orHasntGrown()) {
                    } else {
                        showAOreport("" + r);
                    }
                } else if (r instanceof RuleDMPushNotThroughOr) {
                    if (   andHasntGrown()
                        && orHasntChangedByMoreThanOne()
                        && orHasntGrown()) {
                    } else {
                        showAOreport("" + r);
                    }
                } else if (r instanceof RuleDMPullNotThroughOr) {
                    if (   andHasntChangedByMoreThanOne()
                        && orHasntChangedByMoreThanOne()) {
                    } else {
                        showAOreport("" + r);
                    }
                } else if (r instanceof RuleDMPushNotThroughAnd) {
                    if (andHasntChangedByMoreThanOne()
                        && andHasntGrown()
                        && orHasntGrown()) {
                    } else {
                        showAOreport("" + r);
                    }
                } else if (r instanceof InsertNotNotRule) {
                    if (andHasntChangedByMoreThanOne()
                        && andHasntGrown()
                        && orHasntChangedByMoreThanOne()
                        && orHasntGrown()) {
                    } else {
                        showAOreport("" + r);
                    }
                } else if (r instanceof RemoveNotNotRule) {
                    if (andHasntShrunk()
                        && andHasntChangedByMoreThanOne()
                        && orHasntChangedByMoreThanOne()) {
                    } else {
                        showAOreport("" + r);
                    }
                } else if (r instanceof RuleOrAbsorbtion) {
                    if (   andHasntChangedByMoreThanOne()
                        && andHasntGrown()
                        && orHasntChangedByMoreThanOne()
                        && orHasntGrown()) {
                    } else {
                        showAOreport("" + r);
                    }
                } else if (r instanceof RuleAndAbsorbtion) {
                    if (andHasntChangedByMoreThanOne()
                        && andHasntGrown()
                        && orHasntChangedByMoreThanOne()
                        && orHasntShrunk()
                        && orHasntGrown()) {
                    } else {
                        showAOreport("" + r);
                    }
                } else if (r instanceof RuleDistributeAndOverOr) {
                    if (   andHasntChangedByMoreThanOne()
                        && orHasntChangedByMoreThanOne()
                        && orHasntGrown()) {
                    } else {
                        showAOreport("" + r);
                    }
                } else if (r instanceof RuleDistributeOrOverAnd) {
                    if (   andHasntChangedByMoreThanOne()
                        && andHasntGrown()
                        && orHasntChangedByMoreThanOne()
                        ) {
                    } else {
                        showAOreport("" + r);
                    }
                } else {
                    showAOreport(" unexpected change : " + r);
                }
                ///////////////////////////////////////////////////////
            }
        }
        System.out.println("");
        for (int i = 0; i < HRules.size(); i++) {
            Rule r = (Rule) HRules.get(i);
            System.out.println("" + resultsHM.get(r.name) + " " + r.name);
        }
    }
    
    private boolean isUnitaryOrNotNotNotUnitaryETC(Diagram d)
    {
        if(d instanceof UnitaryDiagram){
            return true;
        } else {
          //  if(!(d instanceof NotCompoundDiagram)){
                return false;
          //  } else {
          //      return isUnitaryOrNotNotNotUnitaryETC((Diagram)((NotCompoundDiagram)d).getChildren().get(0));
          //  }
        }        
    }
}
