package rwd.rules.heuristics;

import rwd.abstractsyntax.eulerdiagram.Contour;
import rwd.abstractsyntax.eulerdiagram.Zone;
import rwd.abstractsyntax.spiderdiagram.Diagram;
import rwd.abstractsyntax.spiderdiagram.DiagramFileUser;
import java.util.Vector;
import rwd.rules.choicespec.RuleDeleteContour;
import rwd.rules.choicespec.RuleWithChoice;
import rwd.rules.inputspec.RuleAddContour;
import rwd.rules.application.RuleAppliesToContour;
import rwd.rules.application.RuleAppliesWithString;
import rwd.rules.application.RuleApplicationSpec;
import rwd.abstractsyntax.spiderdiagram.Spider;
import rwd.rules.choicespec.RuleIntroduceShadedZone;
import rwd.abstractsyntax.spiderdiagram.UnitaryDiagram;

public class HeuristicCalculatorOptimistic implements HeuristicCalculator {
    private boolean[] includeSteps = new boolean[] { true, true, true, true };
    private boolean debug = false;
    // 1 means just do contour bit
    // 2 means do zones
    // 3 means do shading
    // 4 means do spiders

    public HeuristicCalculatorOptimistic(boolean[] contributions) {
        includeSteps = contributions;
    }
    public HeuristicCalculatorOptimistic(String contributions) {
        try {
            boolean b1 = !contributions.substring(0, 1).equals("f");
            boolean b2 = !contributions.substring(1, 2).equals("f");
            boolean b3 = !contributions.substring(2, 3).equals("f");
            boolean b4 = !contributions.substring(3, 4).equals("f");
            includeSteps = new boolean[] { b1, b2, b3, b4 };
        } catch (StringIndexOutOfBoundsException x) {
            throw new IllegalArgumentException();
        }
    }
    public HeuristicCalculatorOptimistic() {
    }
    public double calcHeuristic(Diagram d11, Diagram d21) { //
        d11 = d11.getClone();
        d21 = d21.getClone();
        debug("calculating heuristic from "+DiagramFileUser.toSaveString(d11));
        debug("calculating heuristic to "+DiagramFileUser.toSaveString(d21));
        if (!(d11 instanceof UnitaryDiagram)
            || !(d21 instanceof UnitaryDiagram)) {
            return HeuristicProofWriter.proofLengthLimit;
        }
        if (d11.equals(d21)) {
            return 0.0;
        }
        UnitaryDiagram d1 = (UnitaryDiagram) d11;
        UnitaryDiagram d2 = (UnitaryDiagram) d21;
        double result = 0.0;
        double cappedAmount = 0.0;
        ////////////////////////////////////////////////////////////////
        // first compare contour sets
        Vector justD1 = (Vector) d1.getCloneOfContourSet();
        justD1.removeAll(d2.getCloneOfContourSet());
        Vector justD2 = (Vector) d2.getCloneOfContourSet();
        justD2.removeAll(d1.getCloneOfContourSet());
        //
        if (includeSteps[0]) {
            result = result + justD1.size() + justD2.size();
        } //System.out.println("after comparing contours, heuristic is "+result);
        ////////////////////////////////////////////////////////////////
        // second make the contour sets equal
        while (justD1.size() > 0) {
            // remove a contour from d1
            Contour c = (Contour) justD1.get(0);
            d1 =
                (UnitaryDiagram) RuleDeleteContour.getRule().applyRule(
                    new RuleAppliesToContour(d1, c));
            justD1.remove(0);
        }
        while (justD2.size() > 0) { // add a contour to d1
            String s = "" + justD2.get(0);
            d1 =
                (UnitaryDiagram) RuleAddContour.getRule().applyRule(
                    new RuleAppliesWithString(d1, s));
            justD2.remove(0);
        } //System.out.println("d1 is "+d1.toSaveString());
        //System.out.println("d2 is "+d2.toSaveString());
        ///////////////////////////////////////////////////////////////
        // compare the zone sets : are they equal?
        Vector zones1 = d1.getCloneOfZoneSet();
        Vector zones2 = d2.getCloneOfZoneSet();
        boolean condition1 = !zones1.containsAll(zones2);
        boolean condition2 = !zones2.containsAll(zones1);
        if (condition1 && includeSteps[1]) { // zones2 has extra
            cappedAmount++;
        }
        if (condition2 && includeSteps[1]) { // zones1 has extra
            result++;
        } //System.out.println("after comparing zones, heuristic is "+result);
        ///////////////////////////////////////////////////////////////
        ///////////////////////////////////////////////////////////////
        // for shading, insert missing zones where possible
        RuleWithChoice rasz =
            (RuleWithChoice) RuleIntroduceShadedZone.getRule();
        Vector appspecs = rasz.getApplicationPossibilities(d1);
        while (appspecs.size() > 0) {
            d1 =
                (UnitaryDiagram) rasz.applyRule(
                    (RuleApplicationSpec) appspecs.get(0));
            appspecs = rasz.getApplicationPossibilities(d1);
        }
        appspecs = rasz.getApplicationPossibilities(d2);
        while (appspecs.size() > 0) {
            d2 =
                (UnitaryDiagram) rasz.applyRule(
                    (RuleApplicationSpec) appspecs.get(0));
            appspecs = rasz.getApplicationPossibilities(d2);
        } // find out whether all shaded zones in d2
        // are also shaded in d1 - and is there more shading in d1
        if (!d1.containsShadedAll(d2.getCloneOfShadedZones())
            && includeSteps[2]) {
                debug("returning infinity from HCalc bcs d2 has extra shading!");
                debug("d1 is "+DiagramFileUser.toSaveString(d1));
                debug("d2 is "+DiagramFileUser.toSaveString(d2));
            return HeuristicProofWriter.proofLengthLimit;
        }
        if (d1.getNumShadedZones() > d2.getNumShadedZones()
            && includeSteps[2]) {
            cappedAmount++;
        } /////////////////////////////////////////////////////////////////
        // you can't introduce spiders
        if (d2.getSpiders().size() > d1.getSpiders().size()
            && includeSteps[3]) {
            return HeuristicProofWriter.proofLengthLimit;
        } else {
            // add one for each extra spider at start
            int excessSpiderCount =
                d1.getSpiders().size() - d2.getSpiders().size();
            if (includeSteps[3]) {
                result = result + excessSpiderCount;
            } // find the number of unmatched spiders: ie.
            // take each spider in d2
            // look for a spider in d1 which has the same habitat
            // if you find a matching spider
            // then remove each from each diagram
            Vector d2sSpiders = (Vector) d2.getSpiders().clone();
            d2SpiderLoop : for (int i = 0; i < d2sSpiders.size(); i++) {
                Spider s = (Spider) (d2sSpiders.get(i));
                Vector habitat = s.getHabitat();
                for (int j = 0; j < d1.getSpiders().size(); j++) {
                    Spider s1 = (Spider) d1.getSpiders().get(j);
                    Vector hab1 = s1.getHabitat();
                    if (habitat.containsAll(hab1)
                        && hab1.containsAll(habitat)) { // found match
                        d2.removeSpider(s);
                        d1.removeSpider(s1);
                        continue d2SpiderLoop;
                    }
                }
            } // clean this logic up!!
            // if d2 still >=1 spiders left 
            if (d2.getSpiders().size() >= 1 && includeSteps[3]) {
                cappedAmount++;
            }
        }
        if (cappedAmount > 2.0) {
            cappedAmount = 2.0;
        }
        return result + cappedAmount;
    }

    public static void main(String args[]) {
        Vector zones1 = new Vector();
        Vector inside = new Vector();
        Vector outside = new Vector();
        inside.add(Contour.getContour("a"));
        zones1.add(Zone.getZone(inside, outside));
        inside = new Vector();
        outside = new Vector();
        outside.add(Contour.getContour("a"));
        zones1.add(Zone.getZone(inside, outside));
        UnitaryDiagram ud1 = new UnitaryDiagram(zones1, "d1");
        //////////////////////////////////////////////////////////////////////
        Vector zones2 = new Vector();
        inside = new Vector();
        outside = new Vector();
        outside.add(Contour.getContour("b"));
        zones2.add(Zone.getZone(inside, outside));
        inside = new Vector();
        outside = new Vector();
        inside.add(Contour.getContour("b"));
        zones2.add(Zone.getZone(inside, outside));
        UnitaryDiagram ud2 = new UnitaryDiagram(zones2, "d2");
        //////////////////////////////////////////////////////////////////////
        /**/
        System.out.println(
            "heuristic for diagrams \n"
                + DiagramFileUser.toSaveString(ud1)
                + " and\n "
                + DiagramFileUser.toSaveString(ud2)
                + "\nis "
                + new HeuristicCalculatorOptimistic().calcHeuristic(ud1, ud2));
    }

    private void debug(String message) {
        if (debug) {
            System.out.println("" + message);
        }
    }
}
