package rwd.rules.heuristics;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Vector;

import rwd.rules.NoProofException;
import rwd.rules.Proof;
import rwd.rules.ProofFileUser;
import rwd.rules.ProofGetterRandom;
import rwd.abstractsyntax.spiderdiagram.Diagram;
import rwd.abstractsyntax.spiderdiagram.DiagramFileUser;
import rwd.gui.FrameProgressShower;
import rwd.rules.choicespec.*;
import rwd.rules.inputspec.*;
import rwd.rules.nospec.AndIdempotentRule;
import rwd.rules.nospec.InsertNotNotRule;
import rwd.rules.nospec.OrIdempotentRule;
import rwd.rules.nospec.RemoveNotNotRule;

/**
 * @author jaf28
 */

public class HeuristicProofAssessorCompound {

    public static void main(String[] args) {
        Vector compoundHRules = new Vector();
        compoundHRules.add(RuleAddContour.getRule());
        compoundHRules.add(RuleDeleteContourSimple.getRule());
        compoundHRules.add(RuleIntroduceShadedZone.getRule());
        compoundHRules.add(RuleDeleteShadedZone.getRule());
        compoundHRules.add(RuleUnshadeZone.getRule());
        compoundHRules.add(RuleDeleteSpider.getRule());
        compoundHRules.add(RuleAddSpiderFoot.getRule());
        compoundHRules.add(RuleSplitSpider.getRule());
        compoundHRules.add(RuleJoinSpiders.getRule());
        compoundHRules.add(RuleExcludedMiddle.getRule());
        //compoundHRules.add(MergeContradictionAvoidingFalseRule.getRule());
        //compoundHRules.add(RemoveNotRule.getRule());
        compoundHRules.add(AndIdempotentRule.getRule());
        compoundHRules.add(RuleReverseAndIdempotent.getRule());
        compoundHRules.add(OrIdempotentRule.getRule());
        compoundHRules.add(RuleReverseOrIdempotent.getRule());
        compoundHRules.add(RuleDMPullNotThroughAnd.getRule());
        compoundHRules.add(RuleDMPullNotThroughOr.getRule());
        compoundHRules.add(RuleDMPushNotThroughAnd.getRule());
        compoundHRules.add(RuleDMPushNotThroughOr.getRule());
        compoundHRules.add(RemoveNotNotRule.getRule());
        compoundHRules.add(InsertNotNotRule.getRule());
        compoundHRules.add(RuleAndAbsorbtion.getRule());
        compoundHRules.add(RuleOrAbsorbtion.getRule());
        compoundHRules.add(RuleDistributeAndOverOr.getRule());
        compoundHRules.add(RuleDistributeOrOverAnd.getRule());
        compoundHRules.add(RuleDistributeMergeAnds.getRule());
        compoundHRules.add(RuleDistributeMergeOrs.getRule());

        System.out.println("start assessment");
        String dataFilename = args[0];
        int numberOfDataPoints = Integer.parseInt(args[1]);
        int numInitialContours = Integer.parseInt(args[2]);
        int maxNumContours = numInitialContours;
        int numSpiders = Integer.parseInt(args[3]);
        int numComponents = Integer.parseInt(args[4]);
        int depthOfProofs = Integer.parseInt(args[5]);
        System.out.println("depth of proofs is " + depthOfProofs);
        boolean collectAll = (args[6].equals("all"));
        System.out.println("collecting all :" + collectAll);
        FrameProgressShower fps = new FrameProgressShower();

        try {
            ProofGetterRandom pgr =
                new ProofGetterRandom(
                    numInitialContours,
                    maxNumContours,
                    numSpiders,
                    numComponents,
                    compoundHRules);
            dataPointLoop : for (int i = 0; i < numberOfDataPoints; i++) {
                //System.out.println("get a proof task..");
                Proof p = null;
                while (p == null) {
                    p = pgr.getProof(depthOfProofs);
                    if (p.numSteps() < 2) {
                        p = null;
                    }
                }
                //System.out.println("got a proof task");
                //ProofManager.showNewProof(p,false,true);

                // pass the start/end diagrams to the proof-writer
                Diagram start = p.getFirstDiagram();
                Diagram target = p.getLastDiagram();
                //ProofManager.showNewProof(p,false,true);

                int optimalLength = -1;
                int[] lengthBuilt = new int[2];
                int[] numBuilt = new int[2];
                try {
                    BufferedWriter bf =
                        new BufferedWriter(new FileWriter(new File(dataFilename)));
                    Vector outcome =
                        HeuristicProofWriter.seekProof(
                            start,
                            target,
                            p.numSteps(),
                            new HeuristicCalculatorZero(),
                            new DefaultCostCalculator(),
                            compoundHRules,
                            fps,
                            false,
                            true,
                            collectAll);
                    //System.out.print(".");
                    if (outcome == null) {
                        System.out.println("null outcome found!!");
                        DiagramFileUser.saveToFile(start);
                        DiagramFileUser.saveToFile(target);
                        System.out.println("" + ProofFileUser.toSaveString(p));
                        System.exit(0);
                    }
                    Proof pfound = (Proof) outcome.get(0);
                    if (pfound == null) {
                        System.out.println("null pfound!!");
                        DiagramFileUser.saveToFile(start);
                        DiagramFileUser.saveToFile(target);
                        System.out.println("" + ProofFileUser.toSaveString(p));
                        System.exit(0);
                    }
                    numBuilt[0] =
                        ((Integer) outcome.get(outcome.size() - 1)).intValue();
                    lengthBuilt[0] = pfound.numSteps();
                    System.out.print("length "+lengthBuilt[0]+", Zero/Full : " + numBuilt[0] + "/");
                    bf.write("length "+lengthBuilt[0]+", Zero/Full : " + numBuilt[0] + "/");
                    
                    outcome =
                        HeuristicProofWriter.seekProof(
                            start,
                            target,
                            p.numSteps(),
                            new HeuristicCalculatorCompound(),
                            new DefaultCostCalculator(),
                            compoundHRules,
                            fps,
                            false,
                            true,
                            collectAll);
                    pfound = (Proof) outcome.get(0);
                    if (pfound == null) {
                        System.out.println("null proof found!!");
                        DiagramFileUser.saveToFile(start);
                        DiagramFileUser.saveToFile(target);
                        System.out.println("" + ProofFileUser.toSaveString(p));
                        System.exit(0);
                    }
                    numBuilt[1] =
                        ((Integer) outcome.get(outcome.size() - 1)).intValue();
                    lengthBuilt[1] = pfound.numSteps();
                    System.out.println("" + numBuilt[1]);
                    bf.write("" + numBuilt[1]);
                    bf.newLine();
                } catch (NoProofException x) {
                    System.out.println("no proof found!!");
                    DiagramFileUser.saveToFile(start);
                    DiagramFileUser.saveToFile(target);
                    System.exit(0);
                }
                // check numBuilt decreases
                //if (numBuilt[0] < numBuilt[1]) {
                //    System.out.println(
                //        "heuristic failed to improve !! : numbuilt were "
                //            + numBuilt[0]
                //            + " and "
                //            + numBuilt[1]);
                //    System.exit(0);
                //}
                // log
                //System.out.println(" go to next data point...");
            }
            //System.out.println(" finished data point loop");
        } catch (IOException x) {
            System.out.println("io problem " + x);
            x.printStackTrace();
            System.exit(0);
        }
    }
}
