package rwd.tableaux.diagram;

import java.util.Vector;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import rwd.abstractsyntax.spiderdiagram.AndCompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.CompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.Diagram;
import rwd.abstractsyntax.spiderdiagram.FalseDiagram;
import rwd.abstractsyntax.spiderdiagram.NotCompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.OrCompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.Spider;
import rwd.abstractsyntax.spiderdiagram.TrueDiagram;
import rwd.abstractsyntax.spiderdiagram.UnitaryDiagram;
import rwd.tableaux.semantic.Options;

import uk.ac.kent.cs.kmf.util.ILog;

/**
 * @author Octavian Patrascoiu
 *
 */
public class SpiderSplitterImpl implements SpiderSplitter {
	/** Constructors */
	public SpiderSplitterImpl(Options opt, ILog log) {
		this.opt = opt;
		this.log = log;
	}

	/** Split spiders - create or diagrams and single-footed spiders */
	public Diagram splitSpiders(Diagram d) {
		// Unitary diagram
		Diagram newDiagram = d; 
		if (d instanceof UnitaryDiagram) {
			UnitaryDiagram ud = (UnitaryDiagram) d;
			// true or false
			if (ud instanceof FalseDiagram || ud instanceof TrueDiagram) {
			// User diagram
			} else {
				// Split spiders from the unitary diagram
				newDiagram = splitUnitaryDiagram(ud);
			}
		} else if (d instanceof CompoundDiagram) {
			CompoundDiagram cd = (CompoundDiagram)d;
			// For each child
			Collection children = cd.getChildren();
			Vector newChildren = new Vector();
			for(Iterator i = children.iterator(); i.hasNext();) {
				Diagram child = (Diagram)i.next();
				// Make a new child
				Diagram newChild = splitSpiders(child);
				newChildren.add(newChild);
			}
			// Make a new compound diagram
			CompoundDiagram newParent = null;
			if (cd instanceof NotCompoundDiagram) {
				newParent = new NotCompoundDiagram(new Vector(), "");
				newParent.setChildren(newChildren);
			} else if (cd instanceof AndCompoundDiagram) {
				newParent = new AndCompoundDiagram(newChildren, "");
				newParent.setChildren(newChildren);
			} else if (cd instanceof OrCompoundDiagram) {
				newParent = new OrCompoundDiagram(newChildren, "");
				newParent.setChildren(newChildren);
			}
			newDiagram = newParent;
			if (!opt.isBinaryOperators())
				newDiagram.flatten();
		}
		return newDiagram;
	}

	/** Split a given spider */
	public Diagram splitSpider(Diagram d, List habitat) {
		// Unitary diagram
		Diagram newDiagram = d; 
		if (d instanceof UnitaryDiagram) {
			UnitaryDiagram ud = (UnitaryDiagram) d;
			// true or false
			if (ud instanceof FalseDiagram || ud instanceof TrueDiagram) {
			// User diagram
			} else {
				// Split spiders from the unitary diagram
				newDiagram = splitUnitaryDiagram(ud, habitat);
			}
		} else if (d instanceof CompoundDiagram) {
			CompoundDiagram cd = (CompoundDiagram)d;
			// For each child
			Collection children = cd.getChildren();
			Vector newChildren = new Vector();
			for(Iterator i = children.iterator(); i.hasNext();) {
				Diagram child = (Diagram)i.next();
				// Make a new child
				Diagram newChild = splitSpider(child, habitat);
				newChildren.add(newChild);
			}
			// Make a new compound diagram
			CompoundDiagram newParent = null;
			if (cd instanceof NotCompoundDiagram) {
				newParent = new NotCompoundDiagram(new Vector(), "");
				newParent.setChildren(newChildren);
			} else if (cd instanceof AndCompoundDiagram) {
				newParent = new AndCompoundDiagram(newChildren, "");
				newParent.setChildren(newChildren);
			} else if (cd instanceof OrCompoundDiagram) {
				newParent = new OrCompoundDiagram(newChildren, "");
				newParent.setChildren(newChildren);
			}
			newDiagram = newParent;
			if (!opt.isBinaryOperators())
				newDiagram.flatten();
		}
		return newDiagram;
	}

	/** Split all spiders from a unitary diagram */
	protected Diagram splitUnitaryDiagram(UnitaryDiagram ud) {
		// Compute the list of habitats, sizes
		List oldSpiders = ud.getSpiders();
		int ns = oldSpiders.size();
		Vector habitats[] = new Vector[ns];
		int sizes[] = new int[ns];
		for(int i=0; i<ns; i++) {
			Spider s = (Spider) oldSpiders.get(i);
			Vector habitat = s.getHabitat(); 
			habitats[i] = habitat;
			sizes[i] = habitat.size();
		}
		
		// Generate all possible combinations
		// and compute a list of diagrams
		int solution[] = new int[ns];
		for(int i=0; i<ns; i++) solution[i] = 0;
		boolean flag = true;
		List diagrams = new Vector();
		int no = 1;
		while (flag) {
			// Create a new diagram
			UnitaryDiagram newUD = new UnitaryDiagram(ud.getCloneOfZoneSet(), ud.getName()+"-"+no++);
			// Set the shaded zones and the spiders
			newUD.setShadedZones(ud.getCloneOfShadedZones());
			for(int j=0; j<ns; j++) {
				// Create a new habitat
				Vector newHabitat = new Vector();
				newHabitat.add(habitats[j].get(solution[j]));
				// Add the corresponding spider
				newUD.addSpider(newHabitat); 
			}
			diagrams.add(newUD);
			// Generate a new solution
			int i = ns-1;
			while (i >= 0) {
				int ni = sizes[i];
				int ii = solution[i];
				if (ii == ni-1) {
					solution[i] = 0;
					i--;
				} else {
					solution[i]++;
					break;
				}
			}
			if (i == -1) flag = false;
		}
		// Make binary OR diagrams
		Diagram newDiagram = (Diagram)diagrams.get(0);
		for(int i=1; i<diagrams.size(); i++) {
			Diagram temp = (Diagram)diagrams.get(i);
			OrCompoundDiagram or = new OrCompoundDiagram(new Vector(), ""); 
			or.getChildren().add(newDiagram);
			or.getChildren().add(temp);
			if (!opt.isBinaryOperators())
				or.flatten();
			newDiagram = or;
		}
		return newDiagram;
	}

	/** Split a spider from a unitary diagram */
	protected Diagram splitUnitaryDiagram(UnitaryDiagram ud, List habitat) {
		// Compute the list of habitats, sizes
		Vector oldSpiders = ud.getSpiders();
		Vector restSpiders = new Vector();
		// Check if the habitat exists
		boolean found = true;
		for(int i=0; i<oldSpiders.size(); i++) {
			Spider sp = (Spider)oldSpiders.get(i);
			Vector oldHabitat = sp.getHabitat();
			// Remove spider
			if (oldHabitat.containsAll(habitat) && habitat.containsAll(oldHabitat)) {
				found = true;
			} else {
				restSpiders.add(sp);
			}
		}
		// Split it if found
		if (found) {
			int ns = 1;
			Vector habitats[] = new Vector[ns];
			int sizes[] = new int[ns];
			habitats[0] = new Vector(habitat);
			sizes[0] = habitat.size();
		
			// Generate all possible combinations
			// and compute a list of diagrams
			int solution[] = new int[ns];
			for(int i=0; i<ns; i++) solution[i] = 0;
			boolean flag = true;
			List diagrams = new Vector();
			int no = 1;
			while (flag) {
				// Create a new diagram
				UnitaryDiagram newUD = new UnitaryDiagram(ud.getCloneOfZoneSet(), ud.getName()+"-"+no++);
				// Set the shaded zones and the spiders
				newUD.setShadedZones(ud.getCloneOfShadedZones());
				for(int j=0; j<ns; j++) {
					// Create a new habitat
					Vector newHabitat = new Vector();
					newHabitat.add(habitats[j].get(solution[j]));
					// Add the corresponding spider
					newUD.addSpider(newHabitat);
				}
				// Add rest of spiders
				for(int i=0; i<restSpiders.size(); i++) {
					Spider sp = (Spider)restSpiders.get(i);
					Vector restHabitat = sp.getHabitat();
					newUD.addSpider(restHabitat);
				}
				diagrams.add(newUD);
				// Generate a new solution
				int i = ns-1;
				while (i >= 0) {
					int ni = sizes[i];
					int ii = solution[i];
					if (ii == ni-1) {
						solution[i] = 0;
						i--;
					} else {
						solution[i]++;
						break;
					}
				}
				if (i == -1) flag = false;
			}
			// Make binary OR diagrams
			Diagram newDiagram = (Diagram)diagrams.get(0);
			for(int i=1; i<diagrams.size(); i++) {
				Diagram temp = (Diagram)diagrams.get(i);
				OrCompoundDiagram or = new OrCompoundDiagram(new Vector(), ""); 
				or.getChildren().add(newDiagram);
				or.getChildren().add(temp);
				if (!opt.isBinaryOperators())
					or.flatten();
				newDiagram = or;
			}
			return newDiagram;
		}
		return ud;
	}
	
	//
	// Local attributes
	//
	protected Options opt;
	protected ILog log;
}
