/**
 * Kent Modelling Framework - KMFStudio
 * Copyright (C) 2002 University of Kent at Canterbury, UK 
 * Visit www.cs.ukc.ac.uk/kmf
 *
 */

/**
 * 
 * @author Octavian Patrascoiu
 *
 */

package rwd.tableaux.syntax.parser;

import java.io.Reader;

import rwd.tableaux.syntax.ast.*;
import uk.ac.kent.cs.kmf.util.*;

public interface Parser {
	/** 
	  * Parse the given string 
	  * without debugging info, 
	  * reporting errors on default log: System.out 
	  * 
	  * The result is null if an error is reported 
	  * */
	public LogicalExp parse(String body);

	/** 
	 * Parse the given string 
	 * without debugging info, 
	 * reporting errors on given log 
	 * 
	 * The result is null if an error is reported
	 *  
	 * */
	public LogicalExp parse(String body, ILog log);

	/** 
	  * Parse the given string 
	  * with/without debugging info, 
	  * reporting errors on given log
	  * 
	  * The result is null if an error is reported 
	  *  
	  * */
	public LogicalExp parse(String body, ILog log, boolean debug);

	/** 
	 * Parse the given file 
	 * without debugging info, 
	 * reporting errors on default log: System.out
	 *  
	 * The result is null if an error is reported 
	 * 
	 * */
	public LogicalExp parse(Reader reader);
	
	/** 
	 * Parse the given file 
	 * without debugging info, 
	 * reporting errors on given log
	 *  
	 * The result is null if an error is reported 
	 * 
	 * */
	public LogicalExp parse(Reader reader, ILog log);
	
	/** 
	 * Parse the given file 
	 * with/without debugging info, 
	 * reporting errors on given log
	 * 
	 * The result is null if an error is reported
	 *  
	 * */
	public LogicalExp parse(Reader reader, ILog log, boolean debug);
	
	/** 
	 * Check if the parser encountered an error 
	 * 
	 * */
	public boolean hasErrors();
	
	/** 
	 * Check if the parser encountered a warning 
	 * 
	 * */
	public boolean hasWarnings();
}
