package rwd.views.abstracttoconcrete.converting;
import java.util.Vector;
import java.util.TreeSet;
import java.util.Iterator;
import java.util.NoSuchElementException;

import rwd.abstractsyntax.eulerdiagram.Zone;
//import rwd.views.abstracttoconcrete.oldaed.*;
import rwd.views.graph.*;

public class Circulariser {
	private static Position[] positions = null;
	private static TreeSet parentSet = new TreeSet();
	private static Graph tree;
	private static FaceCycle fc; // goes round the tree
	private static Vector mappings = new Vector();

	public static Position getPosition(int index) {
		return positions[index];
	}

	public static Graph circularise(
		Graph g,
		Vector parents) { //System.out.println("rwd.views.graph is "+g);
		reduceParentSet(g, parents);
		// restricts to parents which are inner zones of this dual rwd.views.graph g
		Graph result = new Graph();
		if (g.nodes().size() == 1) // very small case
			{
			((Node) (g.nodes().iterator().next())).setPosition(
				new Position(0, 0));
			return g;
		}
		Vector graphCycles = FaceCycle.faceCycles(g);
		mappings = new Vector(); // will hold oldDEdge, newEdge,...
		result = mapInEdgesRoundCircle(g);
		// now mappings is populated with dedge, edge for the tree
		// face-cycle and we have to use this info to determine image edges
		// for directedEdges in rwd.views.graph face-cycles.

		// collect together edges in g that need adding in
		Vector toAddIn = new Vector();
		Iterator edgeIt = g.edges().iterator();
		//System.out.println("g is "+g);
		//System.out.println("tree is "+tree);
		while (edgeIt.hasNext()) {
			Edge e = (Edge) (edgeIt.next());
			//System.out.println("found "+e+" in rwd.views.graph "+g);
			if (!tree.edges().contains(e)) {
				toAddIn.add(e);
			}
		}
		//System.out.println("edges across come from "+toAddIn);
		// each of these is represented in two face-cycles
		// and at least on representative of one edge has neighbours
		// which are directedEdges which are already mapped
		while (toAddIn.size() > 0) {
			// iterate through each edge, finding its DEdges in faceCycle(s)
			Iterator edgeToAddIt = toAddIn.iterator();
			boolean shouldReverse = false;
			goThroughEdgesToAdd : while (edgeToAddIt.hasNext()) {
				Edge edgeToAddIn = (Edge) (edgeToAddIt.next());
				//System.out.println("looking to add in "+edgeToAddIn);
				Iterator i = edgeToAddIn.ends().iterator();
				Node from = (Node) (i.next());
				Node to = (Node) (i.next());
				DirectedEdge deToAdd;
				if (!shouldReverse) {
					deToAdd = new DirectedEdge(from, to);
				} else {
					deToAdd = new DirectedEdge(to, from);
				}
				//System.out.println("looking for "+deToAdd+" in a faceCycle");
				// find a directedEdge in graphCycles with these ends
				FaceCycle oneContainingFc = null;
				Iterator faceCycleIt = graphCycles.iterator();
				faceSearch : while (faceCycleIt.hasNext()) {
					fc = (FaceCycle) (faceCycleIt.next());
					Iterator dEdgeIt = fc.directedEdges.iterator();
					while (dEdgeIt.hasNext()) {
						DirectedEdge de = (DirectedEdge) (dEdgeIt.next());
						if (de.compareTo(deToAdd) == 0) {
							if (oneContainingFc == null) {
								oneContainingFc = fc;
							}
						}
					}
				}
				//System.out.println("the edge appears in "+oneContainingFc);
				// Now we know an edge we'd like to add in, and a faceCycle.
				// The edge runs between Nodes "from" and Node "to".
				// Find the preceding and following directedEdges in each facecycle
				// and check to see if they're in mapped already
				DirectedEdge precedingDE = null;
				DirectedEdge followingDE = null;
				Iterator faceIt = oneContainingFc.directedEdges.iterator();
				lookAroundTheFace : while (faceIt.hasNext()) {
					DirectedEdge currentDE = (DirectedEdge) (faceIt.next());
					if (currentDE.from() == deToAdd.from()
						&& currentDE.to() == deToAdd.to()) {
						// BINGO found edge in facecycle - pick up nbrs
						try {
							followingDE = (DirectedEdge) (faceIt.next());
						} catch (NoSuchElementException x) {
							followingDE =
								(DirectedEdge) (oneContainingFc
									.directedEdges
									.iterator()
									.next());
						}
						if (precedingDE == null) {
							while (faceIt.hasNext()) {
								precedingDE = (DirectedEdge) (faceIt.next());
							}
						}
						break lookAroundTheFace;
					}
					precedingDE = currentDE;
				}
				//System.out.println("the preceding DE is "+precedingDE);
				//System.out.println("the following DE is "+followingDE);
				// see if these DEs are mapped already
				Edge imageOfPreceding = null;
				Edge imageOfFollowing = null;
				Iterator mappingsIt = mappings.iterator();
				while (mappingsIt.hasNext()) {
					DirectedEdge mappedDE = (DirectedEdge) (mappingsIt.next());
					//System.out.println("mapped DE "+mappedDE);
					if (mappedDE.compareTo(precedingDE) == 0) {
						imageOfPreceding = (Edge) (mappingsIt.next());
					} else if (mappedDE.compareTo(followingDE) == 0) {
						imageOfFollowing = (Edge) (mappingsIt.next());
					} else {
						mappingsIt.next();
					}
				}
				//System.out.println("images determined are "+imageOfPreceding+
				//  " and "+imageOfFollowing);
				if (imageOfPreceding != null
					&& imageOfFollowing != null) {
					// JACKPOT - we have edge, and we know images of nbrs
					Edge imageEdge = null;
					Iterator precedingNodesIt =
						imageOfPreceding.nodes.iterator();
					Node oneEnd = (Node) (precedingNodesIt.next());
					Node otherEnd = (Node) (precedingNodesIt.next());
					if (oneEnd.label1().equals(deToAdd.from().label1())) {
						imageEdge =
							edgeToAddIn.substitute(deToAdd.from(), oneEnd);
					} else {
						imageEdge =
							edgeToAddIn.substitute(deToAdd.from(), otherEnd);
					}
					Iterator followingNodesIt =
						imageOfFollowing.nodes.iterator();
					oneEnd = (Node) (followingNodesIt.next());
					otherEnd = (Node) (followingNodesIt.next());
					if (oneEnd.label1().equals(deToAdd.to().label1())) {
						imageEdge = imageEdge.substitute(deToAdd.to(), oneEnd);
					} else {
						imageEdge =
							imageEdge.substitute(deToAdd.to(), otherEnd);
					}
					result.add(imageEdge);
					mappings.add(deToAdd.reverse());
					mappings.add(imageEdge);
					//System.out.println("added in "+edgeToAddIn);
					toAddIn.remove(edgeToAddIn);
					break goThroughEdgesToAdd;
				} else { //move on to next edge in
				}
				if (!edgeToAddIt.hasNext() && !shouldReverse) {
					shouldReverse = true;
					edgeToAddIt = toAddIn.iterator();
				}
			}
		}
		//System.out.println("returning "+result);
		//System.out.println("result has "+result.nodes.size()+" nodes and "
		//                                      +result.edges.size()+"edges");
		return result;
	}

	public static Graph circularise(
		Graph g,
		BlockGraph bg) { //System.out.println("rwd.views.graph is "+g);
		// reduceParentSet(g,parents);
		// restricts to parents which are inner zones of this dual rwd.views.graph g
		Graph result = new Graph();
		if (g.nodes().size() == 1) // very small case
			{
			((Node) (g.nodes().iterator().next())).setPosition(
				new Position(0, 0));
			return g;
		}
		Vector graphCycles = FaceCycle.faceCycles(g);
		mappings = new Vector(); // will hold oldDEdge, newEdge,...
		result = mapInEdgesRoundCircle(g);
		// now mappings is populated with dedge, edge for the tree
		// face-cycle and we have to use this info to determine image edges
		// for directedEdges in rwd.views.graph face-cycles.

		// collect together edges in g that need adding in
		Vector toAddIn = new Vector();
		Iterator edgeIt = g.edges().iterator();
		//System.out.println("g is "+g);
		//System.out.println("tree is "+tree);
		while (edgeIt.hasNext()) {
			Edge e = (Edge) (edgeIt.next());
			//System.out.println("found "+e+" in rwd.views.graph "+g);
			if (!tree.edges().contains(e)) {
				toAddIn.add(e);
			}
		}
		//System.out.println("edges across come from "+toAddIn);
		// each of these is represented in two face-cycles
		// and at least on representative of one edge has neighbours
		// which are directedEdges which are already mapped
		while (toAddIn.size() > 0) {
			// iterate through each edge, finding its DEdges in faceCycle(s)
			Iterator edgeToAddIt = toAddIn.iterator();
			boolean shouldReverse = false;
			goThroughEdgesToAdd : while (edgeToAddIt.hasNext()) {
				Edge edgeToAddIn = (Edge) (edgeToAddIt.next());
				//System.out.println("looking to add in "+edgeToAddIn);
				Iterator i = edgeToAddIn.ends().iterator();
				Node from = (Node) (i.next());
				Node to = (Node) (i.next());
				DirectedEdge deToAdd;
				if (!shouldReverse) {
					deToAdd = new DirectedEdge(from, to);
				} else {
					deToAdd = new DirectedEdge(to, from);
				}
				//System.out.println("looking for "+deToAdd+" in a faceCycle");
				// find a directedEdge in graphCycles with these ends
				FaceCycle oneContainingFc = null;
				Iterator faceCycleIt = graphCycles.iterator();
				faceSearch : while (faceCycleIt.hasNext()) {
					fc = (FaceCycle) (faceCycleIt.next());
					Iterator dEdgeIt = fc.directedEdges.iterator();
					while (dEdgeIt.hasNext()) {
						DirectedEdge de = (DirectedEdge) (dEdgeIt.next());
						if (de.compareTo(deToAdd) == 0) {
							if (oneContainingFc == null) {
								oneContainingFc = fc;
							}
						}
					}
				}
				//System.out.println("the edge appears in "+oneContainingFc);
				// Now we know an edge we'd like to add in, and a faceCycle.
				// The edge runs between Nodes "from" and Node "to".
				// Find the preceding and following directedEdges in each facecycle
				// and check to see if they're in mapped already
				DirectedEdge precedingDE = null;
				DirectedEdge followingDE = null;
				Iterator faceIt = oneContainingFc.directedEdges.iterator();
				lookAroundTheFace : while (faceIt.hasNext()) {
					DirectedEdge currentDE = (DirectedEdge) (faceIt.next());
					if (currentDE.from() == deToAdd.from()
						&& currentDE.to() == deToAdd.to()) {
						// BINGO found edge in facecycle - pick up nbrs
						try {
							followingDE = (DirectedEdge) (faceIt.next());
						} catch (NoSuchElementException x) {
							followingDE =
								(DirectedEdge) (oneContainingFc
									.directedEdges
									.iterator()
									.next());
						}
						if (precedingDE == null) {
							while (faceIt.hasNext()) {
								precedingDE = (DirectedEdge) (faceIt.next());
							}
						}
						break lookAroundTheFace;
					}
					precedingDE = currentDE;
				}
				//System.out.println("the preceding DE is "+precedingDE);
				//System.out.println("the following DE is "+followingDE);
				// see if these DEs are mapped already
				Edge imageOfPreceding = null;
				Edge imageOfFollowing = null;
				Iterator mappingsIt = mappings.iterator();
				while (mappingsIt.hasNext()) {
					DirectedEdge mappedDE = (DirectedEdge) (mappingsIt.next());
					//System.out.println("mapped DE "+mappedDE);
					if (mappedDE.compareTo(precedingDE) == 0) {
						imageOfPreceding = (Edge) (mappingsIt.next());
					} else if (mappedDE.compareTo(followingDE) == 0) {
						imageOfFollowing = (Edge) (mappingsIt.next());
					} else {
						mappingsIt.next();
					}
				}
				//System.out.println("images determined are "+imageOfPreceding+
				//  " and "+imageOfFollowing);
				if (imageOfPreceding != null
					&& imageOfFollowing != null) {
					// JACKPOT - we have edge, and we know images of nbrs
					Edge imageEdge = null;
					Iterator precedingNodesIt =
						imageOfPreceding.nodes.iterator();
					Node oneEnd = (Node) (precedingNodesIt.next());
					Node otherEnd = (Node) (precedingNodesIt.next());
					if (oneEnd.label1().equals(deToAdd.from().label1())) {
						imageEdge =
							edgeToAddIn.substitute(deToAdd.from(), oneEnd);
					} else {
						imageEdge =
							edgeToAddIn.substitute(deToAdd.from(), otherEnd);
					}
					Iterator followingNodesIt =
						imageOfFollowing.nodes.iterator();
					oneEnd = (Node) (followingNodesIt.next());
					otherEnd = (Node) (followingNodesIt.next());
					if (oneEnd.label1().equals(deToAdd.to().label1())) {
						imageEdge = imageEdge.substitute(deToAdd.to(), oneEnd);
					} else {
						imageEdge =
							imageEdge.substitute(deToAdd.to(), otherEnd);
					}
					result.add(imageEdge);
					mappings.add(deToAdd.reverse());
					mappings.add(imageEdge);
					//System.out.println("added in "+edgeToAddIn);
					toAddIn.remove(edgeToAddIn);
					break goThroughEdgesToAdd;
				} else { //move on to next edge in
				}
				if (!edgeToAddIt.hasNext() && !shouldReverse) {
					shouldReverse = true;
					edgeToAddIt = toAddIn.iterator();
				}
			}
		}
		//System.out.println("returning "+result);
		//System.out.println("result has "+result.nodes.size()+" nodes and "
		//                                      +result.edges.size()+"edges");
		return result;
	}

	private static Position getPosition(
		int posNumber,
		int totalNum,
		int arrayNumber) { //System.out.print("node at array place "+arrayNumber+" at position number "+
		//                                   posNumber+" in "+positions);
		if (positions[arrayNumber] == null) {
			positions[arrayNumber] =
				Position.getRadialPosition(posNumber, totalNum, 0);
		}
		//System.out.println(" is "+positions[arrayNumber]);
		return positions[arrayNumber];
	}
	//////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////
	private static void reduceParentSet(Graph g, Vector parents) {
		parentSet = new TreeSet(parents); // ignores duplicate parents
		Iterator parentIt = parents.iterator(); // iterate through bag
		while (parentIt.hasNext())
			// so we can remove parents which are irrelevant
			{
			Zone z = (Zone) (parentIt.next());
			Iterator nodeIt = g.nodes().iterator();
			boolean foundMatch = false;
			NodeSearch : while (nodeIt.hasNext()) {
				Node n = (Node) (nodeIt.next());
				if (n.label1().equals(z.getContainingContours())) {
					foundMatch = true;
					break NodeSearch;
				}
			}
			if (!foundMatch) {
				parentSet.remove(z);
			}
		} // now parentSet only includes zones for nodes in Graph g
		Iterator nodeIt = g.nodes().iterator(); // also we want to remove the
		Node smallestNode = null; // smallest zone = outside zone
		NodeSearch : while (nodeIt.hasNext()) {
			Node n = (Node) (nodeIt.next());
			if (smallestNode == null
				|| smallestNode.label1().containsAll(n.label1())) {
				smallestNode = n;
				break NodeSearch;
			}
		} // having found the smallest node, now remove it if it appears
		parentIt = parentSet.iterator(); // as a parent
		Zone smallestZone = null;
		while (parentIt.hasNext()) {
			Zone z = (Zone) (parentIt.next());
			//System.out.println("remove parent "+z+"?");
			if (z.getContainingContours().equals(smallestNode.label1())) {
				//System.out.println("yes");
				smallestZone = z;
			}
		}
		if (smallestZone != null) {
			parentSet.remove(smallestZone);
		}
	}
	//////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////
	private static Graph mapInEdgesRoundCircle(Graph g) {
		Graph result = new Graph();
		mappings = new Vector();
		int totalNum = 2 * (g.nodes().size() - 1) + 2 * parentSet.size();
		//System.out.println("declaring position array");
		int posNumber = 0;
		int nodeNumber = 0;
		positions = new Position[totalNum];
		tree = g.getSpanningTree();
		//System.out.println("tree is "+tree);
		Vector treeCycle = FaceCycle.faceCycles(tree);
		fc = (FaceCycle) (treeCycle.iterator().next());
		Iterator treeCycleIt = fc.directedEdges.iterator();
		DirectedEdge de = (DirectedEdge) (treeCycleIt.next());
		//System.out.println(" got edge "+de);
		Node nf = null;
		Node nt = de.from();
		NodeAllowingDuplicates nfd = null;
		NodeAllowingDuplicates ntd = 
			new NodeAllowingDuplicates(nt.label1(), nt.label2());
		//System.out.println("calling for 0th position");
		ntd.setPosition(getPosition(posNumber, totalNum, nodeNumber));
		result.add(ntd);
		NodeAllowingDuplicates startNode = ntd;
		//System.out.println("add node "+ntd);
		treeCycleIt = fc.directedEdges.iterator();
		boolean wasAtParent = false;
		Iterator parentIt = parentSet.iterator();
		findParent : while (parentIt.hasNext()) {
			Zone z = (Zone) (parentIt.next());
			if (z.getContainingContours().equals(nt.label1())) {
				parentSet.remove(z);
				wasAtParent = true;
				break findParent;
			}
		}
		while (treeCycleIt.hasNext()) {
			de = (DirectedEdge) (treeCycleIt.next());
			// check through parents for a zone which shares label with nf
			// ie give extra space after nodes which need it
			nodeNumber++;
			nf = nt;
			nfd = ntd;
			if (wasAtParent) {
				posNumber++;
				//System.out.println("was a parent - posNumber increases to "+posNumber);
			}
			//System.out.println(""+posNumber+": we have node "+nf);
			posNumber++;
			//System.out.println("posNumber increments to "+posNumber);
			nt = de.to();
			// check through parents for a zone which shares label with nf
			// ie give extra space after nodes which need it
			//System.out.println("parentSet is "+parentSet);
			parentIt = parentSet.iterator();
			wasAtParent = false;
			findParent : while (parentIt.hasNext()) {
				Zone z = (Zone) (parentIt.next());
				//System.out.println("look out ahead for zone "+z);
				if (z.getContainingContours().equals(nt.label1())) {
					parentSet.remove(z);
					posNumber++;
					//System.out.println("after removing parent "+z+", posNumber is "+posNumber);
					wasAtParent = true;
					break findParent;
				}
			}
			ntd = new NodeAllowingDuplicates(nt.label1(),nt.label2());
			if (posNumber < totalNum) {
				//System.out.println("calling for position at posNumber "+posNumber);
				ntd.setPosition(getPosition(posNumber, totalNum, nodeNumber));
			} else {
				ntd = startNode;
			}
			Edge e = Edge.getEdge(de.from(), de.to());
			//System.out.println("@@@@@@@@@@@@@@@@@@@@@@edge e is "+e);
			Edge ed = e.substitute(nf, nfd);
			Edge edd = ed.substitute(nt, ntd);
			//System.out.println("add edge "+edd);
			//System.out.println("add node "+ntd);
			result.add(edd);
			result.add(ntd);
			//System.out.println("result has "+result.nodes.size()+" nodes and "
			//                                    +result.edges.size()+"edges");
			// de has moved to edd - remember this for the next step
			mappings.add(de);
			mappings.add(edd);
			//System.out.println(" got edge "+de);
		}
		//System.out.println("mappings are "+mappings);
		return result;
	}
}
