package rwd.views.abstracttoconcrete.forcemodel;

import java.util.*;
import java.io.Serializable;
import java.awt.*;
import java.awt.geom.*;
import rwd.abstractsyntax.eulerdiagram.Contour;
import rwd.concretesyntax.eulerdiagram.*;

/**
 * @author  $Author: op $
 * @version $Id: Diagram.java,v 1.13 2004/10/20 10:08:39 op Exp $
 */
public class Diagram implements Serializable, Constants {
    
    public Diagram(Graph graph) {
        _nodes = graph.getNodes();
        _contours = graph.getContours();
    }
    
    // Draw this graph on the supplied Graphics2D context.
    // THIS METHOD IS NOT ACTUALLY REQUIRED ANY LONGER
    /*public void draw(Graphics2D g, int width, int height) {
        
        // Draw contours.
        g.setColor(Color.black);
        g.setStroke(new BasicStroke(CONTOUR_BORDER_WIDTH));
        Iterator contourIt = _contours.keySet().iterator();
        while (contourIt.hasNext()) {
            Character label = (Character) contourIt.next();
            char ch = label.charValue();
            Polygon polygon = (Polygon) _contours.get(label);
            if (polygon == null) {
                continue;
            }
            g.setColor(new Color((ch * 153) % 256, (ch * 102) % 256, (ch * 51) % 256, 255));
            g.draw(polygon);
            //Point2D rightMost = polygon.getCurrentPoint();
            //g.drawString(label.toString(), (int) rightMost.getX() + 4, (int) rightMost.getY());
            g.setColor(new Color((ch * 153) % 256, (ch * 102) % 256, (ch * 51) % 256, CONTOUR_OPACITY));
            g.fill(polygon);
        }
        
    }*/
    
    public void addContours() {
        Iterator contourIt = _contours.keySet().iterator();
        while (contourIt.hasNext()) {
            Character label = (Character) contourIt.next();
            addContour(label);
        }
    }
    
    public void addContour(Character label) {
        // Find graph points that are involved with this contour.
        Vector graphNodes = new Vector();
        char ch = label.charValue();
        Iterator nodeIt = _nodes.iterator();
        while (nodeIt.hasNext()) {
            Node node = (Node) nodeIt.next();
            String nodeLabel = node.getLabel();
            if (nodeLabel.indexOf(ch) >= 0) {
                graphNodes.add(node);
            }
        }
        
        // Find center point of graph nodes.
        double minX = Double.POSITIVE_INFINITY;
        double maxX = Double.NEGATIVE_INFINITY;
        double minY = Double.POSITIVE_INFINITY;
        double maxY = Double.NEGATIVE_INFINITY;
        nodeIt = graphNodes.iterator();
        while (nodeIt.hasNext()) {
            Node node = (Node) nodeIt.next();
            if (node.getX() > maxX) {
                maxX = node.getX();
            }
            if (node.getX() < minX) {
                minX = node.getX();
            }
            if (node.getY() > maxY) {
                maxY = node.getY();
            }
            if (node.getY() < minY) {
                minY = node.getY();
            }
        }
        double safeOutsideDistance = Math.max(maxX - minX, maxY - minY) + IDEAL_CONTOUR_RADIUS;

        Point2D mean = meanPoint(graphNodes);
        double xMean = mean.getX();
        double yMean = mean.getY();
        
        Vector contourNodes = new Vector();
        int pointCount = POINTS_PER_CONTOUR;
        for (int n = 0; n < pointCount; n++) {
            double angle = (n * Math.PI * 2) / pointCount;
            
            // This is where the contour point will start off.
            double x = xMean + safeOutsideDistance * Math.cos(angle);
            double y = yMean + safeOutsideDistance * Math.sin(angle);
            
            // This is the unit vector that it shall move by.
            double dx = (x - xMean) / safeOutsideDistance;
            double dy = (y - yMean) / safeOutsideDistance;
            
            // Move the node closer until the force model balances.
            double potential = potential(xMean, yMean, x, y, graphNodes);
            while (distance(xMean, yMean, x, y) > 1) {
                x -= dx;
                y -= dy;
                double newPotential = potential(xMean, yMean, x, y, graphNodes);
                //if (newPotential > potential) {
                if (newPotential < 0) {
                    break;
                }
                potential = newPotential;
            }
            
            Node node = new Node();
            node.setX(x);
            node.setY(y);
            contourNodes.add(node);
        }
        
        Polygon polygon = new Polygon();
        Iterator contourNodeIt = contourNodes.iterator();
        Node firstNode = (Node) contourNodeIt.next();
        polygon.addPoint((int) firstNode.getX(), (int) firstNode.getY());
        while (contourNodeIt.hasNext()) {
            Node node = (Node) contourNodeIt.next();
            polygon.addPoint((int) node.getX(), (int) node.getY());
        }
        //path.lineTo((float) firstNode.getX(), (float) firstNode.getY());
        
        _contours.put(label, polygon);
    }
    
    private Point2D meanPoint(Vector nodes) {
        double totalX = 0;
        double totalY = 0;
        for (int i = 0; i < nodes.size(); i++) {
            Node node = (Node) nodes.get(i);
            totalX += node.getX();
            totalY += node.getY();
        }
        return new Point2D.Double(totalX / nodes.size(), totalY / nodes.size());
    }
    
    private double potential(double xCenter, double yCenter, double x, double y, Vector graphNodes) {
        double potential = 0;
        // Attraction towards middle point...
        potential += (distance(xCenter, yCenter, x, y) - (graphNodes.size() * 15)) / IDEAL_CONTOUR_RADIUS;
        // Repulsions...
        for (int i = 0; i < graphNodes.size(); i++) {
            Node node = (Node) graphNodes.get(i);
            potential -= IDEAL_CONTOUR_RADIUS / distance(x, y, node.getX(), node.getY()); 
            //potential -= Math.pow(IDEAL_CONTOUR_RADIUS, 2) / Math.pow(distance(x, y, node.getX(), node.getY()), 2);
        }
        return potential;
    }
    
    private double distance(double x1, double y1, double x2, double y2) {
        double dx = x2 - x1;
        double dy = y2 - y1;
        return Math.sqrt(dx * dx + dy * dy);
    }
    
    public Vector getContourPolygons() {
        Vector contourPolygons = new Vector();
        Iterator it = _contours.keySet().iterator();
        while (it.hasNext()) {
            Character label = (Character) it.next();
            Polygon polygon = (Polygon) _contours.get(label);
            contourPolygons.add(new ConcreteContour(Contour.getContour(label.toString()), polygon));
        }
        return contourPolygons;
    }

    private LinkedHashSet _nodes;    // Node
    private HashMap _contours; // Char -> Polygon
    
}
