package rwd.views.graph;

import java.util.TreeSet;
import java.util.Iterator;
import java.util.Vector;

import rwd.abstractsyntax.eulerdiagram.Zone;
import rwd.abstractsyntax.spiderdiagram.UnitaryDiagram;
//import rwd.views.abstracttoconcrete.oldaed.*;
/**
* A rwd.views.graph has nodes and edges between those nodes.
*/
public class Graph {
	private TreeSet nodes;
	private TreeSet edges;
	private TreeSet labels;

	public TreeSet nodes() {
		return nodes;
	}

	public TreeSet edges() {
		return edges;
	}

	public TreeSet labels() {
		return labels;
	}
	////////////////////////////////////////////////////////////////////////////
	/**
	 * Initialises a Graph object with no Nodes and no Edges.
	 */
	public Graph() {
		nodes = new TreeSet();
		edges = new TreeSet();
		labels = new TreeSet();
	}
	////////////////////////////////////////////////////////////////////////////
	/**
	 * Initialises a Graph object with given Nodes and given Edges.
	 * If an Edge has ends not included in the NodeSet, these extra
	 * Nodes are included as well.
	 */
	public Graph(TreeSet nodes, TreeSet edges) {
		this.nodes = nodes;
		this.edges = edges;
		//System.out.println("building labels from "+nodes);
		labels = new TreeSet();
		Iterator i = nodes.iterator();
		while (i.hasNext()) {
			Node n = (Node) (i.next());
			//System.out.println("add in labels "+nodeLabel);
			labels.addAll(n.label1());
			labels.addAll(n.label2());
		}
		//System.out.println("labels is "+labels());
	}
	////////////////////////////////////////////////////////////////////////////
	public String toString() // required by OSElement
	{
		return "Nodes: " + nodes + " and Edges: " + edges;
	}
	////////////////////////////////////////////////////////////////////////////
	/**
	 * Adds a Node to the Graph's nodes attribute.
	 * Returns false if the Node is already there.
	 */
	public void add(Node n) {
		nodes.add(n);
		labels.addAll(n.label1());
		labels.addAll(n.label2());
	}
	////////////////////////////////////////////////////////////////////////////
	/**
	 * Adds an Edge to the Graph's edges attribute.
	 * Returns false if the Edge is already there.
	 * Also adds the nodes at the ends of the edge.
	 */
	public void add(Edge e) {
		edges.add(e);
	}
	public boolean checkAndAdd(Edge e) {
		if (nodes.containsAll(e.nodes)) {
			edges.add(e);
			return true;
		} else {
			return false;
		}
	}
	///////////////////////////////////////////////////////////////////////////
	/**
	 * Returns the set of neighbours of given Nodes in the this Graph context.
	 * If this AbNode isn't in the CGraph, the method will return a null object.
	 */
	private TreeSet addNbrs(TreeSet nodes) {
		Iterator in = nodes.iterator();
		TreeSet neighbours = (TreeSet) (nodes.clone());
		while (in.hasNext()) {
			Node n = (Node) (in.next());
			Iterator ie = edges.iterator();
			while (ie.hasNext()) {
				Edge e = (Edge) (ie.next());
				if (e.ends().contains(n)) {
					//System.out.println("adding ends of "+e);
					neighbours.addAll(e.nodes);
					//System.out.println("neighbours is "+neighbours);
				}
			}
		}
		return neighbours;
	}
	/////////////////////////////////////////////////////////////////////////////
	private TreeSet getReachFrom(Node n) {
		TreeSet component = new TreeSet();
		component.add(n);
		Iterator i = nodes.iterator();
		while (i.hasNext()) // just a count
		{
			i.next(); // increment the count
			component = addNbrs(component);
		}
		return component;
	}
	/////////////////////////////////////////////////////////////////////////////
	private int countComponents() {

		//System.out.println("counting components of " + this);
		int result = 0;
		if (nodes.size() == 0) {
			return 0;
		}
		TreeSet allNodes = (TreeSet) (nodes.clone());
		while (allNodes.size() > 0) {
			//System.out.println("looking at node set "+allNodes);
			Node n = (Node) (allNodes.iterator().next());
			//System.out.println("looking at node "+n);
			TreeSet reach = getReachFrom(n);
			//System.out.println("reach from node "+n+" is "+reach);
			result++;
			//System.out.println("reduce node set by "+reach);
			Iterator reachIt = reach.iterator();
			while(reachIt.hasNext())
			{ Node reachedNode = (Node)reachIt.next();
				//System.out.println("reached node "+reachedNode+","+reachedNode.hashCode());
				allNodes.remove(reachedNode);
				//System.out.println("node set is now "+allNodes);
				//Iterator allIt = allNodes.iterator();
				//while(allIt.hasNext())
				//{ Node aNode = (Node)allIt.next();
					//System.out.println("a node "+aNode+","+aNode.hashCode());
				//}
			}
			//System.out.println("reduced nodes set to "+allNodes);
		}
		//System.out.println("it has "+result+" components");
		return result;
	}
	public Vector components() {
		Vector components = new Vector();
		if (nodes.size() == 0) {
			return components;
		}
		TreeSet allNodes = (TreeSet) (nodes.clone());
		while (allNodes.size() > 0) {
			//System.out.println("looking at node set "+allNodes);
			Node n = (Node) (allNodes.iterator().next());
			TreeSet reach = getReachFrom(n);
			components.add(new Graph(reach, new TreeSet()));
			//System.out.println("reduce node set by "+reach);
			allNodes.removeAll(reach);
		}
		Iterator edgeIt = edges.iterator();
		while (edgeIt.hasNext()) {
			Edge e = (Edge) (edgeIt.next());
			Iterator componentIt = components.iterator();
			compLoop : while (componentIt.hasNext()) {
				Graph component = (Graph) (componentIt.next());
				if (component.checkAndAdd(e)) {
					break compLoop;
				}
			}
		}
		return components;
	}
	/////////////////////////////////////////////////////////////////////////////
	/**
	 * Returns true for connected Graph objects.
	 */
	private boolean isConnected() {
		return countComponents() == 1;
	}
	/////////////////////////////////////////////////////////////////////////////
	public String getConnectivityFix() {
		return "add" + getFix();
	}
	////////////////////////////////////
	private String getFix() {
		if (!isConnected()) {
			return "" + getDisconnectedFix();
		}
		TreeSet firstFix = null;
		Iterator i = labels.iterator();
		contourLoop : while (i.hasNext()) {
			Comparable c = (Comparable) (i.next());
			// make a subgraph including only nodes including c
			Graph subGraph = new Graph();
			Iterator nodeIt = nodes.iterator();
			while (nodeIt.hasNext()) {
				Node n = (Node) (nodeIt.next());
				if (n.label1().contains(c)) {
					subGraph.add(n);
				}
			}
			Iterator edgeIt = edges.iterator();
			while (edgeIt.hasNext()) {
				Edge e = (Edge) (edgeIt.next());
				if (subGraph.nodes.containsAll(e.ends())) {
					subGraph.add(e);
				}
			}
			if (!subGraph.isConnected()) {
				firstFix = subGraph.getDisconnectedFix();
				break contourLoop;
			}
			// now build rwd.views.graph which excludes c
			subGraph = new Graph();
			nodeIt = nodes.iterator();
			while (nodeIt.hasNext()) {
				Node n = (Node) (nodeIt.next());
				if (!n.label1().contains(c)) {
					subGraph.add(n);
				}
			}
			edgeIt = edges.iterator();
			while (edgeIt.hasNext()) {
				Edge e = (Edge) (edgeIt.next());
				if (subGraph.nodes.containsAll(e.ends())) {
					subGraph.add(e);
				}
			}
			if (!subGraph.isConnected()) {
				firstFix = subGraph.getDisconnectedFix();
				break contourLoop;
			}
		}
		if (firstFix == null) {
			return "";
		}
		///// firstFix is a set of Nodes which, when added, fixes something
		///// add these nodes to this and iterate
		Iterator fixIt = firstFix.iterator();
		Graph extendedGraph =
			new Graph(
				(TreeSet) (this.nodes.clone()),
				(TreeSet) (this.edges.clone()));
		while (fixIt.hasNext()) {
			Node n = (Node) (fixIt.next());
			extendedGraph.add(n);
		}
		// populate extendedGraph with edges
		Iterator extend1 = extendedGraph.nodes.iterator();
		while (extend1.hasNext()) {
			Node n1 = (Node) (extend1.next());
			Iterator extend2 = extendedGraph.nodes.iterator();
			while (extend2.hasNext()) {
				Node n2 = (Node) (extend2.next());
				if (n1.symmDiff(n2).size() == 1)
					extendedGraph.add(Edge.getEdge(n1, n2));
			}
		}
		return "" + firstFix + extendedGraph.getFix();
	}
	//////////////////////////////////////////////////////////////////
	private Graph getGraphWithoutContour(Comparable c) { //System.out.println("building rwd.views.graph without "+c);
		Graph g = new Graph();
		Iterator i = nodes.iterator();
		while (i.hasNext()) {
			Node n = (Node) (i.next());
			g.add(n);
		}
		i = edges.iterator();
		while (i.hasNext()) {
			Edge e = (Edge) (i.next());
			if (e.label().compareTo(c) != 0) {
				g.add(e);
			}
		}
		return g;
	}
	//////////////////////////////////////////////////////////////////
	private TreeSet getDisconnectedFix() { //System.out.println("fixing disconnectedness of "+this);
		//System.out.println("nodes is "+nodes);
		TreeSet result = new TreeSet();
		TreeSet reach1 = getReachFrom((Node) (nodes.iterator()).next());
		//System.out.println("reach1 is "+reach1);
		if (reach1.equals(nodes)) {
			return result;
		}
		TreeSet reach2 = null;
		Iterator nodeIt = nodes.iterator();
		nodeLoop : while (nodeIt.hasNext()) {
			Node n2 = (Node) (nodeIt.next());
			if (!reach1.contains(n2)) {
				reach2 = getReachFrom(n2);
				break nodeLoop;
			}
		}
		//System.out.println("reach2 is "+reach2);
		// found two components - now link them together
		Iterator i1 = reach1.iterator();
		Iterator i2 = reach2.iterator();
		Node first = (Node) (i1.next());
		Node second = (Node) (i2.next());
		int steps = first.distance(second);
		i1 = reach1.iterator();
		while (i1.hasNext()) {
			Node n1 = (Node) (i1.next());
			i2 = reach2.iterator();
			while (i2.hasNext()) {
				Node n2 = (Node) (i2.next());
				if (n1.distance(n2) < steps) {
					first = n1;
					second = n2;
					steps = n1.distance(n2);
				}
			}
		}
		// found two closest nodes
		// build rwd.views.graph with path added in and then use recursion
		Graph bigger =
			new Graph((TreeSet) (nodes.clone()), (TreeSet) (edges.clone()));
		TreeSet path = first.path(second);
		if (path.size() == 0) {
			return result;
		}
		Iterator i = path.iterator();
		while (i.hasNext()) {
			Node n = (Node) (i.next());
			bigger.add(n);
			Iterator bigI = bigger.nodes.iterator();
			while (bigI.hasNext()) {
				Node nAlready = (Node) (bigI.next());
				if (n.distance(nAlready) == 1) {
					bigger.add(Edge.getEdge(n, nAlready));
				}
			}
		}
		TreeSet nextFix = bigger.getDisconnectedFix();
		if (nextFix.size() == 0) {
			return path;
		} else {
			path.addAll(nextFix);
			return path;
		}
	}
	/*	/////////////////////////////////////////////////////////////////////////
		public static Graph maxDual(OldAbstractEulerDiagram aed) {
			Graph result = new Graph();
			Iterator zoneIt = aed.zones().iterator();
			while (zoneIt.hasNext()) {
				Zone z = (Zone) (zoneIt.next());
				result.add(Node.getNode(
				new TreeSet(z.getContainingContours()), 
				new TreeSet(z.getExcludingContours())));
			}
			// compare zones pairwise
			zoneIt = aed.zones().iterator();
			while (zoneIt.hasNext()) {
				Zone z1 = (Zone) (zoneIt.next());
				Iterator zoneIt2 = aed.zones().iterator();
				while (zoneIt2.hasNext() && !zoneIt2.next().equals(z1)) {
				}
				while (zoneIt2.hasNext()) {
					Zone z2 = (Zone) (zoneIt2.next());
					// compare z1 and z2
					Vector contours1 = z1.getContainingContours();
					Vector contours2 = z2.getContainingContours();
					if ((contours1.size() == contours2.size() + 1
						&& contours1.containsAll(contours2))
						|| (contours2.size() == contours1.size() + 1
							&& contours2.containsAll(contours1))) {
						result.add(
							Edge.getEdge(
								Node.getNode(
								new TreeSet(contours1),
								new TreeSet(z1.getExcludingContours())),
								Node.getNode(
								new TreeSet(contours2),
								new TreeSet(z2.getExcludingContours()))));
					}
				}
			}
			//System.out.println("maxDual is "+result+" with labels "+result.labels());
			//if(result.passesConnectivity())
			{
				return result;
			}
			//else
			//{ return null; }
		}
		*/
	/////////////////////////////////////////////////////////////////////////
	public static Graph maxDual(UnitaryDiagram ud) {
		Graph result = new Graph();
		Iterator zoneIt = ud.getCloneOfZoneSet().iterator();
		while (zoneIt.hasNext()) {
			Zone z = (Zone) (zoneIt.next());
			Vector a = z.getContainingContours();
			result.add(
				Node.getNode(
					new TreeSet(a),
					new TreeSet(z.getExcludingContours())));
		}
		// compare zones pairwise
		zoneIt = ud.getCloneOfZoneSet().iterator();
		while (zoneIt.hasNext()) {
			Zone z1 = (Zone) (zoneIt.next());
			Iterator zoneIt2 = ud.getCloneOfZoneSet().iterator();
			while (zoneIt2.hasNext() && !zoneIt2.next().equals(z1)) {
			}
			while (zoneIt2.hasNext()) {
				Zone z2 = (Zone) (zoneIt2.next());
				// compare z1 and z2
				TreeSet contours1 = new TreeSet(z1.getContainingContours());
				TreeSet contours2 = new TreeSet(z2.getContainingContours());
				if ((contours1.size() == contours2.size() + 1
					&& contours1.containsAll(contours2))
					|| (contours2.size() == contours1.size() + 1
						&& contours2.containsAll(contours1))) {
					result.add(
						Edge.getEdge(
							Node.getNode(
								contours1,
								new TreeSet(z1.getExcludingContours())),
							Node.getNode(
								contours2,
								new TreeSet(z2.getExcludingContours()))));
				}
			}
		}
		//System.out.println("maxDual is "+result+" with labels "+result.labels());
		//if(result.passesConnectivity())
		{
			return result;
		}
		//else
		//{ return null; }
	}
	//////////////////////////////////////////////////////////////////////
	public boolean passesConnectivity() {
		//System.out.println("checking connectivity of " + this);
		if (!isConnected()) {
			return false;
		}
		//System.out.println("is connected : now check subgraphs");

		Iterator i = labels.iterator();
		while (i.hasNext()) {
			Comparable c = (Comparable) (i.next());
			Graph withoutC = getGraphWithoutContour(c);
			//System.out.println("got a subgraph without " + c + ":" + withoutC);
			if (withoutC.countComponents() > 2) {
				return false;
			}
		}
		return true;
	}
	//////////////////////////////////////////////////////////////////////
	Vector edgesRemoved = new Vector();
	Vector history = new Vector(); // stores sets of removed edges

	public void clearHistory() {
		edgesRemoved = new Vector();
		history = new Vector();
	}

	public Graph shrink() { // first look for an edge to remove, maintaining connectivity
		//System.out.println("try to shrink "+this+" labelled "+labels());
		//TreeSet nextEdgeSet = new TreeSet();
		Iterator edgeIt = null;
		TreeSet edgeSetCopy = new TreeSet();
		Iterator histIt = null;
		Iterator compIt = null;
		Vector checkTwoComponents = new Vector();
		Vector newSubGraphs = null;
		Graph subGraph = null;
		Graph newSubGraph = null;
		Iterator labelIt = labels().iterator();
		while (labelIt.hasNext()) {
			checkTwoComponents.add(
				getGraphWithoutContour((Comparable) (labelIt.next())));
		}
		// start with a graph which possibly has some edges
		// removed - stored in edgesRemoved
		// also I know checkTwoComponents - keeps track of which edges should be
		// included
		//System.out.println("graph is "+this);
		//System.out.println("edgesRemoved are "+edgesRemoved);
		//System.out.println("checkTwoComponents is "+checkTwoComponents);
		//System.out.println("history is "+history);

		edgeSetCopy = (TreeSet) (edges.clone());
		edgeIt = edgeSetCopy.iterator();
		nextEdge : while (edgeIt.hasNext()) {
			Edge e = (Edge) (edgeIt.next());
			//System.out.println("shrink loop:remove "+e+"?");
			TreeSet ends = e.ends();
			//iterate through ends checking if contaiing contours = 0
			Iterator i = ends.iterator();
			while (i.hasNext()) {
				Node n = (Node) i.next();
				if (n.label1().size() == 0) {
					continue nextEdge;
				}
			}
			edgesRemoved.add(e);
			// check history first
			histIt = history.iterator();
			while (histIt.hasNext()) {
				Vector historical = (Vector) (histIt.next());
				if (historical.equals(edgesRemoved)) {
					edgesRemoved.remove(e);
					continue nextEdge;
				}
			}
			// not in history: add it and continue
			Vector al = (Vector) (edgesRemoved.clone());
			history.add(al);
			//System.out.println("history grows to "+history);
			// check does it pass connectivity
			// but use checkTwoComponents
			//System.out.println("see if removing "+e+" breaks connectivity");
			boolean passesAll = true;
			newSubGraphs = new Vector();
			compIt = checkTwoComponents.iterator();
			compLoop : while (compIt.hasNext()) {
				subGraph = (Graph) (compIt.next());
				newSubGraph =
					new Graph(
						subGraph.nodes,
						(TreeSet) (subGraph.edges.clone()));
				newSubGraph.edges.remove(e);
				newSubGraphs.add(newSubGraph);
				if (newSubGraph.countComponents() != 2) {
					//System.out.println("newSubGraph "
					//                   +newSubGraph+" doesn't have 2 components");
					passesAll = false;
					break compLoop;
				}
			}
			if (passesAll) // BINGO - can remove this edge!!
				{ //System.out.println("can remove "+e);
				checkTwoComponents = newSubGraphs;
				this.edges.remove(e);
				return this;
			} else // can't remove this edge without violating connectivity
				{
				edgesRemoved.remove(e);
			} // goes on to next edge
		}
		// here, I went through every edge and can't remove any
		// so I have to start putting some back from edgesRemoved
		if (edgesRemoved.size() != 0) {
			Iterator i = edgesRemoved.iterator();
			Edge e = null;
			while (i.hasNext()) {
				e = (Edge) (i.next());
			}
			edgesRemoved.remove(e);
			edges.add(e);
			//System.out.println("replacing "+e);
			return shrink();
		} else {
			return null;
		}
	}
	//////////////////////////////////////////////////////////////////////
	public Vector blocksAndCuts() { // returns a Vector with blocks
		Vector cutVxs = new Vector();
		{ // create a code block : local variables are i, n, nodesCopy, etc
			Iterator i = nodes.iterator();
			while (i.hasNext()) {
				Node n = (Node) (i.next()); // we will remove this
				// create a copy of nodes, edges
				TreeSet nodesCopy = (TreeSet) (nodes.clone());
				TreeSet edgesCopy = (TreeSet) (edges.clone());
				nodesCopy.remove(n);
				// remove edges adjacent to n, see if remainder is connected
				Iterator j = edges.iterator();
				while (j.hasNext()) {
					Edge e = (Edge) (j.next());
					if (e.nodes.contains(n)) {
						edgesCopy.remove(e);
					}
				}
				Graph temp = new Graph(nodesCopy, edgesCopy);
				// see if this rwd.views.graph is connected
				if (!temp.isConnected()) // found cutvx
					{
					cutVxs.add(n);
				}
			}
		}
		//System.out.println("cutvxs are "+cutVxs);

		// next: find blocks
		Vector result = new Vector();
		// first "block" is whole rwd.views.graph
		result.add(
			new Graph((TreeSet) (nodes.clone()), (TreeSet) (edges.clone())));
		// go through each cutvertex in turn, splitting graphs
		Iterator it = cutVxs.iterator();
		while (it.hasNext()) {
			Node cutv = (Node) (it.next());
			// for each cutvx, there is one rwd.views.graph in result which has that vx
			Iterator blockIt = result.iterator();
			searchForBlock : while (blockIt.hasNext()) {
				Graph graph;
				try {
					graph = (Graph) (blockIt.next());
				} catch (ClassCastException x) {
					graph = (Graph) (blockIt.next());
				}
				if (graph.nodes.contains(cutv)) {
					// split the rwd.views.graph up into two (or more) bits
					TreeSet nodesCopy = (TreeSet) (graph.nodes.clone());
					TreeSet edgesCopy = (TreeSet) (graph.edges.clone());
					nodesCopy.remove(cutv);
					// remove adjacent edges
					{
						Iterator edgeIt = graph.edges.iterator();
						Edge e;
						while (edgeIt.hasNext()) {
							e = (Edge) (edgeIt.next());
							if (e.nodes.contains(cutv)) {
								edgesCopy.remove(e);
							}
						}
					}
					Graph temp = new Graph(nodesCopy, edgesCopy);
					// this must be disconnected - go through nodes
					Iterator itNodes = nodesCopy.iterator();
					Node n;
					TreeSet reach;
					TreeSet usedNodes = new TreeSet();
					Edge edge;
					boolean isFirstBlock = true;
					while (itNodes.hasNext()) {
						n = (Node) (itNodes.next());
						if (!usedNodes.contains(n)) {
							//System.out.println("building from node "+na);
							reach = temp.getReachFrom(n);
							usedNodes.addAll(reach);
							TreeSet reachEdges = new TreeSet();
							Iterator edgeIt = edgesCopy.iterator();
							while (edgeIt.hasNext()) {
								edge = (Edge) (edgeIt.next());
								if (reach
									.contains(edge.nodes.iterator().next())) {
									reachEdges.add(edge);
								}
							}
							Graph block = new Graph(reach, reachEdges);
							block.add(cutv);
							edgeIt = edges.iterator();
							while (edgeIt.hasNext()) {
								edge = (Edge) (edgeIt.next());
								Iterator eit = edge.nodes.iterator();
								Node oneNode = (Node) (eit.next());
								Node anotherNode = (Node) (eit.next());
								if ((reach.contains(oneNode)
									&& anotherNode == cutv)
									|| (reach.contains(anotherNode)
										&& oneNode == cutv)) {
									block.add(edge);
								}
							}
							Graph newGraph = new Graph(reach, reachEdges);
							if (!isFirstBlock) // add newGraph at the end
								{ //System.out.println("adding later block");
								result.add(cutv);
								result.add(newGraph);
							} else // replace rwd.views.graph with newGraph
								{ //System.out.println("adding first block");
								int j = result.indexOf(graph);
								result.remove(graph);
								//result.add(j,cutv);
								result.add(j, newGraph);
								isFirstBlock = false; // for  next time
							}
						}
					}
					break searchForBlock;
				}
			}
		}
		return result;
	}
	////////////////////////////////////////////////////////////////
	/*	public static void startAgain() {
			Node.startAgain(); // clears libraries
			Edge.startAgain(); // clears libraries
		}
	*/ ////////////////////////////////////////////////////////////////
	public Graph getSpanningTree() {
		if (!this.isConnected()) {
			return null;
		}
		if (this.nodes.size() < 2) // a rwd.views.graph with 0 or 1 nodes
			{
			return this;
		} // is its own
		Graph result = new Graph();
		//will grow to be a tree
		// find the null vertex, if it's there, and add one vertex to result
		Iterator i = this.nodes.iterator();
		nodeLoop : while (i.hasNext()) {
			Node n = (Node) (i.next());
			if (n.label1().size() == 0) {
				result.add(n);
				break nodeLoop;
			}
		}
		if (result.nodes.size() == 0) {
			result.add(((Node) (nodes.iterator().next())));
		}
		//System.out.println("tree so far is "+result);
		while (result.edges.size() < nodes.size() - 1) {
			//System.out.println("extensible tree is "+result);
			TreeSet addableNodes = new TreeSet();
			TreeSet addableEdges = new TreeSet();
			// iterate through result.nodes, looking at their nbrs
			// in this, and add to addableNodes
			i = result.nodes.iterator();
			while (i.hasNext()) {
				Node n = (Node) (i.next());
				//System.out.println("considering node "+n);
				Iterator edgeIt = edges.iterator();
				edgeLoop : while (edgeIt.hasNext()) {
					Edge e = (Edge) (edgeIt.next());
					//System.out.println("considering edge "+e);
					Iterator endsIt = e.ends().iterator();
					Node n1 = (Node) (endsIt.next());
					Node n2 = (Node) (endsIt.next());
					//System.out.println("ends "+n1+" and "+n2);
					if (n1 == n
						&& !result.nodes.contains(n2)
						&& !addableNodes.contains(
							n2)) { //System.out.println("add "+n2);
						addableNodes.add(n2);
						addableEdges.add(e);
						//break edgeLoop;
					} else if (
						n2 == n
							&& !result.nodes.contains(n1)
							&& !addableNodes.contains(
								n1)) { //System.out.println("add "+n1);
						addableNodes.add(n1);
						addableEdges.add(e);
						//break edgeLoop;
					}
				}
			}
			//System.out.println("addableNodes are "+addableNodes);
			//System.out.println("addableEdges are "+addableEdges);
			i = addableNodes.iterator();
			while (i.hasNext()) {
				result.add((Node) (i.next()));
			}
			i = addableEdges.iterator();
			while (i.hasNext()) {
				result.add((Edge) (i.next()));
			}
			//System.out.println("growing tree has "+result.nodes.size()+" nodes");
		}
		//System.out.println("spanning tree of rwd.views.graph "+this+"\n is "+result);
		return result;
	}
	//////////////////////////////////////////////
	public int size() {
		return nodes.size();
	}

	/*	public static void main(String args[]) {
			Graph g = new Graph();
			TreeSet label;
			label = new TreeSet();
			Node n = Node.getNode(label);
			label = new TreeSet();
			label.add(OldContour.getContour("a"));
			Node na = Node.getNode(label);
			label = new TreeSet();
			label.add(OldContour.getContour("b"));
			Node nb = Node.getNode(label);
			label = new TreeSet();
			label.add(OldContour.getContour("c"));
			Node nc = Node.getNode(label);
			label = new TreeSet();
			label.add(OldContour.getContour("d"));
			Node nd = Node.getNode(label);
			label = new TreeSet();
			label.add(OldContour.getContour("c"));
			label.add(OldContour.getContour("d"));
			Node ncd = Node.getNode(label);
	
			g.add(n);
			//System.out.println(""+g.blocksAndCuts());
			g.add(na);
			g.add(Edge.getEdge(n, na));
			//System.out.println(""+g.blocksAndCuts());
			g.add(nb);
			g.add(Edge.getEdge(n, nb));
			//System.out.println(""+g.blocksAndCuts());
			g.add(nc);
			g.add(Edge.getEdge(n, nc));
			//System.out.println(""+g.blocksAndCuts());
			g.add(nd);
			g.add(Edge.getEdge(n, nd));
			//System.out.println(""+g.blocksAndCuts());
			g.add(ncd);
			g.add(Edge.getEdge(nc, ncd));
			g.add(Edge.getEdge(nd, ncd));
			//System.out.println(""+g.blocksAndCuts());
	
		}
		*/
}
