package rwd.views.graph;
import java.util.TreeSet;
import java.util.Vector;
import java.util.Iterator;
//import rwd.views.abstracttoconcrete.oldaed.OldContour;
import rwd.views.graph.Position;

/**
 * Nodes represent abstract nodes in a graph.
 * We have the option to specify positions and labels (treesets).
 */
public class Node implements Comparable {
	private static TreeSet nodeLibrary = new TreeSet();
	private static TreeSet alternateNodeLibrary = new TreeSet();
	private TreeSet label1;
	private TreeSet label2;
	private Position position;

	public Vector constraints;
	////////////////////////////////////////////////////////////////////////////
	/**
	 * Initialises an Node object with given label.
	 */
	protected Node(TreeSet label1, TreeSet label2) //constructor
	{
		this.label1 = label1;
		this.label2 = label2;
		try {
			nodeLibrary.add(this);
		} catch (ClassCastException x) {
			alternateNodeLibrary.add(this);
		}
	}
	////////////////////////////////////////////////////////////////////////////
	/**
	 * Returns a Node object with given label.
	 */
	public static Node getNode(TreeSet label1, TreeSet label2) {
		Iterator i = nodeLibrary.iterator();
		while (i.hasNext()) {
			Node n = (Node) (i.next());
			try {
				if (n.label1().equals(label1) && n.label2().equals(label2)) {
					return n;
				}
			} catch (ClassCastException x) {
			}

		}
		i = alternateNodeLibrary.iterator();
		while (i.hasNext()) {
			Node n = (Node) (i.next());
			if (n.label1().equals(label1) && n.label2().equals(label2)) {
				return n;
			}
		}
		return new Node(label1,label2);
	}
/*	////////////////////////////////////////////////////////////////////
	protected static void startAgain() {
		nodeLibrary = new TreeSet();
		alternateNodeLibrary = new TreeSet();
		//System.out.println("libraries have "+
		//(nodeLibrary.size()+alternateNodeLibrary.size())+" entries");
	}
*/	////////////////////////////////////////////////////////////////////
	protected Node along(Comparable c) {
		TreeSet newLabel1 = (TreeSet) (label1.clone());
		TreeSet newLabel2 = (TreeSet) (label2.clone());
		if (label1.contains(c)) {
			newLabel1.remove(c);
			newLabel2.add(c);
		} else {
			newLabel1.add(c);
			newLabel2.remove(c);
		}
		return getNode(newLabel1,newLabel2);
	}
	////////////////////////////////////////////////////////////////////
	protected TreeSet path(Node target) {
		TreeSet result = new TreeSet();
		TreeSet symDiff = symmDiff(target);
		if (symDiff.size() < 2) {
			return result;
		} else {
			Node next = this.along((Comparable) (symDiff.iterator().next()));
			result.add(next);
			TreeSet nextOne = next.path(target);
			if (nextOne == null) {
				return result;
			} else {
				result.addAll(nextOne);
				return result;
			}
		}
	}
	////////////////////////////////////////////////////////////////////////////
	public int compareTo(Object o) { //System.out.println("comparing "+this+" with "+o);
		Node other = null;
		try {
			other = (Node) o;
		} catch (ClassCastException x) { /**/
			System.out.println("class cast exception in Node");
		}
		TreeSet labela = this.label1();
		TreeSet labelb = other.label1();
		if (labela.size() < labelb.size()) {
			//System.out.println("returning "+(-1));
			return -1;
		} else if (
			labela.size() > labelb.size()) {
			//System.out.println("returning "+1);
			return +1;
		} else {
			Iterator i1 = labela.iterator();
			Iterator i2 = labelb.iterator();
			while (i1.hasNext()) {
				Comparable c1 = (Comparable) (i1.next());
				Comparable c2 = (Comparable) (i2.next());
				int n = c1.compareTo(c2);
				if (n != 0) { //System.out.println("returning "+n);
					return n;
				}
			}
		}
		labela = this.label2();
		labelb = other.label2();
		if (labela.size() < labelb.size()) {
			//System.out.println("returning "+(-1));
			return -1;
		} else if (
			labela.size() > labelb.size()) {
			//System.out.println("returning "+1);
			return +1;
		} else {
			Iterator i1 = labela.iterator();
			Iterator i2 = labelb.iterator();
			while (i1.hasNext()) {
				Comparable c1 = (Comparable) (i1.next());
				Comparable c2 = (Comparable) (i2.next());
				int n = c1.compareTo(c2);
				if (n != 0) { //System.out.println("returning "+n);
					return n;
				}
			}
		}
		
		if (this.position != null && other.position != null) {
			int result = this.position.compare(other.position);
			//System.out.println("returning "+result);
			return result;
		} else { //System.out.println("returning "+0);
			return 0;
		}
	}
	////////////////////////////////////////////////////////////////////////////
	public TreeSet label1() {
		return label1;
	}
	public TreeSet label2() {
		return label2;
	}
	////////////////////////////////////////////////////////////////////////////
	/**
	 * Returns the name of the Node.
	 */
	public String toString() { //if(position==null)
		{
			return "n(" + label1+","+label2+")";
		}
		//else
		//{ return ""+label+""+position; }
	}
	////////////////////////////////////////////////////////////////////////////
	public TreeSet symmDiff(Node n) {
		TreeSet countSet = new TreeSet();
		Iterator i = label1.iterator();
		TreeSet others = n.label1();
		while (i.hasNext()) {
			Object o = i.next();
			if (!others.contains(o)) {
				countSet.add(o);
			}
		}
		i = others.iterator();
		TreeSet these = label1();
		while (i.hasNext()) {
			Object o = i.next();
			if (!these.contains(o)) {
				countSet.add(o);
			}
		}
		return countSet;
	}
	////////////////////////////////////////////////////////////////////
	protected int distance(Node n) {
		return symmDiff(n).size();
	}
	////////////////////////////////////////////////////////////////////
	public Position getPosition() {
		return position;
	}
	////////////////////////////////////////////////////////////////////
	public void setPosition(Position p) {
		position = p;
	}
	////////////////////////////////////
	public void setConstraints(Vector constraints) {
		this.constraints = constraints;
	}

/*	public static void main(String args[]) {
		TreeSet label = new TreeSet();
		label.add("a");
		Node n = Node.getNode(label);
		TreeSet label2 = new TreeSet();
		label2.add("a");
		Node n2 = Node.getNode(label2);
		//System.out.println("node hashcodes are "+n.hashCode()+" "+n2.hashCode());
		label = new TreeSet();
		label.add(Contour.getContour("a"));
		n = Node.getNode(label);
		label2 = new TreeSet();
		label2.add(Contour.getContour("a"));
		n2 = Node.getNode(label2);
		//System.out.println("node hashcodes are "+n.hashCode()+" "+n2.hashCode());
	}
	*/
}
