// Created on 17-Feb-2004

package rwd.views.smoother;

import java.awt.Polygon;

import rwd.concretesyntax.eulerdiagram.ConcreteContour;

/**
 */
public class ContourStep {

	public ConcreteContour startcp;
	public ConcreteContour endcp;
	public int steps;
	public double[] xPoints;
	public double[] yPoints;
	public double[] xStep;
	public double[] yStep;


	ContourStep(ConcreteContour startcp, ConcreteContour endcp, int steps) {
		this.startcp = startcp;
		this.endcp = endcp;
		this.steps = steps;

		Polygon p = startcp.getPolygon();

		xPoints = new double[p.npoints];
		yPoints = new double[p.npoints];

		xStep = new double[p.npoints];
		yStep = new double[p.npoints];

		intializePoints();

		generateStepSize();

	}


	protected void intializePoints() {

		Polygon p = startcp.getPolygon();
		int[] xs = p.xpoints;
		int[] ys = p.ypoints;

		for (int i = 0; i <= p.npoints - 1; i++) {
			xPoints[i] = xs[i];
			yPoints[i] = ys[i];
		}
	}


	public void finalizePoints() {

		Polygon startp = startcp.getPolygon();
		int[] startxs = startp.xpoints;
		int[] startys = startp.ypoints;

		Polygon endp = endcp.getPolygon();
		int[] endxs = endp.xpoints;
		int[] endys = endp.ypoints;

		for (int i = 0; i <= startp.npoints - 1; i++) {
			startxs[i] = endxs[i];
			startys[i] = endys[i];
		}
	}



	protected void generateStepSize() {

		Polygon startp = startcp.getPolygon();
		int[] startxs = startp.xpoints;
		int[] startys = startp.ypoints;

		Polygon endp = endcp.getPolygon();
		int[] endxs = endp.xpoints;
		int[] endys = endp.ypoints;

		for (int i = 0; i <= startp.npoints - 1; i++) {
			xStep[i] = endxs[i] - startxs[i];
			xStep[i] = xStep[i]/steps;
			yStep[i] = endys[i] - startys[i];
			yStep[i] = yStep[i]/steps;
		}

	}


	public void doStep(int numberOfSteps) {
	
		Polygon p = startcp.getPolygon();
	
		int[] xs = p.xpoints;
		int[] ys = p.ypoints;
	
		// move points by step value
		for (int i = 0; i <= p.npoints - 1; i++) {
			xPoints[i] += xStep[i]*numberOfSteps;
			yPoints[i] += yStep[i]*numberOfSteps;
	
			// convert points back to polygon
			xs[i] = (int)xPoints[i];
			ys[i] = (int)yPoints[i];
		}


	}

}
