package rwd.views.smoother;

import java.awt.Polygon;
import java.util.Vector;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.swing.JLabel;

import rwd.concretesyntax.ConcreteUnitaryDiagram;
import rwd.concretesyntax.eulerdiagram.ConcreteContour;
import rwd.concretesyntax.spiderdiagram.ConcreteSpiderFoot;
import rwd.gui.editor.viewers.DiagramViewerInterface;
import rwd.metrics.AreaPolygonAdapter;
import rwd.metrics.DiagramArea;

public class FastHillClimber
	extends AbstractHillClimber 
{
	public FastHillClimber(
		List checkers,
		ConcreteUnitaryDiagram cud,
		Vector metrics,
		DiagramViewerInterface viewer,
		boolean moveContours,
		boolean moveFeet,
		boolean isAnimated) 
	{
		super(checkers, cud, metrics, viewer, moveContours, moveFeet, isAnimated);
		diagramArea = DiagramArea.getDiagramArea(cud);
	}

	protected void makeChange(int numberOfSteps, JLabel changeLabel) {
		//System.out.println("making change");
		setStopFlag(false);
		Set contours = cud.getConcreteContours();
		Set feet = cud.getConcreteFeet();
		Quality quality = currentQuality();
		double moveContourDistance = maxMoveContourDistance;
		double moveFeetDistance = maxMoveFeetDistance;
		
		// Measure performance	
		//		PrintStream out = System.out;
		//		try {
		//			out = new PrintStream(new FileOutputStream(new File("debug.dat")));
		//		} catch (Exception e) {
		//		}
		//		double refreshTime = 0;
		//		double checkTime = 0;
		//		double qualityTime = 0;
		//		double drawTime = 0;
		//		double totalTime = 0;

		// Repeat the process numberOfSteps times       
		for (int j = 1; j <= numberOfSteps; j++) {
			if (stopFlag) {
				viewer.redrawImage();
				return;
			}
			if (changeLabel != null) {
				changeLabel.setText(Integer.toString(j));
			}
			if (outputData) {
				/**/
				//System.out.println("");
				/**/
				//System.out.println("ITERATION in " + getClass() + ": " + j);
			}

			// Move contours
			if (moveContours)
				moveContours(contours, diagramArea, quality, moveContourDistance);
		
			// Move feet
			if (moveFeet)
				moveSpiderFeet(feet, diagramArea, quality, moveFeetDistance);

			if (cooling) {
				moveContourDistance = moveContourDistance - (maxMoveContourDistance / numberOfSteps);
				moveFeetDistance = moveFeetDistance - (maxMoveFeetDistance / numberOfSteps);
			}
			if (Math.random() > 0.5) {
				stepNumber++;
			}
		}

		// Print final measures
		//		out.println("Refresh time = "+refreshTime+" "+(int)(refreshTime/totalTime*100)+"%");
		//		out.println("Check time = "+checkTime+" "+(int)(checkTime/totalTime*100)+"%");
		//		out.println("Quality time = "+qualityTime+" "+(int)(qualityTime/totalTime*100)+"%");
		//		out.println("Draw time = "+drawTime+" "+(int)(drawTime/totalTime*100)+"%");
		//		out.println("Total time = "+totalTime+" "+(int)(totalTime/totalTime*100)+"%");
	}
	protected void moveContours(Set contours, double diagramArea, Quality quality, double move) {
		// For each contour
		Iterator k = contours.iterator();
		while (k.hasNext()) {
			ConcreteContour cp = (ConcreteContour) k.next();

			// For each point of the current contour
			Polygon p = cp.getPolygon();
			int[] xs = p.xpoints;
			int[] ys = p.ypoints;
			double pArea = AreaPolygonAdapter.getAreaOf(p);
			double sizeFactor = Math.sqrt(pArea / diagramArea);
			double adjustedMove = move * sizeFactor;
			for (int posNumber = 0; posNumber < p.npoints; posNumber++) {
				// Check if the stopFlag was set to false and
				// stop the climber
				if (stopFlag) {
					viewer.redrawImage();
					return;
				}
				int oldX = xs[posNumber];
				int oldY = ys[posNumber];

				// For each possible directions
				for (int direction = 0; direction <= 3; direction++) {
					double moveX = 0.0;
					double moveY = 0.0;
					if ((direction + stepNumber) % 4 == 0) {
						moveX = adjustedMove;
					}
					if ((direction + stepNumber) % 4 == 1) {
						moveX = -adjustedMove;
					}
					if ((direction + stepNumber) % 4 == 2) {
						moveY = adjustedMove;
					}
					if ((direction + stepNumber) % 4 == 3) {
						moveY = -adjustedMove;
					}

					xs[posNumber] += moveX;
					ys[posNumber] += moveY;
					
					// Notify observers (metrics and viewers)
					notifyObservers(cp, false, animated);

					// Check structure and quality
					boolean ok = structureStillOK();
					if (ok) {
						// Compute quality
						Quality newQuality = currentQuality();

						// If better quality accept position 
						if (newQuality.compare(quality) <= 0) {
							// ComputeQuality
							quality = newQuality;
							stepNumber++;
							// Notify observers (metrics and viewers)
							notifyObservers(cp, false, animated);
							break;
						// Move back
						} else {
							xs[posNumber] = oldX;
							ys[posNumber] = oldY;
						}
					// Move back
					} else {
						xs[posNumber] = oldX;
						ys[posNumber] = oldY;
					}
					p.invalidate();
				}
			}

			if (contourMove) {
				Vector oldPoints = getPolygonPoints(p);

				for (int direction = 0; direction <= 3; direction++) {
					double moveX = 0.0;
					double moveY = 0.0;
					if ((direction + stepNumber) % 4 == 0) {
						moveX = adjustedMove;
					}
					if ((direction + stepNumber) % 4 == 1) {
						moveX = -adjustedMove;
					}
					if ((direction + stepNumber) % 4 == 2) {
						moveY = adjustedMove;
					}
					if ((direction + stepNumber) % 4 == 3) {
						moveY = -adjustedMove;
					}

					movePolygonPoints(p, (int) moveX, (int) moveY);
					cud.refreshZones(false);
					
					// Check the new diagram
					boolean ok = structureStillOK();
					if (ok) {
						Quality newQuality = currentQuality();
						if (newQuality.compare(quality) < 0) {
							quality = newQuality;
							stepNumber++;
							if (animated) {
								viewer.redrawImage();
							}
							break;
						} else {
							setPolygonPoints(oldPoints, p);
						}
					} else {
						setPolygonPoints(oldPoints, p);
					}
				}
			}
			cp.autoPlaceLabel();
		}
		
	}
	protected void moveSpiderFeet(Set feet, double diagramArea, Quality quality, double move) {
		// For each feet
		Iterator i = feet.iterator();
		double adjustedMove = move;
		while (i.hasNext()) {
			ConcreteSpiderFoot foot = (ConcreteSpiderFoot) i.next();
			int oldX = foot.getX();
			int oldY = foot.getY();

			// For each possible directions
			for (int direction = 0; direction <= 7; direction++) {
				double moveX = 0.0;
				double moveY = 0.0;
				if ((direction + stepNumber) % 8 == 0) {
					moveX = adjustedMove;
				}
				if ((direction + stepNumber) % 8 == 1) {
					moveX = -adjustedMove;
				}
				if ((direction + stepNumber) % 8 == 2) {
					moveY = adjustedMove;
				}
				if ((direction + stepNumber) % 8 == 3) {
					moveY = -adjustedMove;
				}
				if ((direction + stepNumber) % 8 == 4) {
					moveX = adjustedMove;
					moveY = adjustedMove;
				}
				if ((direction + stepNumber) % 8 == 5) {
					moveX = -adjustedMove;
					moveY = adjustedMove;
				}
				if ((direction + stepNumber) % 8 == 6) {
					moveX = -adjustedMove;
					moveY = -adjustedMove;
				}
				if ((direction + stepNumber) % 8 == 7) {
					moveX = adjustedMove;
					moveY = -adjustedMove;
				}

				foot.setLocation((int)(oldX + moveX), (int)(oldY + moveY));

				// Notify observers
				notifyObservers(foot, false, animated);

				// Check structure
				boolean ok = structureStillOK();

				// If ok compute quality 
				if (ok) {
					// Compute quality
					Quality newQuality = currentQuality();

					// If better quality accept position 
					if (newQuality.compare(quality) <= 0) {
						// ComputeQuality
						quality = newQuality;
						stepNumber++;
						// Notify observers
						notifyObservers(foot, false, animated);
						break;
					// Move back
					} else {
						foot.setLocation(oldX, oldY);
					}
				// Move back
				} else {
					foot.setLocation(oldX, oldY);
				}
			}
		}		
	}

	protected int stepNumber = 0;
	protected double diagramArea;
	protected Runtime r = Runtime.getRuntime();
}
